
/*

CILanczosScaleTransform

Demo of scaling an image with the CILanczosScaleTransform filter.

Ken Shmidheiser
6 February 2020

*/

include "Tlbx CocoaUI.incl"
cocoainit

output file "Image Scaler"

include resources "waterfall.jpg"

_window = 1

begin enum output 1
_originalImageView
_scaledImageView
_aspectRatioSlider
_scaleSlider
_aspectValueField
_scaleValueField
_originalLabel
_scaledLabel
_aspectLabel
_ratioLabel
end enum


void local fn BuildWindow
'~'1
dim as SInt32   wndStyleMask
dim as CGRect   r

wndStyleMask  = NSTitledWindowMask
wndStyleMask += NSClosableWindowMask
wndStyleMask += NSMiniaturizableWindowMask

r = fn CGRectMake( 0, 0, 1070, 600 )
cocoa window _window, @"Image Scaler", r, wndStyleMask

r = fn CGRectMake( 20, 20, 440, 540 )
imageview _originalImageView, YES, @"waterfall.jpg", r, NSImageScaleAxesIndependently, NSImageAlignCenter, NSImageFramePhoto, _window

r = fn CGRectMake( 470, 20, 440, 540 )
imageview _scaledImageView, YES,, r, NSImageScaleNone, NSImageAlignCenter, NSImageFramePhoto, _window

// The additional horizontal scaling factor (stretch or shrink horizontally) to use on the image.
r = fn CGRectMake( 935, 40, 40, 510 )
slider _aspectRatioSlider, YES, 1.20, r, 0.01, 5.0, NO, _window
ControlSetContinuous( _aspectRatioSlider, YES )

// The scaling factor to proportionately shrink or enlarge the image.
r = fn CGRectMake( 990, 40, 40, 510 )
slider _scaleSlider, YES, 0.40, r, -0.01, 4.0, NO, _window
ControlSetContinuous( _scaleSlider, YES )

r = fn CGRectMake( 935, 20, 40, 17 )
textlabel _aspectValueField, @"0.4", r, _window
ControlSetAlignment( _aspectValueField, NSCenterTextAlignment )

r = fn CGRectMake( 990, 20, 40, 17 )
textlabel _scaleValueField, @"1.20", r, _window
ControlSetAlignment( _scaleValueField, NSCenterTextAlignment )

r = fn CGRectMake( 20, 570, 440, 17 )
textlabel _originalLabel, @"Original Image", r, _window
ControlSetAlignment( _originalLabel, NSCenterTextAlignment )

r = fn CGRectMake( 470, 570, 440, 17 )
textlabel _scaledLabel, @"Scaled Image", r, _window
ControlSetAlignment( _scaledLabel, NSCenterTextAlignment )

r = fn CGRectMake( 925, 550, 60, 42 )
textlabel _aspectLabel, @"Aspect Ratio", r, _window
ControlSetAlignment( _aspectLabel, NSCenterTextAlignment )

r = fn CGRectMake( 980, 550, 64, 42 )
textlabel _ratioLabel, @"Scale Factor %", r, _window
ControlSetAlignment( _ratioLabel, NSCenterTextAlignment )
end fn


local fn ImageRefToCIImage( image as ImageRef ) as CIImageRef
'~'1
dim as CFDataRef   dta
dim as CIImageRef  ciImage

dta = fn ImageTIFFRepresentationUsingCompression( image, NSTIFFCompressionNone, 0.0 )
ciImage = fn CIImageWithData( dta )
end fn = ciImage


local fn CIImageToImageRef( ciImage as CIImageRef ) as ImageRef
'~'1
dim as CIImageRepRef  rep
dim as CGSize         size
dim as ImageRef       image

rep = fn CIImageRepWithCIImage( ciImage )
size = fn ImageRepSize( rep )
image = fn ImageWithSize( size )
ImageAddRepresentation( image, rep )
end fn = image


local fn ScaleImageWithAspectRatio( inputCIImage as CIImageRef, aspectRatio as CGFloat, scale as CGFloat ) as CIImageRef
'~'1
dim as CIImageRef   scaledCIImage
dim as CIFilterRef  scaleFilter

scaleFilter = fn CIFilterWithName( @"CILanczosScaleTransform" )
ObjectSetValueForkey( scaleFilter, inputCIImage, @"inputImage" )
ObjectSetValueForkey( scaleFilter, @(scale), @"inputScale" )
ObjectSetValueForkey( scaleFilter, @(aspectRatio), @"inputAspectRatio" )
scaledCIImage = fn CIFilterOutputImage( scaleFilter )
end fn = scaledCIImage


void local fn ScaleImage
'~'1
dim as double      aspectRatio, scaleFactor
dim as ImageRef    originalImage, scaledImage
dim as CIImageRef  ciImage, ciScaledImage

aspectRatio = fn ControlDoubleValue( _aspectRatioSlider )
scaleFactor = fn ControlDoubleValue( _scaleSlider )

originalImage = fn ImageViewImage( _originalImageView )
ciImage = fn ImageRefToCIImage( originalImage )
ciScaledImage = fn ScaleImageWithAspectRatio( ciImage, aspectRatio, scaleFactor )
scaledImage = fn CIImageToImageRef( ciScaledImage )
ImageViewSetImage( _scaledImageView, scaledImage )
end fn


void local fn DoDialog( ev as long, tag as long, wnd as long, obj as CFTypeRef )
'~'1
dim as double aspectRatio, scaleFactor

select (ev)
case _btnClick
select (tag )
case _aspectRatioSlider
aspectRatio = fn ControlDoubleValue( tag )
ControlSetDoubleValue( _aspectValueField, aspectRatio )
fn ScaleImage
case _scaleSlider
scaleFactor = fn ControlDoubleValue( tag )
ControlSetDoubleValue( _scaleValueField, scaleFactor )
fn ScaleImage
end select

case _windowWillClose : end
end select
end fn

on dialog fn DoDialog

fn BuildWindow
fn ScaleImage

HandleEvents

