/*
How to put C code in a project and call C functions from FB.

The file-based methods (new in FB 5.4) have advantages over _PASSTHROUGHFUNCTION:
- can edit in Xcode instead of FB editor, avoiding indent problems and incomplete structure warnings.
- Objective-C code can be included without requiring the 'Use Objective-C' settings checkbox to be on.
- the files can be added to a project window; in this standalone demo we use instead  include "SomeSourceFile.c"

When you include a C source file of any kind, to avoid linker errors you must also  
include a .h file that declares all symbols to be exported.
 - functions to be called from FB code must be prototyped in a .h file
 - functions private to the C source file need not be declared in a .h file

   rp 20090904
*/

include "Tlbx CocoaUI.incl"
cocoainit
include "NSLog.incl"

// plain C embedded in translated FB code
BeginCDeclaration
int PlusOnePASSTHROUGH( int x );
EndC

BeginCFunction
int PlusOnePASSTHROUGH( int x ) 
{ 
 return x + 1; 
}
EndC
toolbox fn PlusOnePASSTHROUGH( long ) = long


/*
 plain C file
*/
include "Test.h"
include "Test.c"
toolbox fn PlusOne( long ) = long


/*
 Objective-C file
 The compiler recognises the extension .m
 FBtoC adds the linker command -framework AppKit
*/
include "TestObjC.h"
include "TestObjC.m"
toolbox fn PlusOneObjC( long ) = long


// exercise the methods of calling C code from FB
NSLog(@"%ld\t%ld\t%ld",fn PlusOne( 0 ), fn PlusOnePASSTHROUGH( 1 ), fn PlusOneObjC( 2 ))

HandleEvents





