
/*
   CFArray Additions demo

   Bernie Wylde 20091224

*/

include "ConsoleWindow"
include "CFArray Additions.incl"
include "CFDictionary Additions.incl"
include "CFSortDescriptor.incl"


local fn PrintArrayOfStrings( array as CFArrayRef )
'~'1
dim as CFIndex   count, i

count = fn CFArrayGetCount( array )
long if ( count )
print "index","value"
print "-----","-----"
for i = 0 to count - 1
print i,
fn ConsolePrintCFString( fn CFArrayGetValueAtIndex( array, i ) )
next i
end if
end fn

local fn PrintIndexSet( set as CFIndexSetRef )
'~'1
dim as CFIndex   index

index = fn CFIndexSetGetFirstIndex( set )
while ( index != _kCFIndexSetIndexNotFound )
print index
index = fn CFIndexSetGetIndexGreaterThanIndex( set, index )
wend
end fn

local fn PrintArrayOfDicts( array as CFArrayRef )
'~'1
dim as CFDictionaryRef   dict
dim as CFIndex           index, count
dim as Str255            value1, value2
dim as long              value3

print "index","""key1""","""key2""","""key3"""
print "-----","------","------","------"
count = fn CFArrayGetCount( array )
for index = 0 to count - 1
dict = fn CFArrayGetValueAtIndex( array, index )
fn CFStringGetPascalString( fn CFDictionaryGetValue( dict, @"key1" ), @value1, 256, _kCFStringEncodingMacRoman )
fn CFStringGetPascalString( fn CFDictionaryGetValue( dict, @"key2" ), @value2, 256, _kCFStringEncodingMacRoman )
fn CFNumberGetValue( fn CFDictionaryGetValue( dict, @"key3" ), _kCFNumberSInt32Type, @value3 )
print index,value1,value2,value3
next index
end fn


dim as CFMutableArrayRef      mutArray
dim as CFArrayRef             array1, array2, sortedArray, descriptors
dim as CFMutableIndexSetRef   set
dim as CFSortDescriptorRef    sd1, sd2, sd3
dim as CFDictionaryRef        dict
dim as CFEnumeratorRef        enumerator
dim as CFStringRef            value1, value2
dim as CFNumberRef            value3
dim as CFIndex                index
dim as SInt32                 value
dim as CFURLRef               url

window _FBConsoleWndNum, @"CFArray Additions"

// CFArrayCreateWithValues
print "CFArrayCreateWithValues"
array1 = fn CFArrayCreateWithValues( @"alpha", @"bravo", @"charlie", @"delta", @"echo", 0 )
fn PrintArrayOfStrings( array1 )

// CFArrayIsEqualToArray
print : print "CFArrayCreateCopy"
array2 = fn CFArrayCreateCopy( _kCFAllocatorDefault, array1 )
print "CFArrayIsEqualToArray"
print fn CFArrayIsEqualToArray( array1, array2 )
CFRelease( array2 )

// CFArrayCreateSubArrayWithRange
print : print "CFArrayCreateSubArrayWithRange - 1,3"
array2 = fn CFArrayCreateSubArrayWithRange( array1, fn CFRangeMake( 1, 3 ) )
fn PrintArrayOfStrings( array2 )

// CFArrayGetFirstValueCommonWithArray
print : print "CFArrayGetFirstValueCommonWithArray"
fn ConsolePrintCFString( fn CFArrayGetFirstValueCommonWithArray( array1, array2 ) )
CFRelease( array2 )

// CFArrayGetLastValue
print : print "CFArrayGetLastValue"
fn ConsolePrintCFString( fn CFArrayGetLastValue( array1 ) )

// CFArrayGetFirstIndexOfValueInRange
print : print "CFArrayGetFirstIndexOfValueInRange - ""delta"" 1,4"
print fn CFArrayGetFirstIndexOfValueInRange( array1, @"delta", fn CFRangeMake( 1, 4 ) )
CFRelease( array1 )

// CFArrayCreateMutable
print : print "CFArrayCreateMutable"
mutArray = fn CFArrayCreateMutable( _kCFAllocatorDefault, 0, @kCFTypeArrayCallBacks )
CFArrayAppendValue( mutArray, @"mike" )
CFArrayAppendValue( mutArray, @"november" )
CFArrayAppendValue( mutArray, @"mike" )
CFArrayAppendValue( mutArray, @"oscar" )
CFArrayAppendValue( mutArray, @"november" )
CFArrayAppendValue( mutArray, @"november" )
CFArrayAppendValue( mutArray, @"november" )
CFArrayAppendValue( mutArray, @"oscar" )
CFArrayAppendValue( mutArray, @"papa" )
CFArrayAppendValue( mutArray, @"sierra" )
CFArrayAppendValue( mutArray, @"november" )
fn PrintArrayOfStrings( mutArray )

// CFArrayRemoveLastValue
print : print "CFArrayRemoveLastValue"
CFArrayRemoveLastValue( mutArray )
fn PrintArrayOfStrings( mutArray )

// CFArrayRemoveValue
print : print "CFArrayRemoveValue - ""mike"""
CFArrayRemoveValue( mutArray, @"mike" )
fn PrintArrayOfStrings( mutArray )

// CFArrayRemoveValueInRange
print : print "CFArrayRemoveValueInRange - ""november"" 0,3"
CFArrayRemoveValueInRange( mutArray, @"november", fn CFRangeMake( 0, 3 ) )
fn PrintArrayOfStrings( mutArray )

// CFArrayRemoveValuesInArray
print : print "CFArrayCreateWithValues"
array1 = fn CFArrayCreateWithValues( @"oscar", @"papa", 0 )
fn PrintArrayOfStrings( mutArray )
print : print "CFArrayRemoveValuesInArray"
CFArrayRemoveValuesInArray( mutArray, array1 )
CFRelease( array1 )
fn PrintArrayOfStrings( mutArray )

// CFArrayRemoveValuesInRange
print : print "CFArrayRemoveValuesInRange - 0,2"
CFArrayRemoveValuesInRange( mutArray, fn CFRangeMake( 0, 2 ) )
fn PrintArrayOfStrings( mutArray )

// CFArrayInsertValuesAtIndexes
print : print "CFArrayInsertValuesAtIndexes"
set = fn CFIndexSetCreateWithIndexesInRange( fn CFRangeMake( 0, 2 ) )
array1 = fn CFArrayCreateWithValues( @"alpha", @"bravo", 0 )
CFArrayInsertValuesAtIndexes( mutArray, array1, set )
CFRelease( array1 )
CFRelease( set )
fn PrintArrayOfStrings( mutArray )

// CFArrayReplaceValuesAtIndexes
print : print "CFArrayReplaceValuesAtIndexes"
set = fn CFIndexSetCreateWithIndexesInRange( fn CFRangeMake( 1, 2 ) )
array1 = fn CFArrayCreateWithValues( @"xray", @"yankee", 0 )
CFArrayReplaceValuesAtIndexes( mutArray, array1, set )
CFRelease( array1 )
fn PrintArrayOfStrings( mutArray )
CFRelease( set )
CFRelease( mutArray )

// CFArrayGetFirstIndexOfValueIdenticalTo
array1 = fn CFArrayCreateWithValues( @"alpha", @"bravo", @"charlie", @"delta", @"echo", 0 )
print : print "CFArrayGetFirstIndexOfValueIdenticalTo - ""charlie"""
print fn CFArrayGetFirstIndexOfValueIdenticalTo( array1, @"charlie" )
CFRelease( array1 )

// CFArrayGetFirstIndexOfValueIdenticalToInRange
array1 = fn CFArrayCreateWithValues( @"alpha", @"bravo", @"charlie", @"delta", @"echo", 0 )
print : print "CFArrayGetFirstIndexOfValueIdenticalToInRange - ""charlie"""
print fn CFArrayGetFirstIndexOfValueIdenticalToInRange( array1, @"charlie", fn CFRangeMake( 1, 2 ) )
CFRelease( array1 )

// CFArrayWriteToURL
array1 = fn CFArrayCreateWithValues( @"mike", @"november", @"oscar", @"papa", 0 )
long if ( files$( _CFURLRefSave,,, url ) )
print : print "CFArrayCreateWithValues"
print "CFArrayWriteToURL"
fn CFArrayWriteToURL( array1, url )
CFRelease( url )
end if
CFRelease( array1 )

// CFArrayCreateWithContentsOfURL
long if ( files$( _CFURLRefOpen,,, url ) )
print : print "CFArrayCreateWithContentsOfURL"
array1 = fn CFArrayCreateWithContentsOfURL( url )
fn PrintArrayOfStrings( array1 )
CFRelease( array1 )
CFRelease( url )
end if

// CFArrayCreateByAddingValue
print : print "CFArrayCreateWithValues"
print "CFArrayCreateByAddingValue - ""echo"""
array1 = fn CFArrayCreateWithValues( @"alpha", @"bravo", @"charlie", @"delta", 0 )
array2 = fn CFArrayCreateByAddingValue( array1, @"echo" )
CFRelease( array1 )
fn PrintArrayOfStrings( array2 )

// CFArrayCreateDescription
print : print "CFArrayCreateDescription"
value1 = fn CFArrayCreateDescription( array2 )
CFRelease( array2 )
fn ConsolePrintCFString( value1 )
CFRelease( value1 )


print : print "--------------------------------------------------------------------------"
print "CFIndexSet"
print "--------------------------------------------------------------------------"

// CFArrayCreateMutable
print "CFArrayCreateMutable"
mutArray = fn CFArrayCreateMutable( _kCFAllocatorDefault, 0, @kCFTypeArrayCallBacks )
CFArrayAppendValue( mutArray, @"alpha" )
CFArrayAppendValue( mutArray, @"bravo" )
CFArrayAppendValue( mutArray, @"charlie" )
CFArrayAppendValue( mutArray, @"delta" )
CFArrayAppendValue( mutArray, @"echo" )
CFArrayAppendValue( mutArray, @"foxtrot" )
CFArrayAppendValue( mutArray, @"golf" )
CFArrayAppendValue( mutArray, @"hotel" )
CFArrayAppendValue( mutArray, @"india" )
fn PrintArrayOfStrings( mutArray )

// CFIndexSetCreateMutable
print : print "CFIndexSetCreateMutable"
set = fn CFIndexSetCreateMutable()
print "CFIndexSetAddIndex"
CFIndexSetAddIndex( set, 2 )
CFIndexSetAddIndex( set, 0 )
CFIndexSetAddIndex( set, 5 )

// enumerate set
fn PrintIndexSet( set )

// CFArrayCreateArrayOfValuesAtIndexes
print : print "CFArrayCreateArrayOfValuesAtIndexes"
array1 = fn CFArrayCreateArrayOfValuesAtIndexes( mutArray, set )
fn PrintArrayOfStrings( array1 )
CFRelease( array1 )

// CFArrayRemoveValuesAtIndexes
print : print "CFArrayRemoveValuesAtIndexes"
CFArrayRemoveValuesAtIndexes( mutArray, set )
fn PrintArrayOfStrings( mutArray )

// CFArrayInsertValuesAtIndexes
print : print "CFArrayInsertValuesAtIndexes"
array1 = fn CFArrayCreateWithValues( @"juliet", @"kilo", @"lima", 0 )
CFArrayInsertValuesAtIndexes( mutArray, array1, set )
CFRelease( array1 )
fn PrintArrayOfStrings( mutArray )

// CFArrayReplaceValuesAtIndexes
print : print "CFArrayReplaceValuesAtIndexes"
array1 = fn CFArrayCreateWithValues( @"quebec", @"romeo", @"sierra", 0 )
CFArrayReplaceValuesAtIndexes( mutArray, array1, set )
CFRelease( array1 )
fn PrintArrayOfStrings( mutArray )

CFRelease( mutArray )
CFRelease( set )


print : print "--------------------------------------------------------------------------"
print "CFSortDescriptors"
print "--------------------------------------------------------------------------"

// CFArrayCreateMutable
print "CFArrayCreateMutable"
mutArray = fn CFArrayCreateMutable( _kCFAllocatorDefault, 0, @kCFTypeArrayCallBacks )
for index = 1 to 10
select ( rnd( 3 ) )
case 1 : value1 = @"alpha"
case 2 : value1 = @"bravo"
case 3 : value1 = @"charlie"
end select
select ( rnd( 2 ) )
case 1 : value2 = @"alpha"
case 2 : value2 = @"bravo"
end select
value = rnd( 9 )
value3 = fn CFNumberCreate( _kCFAllocatorDefault, _kCFNumberSInt32Type, @value )
dict = fn CFDictionaryCreateWithValuesAndKeys( value1, @"key1", value2, @"key2", value3, @"key3", 0 )
CFRelease( value3 )
CFArrayAppendValue( mutArray, dict )
CFRelease( dict )
next index
print "Array of dicts"
print : print "Unsorted"
fn PrintArrayOfDicts( mutArray )

// CFSortDescriptorCreate
sd1 = fn CFSortDescriptorCreate( @"key1", _true )
sd2 = fn CFSortDescriptorCreate( @"key2", _true )
sd3 = fn CFSortDescriptorCreate( @"key3", _false )
descriptors = fn CFArrayCreateWithValues( sd1, sd2, sd3, 0 )
CFRelease( sd1 )
CFRelease( sd2 )
CFRelease( sd3 )

// CFArrayCreateSortedArrayUsingDescriptors
sortedArray = fn CFArrayCreateSortedArrayUsingDescriptors( mutArray, descriptors )
CFRelease( mutArray )
CFRelease( descriptors )
print : print "Sorted","[ascending]","[ascending]","[descending]"
fn PrintArrayOfDicts( sortedArray )
CFRelease( sortedArray )


print : print "--------------------------------------------------------------------------"
print "CFEnumerator"
print "--------------------------------------------------------------------------"

array1 = fn CFArrayCreateWithValues( @"one", @"two", @"three", @"four", @"five", 0 )
enumerator = fn CFArrayCreateEnumerator( array1 )
value1 = fn CFEnumeratorGetNextObject( enumerator )
while ( value1 )
fn ConsolePrintCFString( value1 )
value1 = fn CFEnumeratorGetNextObject( enumerator )
wend
CFRelease( enumerator )
CFRelease( array1 )







