//
//  CFArray Additions.h
//
//  Bernie Wylde 20091219
//

#include "CFIndexSet.h"
#include "CFEnumerator.h"

#ifndef CFARRAY_ADDITIONS_H
#define CFARRAY_ADDITIONS_H

// create
CFArrayRef CFArrayCreateWithContentsOfURL( CFURLRef url );
CFArrayRef CFArrayCreateWithContentsOfFile( FSRef *ref );
CFArrayRef CFArrayCreateArrayOfValuesAtIndexes( CFArrayRef inArray, CFIndexSetRef set );
CFArrayRef CFArrayCreateWithValues( CFTypeRef val, ... );

// compare
Boolean CFArrayIsEqualToArray( CFArrayRef array1, CFArrayRef array2 );
CFTypeRef CFArrayGetFirstValueCommonWithArray( CFArrayRef array1, CFArrayRef array2 );

// deriving new arrays
CFArrayRef CFArrayCreateByAddingValue( CFArrayRef array, CFTypeRef value );
CFArrayRef CFArrayCreateSubArrayWithRange( CFArrayRef array, CFRange range );

// sorting
CFArrayRef CFArrayCreateSortedArrayUsingDescriptors( CFArrayRef array, CFArrayRef descriptors );

// query
CFTypeRef CFArrayGetLastValue( CFArrayRef array );
CFEnumeratorRef CFArrayCreateEnumerator( CFArrayRef array );
CFEnumeratorRef CFArrayCreateReverseEnumerator( CFArrayRef array );

// find
CFIndex CFArrayGetFirstIndexOfValueInRange( CFArrayRef array, CFTypeRef value, CFRange range );
CFIndex CFArrayGetFirstIndexOfValueIdenticalTo( CFArrayRef array, CFTypeRef value );
CFIndex CFArrayGetFirstIndexOfValueIdenticalToInRange( CFArrayRef array, CFTypeRef value, CFRange range );

// add
void CFArrayInsertValuesAtIndexes( CFMutableArrayRef inArray, CFArrayRef values, CFIndexSetRef set );

// remove
void CFArrayRemoveLastValue( CFMutableArrayRef array );
void CFArrayRemoveValue( CFMutableArrayRef array, CFTypeRef value );
void CFArrayRemoveValueInRange( CFMutableArrayRef array, CFTypeRef value, CFRange range );
void CFArrayRemoveValuesInArray( CFMutableArrayRef array1, CFArrayRef array2 );
void CFArrayRemoveValuesInRange( CFMutableArrayRef array, CFRange range );
void CFArrayRemoveValuesAtIndexes( CFMutableArrayRef inArray, CFIndexSetRef set );

// replace
void CFArrayReplaceValuesAtIndexes( CFMutableArrayRef inArray, CFArrayRef values, CFIndexSetRef set );

// store
Boolean CFArrayWriteToURL( CFArrayRef array, CFURLRef url );
Boolean CFArrayWriteToPath( CFArrayRef array, CFStringRef path );
Boolean CFArrayWriteToFile( CFArrayRef array, FSRef *parRef, CFStringRef filename );

// description
CFStringRef CFArrayCreateDescription( CFArrayRef array );

#endif	/*CFARRAY_ADDITIONS_H*/
