//
//  CFArray Additions.m
//
//  Bernie Wylde 20091219
//

#import <Foundation/Foundation.h>
#import "CFArray_Additions.h"

// create
CFArrayRef CFArrayCreateWithContentsOfURL( CFURLRef url )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef array = (CFArrayRef)[[NSArray alloc] initWithContentsOfURL:(NSURL *)url];
	[localPool drain];
	return array;
}

CFArrayRef CFArrayCreateWithContentsOfFile( FSRef *ref )
{	
	CFArrayRef array = NULL;
	CFURLRef url = CFURLCreateFromFSRef( kCFAllocatorDefault, ref );
	if ( url != NULL ) {
		array = CFArrayCreateWithContentsOfURL( url );
		CFRelease( url );
	}
	return array;
}

CFArrayRef CFArrayCreateArrayOfValuesAtIndexes( CFArrayRef inArray, CFIndexSetRef set )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef outArray = (CFArrayRef)[[(NSArray *)inArray objectsAtIndexes:(NSIndexSet *)set] retain];
	[localPool drain];
	return outArray;
}

CFArrayRef CFArrayCreateWithValues( CFTypeRef val, ... )
{
	va_list ap;
	CFMutableArrayRef tempArray = CFArrayCreateMutable( kCFAllocatorDefault, 0, &kCFTypeArrayCallBacks );
	va_start( ap, val );
	while ( val != NULL ) {
		CFArrayAppendValue( tempArray, val );
		val = va_arg( ap, CFTypeRef );
	}
	va_end( ap );
	CFArrayRef array = CFArrayCreateCopy( kCFAllocatorDefault, tempArray );
	CFRelease( tempArray );
	return array;
}


// compare
Boolean CFArrayIsEqualToArray( CFArrayRef array1, CFArrayRef array2 )
{ return [(NSArray *)array1 isEqualToArray:(NSArray *)array2]; }

CFTypeRef CFArrayGetFirstValueCommonWithArray( CFArrayRef array1, CFArrayRef array2 )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFTypeRef value = (CFTypeRef)[(NSArray *)array1 firstObjectCommonWithArray:(NSArray *)array2];
	[localPool drain];
	return value;
}

// deriving new arrays
CFArrayRef CFArrayCreateByAddingValue( CFArrayRef inArray, CFTypeRef value )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef outArray = (CFArrayRef)[[(NSArray *)inArray arrayByAddingObject:(id)value] retain];
	[localPool drain];
	return outArray;
}

CFArrayRef CFArrayCreateSubArrayWithRange( CFArrayRef array, CFRange range )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];	
	CFArrayRef subArray = (CFArrayRef)[[(NSArray *)array subarrayWithRange:(*(NSRange*)&range)] retain];
	[localPool drain];
	return subArray;
}

// sorting
CFArrayRef CFArrayCreateSortedArrayUsingDescriptors( CFArrayRef array, CFArrayRef descriptors )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];	
	CFArrayRef outArray = (CFArrayRef)[[(NSArray *)array sortedArrayUsingDescriptors:(NSArray *)descriptors] retain];
	[localPool drain];
	return outArray;
}

// query
CFTypeRef CFArrayGetLastValue( CFArrayRef array )
{ return (CFTypeRef)[(NSArray *)array lastObject]; }

CFEnumeratorRef CFArrayCreateEnumerator( CFArrayRef array )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFEnumeratorRef enumerator = (CFEnumeratorRef)[[(NSArray *)array objectEnumerator] retain];
	[localPool drain];
	return enumerator;
}

CFEnumeratorRef CFArrayCreateReverseEnumerator( CFArrayRef array )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFEnumeratorRef enumerator = (CFEnumeratorRef)[[(NSArray *)array reverseObjectEnumerator] retain];
	[localPool drain];
	return enumerator;
}

// find
CFIndex CFArrayGetFirstIndexOfValueInRange( CFArrayRef array, CFTypeRef value, CFRange range )
{ return (CFIndex)[(NSArray *)array indexOfObject:(id)value inRange:(*(NSRange*)&range)]; }

CFIndex CFArrayGetFirstIndexOfValueIdenticalTo( CFArrayRef array, CFTypeRef value )
{ return (CFIndex)[(NSArray *)array indexOfObjectIdenticalTo:(id)value]; }

CFIndex CFArrayGetFirstIndexOfValueIdenticalToInRange( CFArrayRef array, CFTypeRef value, CFRange range )
{ return (CFIndex)[(NSArray *)array indexOfObjectIdenticalTo:(id)value inRange:(*(NSRange*)&range)]; }

// add
void CFArrayInsertValuesAtIndexes( CFMutableArrayRef inArray, CFArrayRef values, CFIndexSetRef set )
{ [(NSMutableArray *)inArray insertObjects:(NSArray *)values atIndexes:(NSIndexSet *)set]; }

// remove
void CFArrayRemoveLastValue( CFMutableArrayRef array )
{ [(NSMutableArray *)array removeLastObject]; }

void CFArrayRemoveValue( CFMutableArrayRef array, CFTypeRef value )
{ [(NSMutableArray *)array removeObject:(id)value]; }

void CFArrayRemoveValueInRange( CFMutableArrayRef array, CFTypeRef value, CFRange range )
{ [(NSMutableArray *)array removeObject:(id)value inRange:(*(NSRange*)&range)]; }

void CFArrayRemoveValuesInArray( CFMutableArrayRef array1, CFArrayRef array2 )
{ [(NSMutableArray *)array1 removeObjectsInArray:(NSArray *)array2]; }

void CFArrayRemoveValuesInRange( CFMutableArrayRef array, CFRange range )
{ [(NSMutableArray *)array removeObjectsInRange:(*(NSRange*)&range)]; }

void CFArrayRemoveValuesAtIndexes( CFMutableArrayRef inArray, CFIndexSetRef set )
{ [(NSMutableArray *)inArray removeObjectsAtIndexes:(NSIndexSet *)set]; }

// replace
void CFArrayReplaceValuesAtIndexes( CFMutableArrayRef inArray, CFArrayRef values, CFIndexSetRef set )
{ [(NSMutableArray *)inArray replaceObjectsAtIndexes:(NSIndexSet *)set withObjects:(NSArray *)values]; }

// store
Boolean CFArrayWriteToURL( CFArrayRef array, CFURLRef url )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSArray *)array writeToURL:(NSURL *)url atomically:YES];
	[localPool drain];
	return result;
}

Boolean CFArrayWriteToPath( CFArrayRef array, CFStringRef path )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSArray *)array writeToFile:(NSString *)path atomically:YES];
	[localPool drain];
	return result;
}

Boolean CFArrayWriteToFile( CFArrayRef array, FSRef *parRef, CFStringRef filename )
{
	Boolean result = false;
	CFURLRef parUrl = CFURLCreateFromFSRef( kCFAllocatorDefault, parRef );
	if ( parUrl != NULL ) {
		CFURLRef url = CFURLCreateCopyAppendingPathComponent( kCFAllocatorDefault, parUrl, filename, false );
		CFRelease( parUrl );
		if ( url != NULL ) {
			result = CFArrayWriteToURL( array, url );
			CFRelease( url );
		}
	}
	return result;
}

// description
CFStringRef CFArrayCreateDescription( CFArrayRef array )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFStringRef string = (CFStringRef)[[(NSArray *)array description] retain];
	[localPool drain];
	return string;
}




