//
//  CFData Additions.m
//
//  Bernie Wylde 20100103
//

#import <Foundation/Foundation.h>
#import "CFData_Additions.h"

// create
CFDataRef CFDataCreateWithContentsOfURL( CFURLRef url )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFDataRef data = (CFDataRef)[[NSData alloc] initWithContentsOfURL:(NSURL *)url];
	[localPool drain];
	return data;
}

CFDataRef CFDataCreateWithContentsOfFile( FSRef *ref )
{	
	CFDataRef data = NULL;
	CFURLRef url = CFURLCreateFromFSRef( kCFAllocatorDefault, ref );
	if ( url != NULL ) {
		data = CFDataCreateWithContentsOfURL( url );
		CFRelease( url );
	}
	return data;
}


// compare
Boolean CFDataIsEqualToData( CFDataRef data1, CFDataRef data2 )
{ return [(NSData *)data1 isEqualToData:(NSData *)data2]; }


// store
Boolean CFDataWriteToURL( CFDataRef data, CFURLRef url )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSData *)data writeToURL:(NSURL *)url atomically:YES];
	[localPool drain];
	return result;
}

Boolean CFDataWriteToPath( CFDataRef data, CFStringRef path )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSData *)data writeToFile:(NSString *)path atomically:YES];
	[localPool drain];
	return result;
}

Boolean CFDataWriteToFile( CFDataRef data, FSRef *parRef, CFStringRef filename )
{
	Boolean result = false;
	CFURLRef parUrl = CFURLCreateFromFSRef( kCFAllocatorDefault, parRef );
	if ( parUrl != NULL ) {
		CFURLRef url = CFURLCreateCopyAppendingPathComponent( kCFAllocatorDefault, parUrl, filename, false );
		CFRelease( parUrl );
		if ( url != NULL ) {
			result = CFDataWriteToURL( data, url );
			CFRelease( url );
		}
	}
	return result;
}


// description
CFStringRef CFDataCreateDescription( CFDataRef data )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFStringRef string = (CFStringRef)[[(NSData *)data description] retain];
	[localPool drain];
	return string;
}




