//
//  CFDictionary Additions.m
//
//  Bernie Wylde 20091222
//

#import <Foundation/Foundation.h>
#import "CFDictionary_Additions.h"

// create
CFDictionaryRef CFDictionaryCreateWithContentsOfURL( CFURLRef url )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFDictionaryRef dict = (CFDictionaryRef)[[NSDictionary alloc] initWithContentsOfURL:(NSURL *)url];
	[localPool drain];
	return dict;
}

CFDictionaryRef CFDictionaryCreateWithContentsOfFile( FSRef *ref )
{
	CFDictionaryRef dict = NULL;
	CFURLRef url = CFURLCreateFromFSRef( kCFAllocatorDefault, ref );
	if ( url != NULL ) {
		dict = CFDictionaryCreateWithContentsOfURL( url );
		CFRelease( url );
	}
	return dict;	
}

CFDictionaryRef CFDictionaryCreateWithValuesForKeys( CFArrayRef values, CFArrayRef keys )
{
	return (CFDictionaryRef)[[NSDictionary alloc] initWithObjects:(NSArray *)values forKeys:(NSArray *)keys];
}

CFDictionaryRef CFDictionaryCreateWithValuesAndKeys( CFTypeRef val, ... )
{
	CFStringRef key;
	va_list ap;
	CFMutableDictionaryRef tempDict = CFDictionaryCreateMutable( kCFAllocatorDefault, 0, &kCFCopyStringDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks );
	va_start( ap, val );
	while ( val != NULL ) {
		key = va_arg( ap, CFStringRef );
		CFDictionaryAddValue( tempDict, key, val );
		val = va_arg( ap, CFTypeRef );
	}
	va_end( ap );
	CFDictionaryRef dict = CFDictionaryCreateCopy( kCFAllocatorDefault, tempDict );	
	CFRelease( tempDict );
	return dict;
}

// add
void CFDictionaryAddDictionary( CFMutableDictionaryRef dict, CFDictionaryRef otherDict )
{ [(NSMutableDictionary *)dict addEntriesFromDictionary:(NSDictionary *)otherDict]; }

void CFDictionarySetDictionary( CFMutableDictionaryRef dict, CFDictionaryRef otherDict )
{ [(NSMutableDictionary *)dict setDictionary:(NSDictionary *)otherDict]; }

// remove
void CFDictionaryRemoveValuesForKeys( CFMutableDictionaryRef dict, CFArrayRef keys )
{ [(NSMutableDictionary *)dict removeObjectsForKeys:(NSArray *)keys]; }

// compare
Boolean CFDictionaryIsEqualToDictionary( CFDictionaryRef dict1, CFDictionaryRef dict2 )
{ return [(NSDictionary *)dict1 isEqualToDictionary:(NSDictionary *)dict2]; }

// accessing keys and values
CFArrayRef CFDictionaryCreateArrayOfAllKeys( CFDictionaryRef dict )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef array = (CFArrayRef)[[(NSDictionary *)dict allKeys] retain];
	[localPool drain];
	return array;
}

CFArrayRef CFDictionaryCreateArrayOfAllKeysForValue( CFDictionaryRef dict, CFTypeRef value )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef array = (CFArrayRef)[[(NSDictionary *)dict allKeysForObject:(id)value] retain];
	[localPool drain];
	return array;
}

CFArrayRef CFDictionaryCreateArrayOfAllValues( CFDictionaryRef dict )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef array = (CFArrayRef)[[(NSDictionary *)dict allValues] retain];
	[localPool drain];
	return array;
}

CFArrayRef CFDictionaryCreateArrayOfValuesForKeys( CFDictionaryRef dict, CFArrayRef keys, CFTypeRef notFoundMarker )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFArrayRef array = (CFArrayRef)[[(NSDictionary *)dict objectsForKeys:(NSArray *)keys notFoundMarker:(id)notFoundMarker] retain];
	[localPool drain];
	return array;
}

// enumerate
CFEnumeratorRef CFDictionaryCreateKeyEnumerator( CFDictionaryRef dict )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFEnumeratorRef enumerator = (CFEnumeratorRef)[[(NSDictionary *)dict keyEnumerator] retain];
	[localPool drain];
	return enumerator;	
}

CFEnumeratorRef CFDictionaryCreateValueEnumerator( CFDictionaryRef dict )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFEnumeratorRef enumerator = (CFEnumeratorRef)[[(NSDictionary *)dict objectEnumerator] retain];
	[localPool drain];
	return enumerator;		
}

// store
Boolean CFDictionaryWriteToURL( CFDictionaryRef dict, CFURLRef url )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSDictionary *)dict writeToURL:(NSURL *)url atomically:YES];
	[localPool drain];
	return result;
}

Boolean CFDictionaryWriteToPath( CFDictionaryRef dict, CFStringRef path )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSDictionary *)dict writeToFile:(NSString *)path atomically:YES];
	[localPool drain];
	return result;
}

Boolean CFDictionaryWriteToFile( CFDictionaryRef dict, FSRef *parRef, CFStringRef filename )
{
	Boolean result = false;
	CFURLRef parUrl = CFURLCreateFromFSRef( kCFAllocatorDefault, parRef );
	if ( parUrl != NULL ) {
		CFURLRef url = CFURLCreateCopyAppendingPathComponent( kCFAllocatorDefault, parUrl, filename, false );
		CFRelease( parUrl );
		if ( url != NULL ) {
			result = CFDictionaryWriteToURL( dict, url );
			CFRelease( url );
		}
	}
	return result;	
}

// description
CFStringRef CFDictionaryCreateDescription( CFDictionaryRef dict )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFStringRef string = (CFStringRef)[[(NSDictionary *)dict description] retain];
	[localPool drain];
	return string;
}

CFStringRef CFDictionaryCreateDescriptionInStringsFileFormat( CFDictionaryRef dict )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFStringRef string = (CFStringRef)[[(NSDictionary *)dict descriptionInStringsFileFormat] retain];
	[localPool drain];
	return string;
}




