//
//  CFScanner.m
//  
//  Bernie Wylde 20100119
//

#import "CFScanner.h"

// create
CFScannerRef CFScannerCreate( CFStringRef string )
{
	return (CFScannerRef)[[NSScanner alloc] initWithString:(NSString *)string];
}

CFScannerRef CFScannerCreateLocalized( CFStringRef string )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	CFScannerRef scanner = [[NSScanner localizedScannerWithString:(NSString *)string] retain];
	[localPool drain];
	return scanner;
}

// string
CFStringRef CFScannerCopyString( CFScannerRef scanner )
{ return (CFStringRef)[[(NSScanner *)scanner string] retain]; }

// configure
void CFScannerSetScanLocation( CFScannerRef scanner, CFIndex index )
{ [(NSScanner *)scanner setScanLocation:(NSUInteger)index]; }

CFIndex CFScannerGetScanLocation( CFScannerRef scanner )
{ return [(NSScanner *)scanner scanLocation]; }

void CFScannerSetCaseSensitive( CFScannerRef scanner, Boolean flag )
{ [(NSScanner *)scanner setCaseSensitive:flag]; }

Boolean CFScannerIsCaseSensitive( CFScannerRef scanner )
{ return [(NSScanner *)scanner caseSensitive]; }

void CFScannerSetCharactersToBeSkipped( CFScannerRef scanner, CFCharacterSetRef set )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	[(NSScanner *)scanner setCharactersToBeSkipped:(NSCharacterSet *)set];
	[localPool drain];
}

CFCharacterSetRef CFScannerCopyCharactersToBeSkipped( CFScannerRef scanner )
{ return (CFCharacterSetRef)[[(NSScanner *)scanner charactersToBeSkipped] retain]; }

void CFScannerSetLocale( CFScannerRef scanner, CFLocaleRef locale )
{ [(NSScanner *)scanner setLocale:(NSLocale *)locale]; }

CFLocaleRef CFScannerCopyLocale( CFScannerRef scanner )
{ return (CFLocaleRef)[[(NSScanner *)scanner locale] retain]; }

// scan
Boolean CFScannerScanCharactersFromSet( CFScannerRef scanner, CFCharacterSetRef set, CFStringRef *string )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	NSString *nsString = [[[NSString alloc] init] autorelease];
	BOOL flag = [(NSScanner *)scanner scanCharactersFromSet:(NSCharacterSet *)set intoString:&nsString];
	if ( string != NULL ) *string = CFStringCreateCopy( kCFAllocatorDefault, (CFStringRef)nsString );
	[localPool drain];
	return flag;
}

Boolean CFScannerScanUpToCharactersFromSet( CFScannerRef scanner, CFCharacterSetRef set, CFStringRef *string )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	NSString *nsString = [[[NSString alloc] init] autorelease];
	BOOL flag = [(NSScanner *)scanner scanUpToCharactersFromSet:(NSCharacterSet *)set intoString:&nsString];
	if ( string != NULL ) *string = CFStringCreateCopy( kCFAllocatorDefault, (CFStringRef)nsString );
	[localPool drain];
	return flag;
}

Boolean CFScannerScanDouble( CFScannerRef scanner, double *value )
{ return [(NSScanner *)scanner scanDouble:value]; }

Boolean CFScannerScanSingle( CFScannerRef scanner, float *value )
{ return [(NSScanner *)scanner scanFloat:value]; }

Boolean CFScannerScanHexDouble( CFScannerRef scanner, double *value )
{ return [(NSScanner *)scanner scanHexDouble:value]; }
			
Boolean CFScannerScanHexSingle( CFScannerRef scanner, float *value )
{ return [(NSScanner *)scanner scanHexFloat:value]; }

Boolean CFScannerScanHexLong( CFScannerRef scanner, unsigned *value )
{ return [(NSScanner *)scanner scanHexInt:value]; }

Boolean CFScannerScanHexUInt64( CFScannerRef scanner, unsigned long long *value )
{ return [(NSScanner *)scanner scanHexLongLong:value]; }

Boolean CFScannerScanLong( CFScannerRef scanner, int *value )
{ return [(NSScanner *)scanner scanInt:value]; }

Boolean CFScannerScanSInt64( CFScannerRef scanner, long long *value )
{ return [(NSScanner *)scanner scanLongLong:value]; }

Boolean CFScannerScanString( CFScannerRef scanner, CFStringRef string, CFStringRef *outString )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	NSString *nsString = [[[NSString alloc] init] autorelease];
	BOOL flag = [(NSScanner *)scanner scanString:(NSString *)string intoString:&nsString];
	if ( outString != NULL ) *outString = CFStringCreateCopy( kCFAllocatorDefault, (CFStringRef)nsString );
	[localPool drain];
	return flag;
}

Boolean CFScannerScanUpToString( CFScannerRef scanner, CFStringRef string, CFStringRef *outString )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	NSString *nsString = [[[NSString alloc] init] autorelease];
	BOOL flag = [(NSScanner *)scanner scanUpToString:(NSString *)string intoString:&nsString];
	if ( outString != NULL ) *outString = CFStringCreateCopy( kCFAllocatorDefault, (CFStringRef)nsString );
	[localPool drain];
	return flag;
}

Boolean CFScannerIsAtEnd( CFScannerRef scanner )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	BOOL flag = [(NSScanner *)scanner isAtEnd];
	[localPool drain];
	return flag;
}


