//
//  CFString Additions.m
//
//  Bernie Wylde 20100104
//

#import <Foundation/Foundation.h>
#import "CFString_Additions.h"

// create
CFStringRef CFStringCreateWithContentsOfURL( CFURLRef url, CFStringEncoding encoding )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];	
	CFStringRef string = (CFStringRef)[[NSString alloc] initWithContentsOfURL:(NSURL *)url encoding:CFStringConvertEncodingToNSStringEncoding( encoding ) error:nil];
	[localPool drain];
	return string;
}

CFStringRef CFStringCreateWithContentsOfFile( FSRef *ref, CFStringEncoding encoding )
{	
	CFStringRef string = NULL;
	CFURLRef url = CFURLCreateFromFSRef( kCFAllocatorDefault, ref );
	if ( url != NULL ) {
		string = CFStringCreateWithContentsOfURL( url, encoding );
		CFRelease( url );
	}
	return string;
}


// dividing
CFArrayRef CFStringCreateArraySeparatedByCharactersInSet( CFStringRef string, CFCharacterSetRef set )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];	
	NSArray *array = [[(NSString *)string componentsSeparatedByCharactersInSet:(NSCharacterSet *)set] retain];
	[localPool drain];
	return (CFArrayRef)array;
}

CFStringRef CFStringCreateByTrimmingCharactersInSet( CFStringRef inString, CFCharacterSetRef set )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];	
	NSString *outString = [[(NSString *)inString stringByTrimmingCharactersInSet:(NSCharacterSet *)set] retain];
	[localPool drain];
	return (CFStringRef)outString;
}


// store
Boolean CFStringWriteToURL( CFStringRef string, CFURLRef url, CFStringEncoding encoding )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSString *)string writeToURL:(NSURL *)url atomically:YES encoding:CFStringConvertEncodingToNSStringEncoding( encoding ) error:nil];
	[localPool drain];
	return result;
}

Boolean CFStringWriteToPath( CFStringRef string, CFStringRef path, CFStringEncoding encoding )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = [(NSString *)string writeToFile:(NSString *)path atomically:YES encoding:CFStringConvertEncodingToNSStringEncoding( encoding ) error:nil];
	[localPool drain];
	return result;
}

Boolean CFStringWriteToFile( CFStringRef string, FSRef *parRef, CFStringRef filename, CFStringEncoding encoding )
{
	Boolean result = false;
	CFURLRef parUrl = CFURLCreateFromFSRef( kCFAllocatorDefault, parRef );
	if ( parUrl != NULL ) {
		CFURLRef url = CFURLCreateCopyAppendingPathComponent( kCFAllocatorDefault, parUrl, filename, false );
		CFRelease( parUrl );
		if ( url != NULL ) {
			result = CFStringWriteToURL( string, url, encoding );
			CFRelease( url );
		}
	}
	return result;
}

Boolean CFStringAppendToPath( CFStringRef string, CFStringRef path, CFStringEncoding encoding )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = false;
	NSFileHandle *output = [NSFileHandle fileHandleForWritingAtPath:(NSString *)path];
	if ( output != nil ) {
		[output seekToEndOfFile];
		[output writeData:[(NSString *)string dataUsingEncoding:CFStringConvertEncodingToNSStringEncoding( encoding )]];
		[output closeFile];
		result = true;
	}
	[localPool drain];
	return result;
}

#if MAC_OS_X_VERSION_10_5 < MAC_OS_X_VERSION_MAX_ALLOWED
Boolean CFStringAppendToURL( CFStringRef string, CFURLRef url, CFStringEncoding encoding )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = false;
	NSFileHandle *output = [NSFileHandle fileHandleForWritingToURL:(NSURL *)url error:nil];
	if ( output != nil ) {
		[output seekToEndOfFile];
		[output writeData:[(NSString *)string dataUsingEncoding:CFStringConvertEncodingToNSStringEncoding( encoding )]];
		[output closeFile];
		result = true;
	}
	[localPool drain];
	return result;
}
#else
Boolean CFStringAppendToURL( CFStringRef string, CFURLRef url, CFStringEncoding encoding )
{
	NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];
	Boolean result = false;
	CFStringRef path = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
	if ( path != NULL ) {
		result = CFStringAppendToPath( string, path, encoding );
		CFRelease( path );
	}
	[localPool drain];
	return result;
}
#endif

Boolean CFStringAppendToFile( CFStringRef string, FSRef *ref, CFStringEncoding encoding )
{
	Boolean result = false;
	CFStringRef path;
	CFURLRef url = CFURLCreateFromFSRef( kCFAllocatorDefault, ref );
	if ( url != NULL ) {
		path = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
		CFRelease( url );
		result = CFStringAppendToPath( string, path, encoding );
		CFRelease( path );
	}	
	return result;
}

