/*
 MDArray.incl
 
 Bernie Wylde 20170208
 
 1.0.6 20200708
 */

#import <Cocoa/Cocoa.h>

@interface MDArray : NSObject {
    NSMutableArray *_array;
    NSUInteger _dims;
}

@property (assign) NSMutableArray *_array;
@property (assign) NSUInteger _dims;

#pragma mark - private
typedef MDArray *MDArrayRef;

// add
- (void)addObjectv:(id)obj args:(va_list)ap;
- (void)addObjectsv:(id)objects args:(va_list)ap;

// set
- (void)setObjectv:(id)obj args:(va_list)ap;

// insert
- (void)insertObjectv:(id)obj args:(va_list)ap;
- (void)insertObjectsv:(id)objects args:(va_list)ap;

// get
- (id)objectv:(NSUInteger)index args:(va_list)ap;

// remove
- (void)removev:(NSUInteger)index args:(va_list)ap;


#pragma mark - public
// create
- (id)initWithDims:(NSUInteger)dims;
- (id)initWithContentsOfURL:(NSURL *)url;

// query
- (NSUInteger)dimCount;
- (NSUInteger)countv:(va_list)ap;

// add
- (void)addObject:(id)obj, ...;
- (void)addObjects:(id)objects, ...;
- (void)addBool:(int)value, ...;
- (void)addInteger:(NSInteger)value, ...;
- (void)addDouble:(double)value, ...;
- (void)addBytes:(void *)p length:(NSUInteger)length, ...;

// set
- (void)setObject:(id)obj, ...;
- (void)setBool:(int)value, ...;
- (void)setInteger:(NSInteger)value, ...;
- (void)setDouble:(double)value, ...;
- (void)setBytes:(void *)p length:(NSUInteger)length, ...;

// insert
- (void)insertObject:(id)obj, ...;
- (void)insertObjects:(id)objects, ...;
- (void)insertBool:(int)value, ...;
- (void)insertInteger:(NSInteger)value, ...;
- (void)insertDouble:(double)value, ...;
- (void)insertBytes:(void *)p length:(NSUInteger)length, ...;

// get
- (id)object:(int)index, ...;
- (BOOL)bool:(int)index, ...;
- (NSInteger)integer:(int)index, ...;
- (double)double:(int)index, ...;
- (void)getBytes:(void *)p length:(NSUInteger)length, ...;

// remove
- (void)remove:(NSUInteger)index, ...;
- (void)removeObjects:(id)objects, ...;
- (void)removeAll;
- (void)removeNulls;

// sort
- (void)sort;

// compress
- (void)compress;

// description
- (NSString *)descriptionWithEmptyArrays:(BOOL)emptyArrays nulls:(BOOL)nulls;
- (NSString *)description;

// write
- (BOOL)writeToURL:(NSURL *)url;
@end

// Create
MDArrayRef MDArrayWithDims( NSUInteger dims );
MDArrayRef MDArrayWithContentsOfURL( NSURL *url );

// Query
NSUInteger MDArrayDimCount( MDArrayRef mda );
NSUInteger MDArrayCountv( MDArrayRef mda, va_list ap );
NSUInteger MDArrayCount( MDArrayRef mda, ... );

// Add
void MDArrayAddObject( MDArrayRef mda, CFTypeRef obj, ...);
void MDArrayAddObjects( MDArrayRef mda, CFTypeRef objects, ...);
void MDArrayAddBool( MDArrayRef mda, int value, ...);
void MDArrayAddInteger( MDArrayRef mda, NSInteger value, ...);
void MDArrayAddDouble( MDArrayRef mda, double value, ...);
void MDArrayAddBytes( MDArrayRef mda, void *p, NSUInteger length, ... );

// Set
void MDArraySetObject( MDArrayRef mda, CFTypeRef obj, ... );
void MDArraySetBool( MDArrayRef mda, int value, ... );
void MDArraySetInteger( MDArrayRef mda, NSInteger value, ... );
void MDArraySetDouble( MDArrayRef mda, double value, ... );
void MDArraySetBytes( MDArrayRef mda, void *p, NSUInteger length, ... );

// Insert
void MDArrayInsertObject( MDArrayRef mda, CFTypeRef obj, ... );
void MDArrayInsertObjects( MDArrayRef mda, CFTypeRef objects, ... );
void MDArrayInsertBool( MDArrayRef mda, int value, ... );
void MDArrayInsertInteger( MDArrayRef mda, NSInteger value, ... );
void MDArrayInsertDouble( MDArrayRef mda, double value, ... );
void MDArrayInsertBytes( MDArrayRef mda, void *p, NSUInteger length, ... );

// Get
CFTypeRef MDArrayObject( MDArrayRef mda, NSUInteger index, ... );
BOOL MDArrayBool( MDArrayRef mda, NSUInteger index, ... );
NSInteger MDArrayInteger( MDArrayRef mda, NSUInteger index, ... );
double MDArrayDouble( MDArrayRef mda, NSUInteger index, ... );
void MDArrayGetBytes( MDArrayRef mda, void *p, NSUInteger length, ... );

// Remove
void MDArrayRemove( MDArrayRef mda, NSUInteger index, ... );
void MDArrayRemoveObjects( MDArrayRef mda, CFTypeRef objects, ... );
void MDArrayRemoveAll( MDArrayRef mda );
void MDArrayRemoveNulls( MDArrayRef mda );

// Sort
void MDArraySort( MDArrayRef mda );

// Compress
void MDArrayCompress( MDArrayRef mda );

// Description
NSString *MDArrayDescription( MDArrayRef mda, BOOL emptyArrays, BOOL nulls );

// Write
BOOL MDArrayWriteToURL( MDArrayRef mda, NSURL *url );

