//
//  MDArray.m
//  MDArray TestApp
//
//  Created by Bernie Wylde on 08/07/2020.
//  Copyright © 2020 Bernie Wylde. All rights reserved.
//

#import "MDArray.h"

@implementation MDArray

#pragma mark - private
@synthesize _array, _dims;

- (void)padArray:(NSMutableArray *)array toIndex:(NSUInteger)index {
    NSUInteger count = [array count];
    while ( index >= count ) {
        [array addObject:[NSMutableArray arrayWithCapacity:0]];
        count++;
    }
}

- (void)nullPadArray:(NSMutableArray *)array toIndex:(NSUInteger)index insert:(BOOL)insert {
    NSUInteger count = [array count];
    if ( insert ) {
        while ( index > count ) {
            [array addObject:[NSNull null]];
            count++;
        }
    } else {
        while ( index >= count ) {
            [array addObject:[NSNull null]];
            count++;
        }
    }
}

- (NSMutableArray *)array:(NSMutableArray *)array subarrayAtIndex:(NSUInteger)index grow:(BOOL)grow {
    NSMutableArray *subarray = nil;
    NSUInteger count = [array count];
    if ( grow ) {
        if ( index >= count ) {
            [self padArray:array toIndex:index];
        }
        subarray = [array objectAtIndex:index];
    } else {
        if ( index < count ) {
            subarray = [array objectAtIndex:index];
        }
    }
    return subarray;
}

// query
- (NSUInteger)countv:(va_list)ap {
NSMutableArray *array = _array;
    if ( _dims == 1 ) {
       return [array count];
    } else {
        for ( NSUInteger d = 0; d < (_dims - 1); d++ ) {
            NSUInteger arg = va_arg(ap,NSUInteger);
            array = [self array:array subarrayAtIndex:arg grow:NO];
        }
          return [array count];
    }
    return 0;
}

// add
- (void)addObjectv:(id)obj args:(va_list)ap {
    NSMutableArray *array = _array;
    if ( _dims == 1 ) {
        [array addObject:obj];
    } else {
        for ( NSUInteger d = 0; d < _dims; d++ ) {
            NSUInteger arg = va_arg(ap,NSUInteger);
            if ( d == _dims - 1 ) {
                [array addObject:obj];
            } else {
                array = [self array:array subarrayAtIndex:arg grow:YES];
            }
        }
    }
}

- (void)addObjectsv:(id)objects args:(va_list)ap {
    if ( [objects isKindOfClass:[NSString class]] ) {
        objects = [objects componentsSeparatedByString:@";"];
    }
    NSMutableArray *array = _array;
    if ( _dims == 1 ) {
        [array addObjectsFromArray:objects];
    } else {
        for ( NSUInteger d = 0; d < _dims; d++ ) {
            NSUInteger arg = va_arg(ap,NSUInteger);
            if ( d == _dims - 1 ) {
                [array addObjectsFromArray:objects];
            } else {
                array = [self array:array subarrayAtIndex:arg grow:YES];
            }
        }
    }
}

// set
- (void)setObjectv:(id)obj args:(va_list)ap {
    NSMutableArray *array = _array;
    for ( NSUInteger d = 0; d < _dims; d++ ) {
        NSUInteger arg = va_arg(ap,NSUInteger);
        if ( _dims == 1 ) {
            [self nullPadArray:array toIndex:arg insert:NO];
            [array replaceObjectAtIndex:arg withObject:obj];
        } else {
            if ( d == _dims - 1 ) {
                [self nullPadArray:array toIndex:arg insert:NO];
                [array replaceObjectAtIndex:arg withObject:obj];
            } else {
                array = [self array:array subarrayAtIndex:arg grow:YES];
            }
        }
    }
}

// insert
- (void)insertObjectv:(id)obj args:(va_list)ap {
    NSMutableArray *array = _array;
    for ( NSUInteger d = 0; d < _dims; d++ ) {
        NSUInteger arg = va_arg(ap,NSUInteger);
        if ( _dims == 1 ) {
            [self nullPadArray:array toIndex:arg insert:YES];
            [array insertObject:obj atIndex:arg];
        } else {
            if ( d == _dims - 1 ) {
                [self nullPadArray:array toIndex:arg insert:YES];
                [array insertObject:obj atIndex:arg];
            } else {
                array = [self array:array subarrayAtIndex:arg grow:YES];
            }
        }
    }
}

- (void)insertObjectsv:(id)objects args:(va_list)ap {
    if ( [objects isKindOfClass:[NSString class]] ) {
        objects = [objects componentsSeparatedByString:@";"];
    }
    NSMutableArray *array = _array;
    for ( NSUInteger d = 0; d < _dims; d++ ) {
        NSUInteger arg = va_arg(ap,NSUInteger);
        if ( _dims == 1 ) {
            [self nullPadArray:array toIndex:arg insert:YES];
            [array insertObjects:objects atIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(arg,[objects count])]];
        } else {
            if ( d == _dims - 1 ) {
                [self nullPadArray:array toIndex:arg insert:YES];
                [array insertObjects:objects atIndexes:[NSIndexSet indexSetWithIndexesInRange:NSMakeRange(arg,[objects count])]];
            } else {
                array = [self array:array subarrayAtIndex:arg grow:YES];
            }
        }
    }
}

// get
- (id)objectv:(va_list)ap {
    id obj = nil;
    NSMutableArray *array = _array;
    NSUInteger arg = va_arg(ap,NSUInteger);
    if ( _dims == 1 ) {
        if ( arg < [array count] ) {
            obj = [array objectAtIndex:arg];
        }
    } else {
        array = [self array:array subarrayAtIndex:arg grow:NO];
        if ( array ) {
            for ( NSUInteger d = 1; d < _dims; d++ ) {
                NSUInteger arg = va_arg(ap,NSUInteger);
                if ( d == _dims - 1 ) {
                    if ( arg < [array count] ) {
                        obj = [array objectAtIndex:arg];
                    }
                } else {
                    array = [self array:array subarrayAtIndex:arg grow:NO];
                    if ( array == nil ) break;
                }
            }
        }
    }
    return obj;
}

- (id)objectv:(NSUInteger)index args:(va_list)ap {
    id obj = nil;
    NSMutableArray *array = _array;
    NSUInteger arg0 = index;
    if ( _dims == 1 ) {
        if ( arg0 < [array count] ) {
            obj = [array objectAtIndex:arg0];
        }
    } else {
        array = [self array:array subarrayAtIndex:arg0 grow:NO];
        if ( array ) {
            for ( NSUInteger d = 1; d < _dims; d++ ) {
                NSUInteger arg = va_arg(ap,NSUInteger);
                if ( d == _dims - 1 ) {
                    if ( arg < (NSUInteger)[array count] ) {
                        obj = [array objectAtIndex:arg];
                    }
                } else {
                    array = [self array:array subarrayAtIndex:arg grow:NO];
                    if ( array == nil ) break;
                }
            }
        }
    }
    if ( [obj isEqualTo:[NSNull null]] ) obj = nil;
    return obj;
}

// remove
- (void)removev:(NSUInteger)index args:(va_list)ap {
    NSMutableArray *array = _array;
    NSUInteger arg0 = index;
    if ( _dims == 1 ) {
        if ( arg0 < (NSUInteger)[array count] ) {
            [array removeObjectAtIndex:arg0];
        }
    } else {
        array = [self array:array subarrayAtIndex:arg0 grow:NO];
        if ( array ) {
            for ( NSUInteger d = 1; d < _dims; d++ ) {
                NSUInteger arg = va_arg(ap,NSUInteger);
                if ( d == _dims - 1 ) {
                    if ( arg < (NSUInteger)[array count] ) {
                        [array removeObjectAtIndex:arg];
                    }
                } else {
                    array = [self array:array subarrayAtIndex:arg grow:NO];
                    if ( array == nil ) break;
                }
            }
        }
    }
}

- (void)removeObjectsv:(id)objects args:(va_list)ap {
    if ( [objects isKindOfClass:[NSString class]] ) {
        objects = [objects componentsSeparatedByString:@";"];
    }
    NSMutableArray *array = _array;
    for ( NSUInteger d = 0; d < _dims; d++ ) {
        NSUInteger arg = va_arg(ap,NSUInteger);
        if ( _dims == 1 ) {
            [array removeObjectsInArray:objects];
        } else {
            if ( d == _dims - 1 ) {
                [array removeObjectsInArray:objects];
            } else {
                array = [self array:array subarrayAtIndex:arg grow:NO];
            }
        }
    }
}

// sort
- (void)removeNullsFromArray:(NSMutableArray *)array {
    [array removeObject:[NSNull null]];
    NSUInteger count = [array count];
    for ( NSUInteger index = 0; index < count; index++ ) {
        id obj = [array objectAtIndex:index];
        if ( [obj isKindOfClass:[NSMutableArray class]] ) {
            [self removeNullsFromArray:obj];
        }
    }
}

- (void)sortArray:(NSMutableArray *)array {
    [self removeNullsFromArray:array];
    if ( _dims == 1 ) {
        [array sortWithOptions:0 usingComparator:^(id obj1, id obj2) {
            return [obj1 compare:obj2];
        }];
    } else {
        NSUInteger count = [array count];
        for ( NSUInteger index = 0; index < count; index++ ) {
            id obj = [array objectAtIndex:index];
            if ( [obj isKindOfClass:[NSMutableArray class]] ) {
                [self sortArray:obj];
            } else {
                [array sortWithOptions:0 usingComparator:^(id obj1, id obj2) {
                    return [obj1 compare:obj2];
                }];
            }
        }
    }
}

// compress
- (void)compress:(NSMutableArray *)array {
    NSInteger count = [array count];
    if ( count ) {
        for ( NSInteger index = count - 1; index >= 0; index-- ) {
            id obj = [array objectAtIndex:index];
            if ( [obj isKindOfClass:[NSMutableArray class]] ) {
                [self compress:obj];
                if ( [obj count] == 0 ) {
                    if ( index == (NSInteger)[array count] - 1 ) {
                        [array removeObjectAtIndex:index];
                    }
                }
            } else {
                if ( [obj isEqualTo:[NSNull null]] ) {
                    [array removeObjectAtIndex:index];
                } else {
                    break;
                }
            }
        }
    }
}

// description
- (NSUInteger)description:(NSMutableArray *)array indent:(NSUInteger)indent string:(NSMutableString *)string emptyArrays:(BOOL)emptyArrays nulls:(BOOL)nulls {
    indent++;
    NSString *indentSpaces = [@"" stringByPaddingToLength:indent*5 withString:@" " startingAtIndex:0];
    NSInteger count = [array count];
    for ( NSInteger index = 0; index < count; index++ ) {
        id obj = [array objectAtIndex:index];
        if ( [obj isKindOfClass:[NSMutableArray class]] ) {
            if ( [obj count] == 0 ) {
                if ( emptyArrays ) {
                    [string appendFormat:@"%@%ld: -array- <empty>\r",indentSpaces,(unsigned long)index];
                }
            } else {
                [string appendFormat:@"%@%ld: -array-\r",indentSpaces,(unsigned long)index];
                indent = [self description:obj indent:indent string:string emptyArrays:emptyArrays nulls:nulls];
            }
        } else {
            if ( !( nulls == NO && [obj isEqualTo:[NSNull null]] ) ) {
                [string appendFormat:@"%@%ld: %@\r",indentSpaces,(unsigned long)index,obj];
            }
        }
    }
    indent--;
    return indent;
}


#pragma mark - public
- (id)initWithDims:(NSUInteger)dims {
    if ( self = [super init] ) {
        _array = [NSMutableArray arrayWithCapacity:0];
        _dims = dims;
    }
    return self;
}

- (id)initWithContentsOfURL:(NSURL *)url {
    if ( self = [super init] ) {
        _array = [NSKeyedUnarchiver unarchiveObjectWithFile:[url path]];
        _dims = [[_array objectAtIndex:0] integerValue];
        [_array removeObjectAtIndex:0];
    }
    return self;
}

// query
- (NSUInteger)dimCount {
    return _dims;
}

// add
- (void)addObject:(id)obj, ... {
    va_list ap;
    va_start(ap,obj);
    [self addObjectv:obj args:ap];
    va_end(ap);
}

- (void)addObjects:(id)objects, ... {
    va_list ap;
    va_start(ap,objects);
    [self addObjectsv:objects args:ap];
    va_end(ap);
}

- (void)addBool:(int)value, ... {
    va_list ap;
    va_start(ap,value);
    [self addObjectv:[NSNumber numberWithBool:value] args:ap];
    va_end(ap);
}

- (void)addInteger:(NSInteger)value, ... {
    va_list ap;
    va_start(ap,value);
    [self addObjectv:[NSNumber numberWithInteger:value] args:ap];
    va_end(ap);
}

- (void)addDouble:(double)value, ... {
    va_list ap;
    va_start(ap,value);
    [self addObjectv:[NSNumber numberWithDouble:value] args:ap];
    va_end(ap);
}

- (void)addBytes:(void *)p length:(NSUInteger)length, ... {
    va_list ap;
    va_start(ap,length);
    [self addObjectv:[NSData dataWithBytes:p length:length] args:ap];
    va_end(ap);
}

// set
- (void)setObject:(id)obj, ... {
    va_list ap;
    va_start(ap,obj);
    [self setObjectv:obj args:ap];
    va_end(ap);
}

- (void)setBool:(int)value, ... {
    va_list ap;
    va_start(ap,value);
    [self setObjectv:[NSNumber numberWithBool:value] args:ap];
    va_end(ap);
}

- (void)setInteger:(NSInteger)value, ... {
    va_list ap;
    va_start(ap,value);
    [self setObjectv:[NSNumber numberWithInteger:value] args:ap];
    va_end(ap);
}

- (void)setDouble:(double)value, ... {
    va_list ap;
    va_start(ap,value);
    [self setObjectv:[NSNumber numberWithDouble:value] args:ap];
    va_end(ap);
}

- (void)setBytes:(void *)p length:(NSUInteger)length, ... {
    va_list ap;
    va_start(ap,length);
    [self setObjectv:[NSData dataWithBytes:p length:length] args:ap];
    va_end(ap);
}

// insert
- (void)insertObject:(id)obj, ... {
    va_list ap;
    va_start(ap,obj);
    [self insertObjectv:obj args:ap];
    va_end(ap);
}

- (void)insertObjects:(id)objects, ... {
    va_list ap;
    va_start(ap,objects);
    [self insertObjectsv:objects args:ap];
    va_end(ap);
}

- (void)insertBool:(int)value, ... {
    va_list ap;
    va_start(ap,value);
    [self insertObjectv:[NSNumber numberWithBool:value] args:ap];
    va_end(ap);
}

- (void)insertInteger:(NSInteger)value, ... {
    va_list ap;
    va_start(ap,value);
    [self insertObjectv:[NSNumber numberWithInteger:value] args:ap];
    va_end(ap);
}

- (void)insertDouble:(double)value, ... {
    va_list ap;
    va_start(ap,value);
    [self insertObjectv:[NSNumber numberWithDouble:value] args:ap];
    va_end(ap);
}

- (void)insertBytes:(void *)p length:(NSUInteger)length, ... {
    va_list ap;
    va_start(ap,length);
    [self insertObjectv:[NSData dataWithBytes:p length:length] args:ap];
    va_end(ap);
}

// get
- (id)object:(int)index, ... {
    va_list ap;
    va_start(ap,index);
    id obj = [self objectv:index args:ap];
    va_end(ap);
    return obj;
}

- (BOOL)bool:(int)index, ... {
    va_list ap;
    va_start(ap,index);
    BOOL value = NO;
    id obj = [self objectv:index args:ap];
    if ( obj ) value = [obj boolValue];
    va_end(ap);
    return value;
}

- (NSInteger)integer:(int)index, ... {
    va_list ap;
    va_start(ap,index);
    NSInteger value = 0;
    id obj = [self objectv:index args:ap];
    if ( obj ) value = [obj integerValue];
    va_end(ap);
    return value;
}

- (double)double:(int)index, ... {
    va_list ap;
    va_start(ap,index);
    double value = 0.0;
    id obj = [self objectv:index args:ap];
    if ( obj ) value = [obj doubleValue];
    va_end(ap);
    return value;
}

- (void)getBytes:(void *)p length:(NSUInteger)length, ... {
    va_list ap;
    va_start(ap,length);
    NSData *data = [self objectv:ap];
    [data getBytes:p length:length];
    va_end(ap);
}


// remove
- (void)remove:(NSUInteger)index, ... {
    va_list ap;
    va_start(ap,index);
    [self removev:index args:ap];
    va_end(ap);
}

- (void)removeObjects:(id)objects, ... {
    va_list ap;
    va_start(ap,objects);
    [self removeObjectsv:objects args:ap];
    va_end(ap);
}

- (void)removeAll {
    [_array removeAllObjects];
}

// sort
- (void)sort {
    [self sortArray:_array];
}

// compress
- (void)compress {
    [self compress:_array];
}

- (void)removeNulls {
    [self removeNullsFromArray:_array];
}

// description
- (NSString *)descriptionWithEmptyArrays:(BOOL)emptyArrays nulls:(BOOL)nulls {
    NSMutableString *string = [[[NSMutableString alloc] initWithString:@"-array-\r"] autorelease];
    [self description:_array indent:0 string:string emptyArrays:emptyArrays nulls:nulls];
    return string;
}

- (NSString *)description {
    return [self descriptionWithEmptyArrays:YES nulls:YES];
}

// write
- (BOOL)writeToURL:(NSURL *)url {
    [_array insertObject:[NSNumber numberWithInteger:_dims] atIndex:0];
    BOOL result = [NSKeyedArchiver archiveRootObject:_array toFile:[url path]];
    [_array removeObjectAtIndex:0];
    return result;
}
@end

// Create
MDArrayRef MDArrayWithDims( NSUInteger dims )
{ return [[[MDArray alloc] initWithDims:dims] autorelease]; }

MDArrayRef MDArrayWithContentsOfURL( NSURL *url )
{ return [[[MDArray alloc] initWithContentsOfURL:url] autorelease]; }

// Query
NSUInteger MDArrayDimCount( MDArrayRef mda )
{ return [mda dimCount]; }


NSUInteger MDArrayCountv( MDArrayRef mda, va_list ap )
{ return [mda countv:ap]; }

NSUInteger MDArrayCount( MDArrayRef mda, ... )
{
    va_list ap;
    va_start(ap,mda);
    NSUInteger count = MDArrayCountv( mda, ap );
    va_end(ap);
    return count;
}

// Add
void MDArrayAddObjectv( MDArrayRef mda, id obj, va_list ap )
{ [mda addObjectv:obj args:ap]; }

void MDArrayAddObjectsv( MDArrayRef mda, id objects, va_list ap )
{ [mda addObjectsv:objects args:ap]; }

void MDArrayAddObject( MDArrayRef mda, CFTypeRef obj, ... )
{
    va_list ap;
    va_start(ap,obj);
    MDArrayAddObjectv( mda, obj, ap );
    va_end(ap);
}

void MDArrayAddObjects( MDArrayRef mda, CFTypeRef objects, ...)
{
    va_list ap;
    va_start(ap,objects);
    MDArrayAddObjectsv( mda, objects, ap );
    va_end(ap);
}

void MDArrayAddBoolv( MDArrayRef mda, int value, va_list ap )
{ MDArrayAddObjectv( mda, [NSNumber numberWithBool:value], ap ); }

void MDArrayAddBool( MDArrayRef mda, int value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArrayAddBoolv( mda, value, ap );
    va_end(ap);
}

void MDArrayAddIntegerv( MDArrayRef mda, NSInteger value, va_list ap )
{ MDArrayAddObjectv( mda, [NSNumber numberWithInteger:value], ap ); }

void MDArrayAddInteger( MDArrayRef mda, NSInteger value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArrayAddIntegerv( mda, value, ap );
    va_end(ap);
}

void MDArrayAddDoublev( MDArrayRef mda, double value, va_list ap )
{ MDArrayAddObjectv( mda, [NSNumber numberWithDouble:value], ap ); }

void MDArrayAddDouble( MDArrayRef mda, double value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArrayAddDoublev( mda, value, ap );
    va_end(ap);
}

void MDArrayAddBytesv( MDArrayRef mda, void *p, NSUInteger length, va_list ap )
{ MDArrayAddObjectv( mda, [NSData dataWithBytes:p length:length], ap ); }

void MDArrayAddBytes( MDArrayRef mda, void *p, NSUInteger length, ... )
{
    va_list ap;
    va_start(ap,length);
    MDArrayAddBytesv( mda, p, length, ap );
    va_end(ap);
}

// Set
void MDArraySetObjectv( MDArrayRef mda, id obj, va_list ap )
{ [mda setObjectv:obj args:ap]; }

void MDArraySetObject( MDArrayRef mda, CFTypeRef obj, ... )
{
    va_list ap;
    va_start(ap,obj);
    MDArraySetObjectv( mda, obj, ap );
    va_end(ap);
}

void MDArraySetBoolv( MDArrayRef mda, int value, va_list ap )
{ MDArraySetObjectv( mda, [NSNumber numberWithBool:value], ap ); }

void MDArraySetBool( MDArrayRef mda, int value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArraySetBoolv( mda, value, ap );
    va_end(ap);
}

void MDArraySetIntegerv( MDArrayRef mda, NSInteger value, va_list ap )
{ MDArraySetObjectv( mda, [NSNumber numberWithInteger:value], ap ); }

void MDArraySetInteger( MDArrayRef mda, NSInteger value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArraySetIntegerv( mda, value, ap );
    va_end(ap);
}

void MDArraySetDoublev( MDArrayRef mda, double value, va_list ap )
{ MDArraySetObjectv( mda, [NSNumber numberWithDouble:value], ap ); }

void MDArraySetDouble( MDArrayRef mda, double value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArraySetDoublev( mda, value, ap );
    va_end(ap);
}

void MDArraySetBytesv( MDArrayRef mda, void *p, NSUInteger length, va_list ap )
{ MDArraySetObjectv( mda, [NSData dataWithBytes:p length:length], ap ); }

void MDArraySetBytes( MDArrayRef mda, void *p, NSUInteger length, ... )
{
    va_list ap;
    va_start(ap,length);
    MDArraySetBytesv( mda, p, length, ap );
    va_end(ap);
}

// Insert
void MDArrayInsertObjectv( MDArrayRef mda, id obj, va_list ap )
{ [mda insertObjectv:obj args:ap]; }

void MDArrayInsertObjectsv( MDArrayRef mda, id objects, va_list ap )
{ [mda insertObjectsv:objects args:ap]; }

void MDArrayInsertObject( MDArrayRef mda, CFTypeRef obj, ... )
{
    va_list ap;
    va_start(ap,obj);
    MDArrayInsertObjectv( mda, obj, ap );
    va_end(ap);
}

void MDArrayInsertObjects( MDArrayRef mda, CFTypeRef objects, ... )
{
    va_list ap;
    va_start(ap,objects);
    MDArrayInsertObjectsv( mda, objects, ap );
    va_end(ap);
}

void MDArrayInsertBoolv( MDArrayRef mda, int value, va_list ap )
{ [mda insertObjectv:[NSNumber numberWithBool:value] args:ap]; }

void MDArrayInsertBool( MDArrayRef mda, int value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArrayInsertBoolv( mda, value, ap );
    va_end(ap);
}

void MDArrayInsertIntegerv( MDArrayRef mda, NSInteger value, va_list ap )
{ [mda insertObjectv:[NSNumber numberWithInteger:value] args:ap]; }

void MDArrayInsertInteger( MDArrayRef mda, NSInteger value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArrayInsertIntegerv( mda, value, ap );
    va_end(ap);
}

void MDArrayInsertDoublev( MDArrayRef mda, double value, va_list ap )
{ [mda setObjectv:[NSNumber numberWithDouble:value] args:ap]; }

void MDArrayInsertDouble( MDArrayRef mda, double value, ... )
{
    va_list ap;
    va_start(ap,value);
    MDArrayInsertDoublev( mda, value, ap );
    va_end(ap);
}

void MDArrayInsertBytesv( MDArrayRef mda, void *p, NSUInteger length, va_list ap )
{ [mda insertObjectv:[NSData dataWithBytes:p length:length] args:ap]; }

void MDArrayInsertBytes( MDArrayRef mda, void *p, NSUInteger length, ... )
{
    va_list ap;
    va_start(ap,length);
    MDArrayInsertBytesv( mda, p, length, ap );
    va_end(ap);
}

// Get
id MDArrayObjectv( MDArrayRef mda, NSUInteger index, va_list ap )
{ return [mda objectv:index args:ap]; }

CFTypeRef MDArrayObject( MDArrayRef mda, NSUInteger index, ... )
{
    va_list ap;
    va_start(ap,index);
    id obj = MDArrayObjectv( mda, index, ap );
    va_end(ap);
    return obj;
}

BOOL MDArrayBoolv( MDArrayRef mda, NSUInteger index, va_list ap )
{
    BOOL value = NO;
    id obj = [mda objectv:index args:ap];
    if ( obj ) value = [obj boolValue];
    return value;
}

BOOL MDArrayBool( MDArrayRef mda, NSUInteger index, ... )
{
    va_list ap;
    va_start(ap,index);
    BOOL value = MDArrayBoolv( mda, index, ap );
    va_end(ap);
    return value;
}

NSInteger MDArrayIntegerv( MDArrayRef mda, NSUInteger index, va_list ap )
{
    NSInteger value = 0;
    id obj = [mda objectv:index args:ap];
    if ( obj ) value = [obj integerValue];
    return value;
}

NSInteger MDArrayInteger( MDArrayRef mda, NSUInteger index, ... )
{
    va_list ap;
    va_start(ap,index);
    NSInteger value = MDArrayIntegerv( mda, index, ap );
    va_end(ap);
    return value;
}

double MDArrayDoublev( MDArrayRef mda, NSUInteger index, va_list ap )
{
    double value = 0.0;
    id obj = [mda objectv:index args:ap];
    if ( obj ) value = [obj doubleValue];
    return value;
}

double MDArrayDouble( MDArrayRef mda, NSUInteger index, ... )
{
    va_list ap;
    va_start(ap,index);
    double value = MDArrayDoublev( mda, index, ap );
    va_end(ap);
    return value;
}

void MDArrayGetBytesv( MDArrayRef mda, void *p, NSUInteger length, va_list ap )
{
    NSData *data = [mda objectv:ap];
    [data getBytes:p length:length];
}

void MDArrayGetBytes( MDArrayRef mda, void *p, NSUInteger length, ... )
{
    va_list ap;
    va_start(ap,length);
    MDArrayGetBytesv( mda, p, length, ap );
    va_end(ap);
}

// Remove
void MDArrayRemovev( MDArrayRef mda, NSUInteger index, va_list ap )
{ [mda removev:index args:ap]; }

void MDArrayRemoveObjectsv( MDArrayRef mda, CFTypeRef objects, va_list ap )
{ [mda removeObjectsv:objects args:ap]; }

void MDArrayRemove( MDArrayRef mda, NSUInteger index, ... )
{
    va_list ap;
    va_start(ap,index);
    MDArrayRemovev( mda, index, ap );
    va_end(ap);
}

void MDArrayRemoveObjects( MDArrayRef mda, CFTypeRef objects, ... )
{
    va_list ap;
    va_start(ap,objects);
    MDArrayRemoveObjectsv( mda, objects, ap );
    va_end(ap);
}

void MDArrayRemoveAll( MDArrayRef mda )
{ [mda removeAll]; }

void MDArrayRemoveNulls( MDArrayRef mda )
{ [mda removeNulls]; }

// Sort
void MDArraySort( MDArrayRef mda )
{ [mda sort]; }

// Compress
void MDArrayCompress( MDArrayRef mda )
{ [mda compress]; }

// Description
NSString *MDArrayDescription( MDArrayRef mda, BOOL emptyArrays, BOOL nulls )
{ return [mda descriptionWithEmptyArrays:emptyArrays nulls:nulls]; }

// Write
BOOL MDArrayWriteToURL( MDArrayRef mda, NSURL *url )
{ return [mda writeToURL:url]; }

