//
//  NSLog.m
//
//  Bernie Wylde 20130725
//

#import "NSLog.h"

const NSString *NSLogVers = @"1.3.5";

#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1060

@interface NSLogController : NSObject <NSWindowDelegate,NSURLDownloadDelegate,NSURLConnectionDelegate> {
    NSPanel *window;
    NSScrollView *scrollView;
    NSTextView *textView;
    NSTextFinder *textFinder;
    NSDrawer *drawer;
    NSButton *settingsBtn;
    NSButton *wrapBtn;
    NSButton *nwVersionAvailableBtn;
    BOOL hideNwVersionAvailableBtn;
    NSButton *findBtn;
    NSButton *saveBtn;
    NSButton *clearBtn;
    NSMutableData *responseData;
    BOOL terminateAppWhenWindowClosed;
}
@end

#else

@interface NSLogController : NSObject <NSWindowDelegate> {
    NSPanel *window;
    NSScrollView *scrollView;
    NSTextView *textView;
    NSDrawer *drawer;
    NSButton *settingsBtn;
    NSButton *wrapBtn;
    NSButton *nwVersionAvailableBtn;
    BOOL hideNwVersionAvailableBtn;
    NSButton *findBtn;
    NSButton *saveBtn;
    NSButton *clearBtn;
    NSMutableData *responseData;
    BOOL terminateAppWhenWindowClosed;
}

- (void)saveWithMessage:(NSString *)message nameFieldStringValue:(NSString *)filename;
@end

#endif

#define kNSLogVersPath @"http://homepage.ntlworld.com/bernie.w/NSLogVers"
#define kNSLogDownloadPath @"http://homepage.ntlworld.com/bernie.w/NSLog.zip"

#define kNSLogApplicationID CFSTR( "com.berniewylde.nslog" )
#define kNSLogFontNameKey CFSTR( "FontName" )
#define kNSLogFontSizeKey CFSTR( "FontSize" )
#define kNSLogWindowBoundsKey @"NSLogWindowBounds"

#define kNSLogDefaultFontName @"Menlo-Bold"
#define kNSLogDefaultFontSize 11.0

#define kNSLogWindowBorderThickness 23

enum {
    kNSLogDateBtnTag            = 1,
    kNSLogTimeBtnTag            = 2,
    kNSLogAppNameBtnTag         = 3,
    kNSLogProcessIDBtnTag       = 4,
    kNSLogConsoleBtnTag         = 5,
    kNSLogFloatingBtnTag        = 6,
    kNSLogCheckForUpdatesBtnTag = 7
};

BOOL sNSLogSuppress = NO;
BOOL sNSLogAlert = NO;

@interface NSLogController()

@property (retain, nonatomic) NSPanel *window;
@property (retain, nonatomic) NSTextView *textView;
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1060
@property (retain, nonatomic) NSTextFinder *textFinder;
#endif
@property (retain, nonatomic) NSDrawer *drawer;
@property (retain, nonatomic) NSButton *settingsBtn;
@property (retain, nonatomic) NSButton *wrapBtn;
@property (retain, nonatomic) NSButton *nwVersionAvailableBtn;
@property (assign) BOOL hideNwVersionAvailableBtn;
@property (retain, nonatomic) NSButton *findBtn;
@property (retain, nonatomic) NSButton *saveBtn;
@property (retain, nonatomic) NSButton *clearBtn;
@property (retain, nonatomic) NSMutableData *responseData;
@property (assign) BOOL terminateAppWhenWindowClosed;
@end


// private functions
NSLogController *NSLogGetController();
void NSLogV2( NSString *formatString, va_list ap, BOOL cr );
void NSLogV( NSString *formatString, va_list ap );
void NSLogReleaseObject( id obj );


@implementation NSLogController

@synthesize window;
@synthesize textView;
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1060
@synthesize textFinder;
#endif
@synthesize drawer;
@synthesize settingsBtn;
@synthesize wrapBtn;
@synthesize nwVersionAvailableBtn;
@synthesize hideNwVersionAvailableBtn;
@synthesize findBtn;
@synthesize saveBtn;
@synthesize clearBtn;
@synthesize responseData;
@synthesize terminateAppWhenWindowClosed;

#pragma mark - preferences
- (void)setPrefsForButton:(NSButton *)btn {
    CFBooleanRef flag;
    if ( [btn state] == NSOnState ) {
        flag = kCFBooleanTrue;
    } else {
        flag = kCFBooleanFalse;
    }
    
#if !__has_feature(objc_arc)
    CFPreferencesSetAppValue( (CFStringRef)[btn title], flag, kNSLogApplicationID );
#else
    CFPreferencesSetAppValue( (__bridge CFStringRef)[btn title], flag, kNSLogApplicationID );
#endif
    
    CFPreferencesAppSynchronize( kNSLogApplicationID );
}

- (void)setPrefsForButtonWithTag:(NSInteger)tag {
    [self setPrefsForButton:[self buttonWithTag:tag]];
}

- (void)setPrefsForWrapButton {
    CFBooleanRef flag;
    if ( [wrapBtn state] == NSOnState ) {
        flag = kCFBooleanTrue;
    } else {
        flag = kCFBooleanFalse;
    }
    
    // set prefs
    #if !__has_feature(objc_arc)
        CFPreferencesSetAppValue( (CFStringRef)@"Wrap", flag, kNSLogApplicationID );
#else
    CFPreferencesSetAppValue( (__bridge CFStringRef)@"Wrap", flag, kNSLogApplicationID );
#endif
    CFPreferencesAppSynchronize( kNSLogApplicationID );
}



- (void)loadPrefsForButtonWithTag:(NSInteger)tag {
    NSButton *btn = [self buttonWithTag:tag];
    
#if !__has_feature(objc_arc)
    CFBooleanRef ref = CFPreferencesCopyAppValue( (CFStringRef)[btn title], kNSLogApplicationID );
#else
    CFBooleanRef ref = CFPreferencesCopyAppValue( (__bridge CFStringRef)[btn title], kNSLogApplicationID );
#endif
    
    if ( ref != NULL ) {
        if ( ref == kCFBooleanTrue ) {
            [btn setState:NSOnState];
        } else {
            [btn setState:NSOffState];
        }
        CFRelease( ref );
    } else {
        switch ( tag ) {
            case kNSLogFloatingBtnTag:
            case kNSLogCheckForUpdatesBtnTag:
                [btn setState:NSOnState];
                break;
            default:
                [btn setState:NSOffState];
                break;
        }
    }
}

- (void)loadPrefsForWrapButton {
#if !__has_feature(objc_arc)
    CFBooleanRef ref = CFPreferencesCopyAppValue( (CFStringRef)@"Wrap", kNSLogApplicationID );
#else
    CFBooleanRef ref = CFPreferencesCopyAppValue( (__bridge CFStringRef)@"Wrap", kNSLogApplicationID );
#endif
    
    if ( ref != NULL ) {
        if ( ref == kCFBooleanTrue ) {
            [wrapBtn setState:NSOnState];
        } else {
            [wrapBtn setState:NSOffState];
        }
        CFRelease( ref );
    } else {
        [wrapBtn setState:NSOnState];
    }
}

- (void)loadPrefs {
    [self loadPrefsForButtonWithTag:kNSLogDateBtnTag];
    [self loadPrefsForButtonWithTag:kNSLogTimeBtnTag];
    [self loadPrefsForButtonWithTag:kNSLogAppNameBtnTag];
    [self loadPrefsForButtonWithTag:kNSLogProcessIDBtnTag];
    [self loadPrefsForButtonWithTag:kNSLogConsoleBtnTag];
    [self loadPrefsForButtonWithTag:kNSLogFloatingBtnTag];
    [self loadPrefsForButtonWithTag:kNSLogCheckForUpdatesBtnTag];
    
    [self loadPrefsForWrapButton];
}


- (NSInteger)sysVersion {
    NSDictionary *sv = [NSDictionary dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"];
    NSString *string = [sv objectForKey:@"ProductVersion"];
    NSArray *array = [string componentsSeparatedByString:@"."];
    NSString *versString = [NSString stringWithFormat:@"%@%@",[array objectAtIndex:0],[array objectAtIndex:1]];
    return [versString integerValue];
}

#pragma mark - window delegate methods
- (BOOL)windowShouldClose:(NSNotification *)notification {
    [window orderOut:nil];
    return terminateAppWhenWindowClosed;
}

- (void)windowWillClose:(NSNotification *)notification {
    if ( terminateAppWhenWindowClosed ) [NSApp terminate:nil];
}

#pragma mark - button actions
- (void)clearBtnAction:(id)sender {
    textView.string = @"";
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1060
- (void)findBtnAction:(id)sender {
    if ( [[textFinder findBarContainer] isFindBarVisible] ) {
        [textFinder performAction:NSTextFinderActionHideFindInterface];
    } else {
        [textFinder performAction:NSTextFinderActionShowFindInterface];
        [window makeKeyAndOrderFront:nil];
    }
}
#endif

- (void)saveWithMessage:(NSString *)message nameFieldStringValue:(NSString *)filename {
    [window makeKeyAndOrderFront:nil];
    NSSavePanel *sp = [NSSavePanel savePanel];
    [sp setMessage:message];
    [sp setNameFieldStringValue:filename];
    [sp beginSheetModalForWindow:window completionHandler:^(NSInteger result) {
        if ( result == NSFileHandlingPanelOKButton ) {
            NSLogWriteToURL( [sp URL] );
        }
    }];
}

- (void)saveBtnAction:(id)sender {
    [self saveWithMessage:@"Save log" nameFieldStringValue:@"Untitled"];
}

- (void)settingsBtnAction:(id)sender {
    [drawer toggle:nil];
}

- (void)drawerSwitchButtonAction:(id)sender {
    [self setPrefsForButton:sender];
    switch ( [sender tag] ) {
        case kNSLogFloatingBtnTag:
            [window setFloatingPanel:([[self buttonWithTag:kNSLogFloatingBtnTag] state] == NSOnState)];
            if ( ![window isFloatingPanel] ) [drawer toggle:nil];// workaround for bug
            break;
        case kNSLogCheckForUpdatesBtnTag:
            if ( [sender state] == NSOnState ) [self checkForNewVersion];
            break;
        default:
            break;
    }
}


- (NSImage *)imageForResource:(NSString *)name {
    NSImage *image = [NSImage imageNamed:name];
    if ( image == nil ) {
        NSURL *url = [[NSWorkspace sharedWorkspace] URLForApplicationWithBundleIdentifier:@"com.stazsoftware.fbtoc.editor"];
        NSBundle *bundle = [NSBundle bundleWithURL:url];
        NSString *imagePath = [bundle pathForResource:name ofType:@"png"];
#if !__has_feature(objc_arc)
        image = [[[NSImage alloc] initWithContentsOfFile:imagePath] autorelease];
#else
        image = [[NSImage alloc] initWithContentsOfFile:imagePath];
#endif
    }
    return image;
}

- (void)setTextWrap:(id)sender {
    if ( [sender state] == NSOnState ) {
        [[textView textContainer] setContainerSize:NSMakeSize([scrollView frame].size.width, FLT_MAX)];
        [[textView textContainer] setWidthTracksTextView:YES];
        [scrollView setHasHorizontalScroller:NO];
        [textView setHorizontallyResizable:NO];
        
        // workaround to refresh text view wrapping
        NSRect r = [scrollView frame];
        r.size.width++;
        [scrollView setFrame:r];
        r.size.width--;
        [scrollView setFrame:r];
        
        r.origin.x = 0;
        r.origin.y = 0;
        [textView setFrame:r];
        
        [wrapBtn setTitle:[NSString stringWithFormat:@"%C",10619]];
        
    } else {
        [[textView textContainer] setContainerSize:NSMakeSize(FLT_MAX, FLT_MAX)];
        [[textView textContainer] setWidthTracksTextView:NO];
        [scrollView setHasHorizontalScroller:YES];
        [textView setHorizontallyResizable:YES];
        
        NSRect r = [[scrollView contentView] documentVisibleRect];
        NSPoint pt = r.origin;
        [[scrollView contentView] scrollToPoint:NSMakePoint(0,pt.y)];
        [scrollView reflectScrolledClipView:[scrollView contentView]];
        
        [wrapBtn setTitle:[NSString stringWithFormat:@"%C",8649]];
    }
    [scrollView flashScrollers];
}


- (void)wrapButtonAction:(id)sender {
    [self setTextWrap:sender];
    [self setPrefsForWrapButton];
}


#pragma mark - convenience methods
- (NSButton *)buttonWithTag:(NSInteger)tag {
    return [[drawer contentView] viewWithTag:tag];
}

- (void)buttonWithTag:(NSInteger)tag setStateWithBool:(BOOL)flag {
    NSInteger state = NSOffState;
    if ( flag ) state = NSOnState;
    [[self buttonWithTag:tag] setState:state];
    [self setPrefsForButtonWithTag:tag];
}

- (void)switchButtonInitWithFrame:(NSRect)rect title:(NSString *)title tag:(NSInteger)tag toolTip:(NSString *)tooltip {
    NSButton *btn = [[NSButton alloc ]initWithFrame:rect];
    [btn setTitle:title];
    [btn setButtonType:NSSwitchButton];
    [[btn cell] setControlSize:NSSmallControlSize];
    [[btn cell] setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
    [btn setTarget:self];
    [btn setAction:@selector(drawerSwitchButtonAction:)];
    [btn setAutoresizingMask:NSViewMinYMargin];
    [btn setTag:tag];
    [btn setToolTip:tooltip];
    [[drawer contentView] addSubview:btn];
    NSLogReleaseObject( btn );
}

- (void)pauseWithMessage:(NSString *)msg {
    NSAlert *alert = [[NSAlert alloc] init];
    [[alert window] setTitle:@"NSLog Paused"];
    [alert setMessageText:msg];
    [alert addButtonWithTitle:@"Continue"];
    [alert addButtonWithTitle:@"Stop"];
    [alert setInformativeText:@"Stop to terminate this app. Continue to resume."];
    NSArray *buttons = [alert buttons];
    [[buttons objectAtIndex:1] setKeyEquivalent: @"\033"];
    if ( [alert runModal] != NSAlertFirstButtonReturn ) [NSApp terminate:nil];
    NSLogReleaseObject( alert );
}

#pragma mark - new version
- (void)revealFBHeadersFolder {
    NSWorkspace *ws = [NSWorkspace sharedWorkspace];
    NSURL *editorURL = [ws URLForApplicationWithBundleIdentifier:@"com.stazsoftware.fbtoc.editor"];
    if ( editorURL ) {
        NSURL *headersURL = [editorURL URLByAppendingPathComponent:@"Contents/Resources/FBtoC/Headers"];
        if ( headersURL ) [ws openURL:headersURL];
    }
}

- (void)download:(NSURLDownload *)download decideDestinationWithSuggestedFilename:(NSString *)filename {
    NSString *destFilename = [[NSHomeDirectory() stringByAppendingPathComponent:@"Downloads"] stringByAppendingPathComponent:filename];
    [download setDestination:destFilename allowOverwrite:NO];
    
    [self revealFBHeadersFolder];
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setMessageText:@"Download successful."];
    [alert addButtonWithTitle:@"OK"];
    [alert setInformativeText:@"Drag 'NSLog.incl' into FB's Headers folder.\rI've opened it for you."];
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1090
    [alert beginSheetModalForWindow:window completionHandler:^(NSModalResponse returnCode) {
        // ...
    }];
#else
    [alert beginSheetModalForWindow:window modalDelegate:self didEndSelector:nil contextInfo:nil];
#endif
    
}

- (void)download:(NSURLDownload *)download didFailWithError:(NSError *)error {
    NSLogReleaseObject( download );
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setMessageText:@"Download failed!"];
    [alert addButtonWithTitle:@"OK"];
    [alert setInformativeText:[NSString stringWithFormat:@"Error - %@\n%@", [error localizedDescription], [[error userInfo] objectForKey:NSURLErrorFailingURLStringErrorKey]]];
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1090
    [alert beginSheetModalForWindow:window completionHandler:^(NSModalResponse returnCode) {
        // ...
    }];
#else
    [alert beginSheetModalForWindow:window modalDelegate:self didEndSelector:nil contextInfo:nil];
#endif
    
}

- (void)downloadNewVersion {
    NSURLRequest *theRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:kNSLogDownloadPath] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:60.0];
    NSURLDownload *theDownload = [[NSURLDownload alloc] initWithRequest:theRequest delegate:self];
    if ( !theDownload ) {
        // download not possible
    }
}

- (void)alertDidEnd:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo {
    if ( returnCode == NSAlertFirstButtonReturn ) {
        [self downloadNewVersion];
    }
}

- (void)download:(id)sender {
    [window makeKeyAndOrderFront:nil];     // if we don't do this, the popup could be left with a horrible background
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setMessageText:@"A new version of NSLog is available."];
    [alert addButtonWithTitle:@"Download"];
    [alert addButtonWithTitle:@"Don't Download"];
    [alert setInformativeText:@"Do you want to download this now?"];
    NSArray *buttons = [alert buttons];
    [[buttons objectAtIndex:1] setKeyEquivalent: @"\033"];
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1090
    [alert beginSheetModalForWindow:window completionHandler:^(NSModalResponse returnCode) {
        if ( returnCode == NSAlertFirstButtonReturn ) {
            [self downloadNewVersion];
        }
    }];
#else
    [alert beginSheetModalForWindow:window modalDelegate:self didEndSelector:@selector(alertDidEnd:returnCode:contextInfo:) contextInfo:nil];
#endif
    
}

- (void)checkForNewVersion {
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:kNSLogVersPath]];
    [NSURLConnection connectionWithRequest:request delegate:self];
}

#pragma mark NSURLConnection delegate methods
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    responseData = [[NSMutableData alloc] init];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [responseData appendData:data];
}

- (NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse*)cachedResponse {
    return nil;
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    NSString *shortVers = [NSLogVers stringByReplacingOccurrencesOfString:@"." withString:@""];
    NSString *versionString = [[NSString alloc] initWithData:responseData encoding:NSUTF8StringEncoding];
    if ( [shortVers intValue] < [versionString intValue] ) {
        
        if ( hideNwVersionAvailableBtn == NO ) {
            nwVersionAvailableBtn = [[NSButton alloc] initWithFrame:NSMakeRect( 55, 3, 122, 16 )];
            [nwVersionAvailableBtn setFont:[NSFont boldSystemFontOfSize:9.5]];
            [nwVersionAvailableBtn setTitle:@"New version available"];
            [nwVersionAvailableBtn setShowsBorderOnlyWhileMouseInside:YES];
            [nwVersionAvailableBtn setButtonType:NSMomentaryPushInButton];
            [nwVersionAvailableBtn setBezelStyle:NSRecessedBezelStyle];
            [nwVersionAvailableBtn setTarget:self];
            [nwVersionAvailableBtn setAction:@selector(download:)];
            [[window contentView] addSubview:nwVersionAvailableBtn];
            NSLogReleaseObject( nwVersionAvailableBtn );
        }
        
    }
    NSLogReleaseObject( versionString );
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    // Request failed!
}

#pragma mark - init
- (id)init {
    self = [super init];
    if ( ![NSApp mainMenu] ) NSApplicationLoad();     // if app doesn't have Cocoa main menu (Carbon app)
    
    // window
    window = [[NSPanel alloc] initWithContentRect:NSMakeRect( 100, 100, 400, 200 ) styleMask:NSTitledWindowMask | NSClosableWindowMask | NSResizableWindowMask backing:NSBackingStoreBuffered defer:NO];
    [window setTitle:@"NSLog"];
    [window setDelegate:self];
    [window setOpaque:NO];
    [window setAutorecalculatesContentBorderThickness:YES forEdge:NSMinYEdge];
    [window setContentBorderThickness:kNSLogWindowBorderThickness forEdge:NSMinYEdge];
    [window setBecomesKeyOnlyIfNeeded:YES];
    [window setMinSize:NSMakeSize(275, 202)];
    [window setHidesOnDeactivate:NO];
    [window setFrameAutosaveName:kNSLogWindowBoundsKey];
    
    // contentView
    NSView *contentView = [window contentView];
    NSRect frame = [contentView frame];
    frame.size.height -= kNSLogWindowBorderThickness;
    frame.origin.y += kNSLogWindowBorderThickness;
    
    // scrollView
    scrollView = [[NSScrollView alloc] initWithFrame:frame];
    NSSize contentSize = [scrollView contentSize];
    [scrollView setBorderType:NSNoBorder];
    [scrollView setHasVerticalScroller:YES];
    [scrollView setAutohidesScrollers:YES];
    [scrollView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
    
    // textView
    NSRect textViewFrame = NSMakeRect( 0, 0, frame.size.width,frame.size.height );
    textView = [[NSTextView alloc] initWithFrame:textViewFrame];
    [textView setMinSize:NSMakeSize(0.0, contentSize.height)];
    [textView setMaxSize:NSMakeSize(FLT_MAX, FLT_MAX)];
    [textView setVerticallyResizable:YES];
    [textView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
    [textView setEditable:NO];
    
#if __MAC_OS_X_VERSION_MAX_ALLOWED > 1060
    [textView setUsesFindBar:YES];
    textFinder = [[NSTextFinder alloc] init];
    [textFinder setClient:(id<NSTextFinderClient>)textView];
    [textFinder setFindBarContainer:(id<NSTextFinderBarContainer>)scrollView];
#endif
    
    [[textView textContainer] setContainerSize:NSMakeSize(contentSize.width, FLT_MAX)];
    [[textView textContainer] setWidthTracksTextView:YES];
    CFStringRef fontName = CFPreferencesCopyAppValue( kNSLogFontNameKey, kNSLogApplicationID );
    NSFont *font = nil;
    if ( fontName ) {
        CFNumberRef number = CFPreferencesCopyAppValue( kNSLogFontSizeKey, kNSLogApplicationID );
        CGFloat size;
        CFNumberGetValue( number, kCFNumberCGFloatType, &size );
        CFRelease( number );
        
#if !__has_feature(objc_arc)
        font = [NSFont fontWithName:(NSString*)fontName size:size];
#else
        font = [NSFont fontWithName:(__bridge NSString*)fontName size:size];
#endif
        
        CFRelease( fontName );
    } else {
        font = [NSFont fontWithName:(NSString *)kNSLogDefaultFontName size:kNSLogDefaultFontSize];
    }
    if ( font ) [textView setFont:font];
    
    [contentView addSubview:scrollView];
    [scrollView setDocumentView:textView];
    NSLogReleaseObject( scrollView );
    
    // settingsBtn
    settingsBtn = [[NSButton alloc] initWithFrame:NSMakeRect( 10, 1, 20, 20 )];
    [settingsBtn setBordered:NO];
    [settingsBtn setImage:[NSImage imageNamed:NSImageNameSmartBadgeTemplate]];
    [[settingsBtn cell] setHighlightsBy:NSContentsCellMask];
    [settingsBtn setToolTip:@"Settings"];
    [settingsBtn setTarget:self];
    [settingsBtn setAction:@selector(settingsBtnAction:)];
    [contentView addSubview:settingsBtn];
    NSLogReleaseObject( settingsBtn );
    
    // wrapBtn
    wrapBtn = [[NSButton alloc] initWithFrame:NSMakeRect(35,1,20,20)];
    //[wrapBtn setBezelStyle:NSTexturedSquareBezelStyle];
    [[wrapBtn cell] setHighlightsBy:NSContentsCellMask];
    [wrapBtn setTitle:@""];
    [wrapBtn setBordered:NO];
    [wrapBtn setFont:[NSFont fontWithName:@"Symbol" size:15]];
    [wrapBtn setTitle:[NSString stringWithFormat:@"%C",10619]];
    [wrapBtn setToolTip:@"Wrap/Unwrap text"];
    [wrapBtn setTarget:self];
    [wrapBtn setAction:@selector(wrapButtonAction:)];
    [contentView addSubview:wrapBtn];
    NSLogReleaseObject( wrapBtn );
    
    // clearBtn
    clearBtn = [[NSButton alloc] initWithFrame:NSMakeRect( frame.size.width - 35, 1, 20, 20 )];
    [clearBtn setBordered:NO];
    NSImage *trashImage = [NSImage imageNamed:NSImageNameTrashEmpty];
    [trashImage setSize:NSMakeSize(18,18)];
    
    if ( [self sysVersion] < 1010 ) {
        [clearBtn setImage:trashImage];
    } else {
        
        // make OS 10.10 trash icon darker
        NSSize trashImageSize = [trashImage size];
        NSImage *darkImage = [[NSImage alloc] initWithSize:trashImageSize];
        [darkImage lockFocus];
        [trashImage drawAtPoint:NSZeroPoint fromRect:NSMakeRect(0,0,trashImageSize.width,trashImageSize.height) operation:NSCompositeSourceOver fraction:1.0];
        [[[NSColor blackColor] colorWithAlphaComponent:0.15] set];
        NSRectFillUsingOperation(NSMakeRect(0,0,trashImageSize.width,trashImageSize.height), NSCompositeSourceAtop);
        [darkImage unlockFocus];
        [clearBtn setImage:darkImage];
        
#if !__has_feature(objc_arc)
        [darkImage release];
#endif
    }
    
    [[clearBtn cell] setHighlightsBy:NSContentsCellMask];
    [clearBtn setToolTip:@"Clear log"];
    [clearBtn setTarget:self];
    [clearBtn setAction:@selector(clearBtnAction:)];
    [contentView addSubview:clearBtn];
    [clearBtn setAutoresizingMask:NSViewMinXMargin];
    NSLogReleaseObject( clearBtn );
    
    // saveBtn
    saveBtn = [[NSButton alloc] initWithFrame:NSMakeRect( frame.size.width - 65, 1, 20, 20 )];
    [saveBtn setBordered:NO];
    NSImage *saveImage = [[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(kGenericHardDiskIcon)];
    [saveImage setSize:NSMakeSize(15,15)];
    [saveBtn setImage:saveImage];
    [[saveBtn cell] setHighlightsBy:NSContentsCellMask];
    [saveBtn setToolTip:@"Save log to disk"];
    [saveBtn setTarget:self];
    [saveBtn setAction:@selector(saveBtnAction:)];
    [contentView addSubview:saveBtn];
    [saveBtn setAutoresizingMask:NSViewMinXMargin];
    NSLogReleaseObject( saveBtn );
    
    // findBtn
    #if __MAC_OS_X_VERSION_MAX_ALLOWED > 1060
        findBtn = [[NSButton alloc] initWithFrame:NSMakeRect( frame.size.width - 95, 1, 20, 20 )];
    [findBtn setBordered:NO];
    [findBtn setImage:[NSImage imageNamed:NSImageNameRevealFreestandingTemplate]];
    [[findBtn cell] setHighlightsBy:NSContentsCellMask];
    [findBtn setToolTip:@"Search log"];
    [findBtn setTarget:self];
    [findBtn setAction:@selector(findBtnAction:)];
    [contentView addSubview:findBtn];
    [findBtn setAutoresizingMask:NSViewMinXMargin];
    NSLogReleaseObject( findBtn );
#endif
    
    // drawer
    CGFloat height = [window frame].size.height;
    drawer = [[NSDrawer alloc] initWithContentSize:NSMakeSize( 130.0, height ) preferredEdge:NSMinXEdge];
    [drawer setMinContentSize:NSMakeSize( 90.0, 0.0 )];
    [drawer setMaxContentSize:NSMakeSize( 90.0, FLT_MAX )];
    [drawer setParentWindow:window];
    
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 20, 47, 16 ) title:@"Date" tag:kNSLogDateBtnTag toolTip:@"Print date before message"];
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 40, 49, 16 ) title:@"Time" tag:kNSLogTimeBtnTag toolTip:@"Print time before message"];
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 60, 77, 16 ) title:@"App name" tag:kNSLogAppNameBtnTag toolTip:@"Print app name before message"];
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 80, 85, 16 ) title:@"Process ID" tag:kNSLogProcessIDBtnTag toolTip:@"Print app process ID before message"];
    
    NSBox *separator1 = [[NSBox alloc] initWithFrame:NSMakeRect( 2, height - 88, 125, 1 )];
    [separator1 setBoxType:NSBoxCustom];
    [separator1 setBorderType:NSLineBorder];
    [[drawer contentView] addSubview:separator1];
    [separator1 setAutoresizingMask:NSViewMinYMargin];
    NSLogReleaseObject( separator1 );
    
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 110, 85, 16 ) title:@"Console" tag:kNSLogConsoleBtnTag toolTip:@"Forward messages to Console.app"];
    
    NSBox *separator2 = [[NSBox alloc] initWithFrame:NSMakeRect( 2, height - 118, 125, 1 )];
    [separator2 setBoxType:NSBoxCustom];
    [separator2 setBorderType:NSLineBorder];
    [[drawer contentView] addSubview:separator2];
    [separator2 setAutoresizingMask:NSViewMinYMargin];
    NSLogReleaseObject( separator2 );
    
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 140, 65, 16 ) title:@"Floating" tag:kNSLogFloatingBtnTag toolTip:@"Make NSLog window floating window"];
    
    NSText *label = [[NSText alloc] initWithFrame:NSMakeRect( 0, height - 280, 125, 100 )];
    [label setEditable:NO];
    [label setSelectable:NO];
    [label setFont:[NSFont systemFontOfSize:[NSFont systemFontSizeForControlSize:NSMiniControlSize]]];
    NSString *versText = [NSString stringWithFormat:@"NSLog v%@\r\u2022\rBernie Wylde", NSLogVers];
    [label setString:versText];
    [label setAlignment:NSCenterTextAlignment];
    [label setDrawsBackground:NO];
    [label setAutoresizingMask:NSViewMinYMargin];
    [[drawer contentView] addSubview:label];
    NSLogReleaseObject( label );
    
    [self switchButtonInitWithFrame:NSMakeRect( 4, height - 270, 125, 16 ) title:@"Check for Updates" tag:kNSLogCheckForUpdatesBtnTag toolTip:@"Automatically check for updates"];
    
    [self loadPrefs];
    [self setTextWrap:wrapBtn];
    
    if ( [[self buttonWithTag:kNSLogCheckForUpdatesBtnTag] state] == NSOnState ) [self checkForNewVersion];
    
    return self;
}
@end

#pragma mark - private functions
NSLogController *NSLogGetController()
{
    if ( sNSLogSuppress ) return nil;
    static NSLogController *sNSLogController;
    if ( sNSLogController == nil ) sNSLogController = [[NSLogController alloc] init];
    return sNSLogController;
}

void NSLogV2( NSString *formatString, va_list ap, BOOL cr )
{
    NSLogController *controller = NSLogGetController();
    if ( controller != nil ) {
        NSMutableString *temp = [NSMutableString stringWithCapacity:0];
        BOOL dateFlag = [[controller buttonWithTag:kNSLogDateBtnTag] state] == NSOnState;
        BOOL timeFlag = [[controller buttonWithTag:kNSLogTimeBtnTag] state] == NSOnState;
        if ( dateFlag || timeFlag ) {
            NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
            NSMutableString *format = [[NSMutableString alloc] initWithCapacity:0];
            if ( dateFlag ) {
                [format appendString:@"yyyy-MM-dd"];
                if ( timeFlag ) [format appendString:@" "];
            }
            if ( timeFlag ) [format appendString:@"HH:mm:ss.SSS"];
            [dateFormatter setDateFormat:format];
            NSLogReleaseObject( format );
            [temp appendString:[dateFormatter stringFromDate:[NSDate date]]];
            NSLogReleaseObject( dateFormatter );
        }
        
        if ( [[controller buttonWithTag:kNSLogAppNameBtnTag] state] == NSOnState ) {
            if ( [temp length] > 0 ) [temp appendString:@" "];
            [temp appendString:[[NSProcessInfo processInfo] processName]];
        }
        
        if ( [[controller buttonWithTag:kNSLogProcessIDBtnTag] state] == NSOnState ) {
            if ( [temp length] > 0 ) [temp appendString:@" "];
            [temp appendFormat:@"[%d]",[[NSProcessInfo processInfo] processIdentifier]];
        }
        
        if ( [temp length] > 0 ) [temp appendString:@": "];
        
        [temp appendString:(NSString *)formatString];
        if ( cr ) [temp appendString:@"\n"];
        
        va_list apCopy;
        va_copy(apCopy, ap);
        NSString *string = [[NSString alloc] initWithFormat:temp arguments:apCopy];
        va_end(apCopy);
        
        [[controller textView] setSelectedRange:NSMakeRange( [[[controller textView] string] length], 0 )];
        [[controller textView] setEditable:YES];
        [[controller textView] insertText:string];
        [[controller textView] setEditable:NO];
        
        NSLogShow();
        if ( [[controller buttonWithTag:kNSLogConsoleBtnTag] state] == NSOnState ) NSLogv( formatString, ap );
        
        // alert
        if ( sNSLogAlert ) {
            [controller pauseWithMessage:string];
            sNSLogAlert = NO;
        }
        
        NSLogReleaseObject( string );
    } else {
        NSLogv( formatString, ap );
    }
}

void NSLogV( NSString *formatString, va_list ap )
{ NSLogV2( formatString, ap, YES ); }

void NSLogSetButtonState( NSInteger tag, BOOL flag )
{ [NSLogGetController() buttonWithTag:tag setStateWithBool:flag]; }

void NSLogReleaseObject( id obj )
{
#if !__has_feature(objc_arc)
    [obj release];
#endif
}


#pragma mark - sending messages
void NSLog( NSString *formatString, ... )
{
#if !__has_feature(objc_arc)
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#endif
    va_list ap;
    va_start( ap, formatString );
    NSLogV( formatString, ap );
    va_end( ap );
#if !__has_feature(objc_arc)
    [pool drain];
#endif
}

void NSLogAlert( NSString *formatString, ... )
{
#if !__has_feature(objc_arc)
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#endif
    va_list ap;
    va_start( ap, formatString );
    sNSLogAlert = YES;
    NSLogV( formatString, ap );
    va_end( ap );
#if !__has_feature(objc_arc)
    [pool drain];
#endif
}

int printf( const char *format, ... )
{
    va_list ap;
    va_start( ap, format );
    NSString *string = [NSString stringWithCString:format encoding:NSMacOSRomanStringEncoding];
    NSLogV2( string, ap, NO );
    int result = vprintf( format, ap );
    va_end( ap );
    return result;
}


#pragma mark - clearing the log
void NSLogClear()
{ [NSLogGetController() clearBtnAction:nil]; }


#pragma mark - message appearance and format
void NSLogSetShowsDate( BOOL flag )
{ NSLogSetButtonState( kNSLogDateBtnTag, flag ); }

void NSLogSetShowsTime( BOOL flag )
{ NSLogSetButtonState( kNSLogTimeBtnTag, flag ); }

void NSLogSetShowsAppName( BOOL flag )
{ NSLogSetButtonState( kNSLogAppNameBtnTag, flag ); }

void NSLogSetShowsProcessID( BOOL flag )
{ NSLogSetButtonState( kNSLogProcessIDBtnTag, flag ); }

void NSLogSetFont( NSString *name, CGFloat size )
{
    NSLogController *controller = NSLogGetController();
    if ( name == nil ) name = [[[controller textView] font] fontName];
    NSFont *font = [NSFont fontWithName:name size:size];
    if ( font ) {
        [[controller textView] setFont:font];
        
#if !__has_feature(objc_arc)
        CFPreferencesSetAppValue( kNSLogFontNameKey, (CFStringRef)name, kNSLogApplicationID );
#else
        CFPreferencesSetAppValue( kNSLogFontNameKey, (__bridge CFStringRef)name, kNSLogApplicationID );
#endif
        
        CFNumberRef number = CFNumberCreate( kCFAllocatorDefault, kCFNumberCGFloatType, &size );
        CFPreferencesSetAppValue( kNSLogFontSizeKey, number, kNSLogApplicationID );
        CFRelease( number );
        CFPreferencesAppSynchronize( kNSLogApplicationID );
    }
}

void NSLogSetTempFont( NSString *name, CGFloat size )
{
    NSLogController *controller = NSLogGetController();
    if ( name == nil ) name = [[[controller textView] font] fontName];
    NSFont *font = [NSFont fontWithName:name size:size];
    if ( font ) [[controller textView] setFont:font];
}

NSFont *NSLogFont()
{ return [[NSLogGetController() textView] font]; }

void NSLogRestoreDefaultFont()
{ NSLogSetFont( (NSString *)kNSLogDefaultFontName, kNSLogDefaultFontSize ); }


#pragma mark - suppressing log output
void NSLogSuppress( BOOL flag )
{ sNSLogSuppress = flag; }


#pragma mark - accessing message text
NSString *NSLogString()
{ return [[NSLogGetController() textView] string]; }

void NSLogWriteToURL( NSURL *url )
{ [[[NSLogGetController() textView] string]  writeToURL:url atomically:YES encoding:NSUTF8StringEncoding error:nil]; }

void NSLogSave( NSString *message, NSString *filename )
{ [NSLogGetController() saveWithMessage:message nameFieldStringValue:filename]; }


#pragma mark - window
void NSLogSetTitle( NSString *title )
{ [[NSLogGetController() window] setTitle:title]; }

void NSLogSetFrame( NSRect r )
{ [[NSLogGetController() window] setFrame:r display:YES]; }

NSRect NSLogFrame()
{ return [[NSLogGetController() window] frame]; }

void NSLogShow()
{
    NSLogController *controller = NSLogGetController();
    if ( ![[controller window] isVisible] ) {
        [[controller window] orderFront:nil];
        [[controller window] setFloatingPanel:([[controller buttonWithTag:kNSLogFloatingBtnTag] state] == NSOnState)];
    }
}

void NSLogHide()
{ [[NSLogGetController() window] orderOut:nil]; }

NSPanel *NSLogWindow()
{ return [NSLogGetController() window]; }

void NSLogSetFloatingWindow( BOOL flag )
{
    NSLogSetButtonState( kNSLogFloatingBtnTag, flag );
    [[NSLogGetController() window] setFloatingPanel:flag];
}

void NSLogSetBecomesKeyOnlyIfNeeded( BOOL flag )
{ [[NSLogGetController() window] setBecomesKeyOnlyIfNeeded:flag]; }

void NSLogSetTerminateAppWhenWindowClosed( BOOL flag )
{ [NSLogGetController() setTerminateAppWhenWindowClosed:flag]; }


#pragma mark - buttons
void NSLogSettingsButtonSetHidden( BOOL flag )
{ [[NSLogGetController() settingsBtn] setHidden:flag]; }

void NSLogWrapTextButtonSetHidden( BOOL flag )
{ [[NSLogGetController() wrapBtn] setHidden:flag]; }

void NSLogNewVersionAvailableButtonSetHidden( BOOL flag )
{
    NSLogController *controller = NSLogGetController();
    [[controller nwVersionAvailableBtn] setHidden:flag];
    [controller setHideNwVersionAvailableBtn:flag];
}

void NSLogFindButtonSetHidden( BOOL flag )
{ [[NSLogGetController() findBtn] setHidden:flag]; }

void NSLogSaveButtonSetHidden( BOOL flag )
{ [[NSLogGetController() saveBtn] setHidden:flag]; }

void NSLogClearButtonSetHidden( BOOL flag )
{ [[NSLogGetController() clearBtn] setHidden:flag]; }

void NSLogButtonsSetHidden( BOOL flag )
{
    NSLogSettingsButtonSetHidden( flag );
    NSLogWrapTextButtonSetHidden( flag );
    NSLogNewVersionAvailableButtonSetHidden( flag );
    NSLogFindButtonSetHidden( flag );
    NSLogSaveButtonSetHidden( flag );
    NSLogClearButtonSetHidden( flag );
}

#pragma mark - console
void NSLogSetForwardToConsole( BOOL flag )
{ NSLogSetButtonState( kNSLogConsoleBtnTag, flag ); }

#pragma mark - check for updates
void NSLogSetCheckForUpdates( BOOL flag )
{
    NSLogSetButtonState( kNSLogCheckForUpdatesBtnTag, flag );
    if ( flag ) [NSLogGetController() checkForNewVersion];
}


