/*
  Shows basics of working with updated Window statement in FB 5.7.102
  Window Statement in FB 5.7.102 changes the way a rectangle and window attributes are specified.
  Brian 20170625
*/

dim as CGRect      cg
dim as Rect        r
dim as SInt32      wa(7)

wa(0) = _kHIWindowBitStandardHandler
wa(1) = _kHIWindowBitCloseBox
wa(2) = _kHIWindowBitZoomBox
wa(3) = _kHIWindowBitCollapseBox
wa(4) = _kHIWindowBitResizable
wa(5) = _kHIWindowBitLiveResize
wa(6) = _kHIWindowBitCompositing
wa(7) = 0
SetRect( r, 4, 44, 484, 404 )   // not required for Window statement but for later conversion to CGRect.

// Shows different ways to create a window with a CGRect


// remember: the parameters for fn CGRectMake are x, y, width, height
window 1, @"Window 1", fn CGRectMake(500,50,200,350), _kDocumentWindowClass, @wa(0)


// create a CGRect from a previously created QuickDraw Rect ( which we **assume** is inside variable 'r'
window 2, @"Window 2", fn CGRectMake( r.left, r.top, r.right - r.left, r.bottom - r.top), _kDocumentWindowClass, @wa(0)


// this is the x1,y1-x2,y2 format. Internally, the runtime converts these to a CGRect that contains x, y, width, height
window 3, @"Window 3", ( 50,100)-(500,600), _kDocumentWindowClass, @wa(0)


// if you already have a CGRect variable, simply pass it in the Window statement
cg = fn CGRectMake( r.left, r.top, r.right - r.left, r.bottom - r.top)
window 4, @"Window 4", cg, _kDocumentWindowClass, @wa(0)

// window attributes may be specified as NULL
// NULLed window attributes means the window has *no* attributes and no default attributes are assigned.
cg = fn CGRectMake( 0, 350, 360, 240 )
window 2, @"NULL param", cg, _kDocumentWindowClass, NULL

// Missing window attributes may be omitted.
// Missing window attributes receive the default attributes for document windows but
// other window types receive no attributes.
//
cg = fn CGRectMake( 400, 350, 360, 240 )
window 4, @"Missing param", cg, _kDocumentWindowClass



RunApplicationEventLoop()




