/*
   EnumerateDirectory.m

   Bernie Wylde 20090702
   rp:20100209 
*/

#import <Cocoa/Cocoa.h>

void EnumerateDirectory( FSRef *inRef, Boolean recursive, OSStatus (*callBack)( FSRef*, void* ), void* userData ) {
	NSAutoreleasePool	*pool = [[NSAutoreleasePool alloc] init];
	FSRef				outRef;
	NSString			*file;
	BOOL				isDir;
	
	CFURLRef dirUrl = CFURLCreateFromFSRef( kCFAllocatorDefault, inRef );
	if ( dirUrl ) {
		CFStringRef dirPath = CFURLCopyFileSystemPath( dirUrl, kCFURLPOSIXPathStyle );
		if ( dirPath ) {
			NSDirectoryEnumerator *dirEnum = [[NSFileManager defaultManager] enumeratorAtPath:(NSString *)dirPath];
			while ((file = [dirEnum nextObject])) {
				NSString *objPath = [(NSString *)dirPath stringByAppendingPathComponent:file];
				if ( objPath ) {
					CFIndex length = CFStringGetMaximumSizeForEncoding( CFStringGetLength( (CFStringRef)objPath ), kCFStringEncodingUTF8 ) + 1;
					char *buffer = malloc( length );
					if ( buffer ) {
						CFStringGetCString( (CFStringRef)objPath, buffer, length, kCFStringEncodingUTF8 );
						FSPathMakeRef( (UInt8 *)buffer, &outRef, NULL );
						free( buffer );
						(*callBack)( &outRef, userData );
					}
					if ( recursive == false ) {
						[[NSFileManager defaultManager] fileExistsAtPath:objPath isDirectory:&isDir];
						if ( isDir ) [dirEnum skipDescendents];						
					}
				}
			}
			CFRelease( dirPath );
		}
		CFRelease( dirUrl );
	}
	[pool release];
}



