
/*
Problem:  How to create a file URL to a non-existent file when all you have is an FSSpec for guidance. 
          Further, OPEN "O" with a FSSpec won't work in FB 5.7.99, so that's not a solution.
           
Solution: Even if the file doesn't exist the parent directory should. Getting the parent directory from the FSSpec is the key, 
          convert that to a full posix path or URL and then append the file name ( from FSSpec.name )

This solution was helpful ( and born ) when Brian had to upgrade the FB Editor source to be compatible with FB 5.7.99 ( i.e. OPEN changed to use URLs etc. ).
The Editor uses FSSpecs extensively and there wasn't time to completely revamp the code to use URLs. It was necessary to do conversions near 
the OPEN statements to allow it to build with FB 5.7.99

Bernie and Brian 20170316. 
*/

// ---------------
include "NSLog.incl"
include "Util_FileDirectory.incl"

dim as FSSpec fileSpec, parSpec
dim as CFURLRef parURL, fileURL
dim as CFStringRef filename

if ( files$(_FSSpecOpen,,,@fileSpec) )  // this is done just to give us an FSSpec. FB 5.7.99 doesn't allow this, so just imagine your code provides an FSSpec
if ( fn FSMakeFSSpec( fileSpec.vRefNum, fileSpec.parID, "", @parSpec ) == _noErr )   // get the parent directory/folder of the file
if ( fn FD_FSSpecCreateCFURL( @parSpec, @parURL ) == _noErr )                      //  create parent directory URL from the parent directory FSSpec 
filename = fn CFStringCreateWithPascalString( _kCFAllocatorDefault, fileSpec.name, _kCFStringEncodingMacRoman )   // create a CFString from the FSSpec's filename
fileURL = fn CFURLCreateCopyAppendingPathComponent( _kCFAllocatorDefault, parURL, filename, _false )  // append the path ( i.e. the filename ) to the parent URL to create the full file URL.

NSLog(@"The full file URL is: %@",fileURL)

CFRelease( parURL )
CFRelease( filename )
CFRelease( fileURL )
end if
end if
end if

RunApplicationEventLoop()


