/*
 Appearance.m
 
 Bernie Wylde
 */

#import "Appearance.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

// Class
Class AppearanceClass( void )
{ return [NSAppearance class]; }

// Name
CFStringRef AppearanceName( NSAppearance *ref )
{ return (__bridge CFStringRef)[ref name]; }

// Appearance
NSAppearance *AppearanceNamed( CFStringRef name )
{ return [NSAppearance appearanceNamed:(__bridge NSString *)name]; }

// Most appropriate appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
CFStringRef AppearanceBestMatchFromAppearancesWithNames( NSAppearance *ref, CFArrayRef names )
{
    if (@available(macOS 10.14, *)) {
        return (__bridge CFStringRef)[ref bestMatchFromAppearancesWithNames:(__bridge NSArray *)names];
    }
    return nil;
}
#endif

// Current appearance
NSAppearance *AppearanceCurrent( void )
{ return [NSAppearance currentAppearance]; }

void AppearanceSetCurrent( NSAppearance *ref )
{ [NSAppearance setCurrentAppearance:ref]; }

// Vibrancy
BOOL AppearanceAllowsVibrancy( NSAppearance *ref )
{ return [ref allowsVibrancy]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
// Type properties
NSAppearance *AppearanceCurrentDrawing( void )
{
    if (@available(macOS 11.0, *)) {// 20210101
        return [NSAppearance currentDrawingAppearance];
    }
    return nil;
}

// Instance methods
void AppearancePerformAsCurrentDrawing( NSAppearance *ref, CUI_AppearancePerformAsCurrentDrawingCallbackType callback, void *userData )
{
    if (@available(macOS 11.0, *)) {// 20210101
        [ref performAsCurrentDrawingAppearance:^( void ) {
            (*callback)(ref,userData);
        }];
    }
}
#endif// 101600

#endif// 101000
