/*
 Button.m
 
 Bernie Wylde
 */

#import "Button.h"

#pragma mark - subclass
@implementation ButtonSC
- (void)drawRect:(NSRect)dirtyRect {
    CUI_ViewSubclassDrawRect( self, dirtyRect );
    if ( !DialogEventBool() ) [super drawRect:dirtyRect];
}

- (BOOL)acceptsFirstResponder {
    DialogEventSetBool(NO);
    CUI_ViewSubclassAcceptsFirstResponder( self );
    return DialogEventBool();
}

- (void)mouseDown:(NSEvent *)event {
    CUI_ViewSubclassMouseDown( self, event );
    if ( !DialogEventBool() ) [super mouseDown:event];
}

- (void)mouseUp:(NSEvent *)event {
    CUI_ViewSubclassMouseUp( self, event );
}

- (void)mouseEntered:(NSEvent *)event {
    CUI_ViewSubclassMouseEntered( self, event );
}

- (void)mouseExited:(NSEvent *)event {
    CUI_ViewSubclassMouseExited( self, event );
}

- (void)mouseDragged:(NSEvent *)event {
    CUI_ViewSubclassMouseDragged( self, event );
}

- (void)mouseMoved:(NSEvent *)event {
    CUI_ViewSubclassMouseMoved( self, event );
}

- (void)rightMouseDown:(NSEvent *)event {
    CUI_ViewSubclassRightMouseDown( self, event );
    if ( !DialogEventBool() ) [super rightMouseDown:event];
}

- (void)rightMouseUp:(NSEvent *)event {
    CUI_ViewSubclassRightMouseUp( self, event );
}

- (void)rightMouseDragged:(NSEvent *)event {
    CUI_ViewSubclassRightMouseDragged( self, event );
}

- (void)keyDown:(NSEvent *)event {
    CUI_ViewSubclassKeyDown( self, event );
    if ( !DialogEventBool() ) [super keyDown:event];
}

- (void)keyUp:(NSEvent *)event {
    CUI_ViewSubclassKeyUp( self, event );
}

- (void)flagsChanged:(NSEvent *)event {
    CUI_ViewSubclassFlagsChanged( self, event );
}

//- (void)updateTrackingAreas {
//    CUI_ViewSubclassUpdateTrackingAreas( self );
//    [super updateTrackingAreas];
//}

- (void)awakeFromNib {
    [super awakeFromNib];
    CUI_ViewSubclassAwakeFromNib(self);
}

#pragma mark - menu
- (NSMenu *)menuForEvent:(NSEvent*)theEvent {
    CUI_ViewSubclassMenuForEvent( self, theEvent );
    return DialogEventNSMenu();
}
@end

#pragma mark - functions
// Class
Class ButtonClass( void )
{ return [NSButton class]; }

NSButton *ButtonWithTag( NSInteger tag )
{ return (NSButton *)CUI_ValidateViewKindOfClass( tag, [NSButton class] ); }

BOOL ButtonExists( NSInteger tag )
{ return CUI_ViewWithKindOfClassExists( tag, [NSButton class] ); }

void ButtonSetType( NSInteger tag, NSButtonType type )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setButtonType:type];
}

void ButtonGetPeriodicDelay( NSInteger tag, float *delay, float *interval )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn getPeriodicDelay:delay interval:interval];
}

CFStringRef ButtonAlternateTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (__bridge CFStringRef)[btn alternateTitle];
    return nil;
}

CFAttributedStringRef ButtonAttributedTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (__bridge CFAttributedStringRef)[btn attributedTitle];
    return nil;
}


CFAttributedStringRef ButtonAttributedAlternateTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (__bridge CFAttributedStringRef)[btn attributedAlternateTitle];
    return nil;
}

void ButtonSetTitle( NSInteger tag, CFStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setTitle:(__bridge NSString *)title];
}

NSSound *ButtonSound( NSInteger tag)
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn sound];
    return nil;
}

NSImage *ButtonImage( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn image];
    return nil;
}

NSImage *ButtonAlternateImage( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn alternateImage];
    return nil;
}

NSCellImagePosition ButtonImagePosition( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn imagePosition];
    return 0;
}

BOOL ButtonIsBordered( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isBordered];
    return NO;
}

BOOL ButtonIsTransparent( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isTransparent];
    return NO;
}

NSBezelStyle ButtonBezelStyle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn bezelStyle];
    return 0;
}

void ButtonSetBezelStyle( NSInteger tag, NSBezelStyle style )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setBezelStyle:style];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101201
NSColor *ButtonBezelColor( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn bezelColor];
    return nil;
}

void ButtonSetBezelColor( NSInteger tag, NSColor *col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setBezelColor:col];
}
#endif

BOOL ButtonShowsBorderOnlyWhileMouseInside( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn showsBorderOnlyWhileMouseInside];
    return NO;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
BOOL ButtonImageHugsTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn imageHugsTitle];
    return NO;
}

void ButtonSetImageHugsTitle( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setImageHugsTitle:flag];
}
#endif

NSImageScaling ButtonImageScaling( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [[btn cell] imageScaling];
    return 0;
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
NSImageSymbolConfiguration *ButtonSymbolConfiguration( NSInteger tag )
{
    if (@available(macOS 11.0, *)) {// 20210101
        NSButton *btn = ButtonWithTag( tag );
        if ( btn ) return [btn symbolConfiguration];
    }
    return nil;
}

void ButtonSetSymbolConfiguration( NSInteger tag, NSImageSymbolConfiguration *config )
{
    if (@available(macOS 11.0, *)) {// 20210101
        NSButton *btn = ButtonWithTag( tag );
        if ( btn ) [btn setSymbolConfiguration:config];
    }
}
#endif// 101600

BOOL ButtonAllowsMixedState( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn allowsMixedState];
    return NO;
}

void ButtonSetState( NSInteger tag, NSCellStateValue state )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setState:state];
}

CFStringRef ButtonKeyEquivalent( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (__bridge CFStringRef)[btn keyEquivalent];
    return nil;
}

NSUInteger ButtonKeyEquivalentModifierMask( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn keyEquivalentModifierMask];
    return 0;
}

BOOL ButtonPerformKeyEquivalent( NSInteger tag, NSEvent *key )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn performKeyEquivalent:key];
    return 0;
}

NSCellStateValue ButtonState( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn state];
    return 0;
}

CFStringRef ButtonTitle( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return (__bridge CFStringRef)[btn title];
    return NULL;
}

void ButtonSetSound( NSInteger tag, NSSound *snd )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setSound:snd];
}

void ButtonSetSoundNamed( NSInteger tag, CFStringRef sndName )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setSound:[NSSound soundNamed:(__bridge NSString *)sndName]];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
BOOL ButtonIsSpringLoaded( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isSpringLoaded];
    return NO;
}

void ButtonSetSpringLoaded( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setSpringLoaded:flag];
}

NSInteger ButtonMaxAcceleratorLevel( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn maxAcceleratorLevel];
    return 0;
}

void ButtonSetMaxAcceleratorLevel( NSInteger tag, NSInteger level )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setMaxAcceleratorLevel:level];
}
#endif// 101003

void ButtonSetKeyEquivalent( NSInteger tag, CFStringRef key )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setKeyEquivalent:(__bridge NSString *)key];
}

void ButtonSetKeyEquivalentModifierMask( NSInteger tag, NSUInteger mask )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setKeyEquivalentModifierMask:mask];
}

void ButtonSetImage( NSInteger tag, void *inImage )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = CUI_FixImageValue( inImage );
        if ( image ) [btn setImage:image];
    }
}

void ButtonSetImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = CUI_ImageResource( (__bridge NSString *)imageName );
        if ( image ) [btn setImage:image];
    }
}

void ButtonSetImagePosition( NSInteger tag, NSCellImagePosition position )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setImagePosition:position];
}

void ButtonSetImageScaling( NSInteger tag, NSImageScaling scaling )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setImageScaling:scaling];
}

void ButtonSetBordered( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setBordered:flag];
}

void ButtonSetShowsBorderOnlyWhileMouseInside( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setShowsBorderOnlyWhileMouseInside:flag];
}

void ButtonPerformClick( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn performClick:nil];
}

void ButtonSetAttributedTitle( NSInteger tag, CFAttributedStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAttributedTitle:(__bridge NSAttributedString *)title];
}

void ButtonSetAlternateTitle( NSInteger tag, CFStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAlternateTitle:(__bridge NSString *)title];
}

void ButtonSetPeriodicDelay( NSInteger tag, float delay, float interval )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setPeriodicDelay:delay interval:interval];
}

void ButtonSetAttributedAlternateTitle( NSInteger tag, CFAttributedStringRef title )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAttributedAlternateTitle:(__bridge NSAttributedString *)title];
}

void ButtonSetAlternateImage( NSInteger tag, void *inImage )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = CUI_FixImageValue( inImage );
        if ( image ) [btn setAlternateImage:image];
    }
}

void ButtonSetAlternateImageNamed( NSInteger tag, CFStringRef imageName )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSImage *image = CUI_ImageResource( (__bridge NSString *)imageName );
        if ( image ) [btn setAlternateImage:image];
    }
}

void ButtonSetTransparent( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setTransparent:flag];
}

void ButtonSetAllowsMixedState( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setAllowsMixedState:flag];
}

void ButtonSetNextState( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setNextState];
}

void ButtonHighlight( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn highlight:flag];
}

BOOL ButtonIsOpaque( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [btn isOpaque];
    return NO;
}

void ButtonSetHighlightsBy( NSInteger tag, NSUInteger mask )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setHighlightsBy:mask];
}


// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
NSColor *ButtonContentTintColor( NSInteger tag )
{
    if (@available(macOS 10.14, *)) {
        NSButton *btn = ButtonWithTag( tag );
        if ( btn ) return [btn contentTintColor];
    }
    return nil;
}

void ButtonSetContentTintColor( NSInteger tag, NSColor *col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn setContentTintColor:col];
}
#endif// 101400

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
BOOL ButtonHasDestructiveAction( NSInteger tag )
{
    if (@available(macOS 11.0, *)) {// 20210101
        NSButton *btn = ButtonWithTag( tag );
        if ( btn ) return [btn hasDestructiveAction];
    }
    return NO;
}

void ButtonSetHasDestructiveAction( NSInteger tag, BOOL flag )
{
    if (@available(macOS 11.0, *)) {// 20210101
        NSButton *btn = ButtonWithTag( tag );
        if ( btn ) [btn setHasDestructiveAction:flag];
    }
}
#endif// 101600


// - buttoncell functions -
CFTypeRef ButtonKeyEquivalentFont( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [[btn cell] keyEquivalentFont];
    return nil;
}

void ButtonSetKeyEquivalentFont( NSInteger tag, CFTypeRef font )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setKeyEquivalentFont:font];
}


// custom
void ButtonClose( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [btn removeFromSuperview];
}

void ButtonSetTitleColor( NSInteger tag, NSColor *col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) {
        NSMutableAttributedString *as = [[[btn attributedTitle] mutableCopy] autorelease];
        [as addAttribute:NSForegroundColorAttributeName value:col range:NSMakeRange(0,[as length])];
        [btn setAttributedTitle:as];
    }
}

void ButtonSetBackgroundColor( NSInteger tag, NSColor *col )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setBackgroundColor:col];
}

BOOL ButtonImageDimsWhenDisabled( NSInteger tag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) return [[btn cell] imageDimsWhenDisabled];
    return NO;
}

void ButtonSetImageDimsWhenDisabled( NSInteger tag, BOOL flag )
{
    NSButton *btn = ButtonWithTag( tag );
    if ( btn ) [[btn cell] setImageDimsWhenDisabled:flag];
}


// Init
NSButton *ButtonInit( NSInteger inTag, CGRect inRect )
{
    // rect
    NSRect rect = NSRectFromCGRect( inRect );
    if ( NSIsEmptyRect( rect ) ) {
        rect = NSMakeRect( 14, 13, 81, 32 );
    }

    // subclass
    BOOL subclass = CUI_IsSubclassFlag();
    
    // init
    NSButton *btn;
    if ( subclass ) {
        btn = [[[ButtonSC alloc] initWithFrame:rect] autorelease];
    } else {
        btn = [[[NSButton alloc] initWithFrame:rect] autorelease];
    }
        
    // tag
    [btn setTag:inTag];
    
    // style
    [btn setBezelStyle:NSRoundedBezelStyle];

    // type
    [btn setButtonType:NSMomentaryLightButton];

    // font
    CUI_ControlFixFont( btn );

    // target/action
    [btn setTarget:CUI_ControlTargetObj()];
    [btn setAction:@selector(controlAction:)];
    
    return btn;
}

#pragma mark - button statement
void CUI_Button( NSInteger inTag, NSInteger inEnabled, NSInteger inState, CFStringRef inTitle, CGRect inRect, NSInteger inType, NSInteger inStyle, NSInteger inWndTag )
{
    BOOL show = ( inTag > 0 );
    inTag = ABS(inTag);
    
    NSString *title = (__bridge NSString *)inTitle;
    NSRect rect = NSRectFromCGRect( inRect );
    
    // target window
    id w = CUI_TargetWindow( &inWndTag );
    
    if ( w ) {
        NSView *contentView = CUI_TargetContentView( w );
        
        NSButton *btn = CUI_ViewSubviewWithTag( contentView, inTag );
        
        if ( !btn ) {
            btn = TitlebarAccessoryViewWithTag( inTag );
        }
        
        if ( btn ) {    // already exists
            // clear subclass flag
            CUI_SetSubclassFlag(NO);

            // make sure it's same view class
            if ( !CUI_ValidateViewClass( inWndTag, inTag, btn, [NSButton class] ) ) {
                return;
            }

            // enabled
            if ( inEnabled != kFBParamMissing ) [btn setEnabled:(BOOL)inEnabled];
            
            // state
            if ( inState != kFBParamMissing ) {
                if ( inState == NSMixedState ) [btn setAllowsMixedState:YES];
                [btn setState:inState];
            }
            
            // title
            if ( title ) [btn setTitle:title];
            
            // rect
            if ( !NSIsEmptyRect( rect ) ) [btn setFrame:rect];
            
            // type
            if ( inType != kFBParamMissing ) [btn setButtonType:inType];
            
            // style
            if ( inStyle != kFBParamMissing ) [btn setBezelStyle:inStyle];
            
        } else {        // create
            btn = ButtonInit( inTag, inRect );
            
            // default autoresizing
            [btn setAutoresizingMask:NSViewMinYMargin];
            
            // state
            if ( inState != kFBParamMissing ) {
                if ( inState == NSMixedState ) [btn setAllowsMixedState:YES];
                [btn setState:inState];
            }
            
            // enabled
            if ( inEnabled != kFBParamMissing ) [btn setEnabled:(BOOL)inEnabled];
            
            // type
            if ( inType == kFBParamMissing ) inType = NSMomentaryLightButton;
            [btn setButtonType:inType];
            
            // style
            if ( inStyle == kFBParamMissing ) inStyle = NSRoundedBezelStyle;
            [btn setBezelStyle:inStyle];
            
            switch ( inStyle ) {
                case NSHelpButtonBezelStyle:
                    if ( !title ) title = @"";
                    break;
                case NSRecessedBezelStyle:
                    [btn setShowsBorderOnlyWhileMouseInside:YES];
                    break;
                default:
                    break;
            }
            
            // title
            if ( title ) {
                [btn setTitle:title];
                if ( inStyle == NSRoundedBezelStyle ) {
                    if ( inType == NSMomentaryLightButton || inType == NSMomentaryPushInButton ) {
                        if ( [title isEqualToString:@"OK"] ) [btn setKeyEquivalent:@"\r"];
                        if ( [title isEqualToString:@"Cancel"] ) [btn setKeyEquivalent:@"\e"];
                    }
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300//101003
                    if ( inType == NSButtonTypeAccelerator || inType == NSButtonTypeMultiLevelAccelerator ) {
                        if ( [title isEqualToString:@"OK"] ) [btn setKeyEquivalent:@"\r"];;
                        if ( [title isEqualToString:@"Cancel"] ) [btn setKeyEquivalent:@"\e"];
                    }
#endif
                }
            }
            
            // wndTag
            [btn setWndTag:inWndTag];
            
            // fix font
            switch ( inStyle ) {
                case NSRoundRectBezelStyle:
                    [btn setFont:[NSFont systemFontOfSize:12]];
                    break;
                case NSRecessedBezelStyle:
                    [[btn cell] setControlSize:NSSmallControlSize];
                    [btn setFont:[NSFont boldSystemFontOfSize:12]];
                    break;
                    
                case NSInlineBezelStyle:
                    [btn setFont:[NSFont boldSystemFontOfSize:11]];
                    break;
                    
                default:
                    CUI_ControlFixFont( btn );
                    break;
            }
            
            // add to window/popover/superview
            id superview = CUI_CurrentSuperview();
            if ( superview ) {
                [superview addSubview:btn];
            } else {
                [contentView addSubview:btn];
            }
        }
        
        [btn setHidden:!show];
    } else {
        CUI_ShowWindowDoesNotExistAlert( inWndTag );
    }
}

void CUI_Checkbox( NSInteger inTag, NSInteger inEnabled, NSInteger inState, CFStringRef inTitle, CGRect inRect, NSInteger inWndTag )
{ CUI_Button( inTag, inEnabled, inState, inTitle, inRect, NSSwitchButton, kFBParamMissing, inWndTag ); }

void CUI_RadioButton( NSInteger inTag, NSInteger inEnabled, NSInteger inState, CFStringRef inTitle, CGRect inRect, NSInteger inWndTag )
{ CUI_Button( inTag, inEnabled, inState, inTitle, inRect, NSRadioButton, kFBParamMissing, inWndTag ); }

