/*
 CocoaUI.m
 
 Bernie Wylde 20170712
 */

#import "CocoaUI.h"

#pragma mark - subclass flag
static BOOL sCUI_SubclassFlag;

BOOL CUI_IsSubclassFlag( void )
{
    BOOL flag = sCUI_SubclassFlag;
    sCUI_SubclassFlag = NO;// reset
    return flag;
}

void CUI_SetSubclassFlag( BOOL flag )
{ sCUI_SubclassFlag = flag; }


#pragma mark - superview
static NSMutableArray *sCUI_Superviews;
static BOOL sCUI_SuperviewFlag;

BOOL CUI_IsSuperview( void )
{
    BOOL flag = sCUI_SuperviewFlag;
    sCUI_SuperviewFlag = NO;// reset
    return flag;
}

void CUI_SetSuperviewFlag( BOOL flag )
{ sCUI_SuperviewFlag = flag; }

void CUI_AddSuperview( id superview )
{
    if ( !sCUI_Superviews ) sCUI_Superviews = [[NSMutableArray alloc] initWithCapacity:0];
    [sCUI_Superviews addObject:superview];
}

id CUI_CurrentSuperview( void )
{
    if ( sCUI_Superviews ) {
        if ( [sCUI_Superviews count] > 0 ) return [sCUI_Superviews lastObject];
    }
    return nil;
}

id CUI_PenultimateSuperview( void )
{
    if ( sCUI_Superviews ) {
        NSInteger count = [sCUI_Superviews count];
        if ( count > 1 ) return sCUI_Superviews[count-2];
    }
    return nil;
}

void CUI_RemoveSuperview( void )
{ if ( sCUI_Superviews ) [sCUI_Superviews removeLastObject]; }

void CUI_SuperviewEndStatement( void )
{
    sCUI_SuperviewFlag = NO;
    if ( sCUI_Superviews ) [sCUI_Superviews removeLastObject];
}


#pragma mark - alerts
void CUI_ShowAlert( NSAlertStyle style, NSString *msg, NSString *info )
{
    dispatch_async( dispatch_get_main_queue(), ^{   // show alert on main thread 20200504
        NSAlert *alert = [[NSAlert alloc] init];
        [alert setAlertStyle:style];
        [alert setMessageText:msg];
        [alert setInformativeText:info];
        [alert addButtonWithTitle:@"OK"];
        [alert addButtonWithTitle:@"Quit"];
        if ( [alert runModal] == NSAlertSecondButtonReturn ) {
            [NSApp terminate:nil];
        }
        [alert release];
    });
}

void CUI_ShowWindowDoesNotExistAlert( NSInteger tag )
{ CUI_ShowAlert( NSWarningAlertStyle, @"Window error.", [NSString stringWithFormat:@"Window %ld does not exist.",(long)tag] ); }


#pragma mark - utilities
NSColor *CUI_FixColor( long col )
{
    if ( col == kFBParamMissing ) return nil;
    
    if ( col >= 0 && col < 100 ) {
        switch ( col ) {
            case zYellow:
                return [NSColor yellowColor];
            case zGreen:
                return [NSColor greenColor];
            case zCyan:
                return [NSColor cyanColor];
            case zBlue:
                return [NSColor blueColor];
            case zMagenta:
                return [NSColor magentaColor];
            case zRed:
                return [NSColor redColor];
            case zBlack:
                return [NSColor blackColor];
            case zWhite:
                return [NSColor whiteColor];
            case zBrown:
                return [NSColor brownColor];
            case zGray:
                return [NSColor grayColor];
            case zDarkGray:
                return [NSColor darkGrayColor];
            case zLightGray:
                return [NSColor lightGrayColor];
            case zOrange:
                return [NSColor orangeColor];
            case zPurple:
                return [NSColor purpleColor];
            case zClear:
                return [NSColor clearColor];
            case zText:
                return [NSColor textColor];
                break;
            default:
                return nil;
        }
    } else {
        return (NSColor *)col;
    }
    return nil;
}

CGPathRef CUI_NSBezierPathToCGPath( NSBezierPath *bezierPath )// taken from Apple documentation
{
    // Need to begin a path here.
    CGPathRef cgPath = NULL;

    // Then draw the path elements.
    NSInteger numElements = [bezierPath elementCount];
    if ( numElements > 0 ) {
        CGMutablePathRef    mutPath = CGPathCreateMutable();
        NSPoint             points[3];
        BOOL                didClosePath = YES;

        for ( NSInteger i = 0; i < numElements; i++ ) {
            switch ( [bezierPath elementAtIndex:i associatedPoints:points] ) {
                case NSMoveToBezierPathElement:
                    CGPathMoveToPoint( mutPath, NULL, points[0].x, points[0].y );
                    break;

                case NSLineToBezierPathElement:
                    CGPathAddLineToPoint( mutPath, NULL, points[0].x, points[0].y );
                    didClosePath = NO;
                    break;

                case NSCurveToBezierPathElement:
                    CGPathAddCurveToPoint( mutPath, NULL, points[0].x, points[0].y,
                                          points[1].x, points[1].y,
                                          points[2].x, points[2].y );
                    didClosePath = NO;
                    break;

                case NSClosePathBezierPathElement:
                    CGPathCloseSubpath( mutPath );
                    didClosePath = YES;
                    break;
            }
        }

        // Be sure the path is closed or Quartz may not do valid hit detection.
        if ( !didClosePath ) CGPathCloseSubpath( mutPath );

        cgPath = CGPathCreateCopy( mutPath );
        CGPathRelease( mutPath );
    }
    return cgPath;
}

NSImage *CUI_ImageResource( NSString *path )
{
    NSImage *image = [NSImage imageNamed:path];
    if ( !image ) {
        NSString *fullPath = [[NSBundle mainBundle] pathForResource:path ofType:nil];
        if ( fullPath ) {
            image = [[[NSImage alloc] initWithContentsOfFile:fullPath] autorelease];
        }
    }
    return image;
}

NSImage *CUI_FixImageValue( void *inImage )
{
    if ( inImage ) {
        if ( CFGetTypeID( inImage ) == CFStringGetTypeID() ) {
            return CUI_ImageResource( inImage );
        } else if ( [(id)inImage isKindOfClass:[NSImage class]] ) {
            return inImage;
        }
    }
    return nil;
}

NSFont *CUI_FixFont( const void *inFont )
{ return (NSFont *)inFont; }

/*
 NSRangeToCFRange
 NSRangeFromCFRange
 */
CFRange NSRangeToCFRange( NSRange range )
{ return CFRangeMake((CFIndex)range.location,(CFIndex)range.length); }

NSRange NSRangeFromCFRange( CFRange range )
{ return NSMakeRange((NSUInteger)range.location,(NSUInteger)range.length); }

