/*
 ColorPanel.m

 Bernie Wylde
 */

#import "ColorPanel.h"

static CUI_ColorPanelTarget *sCUI_ColorPanelTarget;

@implementation CUI_ColorPanelTarget

- (id)initWithCallback:(CUI_ColorPanelCallbackType)cb userData:(void *)ud {
    if ( self = [super init] ) {
        _callback = cb;
        _userData = ud;
    }
    return self;
}

- (void)action:(id)sender {
    if ( _callback ) {
        (*_callback)([sender color],_userData);
    } else {
        NSColor *col = [[NSColorPanel sharedColorPanel] color];
        DialogEventSetColor( col );
        CallUserDefinedOnDialogFunction( colorPanelChangeColor, 0, 0, col );
    }
}
@end


void CUI_ColorPanelSetTarget( void )
{
    if ( !sCUI_ColorPanelTarget ) {
        sCUI_ColorPanelTarget = [[CUI_ColorPanelTarget alloc] init];
        NSColorPanel *cp = [NSColorPanel sharedColorPanel];
        [cp setTarget:sCUI_ColorPanelTarget];
        [cp setAction:@selector(action:)];
    }
}

// Class
Class ColorPanelClass( void )
{ return [NSColorPanel class]; }

// - shared color panel -
NSColorPanel *ColorPanelShared( void )
{ return [NSColorPanel sharedColorPanel]; }

// - configure -
NSView *ColorPanelAccessoryView( void )
{ return [[NSColorPanel sharedColorPanel] accessoryView]; }

void ColorPanelSetAccessoryView( NSView *ref )
{ [[NSColorPanel sharedColorPanel] setAccessoryView:ref]; }

BOOL ColorPanelIsContinuous( void )
{ return [[NSColorPanel sharedColorPanel] isContinuous]; }

void ColorPanelSetContinuous( BOOL flag )
{
    [[NSColorPanel sharedColorPanel] setContinuous:flag];
    CUI_ColorPanelSetTarget();
}

NSColorPanelMode ColorPanelMode( void )
{ return [[NSColorPanel sharedColorPanel] mode]; }

void ColorPanelSetMode( NSColorPanelMode mode )
{
    [[NSColorPanel sharedColorPanel] setMode:mode];
    CUI_ColorPanelSetTarget();
}

void ColorPanelSetActionCallback( void *callback, void *userData )
{
    CUI_ColorPanelTarget *target = [[[CUI_ColorPanelTarget alloc] initWithCallback:callback userData:userData] autorelease];
    [[NSColorPanel sharedColorPanel] setTarget:target];
    [[NSColorPanel sharedColorPanel] setAction:@selector(action:)];
}

BOOL ColorPanelShowsAlpha( void )
{ return [[NSColorPanel sharedColorPanel] showsAlpha]; }

void ColorPanelSetShowsAlpha( BOOL flag )
{
    [[NSColorPanel sharedColorPanel] setShowsAlpha:flag];
    CUI_ColorPanelSetTarget();
}

// Color lists
void ColorPanelAttachColorList( NSColorList *cl )
{ [[NSColorPanel sharedColorPanel] attachColorList:cl]; }

void ColorPanelDetachColorList( NSColorList *cl )
{ [[NSColorPanel sharedColorPanel] detachColorList:cl]; }

// - picker modes
void ColorPanelSetPickerMode( NSInteger mode )
{
    [NSColorPanel setPickerMode:mode];
    CUI_ColorPanelSetTarget();
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void ColorPanelSetPickerMask( NSColorPanelOptions mask )
{
    [NSColorPanel setPickerMask:mask];
    CUI_ColorPanelSetTarget();
}
#endif

// - setting color -
NSColor *ColorPanelColor( void )
{ return [[NSColorPanel sharedColorPanel] color]; }

void ColorPanelSetColor( NSColor *col )
{
    [[NSColorPanel sharedColorPanel] setColor:col];
    CUI_ColorPanelSetTarget();
}

BOOL ColorPanelDragColor( NSColor *col, NSEvent *event, NSInteger fromView )
{
    NSView *view = CUI_ViewWithTag( fromView );
    if ( view ) return [NSColorPanel dragColor:col withEvent:event fromView:view];
    return NO;
}

// - color info -
CGFloat ColorPanelAlpha( void )
{ return [[NSColorPanel sharedColorPanel] alpha]; }

// -- custom --
void ColorPanelShow( void )
{
    [[NSColorPanel sharedColorPanel] makeKeyAndOrderFront:nil];
    CUI_ColorPanelSetTarget();
}

void ColorPanelClose( void )
{
    if ( [[NSColorPanel sharedColorPanel] isVisible] ) {
        [[NSColorPanel sharedColorPanel] orderOut:nil];
    }
}

