/*
 Control.h

 Bernie Wylde
 */



//#import "DialogEvent.h"
//#import "Event.h"
//#import "View.h"
//#import "Window.h"

#pragma mark - control formatter
typedef BOOL (*CUI_ControlFormatterIsParticalStringValidCallbackType)(NSInteger,CFStringRef*,CFRange*,CFStringRef,CFRange,CFStringRef*,void*);

// format case options
enum {
    formatCaseSensitive = 1,
    formatLowercase,
    formatUppercase,
    formatCapitalize
};

// control function selectors
enum {
    controlState,
    controlEnabled,
    controlHidden,
    controlIntVal
};


@interface CUI_ControlFormatter : NSFormatter
@property (nonatomic,retain) NSControl *control;
@property (assign) CUI_ControlFormatterIsParticalStringValidCallbackType callback;
@property (assign) void *userData;
@property (nonatomic,retain) NSCharacterSet *set;
@property (assign) BOOL pass;
@property (assign) NSUInteger length;
@property (assign) NSUInteger option;

- (id)initWithControl:(NSControl *)c callback:(CUI_ControlFormatterIsParticalStringValidCallbackType)cb userData:(void *)ud;
- (id)initWithCharacters:(NSString *)inCharacters pass:(BOOL)inPass length:(NSUInteger)inLength caseOption:(NSUInteger)caseOption;
- (void)setCharacters:(NSString *)inCharacters pass:(BOOL)inPass length:(NSUInteger)inLength caseOption:(NSUInteger)caseOption;

@end





#pragma mark - control
// dialog events
enum {
    controlTextDidBeginEditing = 1200,
    controlTextDidChange,
    controlTextDidEndEditing,
    controlTextShouldBeginEditing,
    controlTextShouldEndEditing,
    controlDoCommandBySelector
};

typedef void (*CUI_ControlCallbackType)(NSInteger,NSInteger,void *);

@interface NSControl (Additions)
@property (nonatomic,retain) NSValue *callbackValue;
@property (nonatomic,retain) NSValue *userDataValue;

- (CUI_ControlCallbackType)cuiControlCallback;
- (void *)cuiControlUserData;
- (void)setCuiControlCallback:(CUI_ControlCallbackType)cccb;
- (void)setCuiControlUserData:(void *)userData;
@end


@interface CUI_ControlTarget : NSObject {
//    CUI_ControlCallbackType allControlsCallback;
}
@property (assign) CUI_ControlCallbackType allControlsCallback;

- (void)controlAction:(id)sender;

@end

// private
CUI_ControlTarget *CUI_ControlTargetObj( void );
void CUI_ControlCallUserDefinedOnDialogOrCallback( NSInteger ev, id sender, id obj  );

// fix font to match I.B.
void CUI_ControlFixFont( id c );


#pragma mark - public
// Class
Class ControlClass( void );

NSControl *ControlWithTag( NSInteger tag );
BOOL ControlExists( NSInteger tag );

double ControlDoubleValue( NSInteger tag );
void ControlSetDoubleValue( NSInteger tag, double value );
float ControlFloatValue( NSInteger tag );
void ControlSetFloatValue( NSInteger tag, float value );
SInt32 ControlIntValue( NSInteger tag );
void ControlSetIntValue( NSInteger tag, SInt32 value );
NSInteger ControlIntegerValue( NSInteger tag );
void ControlSetIntegerValue( NSInteger tag, NSInteger value );
CFTypeRef ControlObjectValue( NSInteger tag );
void ControlSetObjectValue( NSInteger tag, CFTypeRef obj );
CFStringRef ControlStringValue( NSInteger tag );
void ControlSetStringValue( NSInteger tag, CFStringRef string );
CFAttributedStringRef ControlAttributedStringValue( NSInteger tag );
void ControlSetAttributedStringValue( NSInteger tag, CFAttributedStringRef string );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSControlSize controlSize( NSInteger tag );
#endif// 101000

void ControlSetSize( NSInteger tag, NSControlSize size );
NSTextAlignment ControlAlignment( NSInteger tag );
void ControlSetAlignment( NSInteger tag, NSTextAlignment alignment );
void ControlSizeToFit( NSInteger tag );

// Displaying a cell
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
BOOL ControlIsHighlighted( NSInteger tag );
void ControlSetHighlighted( NSInteger tag, BOOL flag );
#endif// 101000

NSLineBreakMode ControlLineBreakMode( NSInteger tag );
void ControlSetLineBreakMode( NSInteger tag, NSLineBreakMode mode );
BOOL ControlUsesSingleLineMode( NSInteger tag );
void ControlSetUsesSingleLineMode( NSInteger tag, BOOL flag );
NSFormatter *ControlFormatter( NSInteger tag );
void ControlSetFormatter( NSInteger tag, NSFormatter *formatter );
NSWritingDirection ControlBaseWritingDirection( NSInteger tag );
void ControlSetBaseWritingDirection( NSInteger tag, NSWritingDirection direction );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void ControlDrawWithExpansionFrameInView( NSInteger ctrlTag, CGRect contentFrame, NSInteger vwTag );
#endif

BOOL ControlAllowsExpansionToolTips( NSInteger tag );
void ControlSetAllowsExpansionToolTips( NSInteger tag, BOOL flag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGRect ControlExpansionFrameWithFrame( NSInteger tag, CGRect contentFrame );
#endif// 101000

// Managing the field editor
BOOL ControlAbortEditing( NSInteger tag );
NSText *ControlCurrentEditor( NSInteger tag );
void ControlValidateEditing( NSInteger tag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
void ControlEndEditing( NSInteger tag, NSText *textObj );
#endif

CFTypeRef ControlFont( NSInteger tag );
void ControlSetFont( NSInteger tag, CFTypeRef inFont );
void ControlTakeDoubleValueFrom( NSInteger tag, NSInteger otherCtrlTag );

void ControlTakeFloatValueFrom( NSInteger tag, NSInteger otherCtrlTag );

void ControlTakeIntegerValueFrom( NSInteger tag, NSInteger otherCtrlTag );
void ControlTakeObjectValueFrom( NSInteger tag, NSInteger otherCtrlTag );
void ControlTakeStringValueFrom( NSInteger tag, NSInteger otherCtrlTag );

BOOL ControlIsEnabled( NSInteger tag );
void ControlSetEnabled( NSInteger tag, BOOL flag );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
CGSize ControlSizeThatFits( NSInteger tag, CGSize size );
#endif

void ControlSetContinuous( NSInteger tag, BOOL flag );
void ControlPerformClick( NSInteger tag );

void ControlSendActionOn( NSInteger tag, NSEventMask mask );

NSInteger ControlTag( NSControl *ref );
void ControlSetTag( NSControl *ref, NSInteger tag );

BOOL ControlRefusesFirstResponder( NSInteger tag );
void ControlSetRefusesFirstResponder( NSInteger tag, BOOL flag );

BOOL ControlIgnoresMultiClick( NSInteger tag );
void ControlSetIgnoresMultiClick( NSInteger tag, BOOL flag );

// Constraint-based layout
void ControlInvalidateIntrinsicContentSizeForCell( NSInteger tag, NSCell *cell );

// Deprecated
NSCell *ControlCell( NSInteger tag );

// custom
void ControlSetFontWithName( NSInteger tag, CFStringRef inName, CGFloat size );
void ControlSetTargetAction( NSInteger ctrlTag, NSInteger targetTag, CFStringRef actionName );
void ControlSetAction( NSInteger tag, CFStringRef actionName );
void ControlSetActionCallback( NSInteger tag, void *callback, void *userData );

void ControlSetFormat( NSInteger tag, CFStringRef characters, BOOL pass, NSUInteger length, NSUInteger caseOption );
void ControlFormatterSetIsPartialStringValidCallback( NSInteger tag, CUI_ControlFormatterIsParticalStringValidCallbackType callback, void *userData );

void ControlSetActionID( NSInteger tag, NSInteger actionID );
void ControlRangeSetActionID( CFRange range, NSInteger actionID );

#pragma mark - set attributes of multiple controls
// enable
void ControlRangeSetEnabled( CFRange range, BOOL flag );
void ControlIndexesSetEnabled( NSIndexSet *indexes, BOOL flag );

// size
void ControlRangeSetSize( CFRange range, NSControlSize size );
void ControlIndexesSetSize( NSIndexSet *indexes, NSControlSize size );

// alighment
void ControlRangeSetAlignment( CFRange range, NSTextAlignment alignment );
void ControlIndexesSetAlignment( NSIndexSet *indexes, NSTextAlignment alignment );

// font
void ControlRangeSetFont( CFRange range, NSFont *font );
void ControlIndexesSetFont( NSIndexSet *indexes, NSFont *font );

#pragma mark - control function
NSInteger CUI_ControlFunction( NSInteger inTag, NSInteger inSelector );
