/*
 Cursor.m

 Bernie Wylde
 */

#import "Cursor.h"

static CUI_CursorAnimator *sCUI_CursorAnimator;

@implementation CUI_CursorAnimator

- (void)dealloc {
    [self.timer release];
    [self.name release];
    [self.extension release];
    [super dealloc];
}

- (void)timerFireMethod:(NSTimer *)t {
    _counter++;
    if ( _counter > _numImages ) _counter = 1;
    NSString *string = [NSString stringWithFormat:@"%@%ld.%@",_name,_counter,_extension];
    //NSImage *image = ImageNamed((__bridge CFStringRef)string);
    NSImage *image = CUI_ImageResource( string );
    if ( !NSEqualSizes(_size,NSZeroSize) ) [image setSize:_size];
    NSCursor *cursor = [[[NSCursor alloc] initWithImage:image hotSpot:_hotSpot] autorelease];
    if ( cursor ) [cursor set];
}
@end

// Class
Class CursorClass( void )
{ return [NSCursor class]; }

NSCursor *CursorWithImage( NSImage *image, CGPoint hotSpot )
{ return [[[NSCursor alloc] initWithImage:image hotSpot:NSPointFromCGPoint(hotSpot)] autorelease]; }


NSImage *CursorImage( NSCursor *ref )
{ return [ref image]; }

CGPoint CursorHotSpot( NSCursor *ref )
{ return NSPointToCGPoint([ref hotSpot]); }

void CursorHide( void )
{ [NSCursor hide]; }

void CursorUnhide( void )
{ [NSCursor unhide]; }

void CursorSetHiddenUntilMouseMoves( BOOL flag )
{ [NSCursor setHiddenUntilMouseMoves:flag]; }


void CursorPop( void )
{ [NSCursor pop]; }

void CursorPush( NSCursor *ref )
{ [ref push]; }

void CursorSet( NSCursor *ref )
{ [ref set]; }


NSCursor *CursorCurrent( void )
{ return [NSCursor currentCursor]; }

NSCursor *CursorCurrentSystem( void )
{ return [NSCursor currentSystemCursor]; }

NSCursor *CursorArrow( void )
{ return [NSCursor arrowCursor]; }

NSCursor *CursorContextualMenu( void )
{ return [NSCursor contextualMenuCursor]; }

NSCursor *CursorClosedHand( void )
{ return [NSCursor closedHandCursor]; }

NSCursor *CursorCrosshair( void )
{ return [NSCursor crosshairCursor]; }

NSCursor *CursorDisappearingItem( void )
{ return [NSCursor disappearingItemCursor]; }

NSCursor *CursorDragCopy( void )
{ return [NSCursor dragCopyCursor]; }

NSCursor *CursorDragLink( void )
{ return [NSCursor dragLinkCursor]; }

NSCursor *CursorIBeam( void )
{ return [NSCursor IBeamCursor]; }

NSCursor *CursorOpenHand( void )
{ return [NSCursor openHandCursor]; }

NSCursor *CursorOperationNotAllowed( void )
{ return [NSCursor operationNotAllowedCursor]; }

NSCursor *CursorPointingHand( void )
{ return [NSCursor pointingHandCursor]; }

NSCursor *CursorResizeDown( void )
{ return [NSCursor resizeDownCursor]; }

NSCursor *CursorResizeLeft( void )
{ return [NSCursor resizeLeftCursor]; }

NSCursor *CursorResizeLeftRight( void )
{ return [NSCursor resizeLeftRightCursor]; }

NSCursor *CursorResizeRight( void )
{ return [NSCursor resizeRightCursor]; }

NSCursor *CursorResizeUp( void )
{ return [NSCursor resizeUpCursor]; }

NSCursor *CursorResizeUpDown( void )
{ return [NSCursor resizeUpDownCursor]; }

NSCursor *CursorIBeamForVerticalLayout( void )
{ return [NSCursor IBeamCursorForVerticalLayout]; }


// -- convenience --
void CursorSetImage( void *inImage, CGPoint hotSpot )
{
    NSImage *image = CUI_FixImageValue( inImage );
    if ( image ) {
        NSCursor *cursor = [[[NSCursor alloc] initWithImage:image hotSpot:NSPointFromCGPoint(hotSpot)] autorelease];
        if ( cursor ) [cursor set];
    }
}

void CursorSetImageNamed( CFStringRef imageName, CGPoint hotSpot )
{
    NSImage *image = CUI_ImageResource( (__bridge NSString *)imageName );
    if ( image ) {
        NSCursor *cursor = [[[NSCursor alloc] initWithImage:image hotSpot:NSPointFromCGPoint(hotSpot)] autorelease];
        if ( cursor ) {
            [cursor set];
        }
    }
}

void CursorSetArrow( void )
{ [[NSCursor arrowCursor] set]; }

void CursorSetContextualMenu( void )
{ [[NSCursor contextualMenuCursor] set]; }

void CursorSetClosedHand( void )
{ [[NSCursor closedHandCursor] set]; }

void CursorSetCrosshair( void )
{ [[NSCursor crosshairCursor] set]; }

void CursorSetDisappearingItem( void )
{ [[NSCursor disappearingItemCursor] set]; }

void CursorSetDragCopy( void )
{ [[NSCursor dragCopyCursor] set]; }

void CursorSetDragLink( void )
{ [[NSCursor dragLinkCursor] set]; }

void CursorSetIBeam( void )
{ [[NSCursor IBeamCursor] set]; }

void CursorSetOpenHand( void )
{ [[NSCursor openHandCursor] set]; }

void CursorSetOperationNotAllowed( void )
{ [[NSCursor operationNotAllowedCursor] set]; }

void CursorSetPointingHand( void )
{ [[NSCursor pointingHandCursor] set]; }

void CursorSetResizeDown( void )
{ [[NSCursor resizeDownCursor] set]; }

void CursorSetResizeLeft( void )
{ [[NSCursor resizeLeftCursor] set]; }

void CursorSetResizeRight( void )
{ [[NSCursor resizeRightCursor] set]; }

void CursorSetResizeUp( void )
{ [[NSCursor resizeUpCursor] set]; }

void CursorSetResizeUpDown( void )
{ [[NSCursor resizeUpDownCursor] set]; }

void CursorSetIBeamForVerticalLayout( void )
{ [[NSCursor IBeamCursorForVerticalLayout] set]; }


// Custom
void CursorAnimatorStart( CFTimeInterval ti, CFStringRef inFirstImageName, CGSize size, CGPoint hotSpot, long numImages )
{
    if ( !sCUI_CursorAnimator ) {
        sCUI_CursorAnimator = [[CUI_CursorAnimator alloc] init];
        sCUI_CursorAnimator.counter = 1;
        NSString *firstImageName = (__bridge NSString *)inFirstImageName;
        NSString *extension = [firstImageName pathExtension];
        NSString *name = [firstImageName stringByDeletingPathExtension];
        name = [name substringToIndex:[name length] - 1];
        sCUI_CursorAnimator.name = [name retain];
        sCUI_CursorAnimator.extension = [extension retain];
        sCUI_CursorAnimator.size = NSSizeFromCGSize(size);
        sCUI_CursorAnimator.hotSpot = NSPointFromCGPoint(hotSpot);
        sCUI_CursorAnimator.numImages = numImages;
        NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:ti target:sCUI_CursorAnimator selector:@selector(timerFireMethod:) userInfo:nil repeats:YES];
        sCUI_CursorAnimator.timer = timer;
    }
}

void CursorAnimatorStop( void )
{
    if ( sCUI_CursorAnimator ) {
        [sCUI_CursorAnimator.timer invalidate];
        [sCUI_CursorAnimator release];
        sCUI_CursorAnimator = nil;
    }
    [[NSCursor arrowCursor] set];
}


