/*
 Event.h

 Bernie Wylde
 */

#define NEW_EVENT_FUNCTIONS 0// add 'NSEvent *ref' param to some of the functions - 20211103



#import "View.h"

@interface CUI_Event : NSObject
@property (nonatomic,retain) NSEvent *nsEvent;
@end

typedef void (*CUI_EventAddGlobalMonitorType)(NSEvent*,void*);
typedef id (*CUI_EventAddLocalMonitorType)(NSEvent*,void*);
typedef id (*CUI_EventTrackSwipeEventHandlerType)(NSEvent*,CGFloat,NSEventPhase,BOOL,BOOL*,void*);

CUI_Event *CUI_EventObj( void );
void CUI_EventSetEvent( NSEvent *event );

// Class
//Class EventClass( void );

// Create
NSEvent *EventKeyEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, NSGraphicsContext *ctx, CFStringRef chars, CFStringRef unmodChars, BOOL isRepeat, UInt16 code );
NSEvent *EventMouseEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, NSGraphicsContext *ctx, NSInteger evNum, NSInteger clickCount, float pressure );
NSEvent *EventEnterExitEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, NSGraphicsContext *ctx, NSInteger evNum, NSInteger trackingNum, void *userData );
NSEvent *EventOtherEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, NSGraphicsContext *ctx, SInt16 subtype, NSInteger data1, NSInteger data2 );
NSEvent *EventOtherEvent( NSEventType type, CGPoint location, NSUInteger modifierFlags, CFTimeInterval timestamp, NSInteger wndNum, NSGraphicsContext *ctx, SInt16 subtype, NSInteger data1, NSInteger data2 );
NSEvent *EventWithCGEvent( CGEventRef cgEvent );

// Type
//#if MY_FLAG
//NSEventType eventType( NSEvent *ref );// lowercase 'e' to avoid conflict with Carbon's 'EventType'
//#else
NSEventType eventType( void );// lowercase 'e' to avoid conflict with Carbon's 'EventType'
//#endif

short EventSubtype( NSEvent *ref );

#if NEW_EVENT_FUNCTIONS
NSUInteger EventModifierFlags( NSEvent *ref );
#else
NSUInteger EventModifierFlags( void );
#endif

// General information
#if NEW_EVENT_FUNCTIONS
CGPoint EventLocationInWindow( NSEvent *ref );
CFTimeInterval EventTimestamp( NSEvent *ref );
void *EventWindow( NSEvent *ref );
#else
CGPoint EventLocationInWindow( void );
CFTimeInterval EventTimestamp( void );
void *EventWindow( void );
#endif

NSInteger EventWindowNumber( NSEvent *ref );
CGEventRef EventCGEvent( NSEvent *ref );

// Key info
NSUInteger EventClassModifierFlags( void );
CFTimeInterval EventKeyRepeatDelay( void );
CFTimeInterval EventKeyRepeatInterval( void );

#if NEW_EVENT_FUNCTIONS
CFStringRef EventCharacters( NSEvent *ref );
CFStringRef EventCharactersIgnoringModifiers( NSEvent *ref );
BOOL EventIsARepeat( NSEvent *ref );
unsigned short EventKeyCode( NSEvent *ref );
#else// NEW_EVENT_FUNCTIONS
CFStringRef EventCharacters( void );
CFStringRef EventCharactersIgnoringModifiers( void );
BOOL EventIsARepeat( void );
unsigned short EventKeyCode( void );
#endif// NEW_EVENT_FUNCTIONS

// Mouse info
NSUInteger EventPressedMouseButtons( void );
CFTimeInterval EventDoubleClickInterval( void );
CGPoint EventMouseLocation( void );
NSInteger EventButtonNumber( NSEvent *ref );

#if NEW_EVENT_FUNCTIONS
NSInteger EventClickCount( NSEvent *ref );
#else// NEW_EVENT_FUNCTIONS
NSInteger EventClickCount( void );
#endif// NEW_EVENT_FUNCTIONS

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSEventMask EventAssociatedEventsMask( NSEvent *ref );
#endif// 101003

// Mouse tracking info
NSInteger EventNumber( NSEvent *ref );
NSInteger EventTrackingNumber( NSEvent *ref );
NSTrackingArea *EventTrackingArea( NSEvent *ref );
void *EventUserData( NSEvent *ref );

// Custom event info
NSInteger EventData1( NSEvent *ref );
NSInteger EventData2( NSEvent *ref );

// Scroll wheel info
CGFloat EventDeltaX( NSEvent *ref );
CGFloat EventDeltaY( NSEvent *ref );
CGFloat EventDeltaZ( NSEvent *ref );

// Pressure info
float EventPressure( NSEvent *ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSInteger EventStage( NSEvent *ref );
CGFloat EventStageTransition( NSEvent *ref );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSPressureBehavior EventPressureBehavior( NSEvent *ref );
#endif

// Tablet proximity info
NSUInteger EventCapabilityMask( NSEvent *ref );
NSUInteger EventDeviceID( NSEvent *ref );
BOOL EventIsEnteringProximity( NSEvent *ref );
NSUInteger EventPointingDeviceID( NSEvent *ref );
NSUInteger EventPointingDeviceSerialNumber( NSEvent *ref );
NSPointingDeviceType EventPointingDeviceType( NSEvent *ref );
NSUInteger EventSystemTabletID( NSEvent *ref );
NSUInteger EventTabletID( NSEvent *ref );
UInt64 EventUniqueID( NSEvent *ref );
NSUInteger EventVendorID( NSEvent *ref );
NSUInteger EventVendorPointingDeviceType( NSEvent *ref );

// Tablet pointing info
NSInteger EventAbsoluteX( NSEvent *ref );
NSInteger EventAbsoluteY( NSEvent *ref );
NSInteger EventAbsoluteZ( NSEvent *ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSEventButtonMask EventButtonMask( NSEvent *ref );
#endif
float EventRotation( NSEvent *ref );
float EventTangentialPressure( NSEvent *ref );
CGPoint EventTilt( NSEvent *ref );
CFTypeRef EventVendorDefined( NSEvent *ref );

// Periodic events
void EventStartPeriodicEvents( CFTimeInterval delay, CFTimeInterval period );
void EventStopPeriodicEvents( void );

// Touch and gesture info
CGFloat EventMagnification( NSEvent *ref );
CFSetRef EventTouchesMatchingPhase( NSEvent *ref, NSTouchPhase phase, NSInteger vwTag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFSetRef EventAllTouches( NSEvent *ref );
CFSetRef EventTouchesForView( NSEvent *ref, NSInteger vwTag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
CFArrayRef EventCoalescedTouchesForTouch( NSEvent *ref, NSTouch *touch );
#endif
BOOL EventIsMouseCoalescingEnabled( void );
void EventSetMouseCoalescingEnabled( BOOL flag );

// Monitoring app events
/*
  EventAddGlobalMonitor

  monitor function example:
    void local fn MyEventMonitor( event as CocoaEventRef, userData as ptr )

      // ...

    end fn
*/
CFTypeRef EventAddGlobalMonitor( NSEventMask mask, CUI_EventAddGlobalMonitorType monitorFunction, void *userData );

/*
  EventAddLocalMonitor

  monitor function example:
    local fn MyEventMonitor( event as CocoaEventRef, userData as ptr ) as CocoaEventRef

      // ...

    end fn = event
*/
CFTypeRef EventAddLocalMonitor( NSEventMask mask, CUI_EventAddLocalMonitorType monitorFunction, void *userData );
void EventRemoveMonitor( CFTypeRef eventMonitor );

// Scroll wheel and flick events
BOOL EventHasPreciseScrollingDeltas( NSEvent *ref );
CGFloat EventScrollingDeltaX( NSEvent *ref );
CGFloat EventScrollingDeltaY( NSEvent *ref );
NSEventPhase EventMomentumPhase( NSEvent *ref );
NSEventPhase EventPhase( NSEvent *ref );
BOOL EventIsSwipeTrackingFromScrollEventsEnabled( void );
void EventTrackSwipeEvent( NSEvent *ref, NSEventSwipeTrackingOptions options, CGFloat dampenThresholdMin, CGFloat dampenThresholdMax, CUI_EventTrackSwipeEventHandlerType handler, void *userData );
BOOL EventIsDirectionInvertedFromDevice( NSEvent *ref );

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
CFStringRef EventCharactersByApplyingModifiers( NSEvent *ref, NSEventModifierFlags modifiers );
#endif// 101500


// Custom
#if NEW_EVENT_FUNCTIONS
CGPoint EventLocationInView( NSEvent *ref, NSInteger tag );
#else
CGPoint EventLocationInView( NSInteger tag );
#endif
