/*
 FileManager.h

 Bernie Wylde
 */


#import "CocoaUI.h"

//#import "DialogEvent.h"

// dialog events
enum {
    fileManagerShouldMoveItem = 2400,
    fileManagerShouldProceedMovingItem,
    fileManagerShouldCopyItem,
    fileManagerShouldProceedCopyingItem,
    fileManagerShouldRemoveItem,
    fileManagerShouldProceedRemovingItem,
    fileManagerShouldLinkItem,
    fileManagerShouldProceedLinkingItem
};

@interface CUI_FileManagerDelegate : NSObject <NSFileManagerDelegate>
@end

void CUI_FileManagerDelegateCreate( void );

typedef BOOL (*FileManagerEnumeratorCallbackType)(CFURLRef,NSError *,void *);
typedef void (*FileManagerGetFileProviderServicesHandlerType)(CFDictionaryRef,NSError *,void *);// macOS 10.13+
typedef void (*FileManagerUnmountVolumeHandlerType)(NSError *,void *);// macOS 10.11+

// Class
Class FileManagerClass( void );

// Create
NSFileManager *FileManagerDefaultManager( void );

// User directories
CFURLRef FileManagerTemporaryDirectory( void );
CFURLRef FileManagerHomeDirectoryForCurrentUser( void );
CFURLRef FileManagerHomeDirectoryForUser( CFStringRef userName );

// System directories
CFURLRef FileManagerURLForDirectoryAppropriateForURL( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain, CFURLRef url, BOOL create, NSError **err );
CFURLRef FileManagerURLForDirectory( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain );
CFArrayRef FileManagerURLsForDirectory( NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain );

// Container directories
CFURLRef FileManagerContainerURLForSecurityApplicationGroupIdentifier( CFStringRef groupIdentifier );


CFURLRef FileManagerURLForPreferencesDirectory( void );
CFURLRef FileManagerURLForApplicationDirectory( void );
CFArrayRef FileManagerContentsOfDirectoryAtURL( CFURLRef url, CFArrayRef keys, NSDirectoryEnumerationOptions options );

NSDirectoryEnumerator *FileManagerEnumeratorAtURL( CFURLRef url, CFArrayRef propertyKeys, NSDirectoryEnumerationOptions options, FileManagerEnumeratorCallbackType errorCallback, void *userData );
CFArrayRef FileManagerMountedVolumeURLs( CFArrayRef propertyKeys, NSVolumeEnumerationOptions options );

BOOL FileManagerCreateDirectoryAtURL( CFURLRef url, BOOL withIntermediateDirectories, CFDictionaryRef attributes );
BOOL FileManagerCreateFileAtURL( CFURLRef url, CFDataRef contents, CFDictionaryRef attributes );
BOOL FileManagerRemoveItemAtURL( CFURLRef url );
BOOL FileManagerReplaceItemAtURL( CFURLRef origURL, CFURLRef newURL );
BOOL FileManagerReplaceItemAtURLWithOptions( CFURLRef origURL, CFURLRef newURL, CFStringRef backupItemName, NSFileManagerItemReplacementOptions options, CFURLRef *resultingItemURL, NSError **err );
void FileManagerTrashItemAtURL( CFURLRef url );
BOOL FileManagerCopyItemAtURL( CFURLRef srcURL, CFURLRef dstURL );
BOOL FileManagerMoveItemAtURL( CFURLRef srcURL, CFURLRef dstURL );

// ICloud-based items
CFTypeRef FileManagerUbiquityIdentityToken( void );
CFURLRef FileManagerURLForUbiquityContainerIdentifier( CFStringRef identifier );
BOOL FileManagerIsUbiquitousItem( CFURLRef url );
BOOL FileManagerSetUbiquitousItem( BOOL flag, CFURLRef url, CFURLRef destURL, NSError **err );
BOOL FileManagerStartDownloadingUbiquitousItem( CFURLRef url, NSError **err );
BOOL FileManagerEvictUbiquitousItem( CFURLRef url, NSError **err );
CFURLRef FileManagerURLForPublishingUbiquitousItem( CFURLRef url, CFDateRef *expirationDate, NSError **err );

// File provider services
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
void FileManagerGetFileProviderServicesForItem( CFURLRef url, FileManagerGetFileProviderServicesHandlerType completionHandler, void *userData );
#endif

// Symbolic and hard links
BOOL FileManagerCreateSymbolicLinkAtURL( CFURLRef url, CFURLRef dstURL, NSError **err );
BOOL FileManagerLinkItemAtURL( CFURLRef srcURL, CFURLRef dstURL, NSError **err );
CFURLRef FileManagerDestinationURLOfSymbolicLinkAtURL( CFURLRef url, NSError **err );

BOOL FileManagerFileExistsAtURL( CFURLRef url );
BOOL FileManagerIsReadableFileAtURL( CFURLRef url );
BOOL FileManagerIsWritableFileAtURL( CFURLRef url );
BOOL FileManagerIsExecutableFileAtURL( CFURLRef url );
BOOL FileManagerIsDeletableFileAtURL( CFURLRef url );
BOOL FileManagerIsDirectoryAtURL( CFURLRef url );
CFDictionaryRef FileManagerAttributesOfItemAtURL( CFURLRef url );
BOOL FileManagerSetAttributesOfItemAtURL( CFURLRef url, CFDictionaryRef attributes );
CFDataRef FileManagerContentsAtURL( CFURLRef url );

BOOL FileManagerContentsEqual( CFURLRef url1, CFURLRef url2 );

// Relationships between items
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
BOOL FileManagerGetRelationshipOfDirectoryAtURL( NSURLRelationship *relationship, CFURLRef dirURL, CFURLRef itemURL, NSError **err );
BOOL FileManagerGetRelationshipOfDirectoryInDomain( NSURLRelationship *relationship, NSSearchPathDirectory whichDirectory, NSSearchPathDomainMask domain, CFURLRef itemURL, NSError **err );
#endif

// Current directory
BOOL FileManagerChangeCurrentDirectoryURL( CFURLRef url );
CFURLRef FileManagerCurrentDirectoryURL( void );

// Unmount volumes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void FileManagerUnmountVolumeAtURL( CFURLRef url, NSFileManagerUnmountOptions options, FileManagerUnmountVolumeHandlerType completionHandler, void *userData );
#endif


/*
 DirectoryEnumerator.h
 */

// - attributes -
CFDictionaryRef DirectoryEnumeratorDirectoryAttributes( NSDirectoryEnumerator *enumerator );
CFDictionaryRef DirectoryEnumeratorFileAttributes( NSDirectoryEnumerator *enumerator );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
BOOL DirectoryEnumeratorIsEnumeratingDirectoryPostOrder( NSDirectoryEnumerator *ref );
#endif// 101500

NSUInteger DirectoryEnumeratorLevel( NSDirectoryEnumerator *enumerator );

// - skipping -
void DirectoryEnumeratorSkipDescendants( NSDirectoryEnumerator *enumerator );

