/*
 FileWrapper.m

 Bernie Wylde
 */

#import "FileWrapper.h"

// Class
Class FileWrapperClass( void )
{ return [NSFileWrapper class]; }

// Create
NSFileWrapper *FileWrapperWithURL( CFURLRef url, NSFileWrapperReadingOptions options, NSError **err )
{ return [[[NSFileWrapper alloc] initWithURL:(__bridge NSURL *)url options:options error:err] autorelease]; }

NSFileWrapper *FileWrapperDirectoryWithFileWrappers( CFDictionaryRef childrenByPreferredName )
{ return [[[NSFileWrapper alloc] initDirectoryWithFileWrappers:(__bridge NSDictionary *)childrenByPreferredName] autorelease]; }

NSFileWrapper *FileWrapperRegularFileWithContents( CFDataRef contents )
{ return [[[NSFileWrapper alloc] initRegularFileWithContents:(__bridge NSData *)contents] autorelease]; }

NSFileWrapper *FileWrapperSymbolicLinkWithDestinationURL( CFURLRef url )
{ return [[[NSFileWrapper alloc] initSymbolicLinkWithDestinationURL:(__bridge NSURL *)url] autorelease]; }

NSFileWrapper *FileWrapperWithSerializedRepresentation( CFDataRef serializedRepresentation )
{ return [[[NSFileWrapper alloc] initWithSerializedRepresentation:(__bridge NSData *)serializedRepresentation] autorelease]; }

// Query
BOOL FileWrapperIsRegularFile( NSFileWrapper *ref )
{ return [ref isRegularFile]; }

BOOL FileWrapperIsDirectory( NSFileWrapper *ref )
{ return [ref isDirectory]; }

BOOL FileWrapperIsSymbolicLink( NSFileWrapper *ref )
{ return [ref isSymbolicLink]; }

// Info
CFDictionaryRef FileWrapperFileWrappers( NSFileWrapper *ref )
{ return (__bridge CFDictionaryRef)[ref fileWrappers]; }

CFStringRef FileWrapperAddFileWrapper( NSFileWrapper *ref, NSFileWrapper *child )
{ return (__bridge CFStringRef)[ref addFileWrapper:child]; }

void FileWrapperRemoveFileWrapper( NSFileWrapper *ref, NSFileWrapper *child )
{ [ref removeFileWrapper:child]; }

CFStringRef FileWrapperAddRegularFileWithContents( NSFileWrapper *ref, CFDataRef dta, CFStringRef preferredFilename )
{ return (__bridge CFStringRef)[ref addRegularFileWithContents:(__bridge NSData *)dta preferredFilename:(__bridge NSString *)preferredFilename]; }

CFStringRef FileWrapperKeyForFileWrapper( NSFileWrapper *ref, NSFileWrapper *child )
{ return (__bridge CFStringRef)[ref keyForFileWrapper:child]; }

CFURLRef FileWrapperSymbolicLinkDestinationURL( NSFileWrapper *ref )
{ return (__bridge CFURLRef)[ref symbolicLinkDestinationURL]; }

// Update
BOOL FileWrapperMatchesContentsOfURL( NSFileWrapper *ref, CFURLRef url )
{ return [ref matchesContentsOfURL:(__bridge NSURL *)url]; }

BOOL FileWrapperReadFromURL( NSFileWrapper *ref, CFURLRef url, NSFileWrapperReadingOptions options, NSError **err )
{ return [ref readFromURL:(__bridge NSURL *)url options:options error:err]; }

// Serialize
CFDataRef FileWrapperSerializedRepresentation( NSFileWrapper *ref )
{ return (__bridge CFDataRef)[ref serializedRepresentation]; }

// Files
CFStringRef FileWrapperFilename( NSFileWrapper *ref )
{ return (__bridge CFStringRef)[ref filename]; }

CFStringRef FileWrapperPreferredFilename( NSFileWrapper *ref )
{ return (__bridge CFStringRef)[ref preferredFilename]; }

CFDictionaryRef FileWrapperFileAttributes( NSFileWrapper *ref )
{ return (__bridge CFDictionaryRef)[ref fileAttributes]; }

CFDataRef FileWrapperRegularFileContents( NSFileWrapper *ref )
{ return (__bridge CFDataRef)[ref regularFileContents]; }

// Write files
BOOL FileWrapperWriteToURL( NSFileWrapper *ref, CFURLRef url, NSFileWrapperWritingOptions options, CFURLRef origContentsURL, NSError **err )
{ return [ref writeToURL:(__bridge NSURL *)url options:options originalContentsURL:(__bridge NSURL *)origContentsURL error:err]; }

// Icons
NSImage *FileWrapperIcon( NSFileWrapper *ref )
{ return [ref icon]; }


