/*
 ItemProvider.m

 Bernie Wylde
 */

#import "ItemProvider.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

// Class
Class ItemProviderClass( void )
{ return [NSItemProvider class]; }

// Create
NSItemProvider *ItemProviderWithContentsOfURL( CFURLRef url )
{ return [[[NSItemProvider alloc] initWithContentsOfURL:(__bridge NSURL *)url] autorelease]; }

NSItemProvider *ItemProviderWithItem( id<NSSecureCoding> item, CFStringRef identifier )
{ return [[[NSItemProvider alloc] initWithItem:item typeIdentifier:(__bridge NSString *)identifier] autorelease]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSItemProvider *ItemProviderWithObject( id<NSItemProviderWriting> obj )
{
    if (@available(macOS 10.13, *)) {
        return [[[NSItemProvider alloc] initWithObject:obj] autorelease];
    }
    return nil;
}

// Get data
BOOL ItemProviderCanLoadObjectOfClass( NSItemProvider *ref, Class class )
{
    if (@available(macOS 10.13, *)) {
        return [ref canLoadObjectOfClass:class];
    }
    return NO;
}
#endif// 101300

BOOL ItemProviderHasItemConformingToTypeIdentifier( NSItemProvider *ref, CFStringRef identifier )
{ return [ref hasItemConformingToTypeIdentifier:(__bridge NSString *)identifier]; }

void ItemProviderLoadItemForTypeIdentifier( NSItemProvider *ref, CFStringRef identifier, CFDictionaryRef options, CUI_ItemProviderLoadItemCompletionHandler handler, void *userData )
{
    [ref loadItemForTypeIdentifier:(__bridge NSString *)identifier options:(__bridge NSDictionary *)options completionHandler:^(id<NSSecureCoding> item, NSError *error) {
        (*handler)(ref,item,error,userData);
    }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSProgress *ItemProviderLoadDataRepresentationForTypeIdentifier( NSItemProvider *ref, CFStringRef identifier, CUI_ItemProviderLoadDataCompletionHandler handler, void *userData )
{
    if (@available(macOS 10.13, *)) {
        return [ref loadDataRepresentationForTypeIdentifier:(__bridge NSString *)identifier completionHandler:^(NSData *data, NSError *error) {
            (*handler)(ref,(__bridge CFDataRef)data,error,userData);
        }];
    }
    return nil;
}

NSProgress *ItemProviderLoadFileRepresentationForTypeIdentifier( NSItemProvider *ref, CFStringRef identifier, CUI_ItemProviderLoadFileRepresentationCompletionHandler handler, void *userData )
{
    if (@available(macOS 10.13, *)) {
        return [ref loadFileRepresentationForTypeIdentifier:(__bridge NSString *)identifier completionHandler:^(NSURL *url, NSError *error) {
            (*handler)(ref,(__bridge CFURLRef)url,error,userData);
        }];
    }
    return nil;
}

NSProgress *ItemProviderLoadInPlaceFileRepresentationForTypeIdentifier( NSItemProvider *ref, CFStringRef identifier, CUI_ItemProviderLoadInPlaceFileRepresentationCompletionHandler handler, void *userData )
{
    if (@available(macOS 10.13, *)) {
        return [ref loadInPlaceFileRepresentationForTypeIdentifier:(__bridge NSString *)identifier completionHandler:^(NSURL *url, BOOL isInPlace, NSError *error) {
            (*handler)(ref,(__bridge CFURLRef)url,isInPlace,error,userData);
        }];
    }
    return nil;
}

NSProgress *ItemProviderLoadObjectOfClass( NSItemProvider *ref, Class class, CUI_ItemProviderLoadObjectCompletionHandler handler, void *userData )
{
    if (@available(macOS 10.13, *)) {
        return [ref loadObjectOfClass:class completionHandler:^(id<NSItemProviderReading> object, NSError *error) {
            (*handler)(ref,object,error,userData);
        }];
    }
    return nil;
}

BOOL ItemProviderHasRepresentationConformingToTypeIdentifier( NSItemProvider *ref, CFStringRef identifier, NSItemProviderFileOptions fileOptions )
{
    if (@available(macOS 10.13, *)) {
        return [ref hasRepresentationConformingToTypeIdentifier:(__bridge NSString *)identifier fileOptions:fileOptions];
    }
    return NO;
}

CFArrayRef ItemProviderRegisteredTypeIdentifiersWithFileOptions( NSItemProvider *ref, NSItemProviderFileOptions fileOptions )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFArrayRef)[ref registeredTypeIdentifiersWithFileOptions:fileOptions];
    }
    return nil;
}

// Provide data
//if (@available(macOS 10.13, *)) {
//}
void ItemProviderRegisterDataRepresentation( NSItemProvider *ref, CFStringRef identifier, NSItemProviderRepresentationVisibility visibility, CUI_ItemProviderRegisterDataRepresentationLoadHandler handler, void *userData );
void ItemProviderRegisterDataRepresentationWithOptions( NSItemProvider *ref, CFStringRef identifier, NSItemProviderFileOptions options, NSItemProviderRepresentationVisibility visibility, CUI_ItemProviderRegisterDataRepresentationWithOptionsLoadHandler handler, void *userData );
void ItemProviderRegisterObject( NSItemProvider *ref, id<NSItemProviderWriting> obj, NSItemProviderRepresentationVisibility visibility );
void ItemProviderRegisterObjectOfClass( NSItemProvider *ref, Class class, NSItemProviderRepresentationVisibility visibility, CUI_ItemProviderRegisterObjectOfClassLoadHandler handler, void *userData );
#endif// 101300

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
//if (@available(macOS 10.14, *)) {
//}
CFStringRef ItemProviderSuggestedName( NSItemProvider *ref );
void ItemProviderSetSuggestedName( NSItemProvider *ref, CFStringRef name );
#endif// 101400

// Register custom coercion handlers
void ItemProviderRegisterItemForTypeIdentifier( NSItemProvider *ref, CFStringRef identifier, void *handler, void *userData );
CFArrayRef ItemProviderRegisteredTypeIdentifiers( NSItemProvider *ref );

// Preview image
void ItemProviderLoadPreviewImageWithOptions( NSItemProvider *ref, CFDictionaryRef options, void *handler, void *userData );

// Visual attributes
CGRect ItemProviderSourceFrame( NSItemProvider *ref );
CGRect ItemProviderContainerFrame( NSItemProvider *ref );
CGSize ItemProviderPreferredPresentationSize( NSItemProvider *ref );




#pragma mark - ItemProviderReading
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300

//if (@available(macOS 10.13, *)) {
//}

// Create
id<NSItemProviderReading> ItemProviderReadingObjectWithData( CFDataRef dta, CFStringRef identifier, NSError **err );

// Type identifiers
CFArrayRef ItemProviderReadingReadableTypeIdentifiers( void );

// Load data
NSProgress *ItemProviderWritingLoadData( id<NSItemProviderWriting> ref, CFStringRef identifier, void *completionHandler, void *userData );

#endif// 101300


#pragma mark - ItemProviderWriting
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300

//if (@available(macOS 10.13, *)) {
//}

// Type identifiers
CFArrayRef ItemProviderWritingWritableTypeIdentifiers( void );
CFArrayRef ItemProviderWritingWritableTypeIdentifiersForItemProvider( id<NSItemProviderWriting> ref );

// Visibility
NSItemProviderRepresentationVisibility ItemProviderWritingVisibilityForRepresentation( CFStringRef identifier );
NSItemProviderRepresentationVisibility ItemProviderWritingVisibilityForRepresentationWithTypeIdentifier( id<NSItemProviderWriting> ref, CFStringRef identifier );

#endif// 101300


#endif// 101000
