/*
 LayoutAnchor.m

 Bernie Wylde
 */

#import "LayoutAnchor.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

// Class
Class LayoutAnchorClass( void )
{ return [NSLayoutAnchor class]; }

// Build constraints
NSLayoutConstraint *LayoutAnchorConstraintEqualToAnchor( NSLayoutAnchor *ref, NSLayoutAnchor *anchor )
{ return [ref constraintEqualToAnchor:anchor]; }

NSLayoutConstraint *LayoutAnchorConstraintEqualToAnchorConstant( NSLayoutAnchor *ref, NSLayoutAnchor *anchor, CGFloat constant )
{ return [ref constraintEqualToAnchor:anchor constant:constant]; }

NSLayoutConstraint *LayoutAnchorConstraintGreaterThanOrEqualToAnchor( NSLayoutAnchor *ref, NSLayoutAnchor *anchor )
{ return [ref constraintGreaterThanOrEqualToAnchor:anchor]; }

NSLayoutConstraint *LayoutAnchorConstraintGreaterThanOrEqualToAnchorConstant( NSLayoutAnchor *ref, NSLayoutAnchor *anchor, CGFloat constant )
{ return [ref constraintGreaterThanOrEqualToAnchor:anchor constant:constant]; }

NSLayoutConstraint *LayoutAnchorConstraintLessThanOrEqualToAnchor( NSLayoutAnchor *ref, NSLayoutAnchor *anchor )
{ return [ref constraintLessThanOrEqualToAnchor:anchor]; }

NSLayoutConstraint *LayoutAnchorConstraintLessThanOrEqualToAnchorConstant( NSLayoutAnchor *ref, NSLayoutAnchor *anchor, CGFloat constant )
{ return [ref constraintLessThanOrEqualToAnchor:anchor constant:constant]; }

// Debugging
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFArrayRef LayoutAnchorConstraintsAffectingLayout( NSLayoutAnchor *ref )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFArrayRef)[ref constraintsAffectingLayout];
    }
    return nil;
}

BOOL LayoutAnchorHasAmbiguousLayout( NSLayoutAnchor *ref )
{
    if (@available(macOS 10.13, *)) {
        return [ref hasAmbiguousLayout];
    }
    return NO;
}

CFStringRef LayoutAnchorName( NSLayoutAnchor *ref )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFStringRef)[ref name];
    }
    return nil;
}

CFTypeRef LayoutAnchorItem( NSLayoutAnchor *ref )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFTypeRef)[ref item];
    }
    return nil;
}
#endif  // __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300



/*
 LayoutYAxisAnchor
 */
// Class
Class LayoutYAxisAnchorClass( void )
{ return [NSLayoutYAxisAnchor class]; }

// Create layout dimension
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSLayoutDimension *LayoutYAxisAnchorWithOffsetToAnchor( NSLayoutYAxisAnchor *ref, NSLayoutYAxisAnchor *otherAnchor )
{
    if (@available(macOS 10.13, *)) {
        return [ref anchorWithOffsetToAnchor:otherAnchor];
    }
    return nil;
}
#endif


/*
 LayoutXAxisAnchor
 */
// Class
Class LayoutXAxisAnchorClass( void )
{ return [NSLayoutXAxisAnchor class]; }

// Create layout dimension
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSLayoutDimension *LayoutXAxisAnchorWithOffsetToAnchor( NSLayoutXAxisAnchor *ref, NSLayoutXAxisAnchor *otherAnchor )
{
    if (@available(macOS 10.13, *)) {
        return [ref anchorWithOffsetToAnchor:otherAnchor];
    }
    return nil;
}
#endif



/*
 LayoutDimension
 */
// Class
Class LayoutDimensionClass( void )
{ return [NSLayoutDimension class]; }

// Build constraints
NSLayoutConstraint *LayoutDimensionConstraintEqualToAnchor( NSLayoutDimension *ref, NSLayoutDimension *anchor, CGFloat multiplier )
{ return [ref constraintEqualToAnchor:anchor multiplier:multiplier]; }

NSLayoutConstraint *LayoutDimensionConstraintEqualToAnchorConstant( NSLayoutDimension *ref, NSLayoutDimension *anchor, CGFloat multiplier, CGFloat constant )
{ return [ref constraintEqualToAnchor:anchor multiplier:multiplier constant:constant]; }

NSLayoutConstraint *LayoutDimensionConstraintEqualToConstant( NSLayoutDimension *ref, CGFloat constant )
{ return [ref constraintEqualToConstant:constant]; }

NSLayoutConstraint *LayoutDimensionConstraintGreaterThanOrEqualToAnchor( NSLayoutDimension *ref, NSLayoutDimension *anchor, CGFloat multiplier )
{ return [ref constraintGreaterThanOrEqualToAnchor:anchor multiplier:multiplier]; }

NSLayoutConstraint *LayoutDimensionConstraintGreaterThanOrEqualToAnchorConstant( NSLayoutDimension *ref, NSLayoutDimension *anchor, CGFloat multiplier, CGFloat constant )
{ return [ref constraintGreaterThanOrEqualToAnchor:anchor multiplier:multiplier constant:constant]; }

NSLayoutConstraint *LayoutDimensionConstraintGreaterThanOrEqualToConstant( NSLayoutDimension *ref, CGFloat constant )
{ return [ref constraintGreaterThanOrEqualToConstant:constant]; }

NSLayoutConstraint *LayoutDimensionConstraintLessThanOrEqualToAnchor( NSLayoutDimension *ref, NSLayoutDimension *anchor, CGFloat multiplier )
{ return [ref constraintLessThanOrEqualToAnchor:anchor multiplier:multiplier]; }

NSLayoutConstraint *LayoutDimensionConstraintLessThanOrEqualToAnchorConstant( NSLayoutDimension *ref, NSLayoutDimension *anchor, CGFloat multiplier, CGFloat constant )
{ return [ref constraintLessThanOrEqualToAnchor:anchor multiplier:multiplier constant:constant]; }

NSLayoutConstraint *LayoutDimensionConstraintLessThanOrEqualToConstant( NSLayoutDimension *ref, CGFloat constant )
{ return [ref constraintLessThanOrEqualToConstant:constant]; }


#endif  // __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100

