/*
 LayoutManager.h

 Bernie Wylde
 */


#import "CocoaUI.h"

// Class
Class LayoutManagerClass( void );

// Init
NSLayoutManager *LayoutManagerInit( void );

// Text storage
NSTextStorage *LayoutManagerTextStorage(  NSLayoutManager *ref );
void LayoutManagerReplaceTextStorage( NSLayoutManager *ref, NSTextStorage *ts );

// Text containers
CFArrayRef LayoutManagerTextContainers( NSLayoutManager *ref );
void LayoutManagerAddTextContainer( NSLayoutManager *ref, NSTextContainer *tc );
void LayoutManagerInsertTextContainer( NSLayoutManager *ref, NSTextContainer *tc, NSUInteger index );
void LayoutManagerRemoveTextContainer( NSLayoutManager *ref, NSUInteger index );
void LayoutManagerSetTextContainer( NSLayoutManager *ref, NSTextContainer *tc, CFRange glyphRange );
void LayoutManagerTextContainerChangedGeometry( NSLayoutManager *ref, NSTextContainer *tc );
void LayoutManagerTextContainerChangedTextView( NSLayoutManager *ref, NSTextContainer *tc );
NSTextContainer *LayoutManagerTextContainerForGlyph( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange );
NSTextContainer *LayoutManagerTextContainerForGlyphWithoutAdditionalLayout( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange, BOOL flag );
CGRect LayoutManagerUsedRectForTextContainer( NSLayoutManager *ref, NSTextContainer *tc );

// Global layout manager options
BOOL LayoutManagerAllowsNonContiguousLayout( NSLayoutManager *ref );
void LayoutManagerSetAllowsNonContiguousLayout( NSLayoutManager *ref, BOOL flag );
BOOL LayoutManagerHasNonContiguousLayout( NSLayoutManager *ref );
CGFloat LayoutManagerHyphenationFactor(NSLayoutManager *ref );
void LayoutManagerSetHyphenationFactor(NSLayoutManager *ref, CGFloat factor );
BOOL LayoutManagerShowsInvisibleCharacters( NSLayoutManager *ref );
void LayoutManagerSetShowsInvisibleCharacters( NSLayoutManager *ref, BOOL flag );
BOOL LayoutManagerShowsControlCharacters( NSLayoutManager *ref );
void LayoutManagerSetShowsControlCharacters( NSLayoutManager *ref, BOOL flag );
BOOL LayoutManagerUsesFontLeading( NSLayoutManager *ref );
void LayoutManagerSetUsesFontLeading( NSLayoutManager *ref, BOOL flag );
BOOL LayoutManagerBackgroundLayoutEnabled( NSLayoutManager *ref );
void LayoutManagerSetBackgroundLayoutEnabled( NSLayoutManager *ref, BOOL flag );

// Invalidate glyphs and layout
void LayoutManagerInvalidateDisplayForCharacterRange( NSLayoutManager *ref, CFRange range );
void LayoutManagerInvalidateDisplayForGlyphRange( NSLayoutManager *ref, CFRange range );
void LayoutManagerInvalidateGlyphsForCharacterRange( NSLayoutManager *ref, CFRange range, NSInteger delta, CFRange *actualCharacterRange );
void LayoutManagerInvalidateLayoutForCharacterRange( NSLayoutManager *ref, CFRange range, CFRange *actualCharacterRange );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void LayoutManagerProcessEditingForTextStorage( NSLayoutManager *ref, NSTextStorage *textStorage, NSTextStorageEditActions edited, CFRange range, NSInteger delta, CFRange invalidatedRange );
#endif

// Glyph generation and layout
void LayoutManagerEnsureGlyphsForCharacterRange( NSLayoutManager *ref, CFRange range );
void LayoutManagerEnsureGlyphsForGlyphRange( NSLayoutManager *ref, CFRange range );
void LayoutManagerEnsureLayoutForBoundingRect( NSLayoutManager *ref, CGRect r, NSTextContainer *tc );
void LayoutManagerEnsureLayoutForCharacterRange( NSLayoutManager *ref, CFRange range );
void LayoutManagerEnsureLayoutForGlyphRange( NSLayoutManager *ref, CFRange range );
void LayoutManagerEnsureLayoutForTextContainer( NSLayoutManager *ref, NSTextContainer *tc );
NSGlyphGenerator *LayoutManagerGlyphGenerator( NSLayoutManager *ref );

// Accessing glyphs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUInteger LayoutManagerGetGlyphs( NSLayoutManager *ref, CFRange range, CGGlyph *glyphBuffer, NSGlyphProperty *props, NSUInteger *charIndexBuffer, unsigned char *bidiLevelsBuffer );
CGGlyph LayoutManagerCGGlyphAtIndex( NSLayoutManager *ref, NSUInteger glyphIndex );
CGGlyph LayoutManagerCGGlyphAtIndexIsValidIndex( NSLayoutManager *ref, NSUInteger glyphIndex, BOOL *isValidIndex );
void LayoutManagerSetCGGlyphs( NSLayoutManager *ref, CGGlyph *glyphs, NSGlyphProperty *props, NSUInteger *charIndexes, NSFont *font, CFRange glyphRange );
#endif
NSUInteger LayoutManagerCharacterIndexForGlyphAtIndex( NSLayoutManager *ref, NSUInteger glyphIndex );
NSUInteger LayoutManagerGlyphIndexForCharacterAtIndex( NSLayoutManager *ref, NSUInteger charIndex );
BOOL LayoutManagerIsValidGlyphIndex( NSLayoutManager *ref, NSUInteger glyphIndex );
NSUInteger LayoutManagerNumberOfGlyphs( NSLayoutManager *ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSGlyphProperty LayoutManagerPropertyForGlyphAtIndex( NSLayoutManager *ref, NSUInteger glyphIndex );
#endif

// Set layout info
void LayoutManagerSetAttachmentSize( NSLayoutManager *ref, CGSize size, CFRange glyphRange );
void LayoutManagerSetDrawsOutsideLineFragment( NSLayoutManager *ref, BOOL flag, NSUInteger glyphIndex );
void LayoutManagerSetExtraLineFragmentRect( NSLayoutManager *ref, CGRect r, CGRect usedRect, NSTextContainer *textContainer );
void LayoutManagerSetLineFragmentRect( NSLayoutManager *ref, CGRect r, CFRange glyphRange, CGRect usedRect );
void LayoutManagerSetLocation( NSLayoutManager *ref, CGPoint pt, CFRange startOfGlyphRange );
void LayoutManagerSetNotShownAttribute( NSLayoutManager *ref, BOOL flag, NSUInteger glyphIndex );

// Get layout info
CGSize LayoutManagerAttachmentSizeForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index );
BOOL LayoutManagerDrawsOutsideLineFragmentForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index );
CGRect LayoutManagerExtraLineFragmentRect( NSLayoutManager *ref );
NSTextContainer *LayoutManagerExtraLineFragmentTextContainer( NSLayoutManager *ref );
CGRect LayoutManagerExtraLineFragmentUsedRect( NSLayoutManager *ref );
NSUInteger LayoutManagerFirstUnlaidCharacterIndex( NSLayoutManager *ref );
NSUInteger LayoutManagerFirstUnlaidGlyphIndex( NSLayoutManager *ref );
void LayoutManagerGetFirstUnlaidCharacterIndex( NSLayoutManager *ref, NSUInteger *charIndex, NSUInteger *glyphIndex );
CGRect LayoutManagerLineFragmentRectForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange );
CGRect LayoutManagerLineFragmentRectForGlyphAtIndexWithoutAdditionalLayout( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange, BOOL flag );
CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange );
CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndexWithoutAdditionalLayout( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange, BOOL flag );
CGPoint LayoutManagerLocationForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index );
BOOL LayoutManagerNotShownAttributeForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFRange LayoutManagerTruncatedGlyphRangeInLineFragmentForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index );
#endif

// Advanced layout queries
CGRect LayoutManagerBoundingRectForGlyphRange( NSLayoutManager *ref, CFRange glyphRange, NSTextContainer *tc );
NSUInteger LayoutManagerCharacterIndexForPoint( NSLayoutManager *ref, CGPoint pt, NSTextContainer *tc, CGFloat *partialFraction );
CFRange LayoutManagerCharacterRangeForGlyphRange( NSLayoutManager *ref, CFRange glyphRange, CFRange *actualGlyphRange );

CFRange LayoutManagerGlyphRangeForBoundingRect( NSLayoutManager *ref, CGRect bounds, NSTextContainer *tc );

// Drawing

// Text blocks

// Attachments

// Rulers

// Responder chain

// Typesetter
CGFloat LayoutManagerDefaultLineHeightForFont( NSLayoutManager *ref, NSFont *font );
CGFloat LayoutManagerDefaultBaselineOffsetForFont( NSLayoutManager *ref, NSFont *font );

// Temporary attribute support


