/*
 LayoutManager.m

 Bernie Wylde
 */

#import "LayoutManager.h"

// Class
Class LayoutManagerClass( void )
{ return [NSLayoutManager class]; }

// Init
NSLayoutManager *LayoutManagerInit( void )
{ return [[[NSLayoutManager alloc] init] autorelease]; }

// Text storage
NSTextStorage *LayoutManagerTextStorage(  NSLayoutManager *ref )
{ return [ref textStorage]; }

void LayoutManagerReplaceTextStorage( NSLayoutManager *ref, NSTextStorage *ts )
{ [ref replaceTextStorage:ts]; }

// Text containers
CFArrayRef LayoutManagerTextContainers( NSLayoutManager *ref )
{ return (__bridge CFArrayRef)[ref textContainers]; }

void LayoutManagerAddTextContainer( NSLayoutManager *ref, NSTextContainer *tc )
{ [ref addTextContainer:tc]; }

void LayoutManagerInsertTextContainer( NSLayoutManager *ref, NSTextContainer *tc, NSUInteger index )
{ [ref insertTextContainer:tc atIndex:index]; }

void LayoutManagerRemoveTextContainer( NSLayoutManager *ref, NSUInteger index )
{ [ref removeTextContainerAtIndex:index]; }

void LayoutManagerSetTextContainer( NSLayoutManager *ref, NSTextContainer *tc, CFRange glyphRange )
{ [ref setTextContainer:tc forGlyphRange:NSRangeFromCFRange(glyphRange)]; }

void LayoutManagerTextContainerChangedGeometry( NSLayoutManager *ref, NSTextContainer *tc )
{ [ref textContainerChangedGeometry:tc]; }

void LayoutManagerTextContainerChangedTextView( NSLayoutManager *ref, NSTextContainer *tc )
{ [ref textContainerChangedTextView:tc]; }

NSTextContainer *LayoutManagerTextContainerForGlyph( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange )
{ return [ref textContainerForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange]; }

NSTextContainer *LayoutManagerTextContainerForGlyphWithoutAdditionalLayout( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange, BOOL flag )
{ return [ref textContainerForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange withoutAdditionalLayout:flag]; }

CGRect LayoutManagerUsedRectForTextContainer( NSLayoutManager *ref, NSTextContainer *tc )
{ return NSRectToCGRect([ref usedRectForTextContainer:tc]); }

// Global layout manager options
BOOL LayoutManagerAllowsNonContiguousLayout( NSLayoutManager *ref )
{ return [ref allowsNonContiguousLayout]; }

void LayoutManagerSetAllowsNonContiguousLayout( NSLayoutManager *ref, BOOL flag )
{ [ref setAllowsNonContiguousLayout:flag]; }

BOOL LayoutManagerHasNonContiguousLayout( NSLayoutManager *ref )
{ return [ref hasNonContiguousLayout]; }

CGFloat LayoutManagerHyphenationFactor(NSLayoutManager *ref )
{ return [ref hyphenationFactor]; }

void LayoutManagerSetHyphenationFactor(NSLayoutManager *ref, CGFloat factor )
{ [ref setHyphenationFactor:factor]; }

BOOL LayoutManagerShowsInvisibleCharacters( NSLayoutManager *ref )
{ return [ref showsInvisibleCharacters]; }

void LayoutManagerSetShowsInvisibleCharacters( NSLayoutManager *ref, BOOL flag )
{ [ref setShowsInvisibleCharacters:flag]; }

BOOL LayoutManagerShowsControlCharacters( NSLayoutManager *ref )
{ return [ref showsControlCharacters]; }

void LayoutManagerSetShowsControlCharacters( NSLayoutManager *ref, BOOL flag )
{ [ref setShowsControlCharacters:flag]; }

BOOL LayoutManagerUsesFontLeading( NSLayoutManager *ref )
{ return [ref usesFontLeading]; }

void LayoutManagerSetUsesFontLeading( NSLayoutManager *ref, BOOL flag )
{ [ref setUsesFontLeading:flag]; }

BOOL LayoutManagerBackgroundLayoutEnabled( NSLayoutManager *ref )
{ return [ref backgroundLayoutEnabled]; }

void LayoutManagerSetBackgroundLayoutEnabled( NSLayoutManager *ref, BOOL flag )
{ [ref setBackgroundLayoutEnabled:flag]; }

// Invalidate glyphs and layout
void LayoutManagerInvalidateDisplayForCharacterRange( NSLayoutManager *ref, CFRange range )
{ [ref invalidateDisplayForCharacterRange:NSRangeFromCFRange(range)]; }

void LayoutManagerInvalidateDisplayForGlyphRange( NSLayoutManager *ref, CFRange range )
{ [ref invalidateDisplayForGlyphRange:NSRangeFromCFRange(range)]; }

void LayoutManagerInvalidateGlyphsForCharacterRange( NSLayoutManager *ref, CFRange range, NSInteger delta, CFRange *actualCharacterRange )
{ [ref invalidateGlyphsForCharacterRange:NSRangeFromCFRange(range) changeInLength:delta actualCharacterRange:(NSRange *)actualCharacterRange]; }

void LayoutManagerInvalidateLayoutForCharacterRange( NSLayoutManager *ref, CFRange range, CFRange *actualCharacterRange )
{ [ref invalidateLayoutForCharacterRange:NSRangeFromCFRange(range) actualCharacterRange:(NSRange *)actualCharacterRange]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void LayoutManagerProcessEditingForTextStorage( NSLayoutManager *ref, NSTextStorage *textStorage, NSTextStorageEditActions edited, CFRange range, NSInteger delta, CFRange invalidatedRange )
{ [ref processEditingForTextStorage:textStorage edited:edited range:NSRangeFromCFRange(range) changeInLength:delta invalidatedRange:NSRangeFromCFRange(invalidatedRange)]; }
#endif

// Glyph generation and layout
void LayoutManagerEnsureGlyphsForCharacterRange( NSLayoutManager *ref, CFRange range )
{ [ref ensureGlyphsForCharacterRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureGlyphsForGlyphRange( NSLayoutManager *ref, CFRange range )
{ [ref ensureGlyphsForGlyphRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureLayoutForBoundingRect( NSLayoutManager *ref, CGRect r, NSTextContainer *tc )
{ [ref ensureLayoutForBoundingRect:NSRectFromCGRect(r) inTextContainer:tc]; }

void LayoutManagerEnsureLayoutForCharacterRange( NSLayoutManager *ref, CFRange range )
{ [ref ensureLayoutForCharacterRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureLayoutForGlyphRange( NSLayoutManager *ref, CFRange range )
{ [ref ensureLayoutForGlyphRange:NSRangeFromCFRange(range)]; }

void LayoutManagerEnsureLayoutForTextContainer( NSLayoutManager *ref, NSTextContainer *tc )
{ [ref ensureLayoutForTextContainer:tc]; }

NSGlyphGenerator *LayoutManagerGlyphGenerator( NSLayoutManager *ref )
{ return [ref glyphGenerator]; }

// Accessing glyphs
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUInteger LayoutManagerGetGlyphs( NSLayoutManager *ref, CFRange range, CGGlyph *glyphBuffer, NSGlyphProperty *props, NSUInteger *charIndexBuffer, unsigned char *bidiLevelsBuffer )
{ return [ref getGlyphsInRange:NSRangeFromCFRange(range) glyphs:glyphBuffer properties:props characterIndexes:charIndexBuffer bidiLevels:bidiLevelsBuffer]; }

CGGlyph LayoutManagerCGGlyphAtIndex( NSLayoutManager *ref, NSUInteger glyphIndex )
{ return [ref CGGlyphAtIndex:glyphIndex]; }

CGGlyph LayoutManagerCGGlyphAtIndexIsValidIndex( NSLayoutManager *ref, NSUInteger glyphIndex, BOOL *isValidIndex )
{ return [ref CGGlyphAtIndex:glyphIndex isValidIndex:isValidIndex]; }

void LayoutManagerSetCGGlyphs( NSLayoutManager *ref, CGGlyph *glyphs, NSGlyphProperty *props, NSUInteger *charIndexes, NSFont *font, CFRange glyphRange )
{ [ref setGlyphs:glyphs properties:props characterIndexes:charIndexes font:font forGlyphRange:NSRangeFromCFRange(glyphRange)]; }
#endif

NSUInteger LayoutManagerCharacterIndexForGlyphAtIndex( NSLayoutManager *ref, NSUInteger glyphIndex )
{ return [ref characterIndexForGlyphAtIndex:glyphIndex]; }

NSUInteger LayoutManagerGlyphIndexForCharacterAtIndex( NSLayoutManager *ref, NSUInteger charIndex )
{ return [ref glyphIndexForCharacterAtIndex:charIndex]; }

BOOL LayoutManagerIsValidGlyphIndex( NSLayoutManager *ref, NSUInteger glyphIndex )
{ return [ref isValidGlyphIndex:glyphIndex]; }

NSUInteger LayoutManagerNumberOfGlyphs( NSLayoutManager *ref )
{ return [ref numberOfGlyphs]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSGlyphProperty LayoutManagerPropertyForGlyphAtIndex( NSLayoutManager *ref, NSUInteger glyphIndex )
{ return [ref propertyForGlyphAtIndex:glyphIndex]; }
#endif

// Set layout info
void LayoutManagerSetAttachmentSize( NSLayoutManager *ref, CGSize size, CFRange glyphRange )
{ [ref setAttachmentSize:NSSizeFromCGSize(size) forGlyphRange:NSRangeFromCFRange(glyphRange)]; }

void LayoutManagerSetDrawsOutsideLineFragment( NSLayoutManager *ref, BOOL flag, NSUInteger glyphIndex )
{ [ref setDrawsOutsideLineFragment:flag forGlyphAtIndex:glyphIndex]; }

void LayoutManagerSetExtraLineFragmentRect( NSLayoutManager *ref, CGRect r, CGRect usedRect, NSTextContainer *textContainer )
{ [ref setExtraLineFragmentRect:NSRectFromCGRect(r) usedRect:NSRectFromCGRect(usedRect) textContainer:textContainer]; }

void LayoutManagerSetLineFragmentRect( NSLayoutManager *ref, CGRect r, CFRange glyphRange, CGRect usedRect )
{ [ref setLineFragmentRect:NSRectFromCGRect(r) forGlyphRange:NSRangeFromCFRange(glyphRange) usedRect:NSRectFromCGRect(usedRect)]; }

void LayoutManagerSetLocation( NSLayoutManager *ref, CGPoint pt, CFRange startOfGlyphRange )
{ [ref setLocation:NSPointFromCGPoint(pt) forStartOfGlyphRange:NSRangeFromCFRange(startOfGlyphRange)]; }

void LayoutManagerSetNotShownAttribute( NSLayoutManager *ref, BOOL flag, NSUInteger glyphIndex )
{ [ref setNotShownAttribute:flag forGlyphAtIndex:glyphIndex]; }

// Get layout info
CGSize LayoutManagerAttachmentSizeForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index )
{ return NSSizeToCGSize([ref attachmentSizeForGlyphAtIndex:index]); }

BOOL LayoutManagerDrawsOutsideLineFragmentForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index )
{ return [ref drawsOutsideLineFragmentForGlyphAtIndex:index]; }

CGRect LayoutManagerExtraLineFragmentRect( NSLayoutManager *ref )
{ return NSRectToCGRect([ref extraLineFragmentRect]); }

NSTextContainer *LayoutManagerExtraLineFragmentTextContainer( NSLayoutManager *ref )
{ return [ref extraLineFragmentTextContainer]; }

CGRect LayoutManagerExtraLineFragmentUsedRect( NSLayoutManager *ref )
{ return NSRectToCGRect([ref extraLineFragmentUsedRect]); }

NSUInteger LayoutManagerFirstUnlaidCharacterIndex( NSLayoutManager *ref )
{ return [ref firstUnlaidCharacterIndex]; }

NSUInteger LayoutManagerFirstUnlaidGlyphIndex( NSLayoutManager *ref )
{ return [ref firstUnlaidGlyphIndex]; }

void LayoutManagerGetFirstUnlaidCharacterIndex( NSLayoutManager *ref, NSUInteger *charIndex, NSUInteger *glyphIndex )
{ [ref getFirstUnlaidCharacterIndex:charIndex glyphIndex:glyphIndex]; }

CGRect LayoutManagerLineFragmentRectForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange )
{ return NSRectToCGRect([ref lineFragmentRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange]); }

CGRect LayoutManagerLineFragmentRectForGlyphAtIndexWithoutAdditionalLayout( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange, BOOL flag )
{ return NSRectToCGRect([ref lineFragmentRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange withoutAdditionalLayout:flag]); }

CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange )
{ return NSRectToCGRect([ref lineFragmentUsedRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange]); }

CGRect LayoutManagerExtraLineFragmentUsedRectForGlyphAtIndexWithoutAdditionalLayout( NSLayoutManager *ref, NSUInteger index, CFRange *effectiveRange, BOOL flag )
{ return NSRectToCGRect([ref lineFragmentUsedRectForGlyphAtIndex:index effectiveRange:(NSRange *)effectiveRange withoutAdditionalLayout:flag]); }

CGPoint LayoutManagerLocationForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index )
{ return NSPointToCGPoint([ref locationForGlyphAtIndex:index]); }

BOOL LayoutManagerNotShownAttributeForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index )
{ return [ref notShownAttributeForGlyphAtIndex:index]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFRange LayoutManagerTruncatedGlyphRangeInLineFragmentForGlyphAtIndex( NSLayoutManager *ref, NSUInteger index )
{ return NSRangeToCFRange([ref truncatedGlyphRangeInLineFragmentForGlyphAtIndex:index]); }
#endif

// Advanced layout queries
CGRect LayoutManagerBoundingRectForGlyphRange( NSLayoutManager *ref, CFRange glyphRange, NSTextContainer *tc )
{ return NSRectToCGRect([ref boundingRectForGlyphRange:NSRangeFromCFRange(glyphRange) inTextContainer:tc]); }

NSUInteger LayoutManagerCharacterIndexForPoint( NSLayoutManager *ref, CGPoint pt, NSTextContainer *tc, CGFloat *partialFraction )
{ return [ref characterIndexForPoint:NSPointFromCGPoint(pt) inTextContainer:tc fractionOfDistanceBetweenInsertionPoints:partialFraction]; }

CFRange LayoutManagerCharacterRangeForGlyphRange( NSLayoutManager *ref, CFRange glyphRange, CFRange *actualGlyphRange )
{ return NSRangeToCFRange([ref characterRangeForGlyphRange:NSRangeFromCFRange(glyphRange) actualGlyphRange:(NSRange *)actualGlyphRange]); }

CFRange LayoutManagerGlyphRangeForBoundingRect( NSLayoutManager *ref, CGRect bounds, NSTextContainer *tc )
{ return NSRangeToCFRange([ref glyphRangeForBoundingRect:NSRectFromCGRect(bounds) inTextContainer:tc]); }

// Drawing

// Text blocks

// Attachments

// Rulers

// Responder chain

// Typesetter
CGFloat LayoutManagerDefaultLineHeightForFont( NSLayoutManager *ref, NSFont *font )
{ return [ref defaultLineHeightForFont:font]; }

CGFloat LayoutManagerDefaultBaselineOffsetForFont( NSLayoutManager *ref, NSFont *font )
{ return [ref defaultBaselineOffsetForFont:font]; }

// Temporary attribute support

