/*
 LinguisticTagger.m

 Bernie Wylde
 */

#import "LinguisticTagger.h"

// Class
Class LinguisticTaggerClass( void )
{ return [NSLinguisticTagger class]; }

// - init -
NSLinguisticTagger *LinguisticTaggerInit( CFArrayRef tagSchemes, NSUInteger options )
{ return [[[NSLinguisticTagger alloc] initWithTagSchemes:(__bridge NSArray *)tagSchemes options:options] autorelease]; }

NSLinguisticTagger *LinguisticTaggerWithTagSchemes( CFArrayRef tagSchemes, NSUInteger options )
{ return [[[NSLinguisticTagger alloc] initWithTagSchemes:(__bridge NSArray *)tagSchemes options:options] autorelease]; }

// - string -
CFStringRef LinguisticTaggerString( NSLinguisticTagger *ref )
{ return (__bridge CFStringRef)[ref string]; }

void LinguisticTaggerSetString( NSLinguisticTagger *ref, CFStringRef string )
{ [ref setString:(__bridge NSString *)string]; }

// - tag schemes -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFArrayRef LinguisticTaggerAvailableTagSchemesForUnit( NSLinguisticTaggerUnit unit, CFStringRef language )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFArrayRef)[NSLinguisticTagger availableTagSchemesForUnit:unit language:(__bridge NSString *)language];
    }
    return nil;
}
#endif

CFArrayRef LinguisticTaggerAvailableTagSchemesForLanguage( CFStringRef language )
{ return (__bridge CFArrayRef)[NSLinguisticTagger availableTagSchemesForLanguage:(__bridge NSString *)language]; }

CFArrayRef LinguisticTaggerTagSchemes( NSLinguisticTagger *ref )
{ return (__bridge CFArrayRef)[ref tagSchemes]; }

// - determine dominant language and orthography -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFStringRef LinguisticTaggerDominantLanguageForString( CFStringRef string )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFStringRef)[NSLinguisticTagger dominantLanguageForString:(__bridge NSString *)string];
    }
    return nil;
}

CFStringRef LinguisticTaggerDominantLanguage( NSLinguisticTagger *ref )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFStringRef)[ref dominantLanguage];
    }
    return nil;
}
#endif

NSOrthography *LinguisticTaggerOrthographyAtIndex( NSLinguisticTagger *ref, NSUInteger index, CFRange *effectiveRange )
{ return [ref orthographyAtIndex:index effectiveRange:(NSRangePointer)effectiveRange]; }

void LinguisticTaggerSetOrthography( NSLinguisticTagger *ref, NSOrthography *orthRef, CFRange range )
{ [ref setOrthography:orthRef range:NSRangeFromCFRange(range)]; }

// - enumerate -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
void LinguisticTaggerEnumerateTagsForUnit( NSLinguisticTagger *ref, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, CUI_LinguisticTaggerCallbackType2 callback, void *userData )
{
    [ref enumerateTagsInRange:NSRangeFromCFRange(range)
                         unit:unit
                       scheme:(__bridge NSString *)scheme
                      options:options
                   usingBlock:^(NSString *tag, NSRange tokenRange, BOOL *stop) {
                       (*callback)(ref,(__bridge CFStringRef)tag, NSRangeToCFRange(tokenRange), stop, userData );
                   }];
}
#endif

void LinguisticTaggerEnumerateTagsForScheme( NSLinguisticTagger *ref, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, CUI_LinguisticTaggerCallbackType1 callback, void *userData )
{
    [ref enumerateTagsInRange:NSRangeFromCFRange(range)
                       scheme:(__bridge NSString *)scheme
                      options:options
                   usingBlock:^(NSString *tag, NSRange tokenRange, NSRange sentenceRange, BOOL *stop) {
                       (*callback)(ref,(__bridge CFStringRef)tag, NSRangeToCFRange(tokenRange), NSRangeToCFRange(sentenceRange), stop, userData );
                   }];
}

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
void LinguisticTaggerEnumerateTagsForString( CFStringRef string, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, NSOrthography *orthRef, CUI_LinguisticTaggerCallbackType2 callback, void *userData )
{
    if (@available(macOS 10.13, *)) {
        [NSLinguisticTagger enumerateTagsForString:(__bridge NSString *)string
                                             range:NSRangeFromCFRange(range)
                                              unit:unit
                                            scheme:(__bridge NSString *)scheme
                                           options:options
                                       orthography:orthRef
                                        usingBlock:^(NSString *tag, NSRange tokenRange, BOOL *stop) {
                                            (*callback)(nil, (__bridge CFStringRef)tag, NSRangeToCFRange(tokenRange), stop, userData );
                                        }];
    }
}
#endif

// - get -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFStringRef LinguisticTaggerTagForUnit( NSLinguisticTagger *ref, NSUInteger index, NSLinguisticTaggerUnit unit, CFStringRef scheme, CFRange *tokenRange )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFStringRef)[ref tagAtIndex:index unit:unit scheme:(__bridge NSString *)scheme tokenRange:(NSRangePointer)tokenRange];
    }
    return nil;
}
#endif

CFStringRef LinguisticTaggerTagForScheme( NSLinguisticTagger *ref, NSUInteger index, CFStringRef scheme, CFRange *tokenRange, CFRange *sentenceRange )
{ return (__bridge CFStringRef)[ref tagAtIndex:index scheme:(__bridge NSString *)scheme tokenRange:(NSRangePointer)tokenRange sentenceRange:(NSRangePointer)sentenceRange]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFStringRef LinguisticTaggerTagForString( CFStringRef string, NSUInteger index, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSOrthography *orthRef, CFRange *tokenRange )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFStringRef)[NSLinguisticTagger tagForString:(__bridge NSString *)string atIndex:index unit:unit scheme:(__bridge NSString *)scheme orthography:orthRef tokenRange:(NSRangePointer)tokenRange];
    }
    return nil;
}

CFArrayRef LinguisticTaggerTagsForUnit( NSLinguisticTagger *ref, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, CFArrayRef *tokenRanges )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFArrayRef)[ref tagsInRange:NSMakeRange(range.location,range.length) unit:unit scheme:(__bridge NSString *)scheme options:options tokenRanges:(NSArray **)tokenRanges];
    }
    return nil;
}
#endif

CFArrayRef LinguisticTaggerTagsForScheme( NSLinguisticTagger *ref, CFRange range, CFStringRef scheme, NSLinguisticTaggerOptions options, CFArrayRef *tokenRanges )
{ return (__bridge CFArrayRef)[ref tagsInRange:NSRangeFromCFRange(range) scheme:(__bridge NSString *)scheme options:options tokenRanges:(NSArray **)tokenRanges]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFArrayRef LinguisticTaggerTagsForString( CFStringRef string, CFRange range, NSLinguisticTaggerUnit unit, CFStringRef scheme, NSLinguisticTaggerOptions options, NSOrthography *orthRef, CFArrayRef *tokenRanges )
{
    if (@available(macOS 10.13, *)) {
        return (__bridge CFArrayRef)[NSLinguisticTagger tagsForString:(__bridge NSString *)string range:NSRangeFromCFRange(range) unit:unit scheme:(__bridge NSString *)scheme options:options orthography:orthRef tokenRanges:(NSArray **)tokenRanges];
    }
    return nil;
}
#endif

// - range of unit token -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFRange LinguisticTaggerTokenRangeAtIndex( NSLinguisticTagger *ref, NSUInteger index, NSLinguisticTaggerUnit unit )
{
    if (@available(macOS 10.13, *)) {
        NSRange range = [ref tokenRangeAtIndex:index unit:unit];
        return NSRangeToCFRange(range);
    }
    return CFRangeMake(0,0);
}
#endif

CFRange LinguisticTaggerSentenceRangeForRange( NSLinguisticTagger *ref, CFRange range )
{
    NSRange outRange = [ref sentenceRangeForRange:(NSRangeFromCFRange(range))];
    return NSRangeToCFRange(outRange);
}

// - possible tags -
CFArrayRef LinguisticTaggerPossibleTagsAtIndex( NSLinguisticTagger *ref, NSUInteger index, CFStringRef scheme, CFRange *tokenRange, CFRange *sentenceRange, CFArrayRef *scores )
{ return (__bridge CFArrayRef)[ref possibleTagsAtIndex:index scheme:(__bridge NSString *)scheme tokenRange:(NSRangePointer)tokenRange sentenceRange:(NSRangePointer)sentenceRange scores:(NSArray **)scores]; }

// Notify changes to analyzed string
void LinguisticTaggerStringEditedInRange( NSLinguisticTagger *ref, CFRange newRange, NSInteger delta )
{ [ref stringEditedInRange:NSRangeFromCFRange(newRange) changeInLength:delta]; }

