/*
 Menu.h

 Bernie Wylde
 */

#import "MenuItem.h"
#import "Nib.h"
#import "Object.h"
#import "View.h"

typedef void (*CUI_MenuItemCallbackType)(NSInteger,NSInteger,void *);
typedef BOOL (*CUI_ValidateMenuItemCallbackType)(NSInteger,NSInteger);

//void CUI_BuildMinimalMenu( void );// 20211002 moved to Application.h

@interface CUI_MenuItemTarget : NSObject
@property (assign) CUI_ValidateMenuItemCallbackType validateCallback;
@property (assign) BOOL boolVal;

- (void)menuItemAction:(id)sender;// 20210103
@end


#pragma mark - menu delegate
typedef void (*CUI_MenuDelegateCallbackType)(NSInteger,NSInteger,NSInteger,void*);

// events
enum {// 20220311
    menuHasKeyEquivalent,// not currently implemented
    menuUpdateItem,// not currently implemented
    menuConfinementRect,// not currently implemented
    menuWillHighlightItem,
    menuWillOpen,
    menuDidClose,
    menuNumberOfItems,// not currently implemented
    menuNeedsUpdate
};

@interface CUI_MenuDelegate : NSObject <NSMenuDelegate>// 20220311
@property (assign) CUI_MenuDelegateCallbackType callback;
@property (assign) void *userData;
@end

//CUI_MenuDelegate *CUI_MenuDelegateObj( void );


void CUI_NibMenuSetup( NSMenu *menu );
void CUI_NibMenuFixAppNameItems( NSMenu *mainMenu );

CUI_MenuItemTarget *CUI_MenuItemTargetObj( void );

void CUI_OtherMenuAdd( NSMenu *menu );
NSMenu *CUI_OtherMenuAtIndex( NSInteger index );
NSInteger CUI_MenuIndex( NSMenu *menu );

@interface NSMenu (Additions)
@property (nonatomic,retain) NSNumber *tagNumber;
- (NSInteger)tag;
- (void)setTag:(NSInteger)tag;
@end

// private
void CUI_MenuPadItemsToIndex( NSMenu *menu, NSInteger index );


@interface NSMenuItem (Additions)
@property (nonatomic,retain) NSValue *callbackValue;
@property (nonatomic,retain) NSValue *userDataValue;

- (CUI_MenuItemCallbackType)cuiMenuItemCallback;
- (void *)cuiMenuItemUserData;
- (void)setCuiMenuItemCallback:(CUI_MenuItemCallbackType)cb;
- (void)setCuiMenuItemUserData:(void *)userData;
@end


void MenuEventSetBool( BOOL flag );

// Class
Class MenuClass( void );

NSMenu *MenuAtIndex( NSInteger index );
BOOL MenuExists( NSInteger index );
NSInteger MenuIndex( NSMenu *m );


// MenuBar
BOOL MenuBarVisible( void );
void MenuBarSetVisible( BOOL flag );
CGFloat MenuBarHeight( void );

// Init
NSMenu *MenuWithTitle( CFStringRef title );

// Add remove items
void MenuInsertItem( NSInteger menuIndex, NSMenuItem *item, NSInteger itemIndex );
NSMenuItem *MenuInsertItemWithTitle( NSInteger menuIndex, NSInteger itemIndex, CFStringRef title, CFStringRef keyEquivalent, NSUInteger keyEquivalentModifierMask );
void MenuAddItem( NSInteger menuIndex, NSMenuItem *item );
NSMenuItem *MenuAddItemWithTitle( NSInteger menuIndex, CFStringRef title, CFStringRef keyEquivalent, NSUInteger keyEquivalentModifierMask );
void MenuRemoveItem( NSInteger menuIndex, NSInteger itemIndex );
void MenuRemoveAllItems( NSInteger menuIndex );

// Find items
NSMenuItem *MenuItemWithTag( NSInteger menuIndex, NSInteger itemTag );
NSInteger MenuItemWithTitle( NSInteger menuIndex, CFStringRef title );
NSMenuItem *MenuItemAtIndex( NSInteger menuIndex, NSInteger itemIndex );
NSInteger MenuNumberOfItems( NSInteger menuIndex );
CFArrayRef MenuItemArray( NSInteger menuIndex );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
void MenuSetItemArray( NSInteger menuIndex, CFArrayRef items );
#endif// 101400

// Find indices of items
NSInteger MenuIndexOfItem( NSInteger menuIndex, NSMenuItem *item );
NSInteger MenuIndexOfItemWithTitle( NSInteger menuIndex, CFStringRef title );
NSInteger MenuIndexOfItemWithTag( NSInteger menuIndex, NSInteger tag );
NSInteger MenuIndexOfItemWithRepresentedObject( NSInteger menuIndex, CFTypeRef obj );
NSInteger MenuIndexOfItemWithSubmenu( NSInteger menuIndex, NSInteger submenuID );

// Submenus
void MenuSetSubmenu( NSInteger parMenuIndex, NSInteger parItemIndex, NSInteger submenuIndex );
NSMenu *MenuSupermenu( NSInteger menuIndex );
NSInteger MenuSupermenuIndex( NSInteger menuIndex );

// Enable/disable items
BOOL MenuAutoenablesItems( NSInteger menuIndex );
void MenuSetAutoenablesItems( NSInteger menuIndex, BOOL flag );
void MenuUpdate( NSInteger menuIndex );

// Font
CFTypeRef MenuFont( NSInteger menuIndex );
void MenuSetFont( NSInteger menuIndex, CFTypeRef inFont );
void MenuSetFontWithName( NSInteger menuIndex, CFStringRef inName, CGFloat size );// convenience

// Keyboard equivalents
BOOL MenuPerformKeyEquivalent( NSInteger menuIndex, NSEvent *event );

// Simulate mouse clicks
void MenuPerformActionForItemAtIndex( NSInteger menuIndex, NSInteger itemID );

// Title
CFStringRef MenuTitle( NSInteger menuIndex );
void MenuSetTitle( NSInteger menuIndex, CFStringRef title );

// Size
CGFloat MenuMinimumWidth( NSInteger menuIndex );
void MenuSetMinimumWidth( NSInteger menuIndex, CGFloat width );
CGSize MenuSize( NSInteger menuIndex );

// Properties
NSMenuProperties MenuPropertiesToUpdate( NSInteger menuIndex );

// Contextual menus
BOOL MenuAllowsContextMenuPlugIns( NSInteger menuIndex );
void MenuSetAllowsContextMenuPlugIns( NSInteger menuIndex, BOOL flag );

// Context-sensitive help
void MenuPopUpContextMenuForView( NSInteger menuIndex, NSEvent *event, NSInteger aViewTag );
void MenuPopUpContextMenuForViewWithFont( NSInteger menuIndex, NSEvent *event, NSInteger aViewTag, NSFont *font );
BOOL MenuPopUp( NSInteger menuIndex, NSInteger itemIndex, CGPoint location, NSInteger viewTag );

// State column
BOOL MenuShowsStateColumn( NSInteger menuIndex );
void MenuSetShowsStateColumn( NSInteger menuIndex, BOOL flag );

// Highlighting
NSInteger MenuHighlightedItem( NSInteger menuIndex );

// Delegate
void MenuSetDelegateCallback( NSInteger menuID, CUI_MenuDelegateCallbackType callback, void *userData );

// User interface
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUserInterfaceLayoutDirection MenuUserInterfaceLayoutDirection( NSInteger menuIndex );
void MenuSetUserInterfaceLayoutDirection( NSInteger menuIndex, NSUserInterfaceLayoutDirection direction );
#endif// 101100

// Tracking
void MenuCancelTracking( NSInteger menuIndex );
void MenuCancelTrackingWithoutAnimation( NSInteger menuIndex );

// Custom
void MenuSetTag( NSInteger menuIndex, NSInteger tag );
void MenuSetOneItemOnState( NSInteger menuIndex, NSInteger itemIndex );
void MenuSetValidateItemsCallback( CUI_ValidateMenuItemCallbackType callback );
BOOL MenuIsHidden( NSInteger menuIndex );
void MenuSetHidden( NSInteger menuIndex, BOOL flag );
BOOL MenuIsHiddenOrHasHiddenAncestor( NSInteger menuIndex );

CFTypeRef MenuProperty( NSInteger menuIndex, CFStringRef key );
void MenuSetProperty( NSInteger menuIndex, CFStringRef key, CFTypeRef value );
void MenuRemoveProperty( NSInteger menuIndex, CFStringRef key );
void MenuRemoveAllProperties( NSInteger menuIndex );

// FB keywords
void CUI_Menu( NSInteger inMenuIndex, NSInteger inItemIndex, NSInteger inEnabled, CFStringRef inTitle, CFStringRef inKeyEquivalent, NSInteger inKeyEquivalentModifier );

void CUI_EditMenu( NSInteger inMenuIndex );
void CUI_WindowMenu( NSInteger inMenuIndex );
void CUI_FileMenu( NSInteger inMenuIndex );
void CUI_FindMenu( NSInteger inMenuIndex );
void CUI_FormatMenu( NSInteger inMenuIndex );
void CUI_FontMenu( NSInteger inMenuIndex );
void CUI_TextMenu( NSInteger inMenuIndex );
void CUI_ViewMenu( NSInteger inMenuIndex );
void CUI_HelpMenu( NSInteger inMenuIndex );

#pragma mark - nibmenu
void CUI_NibMenu( CFStringRef inNibName );


#pragma mark - menu events
// all menu stuff moved to here from AppThings.c - 20211004

// this struct deprecated 20220901
//typedef struct FBMenuEventInfo {    // CocoaUI only uses the menu and item fields
//    NSInteger menu;                            // last selected menu id
//    NSInteger item;                            // last selected menu item
//} FBMenuEventInfo;

typedef void (*CUI_OnMenuFunctionType)(NSInteger,NSInteger);
//typedef void (*CUI_OnMenuFunctionType)(NSInteger,NSInteger,NSInteger);// added 3rd param (delegate event) 20220311

static CUI_OnMenuFunctionType sCUI_OnMenuFunction;
CUI_OnMenuFunctionType CUI_OnMenuFunction( void );
void CUI_SetOnMenuFunction( CUI_OnMenuFunctionType onMenuFn );
void CallUserDefinedOnMenuFunction( long menuIndex, long itemIndex );
//void CallUserDefinedOnMenuFunction( long menuIndex, long itemIndex, long delegateEvent );// 20220311

//long FBMenuFunction( long val );// moved to here from CocoaUI.h - 20211004// deprecated
