/*
 MenuItem.m

 Bernie Wylde
 */

#import "MenuItem.h"

// Class
Class MenuItemClass( void )
{ return [NSMenuItem class]; }

NSMenuItem *CUI_MenuItemAtIndex( NSMenu *menu, NSInteger index )
{
    if ( index > 1000000 ) return (NSMenuItem *)index;
    
    NSInteger count = [menu numberOfItems];
    if ( index < count ) return [menu itemAtIndex:index];
    return nil;
}

BOOL MenuItemExists( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    return ( item != nil );
}

// Create
NSMenuItem *MenuItemInit( CFStringRef title, CFStringRef action, CFStringRef keyEquivalent )
{ return [[[NSMenuItem alloc] initWithTitle:(__bridge NSString *)title action:NSSelectorFromString((__bridge NSString *)action) keyEquivalent:(__bridge NSString *)keyEquivalent] autorelease]; }

// Enable
BOOL MenuItemIsEnabled( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item isEnabled];
    return NO;
}

void MenuItemSetEnabled( NSInteger menuIndex, NSInteger itemIndex, BOOL flag )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setEnabled:flag];
}

// Hidden
BOOL MenuItemIsHidden( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item isHidden];
    return NO;
}

void MenuItemSetHidden( NSInteger menuIndex, NSInteger itemIndex, BOOL flag )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setHidden:flag];
}

BOOL MenuItemIsHiddenOrHasHiddenAncestor( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item isHiddenOrHasHiddenAncestor];
    return NO;
}

// Target and action
void MenuItemSetAction( NSInteger menuIndex, NSInteger itemIndex, CFStringRef actionName )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) {
        if ( [(__bridge NSString *)actionName isEqualToString:@"menuItemAction:"]) {
            [item setTarget:CUI_MenuItemTargetObj()];
        } else {
            [item setTarget:nil];
        }
        [item setAction:NSSelectorFromString((__bridge NSString *)actionName)];
    }
}

// Title
CFStringRef MenuItemTitle( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFStringRef)[item title];
    return nil;
}

void MenuItemSetTitle( NSInteger menuIndex, NSInteger itemIndex, CFStringRef title )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setTitle:(__bridge NSString *)title];
}

CFAttributedStringRef MenuItemAttributedTitle( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFAttributedStringRef)[item attributedTitle];
    return nil;
}

void MenuItemSetAttributedTitle( NSInteger menuIndex, NSInteger itemIndex, CFAttributedStringRef title )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setAttributedTitle:(__bridge NSAttributedString *)title];
}

// Tag
NSInteger MenuItemTag( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item tag];
    return 0;
}

void MenuItemSetTag( NSInteger menuIndex, NSInteger itemIndex, NSInteger tag )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setTag:tag];
}

// State
NSCellStateValue MenuItemState( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item state];
    return NSOffState;
}

void MenuItemSetState( NSInteger menuIndex, NSInteger itemIndex, NSCellStateValue state )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setState:state];
}

// Image
NSImage *MenuItemImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item image];
    return nil;
}

void MenuItemSetImage( NSInteger menuIndex, NSInteger itemIndex, NSImage *image )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setImage:image];
}

NSImage *MenuItemOnStateImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item onStateImage];
    return nil;
}

void MenuItemSetOnStateImage( NSInteger menuIndex, NSInteger itemIndex, NSImage *image )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setOnStateImage:image];
}

NSImage *MenuItemOffStateImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item offStateImage];
    return nil;
}

void MenuItemSetOffStateImage( NSInteger menuIndex, NSInteger itemIndex, NSImage *image )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setOffStateImage:image];
}

NSImage *MenuItemMixedStateImage( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item mixedStateImage];
    return nil;
}

void MenuItemSetMixedStateImage( NSInteger menuIndex, NSInteger itemIndex, NSImage *image )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setMixedStateImage:image];
}

// Submenu
NSMenu *MenuItemSubmenu( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item submenu];
    return nil;
}

NSInteger MenuItemSubmenuIndex( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return CUI_MenuIndex( [item submenu] );
    return 0;
}

BOOL MenuItemHasSubmenu( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item hasSubmenu];
    return NO;
}

NSMenuItem *MenuItemParentItem( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item parentItem];
    return nil;
}

// Separator
BOOL MenuItemIsSeparator( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item isSeparatorItem];
    return NO;
}

NSMenuItem *MenuItemSeparatorItem( void )
{ return [NSMenuItem separatorItem]; }

// Menu
NSMenu *MenuItemMenu( NSMenuItem *item )
{ return [item menu]; }

NSInteger MenuItemMenuIndex( NSMenuItem *item )
{
    NSMenu *menu = [item menu];
    if ( menu ) return CUI_MenuIndex( menu );
    return -1;
}

// KeyEquivalent
CFStringRef MenuItemKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFStringRef)[item keyEquivalent];
    return nil;
}

void MenuItemSetKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex, CFStringRef string )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setKeyEquivalent:(__bridge NSString *)string];
}

NSUInteger MenuItemKeyEquivalentModifierMask( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item keyEquivalentModifierMask];
    return 0;
}

void MenuItemSetKeyEquivalentModifierMask( NSInteger menuIndex, NSInteger itemIndex, NSUInteger mask )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setKeyEquivalentModifierMask:mask];
}

// User key equivalents
BOOL MenuItemUsesUserKeyEquivalents( void )
{ return [NSMenuItem usesUserKeyEquivalents]; }

CFStringRef MenuItemUserKeyEquivalent( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFStringRef)[item userKeyEquivalent];
    return nil;
}

// Alternate
BOOL MenuItemIsAlternate( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item isAlternate];
    return NO;
}

void MenuItemSetAlternate( NSInteger menuIndex, NSInteger itemIndex, BOOL flag )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setAlternate:flag];
}

// Indentation level
NSInteger MenuItemIndentationLevel( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item indentationLevel];
    return 0;
}

void MenuItemSetIndentationLevel( NSInteger menuIndex, NSInteger itemIndex, NSInteger level )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setIndentationLevel:level];
}

// ToolTip
CFStringRef MenuItemToolTip( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFStringRef)[item toolTip];
    return nil;
}

void MenuItemSetToolTip( NSInteger menuIndex, NSInteger itemIndex, CFStringRef string )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setToolTip:(__bridge NSString *)string];
}

// RepresentedObject
CFTypeRef MenuItemRepresentedObject( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFTypeRef)[item representedObject];
    return nil;
}

void MenuItemSetRepresentedObject( NSInteger menuIndex, NSInteger itemIndex, CFTypeRef obj )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setRepresentedObject:(__bridge id)obj];
}

// View
NSView *MenuItemView( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item view];
    return nil;
}

void MenuItemSetView( NSInteger menuIndex, NSInteger itemIndex, NSView *aView )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) [item setView:aView];
}

// Highlighted
BOOL MenuItemIsHighlighted( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return [item isHighlighted];
    return NO;
}

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
BOOL MenuItemAllowsKeyEquivalentWhenHidden( NSInteger menuIndex, NSInteger itemIndex )
{
    if (@available(macOS 10.13, *)) {
        NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
        if ( item ) return [item allowsKeyEquivalentWhenHidden];
    }
    return NO;
}

void MenuItemSetAllowsKeyEquivalentWhenHidden( NSInteger menuIndex, NSInteger itemIndex, BOOL flag )
{
    if (@available(macOS 10.13, *)) {
        NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
        if ( item ) [item setAllowsKeyEquivalentWhenHidden:flag];
    }
}
#endif// 101300

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
BOOL MenuItemAllowsAutomaticKeyEquivalentLocalization( NSInteger menuID, NSInteger itemID )
{
    NSMenuItem *item = MenuItemAtIndex( menuID, itemID );
    if ( item ) return [item allowsAutomaticKeyEquivalentLocalization];
    return NO;
}

void MenuItemSetAllowsAutomaticKeyEquivalentLocalization( NSInteger menuID, NSInteger itemID, BOOL flag )
{
    NSMenuItem *item = MenuItemAtIndex( menuID, itemID );
    if ( item ) [item setAllowsAutomaticKeyEquivalentLocalization:flag];
}

BOOL MenuItemAllowsAutomaticKeyEquivalentMirroring( NSInteger menuID, NSInteger itemID )
{
    NSMenuItem *item = MenuItemAtIndex( menuID, itemID );
    if ( item ) return [item allowsAutomaticKeyEquivalentMirroring];
    return NO;
}

void MenuItemSetAllowsAutomaticKeyEquivalentMirroring( NSInteger menuID, NSInteger itemID, BOOL flag )
{
    NSMenuItem *item = MenuItemAtIndex( menuID, itemID );
    if ( item ) [item setAllowsAutomaticKeyEquivalentMirroring:flag];
}
#endif// 120000

// Custom
void MenuItemSetOnMenuAction( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenu *mainMenu = [NSApp mainMenu];
    if ( !mainMenu ) CUI_BuildMinimalMenu();
    
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) {
        [item setTarget:CUI_MenuItemTargetObj()];
        [item setAction:@selector(menuItemAction:)];
    }
}

void MenuItemSetActionCallback( NSInteger menuIndex, NSInteger itemIndex, void *callback, void *userData )
{
    if ( itemIndex < 0 ) {
        NSMenu *menu = MenuAtIndex( menuIndex );
        NSArray *items = [menu itemArray];
        for ( NSMenuItem *item in items ) {
            [item setTarget:CUI_MenuItemTargetObj()];
            [item setAction:@selector(menuItemAction:)];
            [item setCuiMenuItemCallback:callback];
            [item setCuiMenuItemUserData:userData];
        }
    } else {
        NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
        if ( item ) {
            [item setTarget:CUI_MenuItemTargetObj()];
            [item setAction:@selector(menuItemAction:)];
            [item setCuiMenuItemCallback:callback];
            [item setCuiMenuItemUserData:userData];
        }
    }
}

CFTypeRef MenuItemProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) return (__bridge CFTypeRef)ObjectProperty( item, key );
    return nil;
}

void MenuItemSetProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key, CFTypeRef value )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) ObjectSetProperty( item, key, value );
}

void MenuItemRemoveProperty( NSInteger menuIndex, NSInteger itemIndex, CFStringRef key )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) ObjectRemoveProperty( item, key );
}

void MenuItemRemoveAllProperties( NSInteger menuIndex, NSInteger itemIndex )
{
    NSMenuItem *item = MenuItemAtIndex( menuIndex, itemIndex );
    if ( item ) ObjectRemoveAllProperties( item );
}

