/*
 Object.m

 Bernie Wylde
 */

#import "Object.h"

@implementation CUI_ObjectTarget

- (void)objectSelector:(id)object {
    (*_callback)(object);
}
@end

@implementation NSObject (Additions)

@dynamic IDNumber;

//@dynamic tagNumber;// 20200323

@dynamic wndTagNumber;
@dynamic objProperties;

- (NSInteger)ID {
    return [objc_getAssociatedObject(self, @selector(IDNumber)) integerValue];
}

- (void)setID:(NSInteger)ID {
    objc_setAssociatedObject(self, @selector(IDNumber), [NSNumber numberWithInteger:ID], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

//- (NSInteger)tag {// 20200323
//    return [objc_getAssociatedObject(self, @selector(tagNumber)) integerValue];
//}
//
//- (void)setTag:(NSInteger)tag {// 20200323
//    objc_setAssociatedObject(self, @selector(tagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
//}

- (NSInteger)wndTag {
    return [objc_getAssociatedObject(self, @selector(wndTagNumber)) integerValue];
}

- (void)setWndTag:(NSInteger)tag {
    objc_setAssociatedObject(self, @selector(wndTagNumber), [NSNumber numberWithInteger:tag], OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (NSMutableDictionary *)objectProperties {
    return objc_getAssociatedObject(self, @selector(objProperties));
}

- (void)setObjectProperties:(NSMutableDictionary *)properties {
    objc_setAssociatedObject(self, @selector(objProperties), properties, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}
@end

#pragma mark - private
NSMutableArray *CUI_Objects( void )
{
    static NSMutableArray *sCUI_Objects;
    if ( !sCUI_Objects ) sCUI_Objects = [[NSMutableArray alloc] initWithCapacity:0];
    return sCUI_Objects;
}


#pragma mark - public
//NSInteger ObjectTag( CFTypeRef obj )// 20200323
//{ return [(NSObject *)obj tag]; }
//
//void ObjectSetTag( CFTypeRef obj, NSInteger tag )// 20200323
//{ [(NSObject *)obj setTag:tag]; }
//
//CFTypeRef ObjectWithTag( NSInteger tag )// 20200323
//{
//    // ???
//    return nil;
//}


// Init
NSObject *ObjectInit( void )
{ return [[[NSObject alloc] init] autorelease]; }

// - create, copy, dealloc -
CFTypeRef ObjectCopy( CFTypeRef obj )
{ return (__bridge CFTypeRef)[[(__bridge id)obj copy] autorelease]; }

CFTypeRef ObjectMutableCopy( CFTypeRef obj )
{ return (__bridge CFTypeRef)[[(__bridge id)obj mutableCopy] autorelease]; }


//id ObjectNSClassFromCF( CFTypeRef obj )
//{
//    CFTypeID type = CFGetTypeID(obj);
//    switch ( type ) {
//        case CFArrayGetTypeID():
//            return [NSArray class];
//        case CFDictionaryGetTypeID():
//            return [NSDictionary class];
//        case CFStringGetTypeID():
//            return [NSString class];
//        default:
//            break;
//    }
//    return [(__bridge id)obj class]
//}


Class ObjectClass( CFTypeRef obj )
{
    // should check for CF type here and return NS equivalent
    return [(__bridge id)obj class];
}

Class ObjectSuperclass( CFTypeRef obj )
{ return [(__bridge id)obj superclass]; }


// - identify class -
BOOL ObjectIsSubclassOfClass( CFTypeRef obj, Class class )
{ return [(__bridge id)obj isSubclassOfClass:class]; }


// Description
CFStringRef ObjectDescription( CFTypeRef obj )
{ return (__bridge CFStringRef)[(__bridge id)obj description]; }

// Method info
NSMethodSignature *ObjectInstanceMethodSignatureForSelector( CFTypeRef obj, SEL selector )
{ return [(__bridge id)obj instanceMethodSignatureForSelector:selector]; }

NSMethodSignature *ObjectMethodSignatureForSelector( CFTypeRef obj, SEL selector )
{ return [(__bridge id)obj methodSignatureForSelector:selector]; }

// Sending messages
void ObjectPerformSelector( CFTypeRef obj, CFStringRef selector, CFTypeRef withObj, CFTimeInterval afterDelay )
{ [(__bridge id)obj performSelector:NSSelectorFromString((__bridge NSString *)selector) withObject:(__bridge id)withObj afterDelay:afterDelay]; }

void ObjectPerformSelectorOnMainThread( CFTypeRef obj, CFTypeRef selector, CFTypeRef withObj, BOOL waitUntilDone )
{
//    SEL sel = CUI_FixSelector( selector );
//    [(__bridge id)obj performSelectorOnMainThread:sel withObject:(__bridge id)withObj waitUntilDone:waitUntilDone];

    [(__bridge id)obj performSelectorOnMainThread:NSSelectorFromString((__bridge NSString *)selector) withObject:(__bridge id)withObj waitUntilDone:waitUntilDone];
    
}

void ObjectPerformSelectorInBackground( CFTypeRef obj, CFStringRef selector, CFTypeRef withObj )
{ [(__bridge id)obj performSelectorInBackground:NSSelectorFromString((__bridge NSString *)selector) withObject:(__bridge id)withObj]; }

// Scripting
CFStringRef ObjectClassName( CFTypeRef obj )
{ return (__bridge CFStringRef)[(__bridge id)obj className]; }

// - instance methods -
BOOL ObjectIsEqual( CFTypeRef obj1, CFTypeRef obj2 )
{ return [(__bridge id)obj1 isEqual:(__bridge id)obj2]; }

//void ObjectSetValueForKey( CFTypeRef obj, CFTypeRef value, CFStringRef key )
//{ [(NSObject *)obj setValue:(__bridge id)value forKey:(__bridge NSString *)key]; }
//
//CFTypeRef ObjectValueForKey( CFTypeRef obj, CFStringRef key )
//{ return (__bridge CFTypeRef)[(__bridge id)obj valueForKey:(__bridge NSString *)key]; }


// -- NSObject protocol methods --
BOOL ObjectIsKindOfClass( CFTypeRef obj, Class class )
{ return [(__bridge id)obj isKindOfClass:class]; }





/*
 Custom
 */
// Call function
void ObjectCallFunction( void *fnAddress, CFTypeRef withObj, CFTimeInterval afterDelay )
{
    CUI_ObjectTarget *target = [[CUI_ObjectTarget alloc] init];
    [target setCallback:fnAddress];
    [target performSelector:@selector(objectSelector:) withObject:withObj afterDelay:afterDelay];
}

void ObjectCallFunctionOnMainThread( void *fnAddress, CFTypeRef withObj, BOOL waitUntilDone )
{
    CUI_ObjectTarget *target = [[CUI_ObjectTarget alloc] init];
    [target setCallback:fnAddress];
    [target performSelectorOnMainThread:@selector(objectSelector:) withObject:withObj waitUntilDone:waitUntilDone];
}

void ObjectCallFunctionInBackground( void *fnAddress, CFTypeRef withObj )
{
    CUI_ObjectTarget *target = [[CUI_ObjectTarget alloc] init];
    [target setCallback:fnAddress];
    [target performSelectorInBackground:@selector(objectSelector:) withObject:withObj];
}

// Object property
CFTypeRef ObjectProperty( CFTypeRef obj, CFStringRef key )
{
    id property = nil;
    NSMutableDictionary *properties = [(__bridge id)obj objectProperties];
    if ( properties ) {
        property = [properties objectForKey:(__bridge NSString *)key];
    }
    return (__bridge CFTypeRef)property;
}

void ObjectSetProperty( CFTypeRef obj, CFStringRef key, CFTypeRef value )
{
    NSMutableDictionary *properties = [(__bridge id)obj objectProperties];
    if ( !properties ) {
        properties = [[NSMutableDictionary alloc] initWithCapacity:0];
        [(__bridge id)obj setObjectProperties:properties];
    }
    if ( value == NULL ) {
        [properties removeObjectForKey:(__bridge NSString *)key];
        if ( [properties count] == 0 ) {
            [properties release];
            [(__bridge id)obj setObjectProperties:nil];
        }
    } else {
        [properties setObject:(__bridge id)value forKey:(__bridge NSString *)key];
    }
}

void ObjectRemoveProperty( CFTypeRef obj, CFStringRef key )
{
    NSMutableDictionary *properties = [(__bridge id)obj objectProperties];
    if ( properties ) {
        [properties removeObjectForKey:(__bridge NSString *)key];
        if ( [properties count] == 0 ) {
            [properties release];
            [(__bridge id)obj setObjectProperties:nil];
        }
    }
}

void ObjectRemoveAllProperties( CFTypeRef obj )
{
    NSMutableDictionary *properties = [(__bridge id)obj objectProperties];
    if ( properties ) {
        [properties release];
        [(__bridge id)obj setObjectProperties:nil];
    }
}


// Object ID
NSInteger ObjectID( CFTypeRef obj )
{
    NSMutableArray *objects = CUI_Objects();
    for ( NSObject *object in objects ) {
        if ( [object isEqual:(NSObject *)obj] ) return [object ID];
    }
    return NSNotFound;
}

CFTypeRef ObjectWithID( NSInteger objID )
{
    NSMutableArray *objects = CUI_Objects();
    for ( NSObject *obj in objects ) {
        if ( [obj ID] == objID ) return (__bridge CFTypeRef)obj;
    }
    return nil;
}

void ObjectSetID( CFTypeRef obj, NSInteger objID )
{
    NSMutableArray *objects = CUI_Objects();
    [(NSObject *)obj setID:objID];
    NSObject *object = (NSObject *)ObjectWithID( objID );
    if ( object ) {
        NSUInteger index = [objects indexOfObject:object];
        [objects replaceObjectAtIndex:index withObject:(NSObject *)obj];
    } else {
        [objects addObject:(NSObject *)obj];
    }
}

void ObjectRemoveID( NSInteger objID )
{
    NSObject *obj = (NSObject *)ObjectWithID( objID );
    if ( obj ) {
        [CUI_Objects() removeObject:obj];
    }
}

