/*
 OrderedCollectionDifference.m
 
 Bernie Wylde
 */

#import "OrderedCollectionDifference.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500

// Class
Class OrderedCollectionDifferenceClass( void )
{ return [NSOrderedCollectionDifference class]; }

// Init
NSOrderedCollectionDifference *OrderedCollectionDifferenceWithChanges( CFArrayRef changes )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [[[NSOrderedCollectionDifference alloc] initWithChanges:(__bridge NSArray *)changes] autorelease];
    }
    return nil;
}

NSOrderedCollectionDifference *OrderedCollectionDifferenceWithInsertIndexes( NSIndexSet *inserts, CFArrayRef insertedObjects, NSIndexSet *removes, CFArrayRef removedObjects )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [[[NSOrderedCollectionDifference alloc] initWithInsertIndexes:inserts insertedObjects:(__bridge NSArray *)insertedObjects removeIndexes:removes removedObjects:(__bridge NSArray *)removedObjects] autorelease];
    }
    return nil;
}

NSOrderedCollectionDifference *OrderedCollectionDifferenceWithInsertIndexesAdditionalChanges( NSIndexSet *inserts, CFArrayRef insertedObjects, NSIndexSet *removes, CFArrayRef removedObjects, CFArrayRef changes )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [[[NSOrderedCollectionDifference alloc] initWithInsertIndexes:inserts insertedObjects:(__bridge NSArray *)insertedObjects removeIndexes:removes removedObjects:(__bridge NSArray *)removedObjects additionalChanges:(__bridge NSArray *)changes] autorelease];
    }
    return nil;
}

// Instance properties
BOOL OrderedCollectionDifferenceHasChanges( NSOrderedCollectionDifference *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref hasChanges];
    }
    return NO;
}

CFArrayRef OrderedCollectionDifferenceInsertions( NSOrderedCollectionDifference *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return (__bridge CFArrayRef)[ref insertions];
    }
    return nil;
}

CFArrayRef OrderedCollectionDifferenceRemovals( NSOrderedCollectionDifference *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return (__bridge CFArrayRef)[ref removals];
    }
    return nil;
}

// Instance methods
NSOrderedCollectionDifference *OrderedCollectionDifferenceInverse( NSOrderedCollectionDifference *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref inverseDifference];
    }
    return nil;
}

NSOrderedCollectionDifference *OrderedCollectionDifferenceByTransformingChanges( NSOrderedCollectionDifference *ref, CUI_OrderedCollectionDifferenceTransformingChangesFunctionType fnAddress, void *userData )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref differenceByTransformingChangesWithBlock:^(NSOrderedCollectionChange *change) {
            return (*fnAddress)(change,userData);
        }];
    }
    return nil;
}

#endif// 101500

