/*
 ProcessInfo.h

 Bernie Wylde
 */


#import "CocoaUI.h"

typedef void (*CUI_ProcessInfoPerformActivityCallbackType)(void*);

// Class
Class ProcessInfoClass( void );

// - get process info agent
NSProcessInfo *ProcessInfoInit( void );

// - accessing process info -
CFArrayRef ProcessInfoArguments( void );
CFDictionaryRef ProcessInfoEnvironment( void );
CFStringRef ProcessInfoGloballyUniqueString( void );
int ProcessInfoProcessIdentifier( void );
CFStringRef ProcessInfoProcessName( void );

// - user info -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200
CFStringRef ProcessInfoUserName( void );
CFStringRef ProcessInfoFullUserName( void );
#endif

// - sudden app termination -
void ProcessInfoDisableSuddenTermination( void );
void ProcessInfoEnableSuddenTermination( void );

// - control auto termination
void ProcessInfoDisableAutomaticTermination( CFStringRef reason );
void ProcessInfoEnableAutomaticTermination( CFStringRef reason );
BOOL ProcessInfoAutomaticTerminationSupportEnabled( void );

// - host info -
CFStringRef ProcessInfoHostName( void );
CFStringRef ProcessInfoOperatingSystemVersionString( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSOperatingSystemVersion ProcessInfoOperatingSystemVersion( void );
BOOL ProcessInfoIsOperatingSystemAtLeastVersion( NSOperatingSystemVersion version);
#endif

// - computer info -
NSUInteger ProcessInfoProcessorCount( void );
NSUInteger ProcessInfoActiveProcessorCount( void );
unsigned long long ProcessInfoPhysicalMemory( void );
CFTimeInterval ProcessInfoSystemUptime( void );

// - activities -
void *ProcessInfoBeginActivity( NSActivityOptions options, CFStringRef reason );
void ProcessInfoEndActivity( void *obj );
void ProcessInfoPerformActivity( NSActivityOptions options, CFStringRef reason, CUI_ProcessInfoPerformActivityCallbackType callback, void *userData );

// - thermal state -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSProcessInfoThermalState ProcessInfoThermalState( void );
#endif

// Power state
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
BOOL ProcessInfoIsLowPowerModeEnabled( void );
#endif// 120000

// Platform
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
BOOL ProcessInfoIsMacCatalystApp( void );
#endif// 101500
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
BOOL ProcessInfoIsiOSAppOnMac( void );
#endif// 101600

