/*
 Responder.m

 Bernie Wylde
 */

#import "Responder.h"

@implementation CUI_ResponderTarget

// command for selector
NSInteger CUI_ResponderCommandForSelector( SEL selector )
{
    if ( selector == @selector(complete:) ) {
        return selectorComplete;
    } else if ( selector == @selector(deleteBackward:) ) {
        return selectorDeleteBackward;
    } else if ( selector == @selector(insertBacktab:) ) {
        return selectorInsertBacktab;
    } else if ( selector == @selector(insertNewline:) ) {
        return selectorInsertNewline;
    } else if ( selector == @selector(insertTab:) ) {
        return selectorInsertTab;
    } else if ( selector == @selector(moveLeft:) ) {
        return selectorMoveLeft;
    } else if ( selector == @selector(moveRight:) ) {
        return selectorMoveRight;
    }
    // .....
    return 0;
}


static CUI_ResponderTarget *sCUI_ResponderTarget;

- (void)didPresentErrorWithRecovery:(BOOL)didRecover contextInfo:(void *)contextInfo {
    DialogEventSetBool( didRecover );
    DialogEventSetContext( contextInfo );
    NSInteger wndTag = CUI_OutputWindowTag();
    CallUserDefinedOnDialogFunction( responderDidPresentError, 0, wndTag, nil );
}
@end


#pragma mark - forward events
void CUI_ResponderForwardWindowDialogEvent( NSWindow *w, NSInteger dialogEvent, NSEvent *event )
{
    CUI_EventSetEvent( event );
    NSInteger wndTag = [w tag];
    CUI_OutputWindowSetTag( wndTag );
    CallUserDefinedOnDialogFunction( dialogEvent, wndTag, wndTag, event );
    CUI_EventSetEvent( nil );
}

void CUI_ResponderForwardViewDialogEvent( NSView *view, NSInteger dialogEvent, NSEvent *event )
{
    CUI_EventSetEvent( event );
    NSInteger viewTag = CUI_ViewTag(view);
    //NSInteger wndTag = [[view window] tag];
    
    NSInteger wndTag = CUI_ViewWindowTag( view );

    // ===== 20220721 =====
//    if ( wndTag == 0 ) {
//        NSPopover *popover = CUI_PopoverForView( view );
//        if ( popover ) wndTag = [popover tag];
//    }
    // ====================
    
    CUI_OutputWindowSetTag( wndTag );
    CallUserDefinedOnDialogFunction( dialogEvent, viewTag, wndTag, event );
    CUI_EventSetEvent( nil );
}



#pragma mark - events
BOOL CUI_ResponderAcceptsFirstResponder( id responder )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowAcceptsFirstResponder, nil );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewAcceptsFirstResponder, nil );
    } else {
        // ...
    }
    return DialogEventBool();
}

void CUI_ResponderMouseDown( id responder, NSEvent *event )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowMouseDown, event );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewMouseDown, event );
    } else {
        // ...
    }
}

void CUI_ResponderMouseDragged( id responder, NSEvent *event )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowMouseDragged, event );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewMouseDragged, event );
    } else {
        // ...
    }
}

void CUI_ResponderMouseUp( id responder, NSEvent *event )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowMouseUp, event );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewMouseUp, event );
    } else {
        // ...
    }
}

void CUI_ResponderMouseMoved( id responder, NSEvent *event )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowMouseMoved, event );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewMouseMoved, event );
    } else {
        // ...
    }
}

void CUI_ResponderMouseEntered( id responder, NSEvent *event )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowMouseEntered, event );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewMouseEntered, event );
    } else {
        // ...
    }
}

void CUI_ResponderMouseExited( id responder, NSEvent *event )
{
    if ( [responder isKindOfClass:[NSWindow class]] ) {
        CUI_ResponderForwardWindowDialogEvent( (NSWindow *)responder, windowMouseExited, event );
    } else if ( [responder isKindOfClass:[NSView class]] ) {
        CUI_ResponderForwardViewDialogEvent( (NSView *)responder, viewMouseExited, event );
    } else {
        // ...
    }
}










#pragma mark - public
// Class
Class ResponderClass( void )
{ return [NSResponder class]; }

// Manage next responder
NSResponder *ResponderNextResponder( NSResponder *ref )
{ return [ref nextResponder]; }

// Presenting and customizing error info
BOOL ResponderPresentError( NSResponder *ref, NSError *err )
{ return [ref presentError:err]; }

void ResponderPresentErrorModalForWindow( NSResponder *ref, NSError *err, NSInteger wndTag, void *context )
{
    NSWindow *w = CUI_WindowWithTag( wndTag );
    if ( w ) {
        if ( !sCUI_ResponderTarget ) sCUI_ResponderTarget = [[CUI_ResponderTarget alloc] init];
        [ref presentError:err modalForWindow:w delegate:sCUI_ResponderTarget didPresentSelector:@selector(didPresentErrorWithRecovery:contextInfo:) contextInfo:context];
    }
}

// Managing a responder's menu
NSMenu *ResponderMenu( NSResponder *ref )
{ return [ref menu]; }

void ResponderSetMenu( NSResponder *ref, NSInteger menuID )
{
    NSMenu *menu = MenuAtIndex( menuID );
    if ( menu ) [ref setMenu:menu];
}

// Getting the undo manager
NSUndoManager *ResponderUndoManager( NSResponder *ref )
{ return [ref undoManager]; }

