/*
 SegmentedControl.h

 Bernie Wylde
 */


#import "CocoaUI.h"

//#import "Menu.h"
//#import "View.h"
//#import "Window.h"

@interface SegmentedControlSC : NSSegmentedControl
@end

@interface CUI_SegmentedControlTarget : NSObject
@end

// Class
Class SegmentedControlClass( void );

NSSegmentedControl *SegmentedControlWithTag( NSInteger tag );
BOOL SegmentedControlExists( NSInteger tag );

// Behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
NSSegmentSwitchTracking SegmentedControlTrackingMode( NSInteger tag );
void SegmentedControlSetTrackingMode( NSInteger tag, NSSegmentSwitchTracking mode );
#endif
NSSegmentStyle SegmentedControlStyle( NSInteger tag );
void SegmentedControlSetStyle( NSInteger tag, NSSegmentStyle style );

// Number of segments
NSInteger SegmentedControlSegmentCount( NSInteger tag );
void SegmentedControlSetSegmentCount( NSInteger tag, NSInteger count );

// Text
CFStringRef SegmentedControlLabelForSegment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentLabel( NSInteger tag, NSInteger index, CFStringRef string );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSTextAlignment SegmentedControlSegmentAlignment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentAlignment( NSInteger tag, NSInteger index, NSTextAlignment value );
#endif

// Image
NSImage *SegmentedControlImageForSegment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentImage( NSInteger tag, NSInteger index, NSImage *image );
NSImageScaling SegmentedControlImageScaling( NSInteger tag, NSInteger index );
void SegmentedControlSetImageScaling( NSInteger tag, NSInteger index, NSImageScaling scaling );

// Menu
NSMenu *SegmentedControlMenuForSegment( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentMenu( NSInteger tag, NSInteger index, NSMenu *menu );
void SegmentedControlSetSegmentMenuIndex( NSInteger tag, NSInteger index, NSInteger menuIndex );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
BOOL SegmentedControlSegmentShowsMenuIndicator( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentShowsMenuIndicator( NSInteger tag, NSInteger index, BOOL flag );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
BOOL SegmentedControlIsSpringLoaded( NSInteger tag );
void SegmentedControlSetSpringLoaded( NSInteger tag, BOOL flag );
#endif

// Selected segment
NSInteger SegmentedControlSelectedSegment( NSInteger tag );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSInteger SegmentedControlIndexOfSelectedItem( NSInteger tag );
#endif
BOOL SegmentedControlSelectSegmentWithTag( NSInteger ctrlTag, NSInteger segTag );
void SegmentedControlSetSegmentSelected( NSInteger tag, NSInteger index, BOOL flag );
BOOL SegmentedControlIsSegmentSelected( NSInteger tag, NSInteger index );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSColor *SegmentedControlSelectedSegmentBezelColor( NSInteger tag );
void SegmentedControlSetSelectedSegmentBezelColor( NSInteger tag, NSColor *col );
#endif
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101003
double SegmentedControlDoubleValueForSelectedSegment( NSInteger tag );
#endif

// Spacing
CGFloat SegmentedControlSegmentWidth( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentWidth( NSInteger tag, NSInteger index, CGFloat value );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
void SegmentedControlSetSegmentDistribution( NSInteger tag, NSSegmentDistribution value );
#endif

// Enable
void SegmentedControlSetSegmentEnabled( NSInteger tag, NSInteger index, BOOL flag );
BOOL SegmentedControlIsSegmentEnabled( NSInteger tag, NSInteger index );

// Tags and tooltips
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSInteger SegmentedControlSegmentTag( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentTag( NSInteger tag, NSInteger index, NSInteger segTag );
CFStringRef SegmentedControlSegmentToolTip( NSInteger tag, NSInteger index );
void SegmentedControlSetSegmentToolTip( NSInteger tag, NSInteger index, CFStringRef string );
#endif

// Convenience
void SegmentedControlSetSegment( NSInteger tag, NSInteger index, NSInteger enabled, CFStringRef label, NSImage *image, NSUInteger imageScaling, NSInteger width );



// Init
NSSegmentedControl *SegmentedControlInit( NSInteger inTag, CGRect inRect );

// Statement
void CUI_SegmentedControl( NSInteger inTag, NSInteger inEnabled, NSInteger inIndex, CGRect inRect, NSInteger inSegments, NSInteger inWndTag );

