/*
 Sound.m

 Bernie Wylde
 */

#import "Sound.h"

@implementation CUI_SoundDelegate
- (void)sound:(NSSound *)sound didFinishPlaying:(BOOL)flag {
    DialogEventSetBool(flag);
    CallUserDefinedOnDialogFunction( soundDidFinishPlaying, 0, 0, sound );
}
@end

// private
CUI_SoundDelegate *CUI_SoundDelegateObj( void )
{
    static CUI_SoundDelegate *sCUI_SoundDelegate = nil;
    if ( !sCUI_SoundDelegate ) sCUI_SoundDelegate = [[CUI_SoundDelegate alloc] init];
    return sCUI_SoundDelegate;
}

// Class
Class SoundClass( void )
{ return [NSSound class]; }

// -- create --
BOOL SoundCanInitWithPasteboard( NSPasteboard *pb )
{ return [NSSound canInitWithPasteboard:pb]; }

NSSound *SoundWithContentsOfURL( CFURLRef url, BOOL byReference )
{ return [[[NSSound alloc] initWithContentsOfURL:(__bridge NSURL *)url byReference:byReference] autorelease]; }

NSSound *SoundWithData( CFDataRef data )
{ return [[[NSSound alloc] initWithData:(__bridge NSData *)data] autorelease]; }

NSSound *SoundWithPasteboard( NSPasteboard *pb )
{ return [[[NSSound alloc] initWithPasteboard:pb] autorelease]; }

// -- configure --
CFStringRef SoundName( NSSound *ref )
{ return (__bridge CFStringRef)[ref name]; }

BOOL SoundSetName( NSSound *ref, CFStringRef name )
{ return [ref setName:(__bridge NSString *)name]; }

float SoundVolume( NSSound *ref )
{ return [ref volume]; }

void SoundSetVolume( NSSound *ref, float value )
{ return [ref setVolume:value]; }

CFTimeInterval SoundCurrentTime( NSSound *ref )
{ return [ref currentTime]; }

void SoundSetCurrentTime( NSSound *ref, CFTimeInterval ti )
{ [ref setCurrentTime:ti]; }

BOOL SoundLoops( NSSound *ref )
{ return [ref loops]; }

void SoundSetLoops( NSSound *ref, BOOL flag )
{ [ref setLoops:flag]; }

CFStringRef SoundPlaybackDeviceIdentifier( NSSound *ref )
{ return (__bridge CFStringRef)[ref playbackDeviceIdentifier]; }

void SoundSetPlaybackDeviceIdentifier( NSSound *ref, CFStringRef identifier )
{ [ref setPlaybackDeviceIdentifier:(__bridge NSString *)identifier]; }

// -- info --
CFArrayRef SoundUnfilteredTypes( void )
{ return (__bridge CFArrayRef)[NSSound soundUnfilteredTypes]; }

NSSound *SoundNamed( CFStringRef name )
{ return [NSSound soundNamed:(__bridge NSString *)name]; }

CFTimeInterval SoundDuration( NSSound *ref )
{ return [ref duration]; }

// -- play --
BOOL SoundIsPlaying( NSSound *ref )
{ return [ref isPlaying]; }

BOOL SoundPause( NSSound *ref )
{ return [ref pause]; }

BOOL SoundPlay( NSSound *ref )
{
    [ref setDelegate:CUI_SoundDelegateObj()];
    return [ref play];
}

BOOL SoundResume( NSSound *ref )
{ return [ref resume]; }

BOOL SoundStop( NSSound *ref )
{ return [ref stop]; }

// -- write --
void SoundWriteToPasteboard( NSSound *ref, NSPasteboard *pb )
{ [ref writeToPasteboard:pb]; }

