/*
 SplitViewItem.m

 Bernie Wylde
 */

#import "SplitViewItem.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

// Class
Class SplitViewItemClass( void )
{ return [NSSplitViewItem class]; }

// Create
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSSplitViewItem *SplitViewItemSidebarWithViewController( NSViewController *vc )
{
    if ( @available(macOS 10.11, *) ) {
        return [NSSplitViewItem sidebarWithViewController:vc];
    }
    return nil;
}

NSSplitViewItem *SplitViewItemContentListWithViewController( NSViewController *vc )
{
    if ( @available(macOS 10.11, *) ) {
        return [NSSplitViewItem contentListWithViewController:vc];
    }
    return nil;
}
#endif// 101100

NSSplitViewItem *SplitViewItemWithViewController( NSViewController *vc )
{ return [NSSplitViewItem splitViewItemWithViewController:vc]; }

// Thickness
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGFloat SplitViewItemAutomaticMaximumThickness( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref automaticMaximumThickness];
    }
    return 0.0;
}

void SplitViewItemSetAutomaticMaximumThickness( NSSplitViewItem *ref, CGFloat thickness )
{
    if ( @available(macOS 10.11, *) ) {
        [ref setAutomaticMaximumThickness:thickness];
    }
}

CGFloat SplitViewItemPreferredThicknessFraction( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref preferredThicknessFraction];
    }
    return 0.0;
}

void SplitViewItemSetPreferredThicknessFraction( NSSplitViewItem *ref, CGFloat fraction )
{
    if ( @available(macOS 10.11, *) ) {
        [ref setPreferredThicknessFraction:fraction];
    }
}

CGFloat SplitViewItemMinimumThickness( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref minimumThickness];
    }
    return 0.0;
}

void SplitViewItemSetMinimumThicknessFraction( NSSplitViewItem *ref, CGFloat thickness )
{
    if ( @available(macOS 10.11, *) ) {
        [ref setMinimumThickness:thickness];
    }
}

CGFloat SplitViewItemMaximumThickness( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref maximumThickness];
    }
    return 0.0;
}

void SplitViewItemSetMaximumThickness( NSSplitViewItem *ref, CGFloat thickness )
{
    if ( @available(macOS 10.11, *) ) {
        [ref setMaximumThickness:thickness];
    }
}
#endif// 101100

// Autolayout behaviors
NSLayoutPriority SplitViewItemHoldingPriority( NSSplitViewItem *ref )
{ return [ref holdingPriority]; }

void SplitViewItemSetHoldingPriority( NSSplitViewItem *ref, NSLayoutPriority priority )
{ [ref setHoldingPriority:priority]; }

// Behavior
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSSplitViewItemBehavior SplitViewItemBehavior( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref behavior];
    }
    return 0;
}
#endif// 101100

// Collapsing and expanding
BOOL SplitViewItemIsCollapsed( NSSplitViewItem *ref )
{ return [ref isCollapsed]; }

void SplitViewItemSetCollapsed( NSSplitViewItem *ref, BOOL flag )
{ [ref setCollapsed:flag]; }

BOOL SplitViewItemCanCollapse( NSSplitViewItem *ref )
{ return [ref canCollapse]; }

void SplitViewItemSetCanCollapse( NSSplitViewItem *ref, BOOL flag )
{ [ref setCanCollapse:flag]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSSplitViewItemCollapseBehavior SplitViewItemCollapseBehavior( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref collapseBehavior];
    }
    return 0;
}

void SplitViewItemSetCollapseBehavior( NSSplitViewItem *ref, NSSplitViewItemCollapseBehavior behavior )
{
    if ( @available(macOS 10.11, *) ) {
        [ref setCollapseBehavior:behavior];
    }
}

BOOL SplitViewItemIsSpringLoaded( NSSplitViewItem *ref )
{
    if ( @available(macOS 10.11, *) ) {
        return [ref isSpringLoaded];
    }
    return NO;
}

void SplitViewItemSetSpringLoaded( NSSplitViewItem *ref, BOOL flag )
{
    if ( @available(macOS 10.11, *) ) {
        [ref setSpringLoaded:flag];
    }
}
#endif// 101100

// Customizing appearance
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
BOOL SplitViewItemAllowsFullHeightLayout( NSSplitViewItem *ref )
{
    if ( @available(macOS 11.0, *) ) {
        return [ref allowsFullHeightLayout];
    }
    return NO;
}

void SplitViewItemSetAllowsFullHeightLayout( NSSplitViewItem *ref, BOOL flag )
{
    if ( @available(macOS 11.0, *) ) {
        [ref setAllowsFullHeightLayout:flag];
    }
}

NSTitlebarSeparatorStyle SplitViewItemTitlebarSeparatorStyle( NSSplitViewItem *ref )
{
    if ( @available(macOS 11.0, *) ) {
        return [ref titlebarSeparatorStyle];
    }
    return 0;
}

void SplitViewItemSetTitlebarSeparatorStyle( NSSplitViewItem *ref, NSTitlebarSeparatorStyle style )
{
    if ( @available(macOS 11.0, *) ) {
        [ref setTitlebarSeparatorStyle:style];
    }
}
#endif// 101600

// View controller
NSViewController *SplitViewItemViewController( NSSplitViewItem *ref )
{ return [ref viewController]; }

#endif// 101000

