/*
 Task.m

 Bernie Wylde
 */

#import "Task.h"

// Class
Class TaskClass( void )
{ return [NSTask class]; }

// - init -
NSTask *TaskInit( void )
{ return [[[NSTask alloc] init] autorelease]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
NSTask *TaskLaunchedTaskWithExecutableURL( CFURLRef url, CFArrayRef arguments, NSError **err, CUI_TaskTerminationHandlerType handlerFn )
{
    if (@available(macOS 10.13, *)) {
        return [NSTask launchedTaskWithExecutableURL:(__bridge NSURL *)url arguments:(__bridge NSArray *)arguments error:err terminationHandler:^(NSTask *task) {
            if ( handlerFn ) (*handlerFn)(task);
        }];
    }
    return nil;
}
#endif

// - info + configure-
CFArrayRef TaskArguments( NSTask *ref )
{ return (__bridge CFArrayRef)[ref arguments]; }

void TaskSetArguments( NSTask *ref, CFArrayRef arguments )
{ [ref setArguments:(__bridge NSArray *)arguments]; }

CFURLRef TaskCurrentDirectoryURL( NSTask *ref )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
    if (@available(macOS 10.13, *)) {
        return (__bridge CFURLRef)[ref currentDirectoryURL];
    } else {
        return (__bridge CFURLRef)[NSURL fileURLWithPath:[ref currentDirectoryPath]];
    }
#else
    return (__bridge CFURLRef)[NSURL fileURLWithPath:[ref currentDirectoryPath]];
#endif
}

void TaskSetCurrentDirectoryURL( NSTask *ref, CFURLRef url )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
    if (@available(macOS 10.13, *)) {
        [ref setCurrentDirectoryURL:(__bridge NSURL *)url];
    } else {
        [ref setCurrentDirectoryPath:[(__bridge NSURL *)url path]];
    }
#else
    [ref setCurrentDirectoryPath:[(__bridge NSURL *)url path]];
#endif
}

CFURLRef TaskExecutableURL( NSTask *ref )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
    if (@available(macOS 10.13, *)) {
        return (__bridge CFURLRef)[ref executableURL];
    } else {
        return (__bridge CFURLRef)[NSURL fileURLWithPath:[ref launchPath]];
    }
#else
    return (__bridge CFURLRef)[NSURL fileURLWithPath:[ref launchPath]];
#endif
}

void TaskSetExecutableURL( NSTask *ref, CFURLRef url )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
    if (@available(macOS 10.13, *)) {
        [ref setExecutableURL:(__bridge NSURL *)url];
    } else {
        [ref setLaunchPath:[(__bridge NSURL *)url path]];
    }
#else
    [ref setLaunchPath:[(__bridge NSURL *)url path]];
#endif
}

CFDictionaryRef TaskEnvironment( NSTask *ref )
{ return (__bridge CFDictionaryRef)[ref environment]; }

void TaskSetEnvironment( NSTask *ref, CFDictionaryRef environment )
{ [ref setEnvironment:(__bridge NSDictionary *)environment]; }

int TaskProcessIdentifier( NSTask *ref )
{ return [ref processIdentifier]; }

void *TaskStandardError( NSTask *ref )
{ return (void *)[ref standardError]; }

void TaskSetStandardError( NSTask *ref, void *obj )
{ [ref setStandardError:(id)obj]; }

void *TaskStandardInput( NSTask *ref )
{ return (void *)[ref standardInput]; }

void TaskSetStandardInput( NSTask *ref, void *obj )
{ [ref setStandardInput:(id)obj]; }

void *TaskStandardOutput( NSTask *ref )
{ return (void *)[ref standardOutput]; }

void TaskSetStandardOutput( NSTask *ref, void *obj )
{ [ref setStandardOutput:(id)obj]; }

// - running and stopping -
void TaskInterrupt( NSTask *ref )
{ [ref interrupt]; }

BOOL TaskLaunch( NSTask *ref, NSError **err )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
    if (@available(macOS 10.13, *)) {
        return [ref launchAndReturnError:err];
    } else {
        [ref launch];
    }
#else
    [ref launch];
#endif
    return YES;
}

BOOL TaskResume( NSTask *ref )
{ return [ref resume]; }

BOOL TaskSuspend( NSTask *ref )
{ return [ref suspend]; }

void TaskTerminate( NSTask *ref )
{ [ref terminate]; }

void TaskWaitUntilExit( NSTask *ref )
{ [ref waitUntilExit]; }

// - query state -
BOOL TaskIsRunning( NSTask *ref )
{ return [ref isRunning]; }

int TaskTerminationStatus( NSTask *ref )
{ return [ref terminationStatus]; }

NSTaskTerminationReason TaskTerminationReason( NSTask *ref )
{ return [ref terminationReason]; }

// - termination handler -
void TaskSetTerminationHandler( NSTask *ref, CUI_TaskTerminationHandlerType handlerFn )
{
    [ref setTerminationHandler:^(NSTask *task) {
        if ( handlerFn ) (*handlerFn)(task);
    }];
}

