/*
 TextContainer.m

 Bernie Wylde
 */

#import "TextContainer.h"

// Class
Class TextContainerClass( void )
{ return [NSTextContainer class]; }

// - create -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSTextContainer *TextContainerWithSize( CGSize size )
{ return [[[NSTextContainer alloc] initWithSize:NSSizeFromCGSize(size)] autorelease]; }
#endif

// - components -
NSLayoutManager *TextContainerLayoutManager( NSTextContainer *ref )
{ return [ref layoutManager]; }

void TextContainerSetLayoutManager( NSTextContainer *ref, NSLayoutManager *lm )
{ [ref setLayoutManager:lm]; }

void TextContainerReplaceLayoutManager( NSTextContainer *ref, NSLayoutManager *lm )
{ [ref replaceLayoutManager:lm]; }

NSInteger TextContainerTextView( NSTextContainer *ref )
{
    NSTextView *view = [ref textView];
    if ( view ) return [view tag];
    return 0;
}

// - shape -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGSize TextContainerSize( NSTextContainer *ref )
{ return NSSizeToCGSize([ref size]); }

void TextContainerSetSize( NSTextContainer *ref, CGSize size )
{ [ref setSize:NSSizeFromCGSize(size)]; }

CFArrayRef TextContainerExclusionPaths( NSTextContainer *ref )
{ return (__bridge CFArrayRef)[ref exclusionPaths]; }

void TextContainerSetExclusionPaths( NSTextContainer *ref, CFArrayRef paths )
{ [ref setExclusionPaths:(__bridge NSArray *)paths]; }

NSLineBreakMode TextContainerLineBreakMode( NSTextContainer *ref )
{ return [ref lineBreakMode]; }

void TextContainerSetLineBreakMode( NSTextContainer *ref, NSLineBreakMode lineBreakMode )
{ [ref setLineBreakMode:lineBreakMode]; }
#endif

BOOL TextContainerWidthTracksTextView( NSTextContainer *ref )
{ return [ref widthTracksTextView]; }

void TextContainerSetWidthTracksTextView( NSTextContainer *ref, BOOL flag )
{ [ref setWidthTracksTextView:flag]; }

BOOL TextContainerHeightTracksTextView( NSTextContainer *ref )
{ return [ref heightTracksTextView]; }

void TextContainerSetHeightTracksTextView( NSTextContainer *ref, BOOL flag )
{ [ref setHeightTracksTextView:flag]; }

// - constrain -
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
NSUInteger TextContainerMaximumNumberOfLines( NSTextContainer *ref )
{ return [ref maximumNumberOfLines]; }

void TextContainerSetMaximumNumberOfLines( NSTextContainer *ref, NSUInteger lines )
{ return [ref setMaximumNumberOfLines:lines]; }
#endif

CGFloat TextContainerLineFragmentPadding( NSTextContainer *ref )
{ return [ref lineFragmentPadding]; }

void TextContainerSetLineFragmentPadding( NSTextContainer *ref, CGFloat padding )
{ return [ref setLineFragmentPadding:padding]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CGRect TextContainerLineFragmentRectForProposedRect( NSTextContainer *ref, CGRect proposedRect, NSUInteger index, NSWritingDirection direction, CGRect *remainingRect )
{ return NSRectToCGRect([ref lineFragmentRectForProposedRect:NSRectFromCGRect(proposedRect) atIndex:index writingDirection:direction remainingRect:(NSRect *)remainingRect] );
}
#endif

BOOL TextContainerIsSimpleRectangularTextContainer( NSTextContainer *ref )
{ return [ref isSimpleRectangularTextContainer]; }
