/*
 Thread.h

 Bernie Wylde
 */


#import "CocoaUI.h"

typedef void (*CUI_ThreadCallbackType)(id);

@interface CUI_ThreadTarget : NSObject
@property (assign) CUI_ThreadCallbackType callback;
- (void)doThread:(id)object;
@end

// Class
Class ThreadClass( void );

// -- start --
void ThreadStart( NSThread *ref );

// -- stop --
void ThreadSleepUntilDate( CFDateRef date );
void ThreadSleepForTimeInterval( CFTimeInterval ti );
void ThreadExit( void );
void ThreadCancel( NSThread *ref );

// -- execution state --
BOOL ThreadIsExecuting( NSThread *ref );
BOOL ThreadIsFinished( NSThread *ref );
BOOL ThreadIsCancelled( NSThread *ref );

// Main thread
BOOL ThreadIsMainThread( NSThread *ref );
NSThread *ThreadMainThread( void );

// Query
BOOL ThreadIsMultiThreaded( void );
NSThread *ThreadCurrentThread( void );
CFArrayRef ThreadCallStackReturnAddresses( void );
CFArrayRef ThreadCallStackSymbols( void );

// -- properties --
CFMutableDictionaryRef ThreadDictionary( NSThread *ref );
CFStringRef ThreadName( NSThread *ref );
void ThreadSetName( NSThread *ref, CFStringRef name );
NSUInteger ThreadStackSize( NSThread *ref );

// -- prioritizing --
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSQualityOfService ThreadQualityOfService( NSThread *ref );
void ThreadSetQualityOfService( NSThread *ref, NSQualityOfService value );
#endif
double ThreadPriority( NSThread *ref );
void ThreadSetPriority( NSThread *ref, double priority );

// -- custom --
void ThreadDetachNewThreadFunction( void *fnAddress, CFTypeRef object );
NSThread *ThreadWithFunction( void *fnAddress, CFTypeRef object );

