/*
 URLRequest.h

 Bernie Wylde
 */


#import "CocoaUI.h"

// Class
Class URLRequestClass( void );

// Create
NSURLRequest *URLRequestWithURL( CFURLRef url );
NSURLRequest *URLRequestWithURLCachePolicy( CFURLRef url, NSURLRequestCachePolicy policy, CFTimeInterval timeoutInterval );

// Cache policy
NSURLRequestCachePolicy URLRequestCachePolicy( NSURLRequest *ref );

// Components
CFStringRef URLRequestHTTPMethod( NSURLRequest *ref );
CFURLRef URLRequestURL( NSURLRequest *ref );
CFDataRef URLRequestHTTPBody( NSURLRequest *ref );
NSInputStream *URLRequestHTTPBodyStream( NSURLRequest *ref );
CFURLRef URLRequestMainDocumentURL( NSURLRequest *ref );

// Header fields
CFDictionaryRef URLRequestAllHTTPHeaderFields( NSURLRequest *ref );
CFStringRef URLRequestValueForHTTPHeaderField( NSURLRequest *ref, CFStringRef headerField );

// Behavior
CFTimeInterval URLRequestTimeoutInterval( NSURLRequest *ref );
BOOL URLRequestHTTPShouldHandleCookies( NSURLRequest *ref );
BOOL URLRequestHTTPShouldUsePipelining( NSURLRequest *ref );
BOOL URLRequestAllowsCellularAccess( NSURLRequest *ref );

// Limited nodes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
BOOL URLRequestAllowsConstrainedNetworkAccess( NSURLRequest *ref );
BOOL URLRequestAllowsExpensiveNetworkAccess( NSURLRequest *ref );
#endif// 101500

// Service type
NSURLRequestNetworkServiceType URLRequestNetworkServiceType( NSURLRequest *ref );

// Secure coding
BOOL URLRequestSupportsSecureCoding( void );

// Source of request
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
NSURLRequestAttribution URLRequestAttribution( NSURLRequest *ref );
#endif// 120000

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300
BOOL URLRequestAssumesHTTP3Capable( NSURLRequest *ref );
#endif

#pragma mark - MutableURLRequest
// Cache policy
void MutableURLRequestSetCachePolicy( NSMutableURLRequest *ref, NSURLRequestCachePolicy cachePolicy );

// Components
void MutableURLRequestSetHTTPMethod( NSMutableURLRequest *ref, CFStringRef method );
void MutableURLRequestSetURL( NSMutableURLRequest *ref, CFURLRef url );
void MutableURLRequestSetHTTPBody( NSMutableURLRequest *ref, CFDataRef body );
void MutableURLRequestSetHTTPBodyStream( NSMutableURLRequest *ref, NSInputStream *stream );
void MutableURLRequestSetMainDocumentURL( NSMutableURLRequest *ref, CFURLRef docURL );

// Header fields
void MutableURLRequestSetAllHTTPHeaderFields( NSMutableURLRequest *ref, CFDictionaryRef fields );
void MutableURLRequestAddValueForHTTPHeaderField( NSMutableURLRequest *ref, CFStringRef value, CFStringRef headerField );
void MutableURLRequestSetValueForHTTPHeaderField( NSMutableURLRequest *ref, CFStringRef value, CFStringRef headerField );

// Behavior
void MutableURLRequestSetTimeoutInterval( NSMutableURLRequest *ref, CFTimeInterval interval );
void MutableURLRequestSetHTTPShouldHandleCookies( NSMutableURLRequest *ref, BOOL flag );
void MutableURLRequestSetHTTPShouldUsePipelining( NSMutableURLRequest *ref, BOOL flag );
void MutableURLRequestSetAllowsCellularAccess( NSMutableURLRequest *ref, BOOL flag );

// Limited modes
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
void MutableURLRequestSetAllowsConstrainedNetworkAccess( NSMutableURLRequest *ref, BOOL flag );
void MutableURLRequestSetAllowsExpensiveNetworkAccess( NSMutableURLRequest *ref, BOOL flag );
#endif// 101500

// Service type
void MutableURLRequestSetNetworkServiceType( NSMutableURLRequest *ref, NSURLRequestNetworkServiceType type );

// Source of request
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
void MutableURLRequestSetAttribution( NSMutableURLRequest *ref, NSURLRequestAttribution attribution );
#endif// 120000
