/*
 Unit.h

 Bernie Wylde
 */


#import "CocoaUI.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101200

#pragma mark - Unit
// Class
Class UnitClass( void );

// Create
NSUnit *UnitWithSymbol( CFStringRef symbol );

// Properties
CFStringRef UnitSymbol( NSUnit *ref );


#pragma mark - UnitConverter
// Class
Class UnitConverterClass( void );

double UnitConverterBaseUnitValueFromValue( NSUnitConverter *ref, double value );
double UnitConverterValueFromBaseUnitValue( NSUnitConverter *ref, double value );



#pragma mark - UnitConverterLinear
// Class
Class UnitConverterLinearClass( void );

// Create
NSUnitConverterLinear *UnitConverterLinearWithCoefficient( double coefficient );
NSUnitConverterLinear *UnitConverterLinearWithCoefficientAndConstant( double coefficient, double constant );

// Linear parmaeters
double UnitConverterLinearCoefficient( NSUnitConverterLinear *ref );
double UnitConverterLinearConstant( NSUnitConverterLinear *ref );

#pragma mark - Dimension
// Class
Class DimensionClass( void );

// Create
NSDimension *DimensionWithSymbol( CFStringRef symbol, NSUnitConverter *converter );

// Unit converter
NSUnitConverter *DimensionConverter( NSDimension *ref );

// Base unit
NSUnit *DimensionBaseUnit( void );


#pragma mark - UnitAcceleration
// Class
Class UnitAccelerationClass( void );

// Base unit
NSUnitAcceleration *UnitAccelerationBaseUnit( void );

// Predefined units
NSUnitAcceleration *UnitAccelerationMetersPerSecondSquared( void );
NSUnitAcceleration *UnitAccelerationGravity( void );


#pragma mark - UnitAngle
// Class
Class UnitAngleClass( void );

// Base unit
NSUnitAngle *UnitAngleBaseUnit( void );

// Predefined units
NSUnitAngle *UnitAngleDegrees( void );
NSUnitAngle *UnitAngleArcMinutes( void );
NSUnitAngle *UnitAngleArcSeconds( void );
NSUnitAngle *UnitAngleRadians( void );
NSUnitAngle *UnitAngleGradians( void );
NSUnitAngle *UnitAngleRevolutions( void );


#pragma mark - UnitArea
// Class
Class UnitAreaClass( void );

// Base unit
NSUnitArea *UnitAreaBaseUnit( void );

// Predefined units
NSUnitArea *UnitAreaSquareMegameters( void );
NSUnitArea *UnitAreaSquareKilometers( void );
NSUnitArea *UnitAreaSquareMeters( void );
NSUnitArea *UnitAreaSquareCentimeters( void );
NSUnitArea *UnitAreaSquareMillimeters( void );
NSUnitArea *UnitAreaSquareMicrometers( void );
NSUnitArea *UnitAreaSquareNanometers( void );
NSUnitArea *UnitAreaSquareInches( void );
NSUnitArea *UnitAreaSquareFeet( void );
NSUnitArea *UnitAreaSquareYards( void );
NSUnitArea *UnitAreaSquareMiles( void );
NSUnitArea *UnitAreaAcres( void );
NSUnitArea *UnitAreaAres( void );
NSUnitArea *UnitAreaHectares( void );


#pragma mark - UnitConcentrationMass
// Class
Class UnitConcentrationMassClass( void );

// Base unit
NSUnitConcentrationMass *UnitConcentrationMassBaseUnit( void );

// Predefined units
NSUnitConcentrationMass *UnitConcentrationMassGramsPerLiter( void );
NSUnitConcentrationMass *UnitConcentrationMassMilligramsPerDeciliter( void );
NSUnitConcentrationMass *UnitConcentrationMassMillimolesPerLiterWithGramsPerMole( double gramsPerMole );


#pragma mark - UnitDispersion
// Class
Class UnitDispersionClass( void );

// Base unit
NSUnitDispersion *UnitDispersionBaseUnit( void );

// Predefined units
NSUnitDispersion *UnitDispersionPartsPerMillion( void );


#pragma mark - UnitDuration
// Class
Class UnitDurationClass( void );

// Base unit
NSUnitDuration *UnitDurationBaseUnit( void );

// Predefined units
NSUnitDuration *UnitDurationSeconds( void );
NSUnitDuration *UnitDurationMinutes( void );
NSUnitDuration *UnitDurationHours( void );

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
NSUnitDuration *UnitDurationMicroseconds( void );
NSUnitDuration *UnitDurationMilliseconds( void );
NSUnitDuration *UnitDurationNanoseconds( void );
NSUnitDuration *UnitDurationPicoseconds( void );
#endif// 101500


#pragma mark - UnitElectricCharge
// Class
Class UnitElectricChargeClass( void );

// Base unit
NSUnitElectricCharge *UnitElectricChargeBaseUnit( void );

// Predefined units
NSUnitElectricCharge *UnitElectricChargeCoulombs( void );
NSUnitElectricCharge *UnitElectricChargeMegaampereHours( void );
NSUnitElectricCharge *UnitElectricChargeKiloampereHours( void );
NSUnitElectricCharge *UnitElectricChargeAmpereHours( void );
NSUnitElectricCharge *UnitElectricChargeMilliampereHours( void );
NSUnitElectricCharge *UnitElectricChargeMicroampereHours( void );


#pragma mark - UnitElectricCurrent
// Class
Class UnitElectricCurrentClass( void );

// Base unit
NSUnitElectricCurrent *UnitElectricCurrentBaseUnit( void );

// Predefined units
NSUnitElectricCurrent *UnitElectricCurrentMegaamperes( void );
NSUnitElectricCurrent *UnitElectricCurrentKiloamperes( void );
NSUnitElectricCurrent *UnitElectricCurrentAmperes( void );
NSUnitElectricCurrent *UnitElectricCurrentMilliamperes( void );
NSUnitElectricCurrent *UnitElectricCurrentMicroamperes( void );


#pragma mark - UnitPotentialDifference
// Class
Class UnitElectricPotentialDifferenceClass( void );

// Base unit
NSUnitElectricPotentialDifference *UnitElectricPotentialDifferenceBaseUnit( void );

// Predefined units
NSUnitElectricPotentialDifference *UnitElectricPotentialDifferenceMegavolts( void );
NSUnitElectricPotentialDifference *UnitElectricPotentialDifferenceKilovolts( void );
NSUnitElectricPotentialDifference *UnitElectricPotentialDifferenceVolts( void );
NSUnitElectricPotentialDifference *UnitElectricPotentialDifferenceMillivolts( void );
NSUnitElectricPotentialDifference *UnitElectricPotentialDifferenceMicrovolts( void );


#pragma mark - UnitElectricResistance
// Class
Class UnitElectricResistanceClass( void );

// Base unit
NSUnitElectricResistance *UnitElectricResistanceBaseUnit( void );

// Predefined units
NSUnitElectricResistance *UnitElectricResistanceMegaohms( void );
NSUnitElectricResistance *UnitElectricResistanceKiloohms( void );
NSUnitElectricResistance *UnitElectricResistanceOhms( void );
NSUnitElectricResistance *UnitElectricResistanceMilliohms( void );
NSUnitElectricResistance *UnitElectricResistanceMicroohms( void );


#pragma mark - UnitEnergy
// Class
Class UnitEnergyClass( void );

// Base unit
NSUnitEnergy *UnitEnergyBaseUnit( void );

// Predefined units
NSUnitEnergy *UnitEnergyKilojoules( void );
NSUnitEnergy *UnitEnergyJoules( void );
NSUnitEnergy *UnitEnergyKilocalories( void );
NSUnitEnergy *UnitEnergyCalories( void );
NSUnitEnergy *UnitEnergyKilowattHours( void );


#pragma mark - UnitFrequency
// Class
Class UnitFrequencyClass( void );

// Base unit
NSUnitFrequency *UnitFrequencyBaseUnit( void );

// Predefined units
NSUnitFrequency *UnitFrequencyTerahertz( void );
NSUnitFrequency *UnitFrequencyGigahertz( void );
NSUnitFrequency *UnitFrequencyMegahertz( void );
NSUnitFrequency *UnitFrequencyKilohertz( void );
NSUnitFrequency *UnitFrequencyHertz( void );
NSUnitFrequency *UnitFrequencyMillihertz( void );
NSUnitFrequency *UnitFrequencyMicrohertz( void );
NSUnitFrequency *UnitFrequencyNanohertz( void );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
NSUnitFrequency *UnitFrequencyFramesPerSecond( void );
#endif// 101500


#pragma mark - UnitFuelEfficiency
// Class
Class UnitFuelEfficiencyClass( void );

// Base unit
NSUnitFuelEfficiency *UnitFuelEfficiencyBaseUnit( void );

// Predefined units
NSUnitFuelEfficiency *UnitFuelEfficiencyMilesPerImperialGallon( void );
NSUnitFuelEfficiency *UnitFuelEfficiencyLitersPer100Kilometers( void );
NSUnitFuelEfficiency *UnitFuelEfficiencyMilesPerGallon( void );


#pragma mark - UnitInformationStorage
// Class
Class UnitInformationStorageClass( void );

// Base unit
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
NSUnitInformationStorage *UnitInformationStorageBaseUnit( void );

// Predefined units
NSUnitInformationStorage *UnitInformationStorageBits( void );
NSUnitInformationStorage *UnitInformationStorageBytes( void );
NSUnitInformationStorage *UnitInformationStorageExabits( void );
NSUnitInformationStorage *UnitInformationStorageExabytes( void );
NSUnitInformationStorage *UnitInformationStorageExbibits( void );
NSUnitInformationStorage *UnitInformationStorageExbibytes( void );
NSUnitInformationStorage *UnitInformationStorageGibibits( void );
NSUnitInformationStorage *UnitInformationStorageGibibytes( void );
NSUnitInformationStorage *UnitInformationStorageGigabits( void );
NSUnitInformationStorage *UnitInformationStorageGigabytes( void );
NSUnitInformationStorage *UnitInformationStorageKibibits( void );
NSUnitInformationStorage *UnitInformationStorageKibibytes( void );
NSUnitInformationStorage *UnitInformationStorageKilobits( void );
NSUnitInformationStorage *UnitInformationStorageKilobytes( void );
NSUnitInformationStorage *UnitInformationStorageMebibits( void );
NSUnitInformationStorage *UnitInformationStorageMebibytes( void );
NSUnitInformationStorage *UnitInformationStorageMegabits( void );
NSUnitInformationStorage *UnitInformationStorageMegabytes( void );
NSUnitInformationStorage *UnitInformationStorageNibbles( void );
NSUnitInformationStorage *UnitInformationStoragePebibits( void );
NSUnitInformationStorage *UnitInformationStoragePebibytes( void );
NSUnitInformationStorage *UnitInformationStoragePetabits( void );
NSUnitInformationStorage *UnitInformationStoragePetabytes( void );
NSUnitInformationStorage *UnitInformationStorageTebibits( void );
NSUnitInformationStorage *UnitInformationStorageTebibytes( void );
NSUnitInformationStorage *UnitInformationStorageTerabits( void );
NSUnitInformationStorage *UnitInformationStorageTerabytes( void );
NSUnitInformationStorage *UnitInformationStorageYobibits( void );
NSUnitInformationStorage *UnitInformationStorageYobibytes( void );
NSUnitInformationStorage *UnitInformationStorageYottabits( void );
NSUnitInformationStorage *UnitInformationStorageYottabytes( void );
NSUnitInformationStorage *UnitInformationStorageZebibits( void );
NSUnitInformationStorage *UnitInformationStorageZebibytes( void );
NSUnitInformationStorage *UnitInformationStorageZettabits( void );
NSUnitInformationStorage *UnitInformationStorageZettabytes( void );
#endif// 101500


#pragma mark - UnitLength
// Class
Class UnitLengthClass( void );

// Base unit
NSUnitLength *UnitLengthBaseUnit( void );

// Predefined units
NSUnitLength *UnitLengthMegameters( void );
NSUnitLength *UnitLengthKilometers( void );
NSUnitLength *UnitLengthHectometers( void );
NSUnitLength *UnitLengthDecameters( void );
NSUnitLength *UnitLengthMeters( void );
NSUnitLength *UnitLengthDecimeters( void );
NSUnitLength *UnitLengthCentimeters( void );
NSUnitLength *UnitLengthMillimeters( void );
NSUnitLength *UnitLengthMicrometers( void );
NSUnitLength *UnitLengthNanometers( void );
NSUnitLength *UnitLengthPicometers( void );
NSUnitLength *UnitLengthInches( void );
NSUnitLength *UnitLengthFeet( void );
NSUnitLength *UnitLengthYards( void );
NSUnitLength *UnitLengthMiles( void );
NSUnitLength *UnitLengthScandinavianMiles( void );
NSUnitLength *UnitLengthLightyears( void );
NSUnitLength *UnitLengthNauticalMiles( void );
NSUnitLength *UnitLengthFathoms( void );
NSUnitLength *UnitLengthFurlongs( void );
NSUnitLength *UnitLengthAstronomicalUnits( void );
NSUnitLength *UnitLengthParsecs( void );


#pragma mark - UnitIlluminance
// Class
Class UnitIlluminanceClass( void );

// Base unit
NSUnitIlluminance *UnitIlluminanceBaseUnit( void );

// Predefined units
NSUnitIlluminance *UnitIlluminanceLux( void );


#pragma mark - UnitMass
// Class
Class UnitMassClass( void );

// Base unit
NSUnitMass *UnitMassBaseUnit( void );

// Predefined units
NSUnitMass *UnitMassKilograms( void );
NSUnitMass *UnitMassGrams( void );
NSUnitMass *UnitMassDecigrams( void );
NSUnitMass *UnitMassCentigrams( void );
NSUnitMass *UnitMassMilligrams( void );
NSUnitMass *UnitMassMicrograms( void );
NSUnitMass *UnitMassNanograms( void );
NSUnitMass *UnitMassPicograms( void );
NSUnitMass *UnitMassOunces( void );
NSUnitMass *UnitMassPounds( void );
NSUnitMass *UnitMassStones( void );
NSUnitMass *UnitMassMetricTons( void );
NSUnitMass *UnitMassShortTons( void );
NSUnitMass *UnitMassCarats( void );
NSUnitMass *UnitMassOuncesTroy( void );
NSUnitMass *UnitMassSlugs( void );

// Type properties
NSUnitMass *UnitMassPoundsMass( void );


#pragma mark - UnitPower
// Class
Class UnitPowerClass( void );

// Base unit
NSUnitPower *UnitPowerBaseUnit( void );

// Predefined units
NSUnitPower *UnitPowerTerawatts( void );
NSUnitPower *UnitPowerGigawatts( void );
NSUnitPower *UnitPowerMegawatts( void );
NSUnitPower *UnitPowerKilowatts( void );
NSUnitPower *UnitPowerWatts( void );
NSUnitPower *UnitPowerMilliwatts( void );
NSUnitPower *UnitPowerMicrowatts( void );
NSUnitPower *UnitPowerNanowatts( void );
NSUnitPower *UnitPowerPicowatts( void );
NSUnitPower *UnitPowerFemtowatts( void );
NSUnitPower *UnitPowerHorsepower( void );


#pragma mark - UnitPressure
// Class
Class UnitPressureClass( void );

// Base unit
NSUnitPressure *UnitPressureBaseUnit( void );

// Predefined units
NSUnitPressure *UnitPressureNewtonsPerMeterSquared( void );
NSUnitPressure *UnitPressureGigapascals( void );
NSUnitPressure *UnitPressureMegapascals( void );
NSUnitPressure *UnitPressureKilopascals( void );
NSUnitPressure *UnitPressureHectopascals( void );
NSUnitPressure *UnitPressureInchesOfMercury( void );
NSUnitPressure *UnitPressureBars( void );
NSUnitPressure *UnitPressureMillibars( void );
NSUnitPressure *UnitPressureMillimetersOfMercury( void );
NSUnitPressure *UnitPressurePoundsPerSquareInch( void );

// Type properties
NSUnitPressure *UnitPressureNewtonsPerMetersSquared( void );
NSUnitPressure *UnitPressurePoundsForcePerSquareInch( void );


#pragma mark - UnitSpeed
// Class
Class UnitSpeedClass( void );

// Base unit
NSUnitSpeed *UnitSpeedBaseUnit( void );

// Predefined units
NSUnitSpeed *UnitSpeedMetersPerSecond( void );
NSUnitSpeed *UnitSpeedKilometersPerHour( void );
NSUnitSpeed *UnitSpeedMilesPerHour( void );
NSUnitSpeed *UnitSpeedKnots( void );


#pragma mark - UnitTemperature
// Class
Class UnitTemperatureClass( void );

// Base unit
NSUnitTemperature *UnitTemperatureBaseUnit( void );

// Predefined units
NSUnitTemperature *UnitTemperatureKelvin( void );
NSUnitTemperature *UnitTemperatureCelsius( void );
NSUnitTemperature *UnitTemperatureFahrenheit( void );


#pragma mark - UnitVolume
// Class
Class UnitVolumeClass( void );

// Base unit
NSUnitVolume *UnitVolumeBaseUnit( void );

// Predefined units
NSUnitVolume *UnitVolumeMegaliters( void );
NSUnitVolume *UnitVolumeKiloliters( void );
NSUnitVolume *UnitVolumeLiters( void );
NSUnitVolume *UnitVolumeDeciliters( void );
NSUnitVolume *UnitVolumeCentiliters( void );
NSUnitVolume *UnitVolumeMilliliters( void );
NSUnitVolume *UnitVolumeCubicKilometers( void );
NSUnitVolume *UnitVolumeCubicMeters( void );
NSUnitVolume *UnitVolumeCubicDecimeters( void );
NSUnitVolume *UnitVolumeCubicCentimeters( void );
NSUnitVolume *UnitVolumeCubicMillimeters( void );
NSUnitVolume *UnitVolumeCubicInches( void );
NSUnitVolume *UnitVolumeCubicFeet( void );
NSUnitVolume *UnitVolumeCubicYards( void );
NSUnitVolume *UnitVolumeCubicMiles( void );
NSUnitVolume *UnitVolumeAcreFeet( void );
NSUnitVolume *UnitVolumeBushels( void );
NSUnitVolume *UnitVolumeTeaspoons( void );
NSUnitVolume *UnitVolumeTablespoons( void );
NSUnitVolume *UnitVolumeFluidOunces( void );
NSUnitVolume *UnitVolumeCups( void );
NSUnitVolume *UnitVolumePints( void );
NSUnitVolume *UnitVolumeQuarts( void );
NSUnitVolume *UnitVolumeGallons( void );
NSUnitVolume *UnitVolumeImperialTeaspoons( void );
NSUnitVolume *UnitVolumeImperialTablespoons( void );
NSUnitVolume *UnitVolumeImperialFluidOunces( void );
NSUnitVolume *UnitVolumeImperialPints( void );
NSUnitVolume *UnitVolumeImperialQuarts( void );
NSUnitVolume *UnitVolumeImperialGallons( void );
NSUnitVolume *UnitVolumeMetricCups( void );


#endif// 101200
