/*
 UserActivity.h

 Bernie Wylde
 */


#import "CocoaUI.h"

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000

typedef void (*CUI_UserActivityContinuationStreamsCompletionHandlerType)(NSUserActivity*,NSInputStream*,NSOutputStream*,NSError*,void*);

// Delegate
typedef void (*CUI_UserActivityDelegateCallbackType)(NSUserActivity*,NSInteger,void*);

@interface CUI_UserActivityDelegate : NSObject <NSUserActivityDelegate>
@property (assign) CUI_UserActivityDelegateCallbackType callback;
@property (assign) void *userData;
@property (nonatomic,retain) NSInputStream *inputStream;
@property (nonatomic,retain) NSOutputStream *outputStream;
@end

enum {
    userActivityDidReceiveInputOutputStream,
    userActivityWasContinued,
    userActivityWillSave
};

NSInputStream *UserActivityDelegateInputStream( NSUserActivity *ref );
NSOutputStream *UserActivityDelegateOutputStream( NSUserActivity *ref );

// Class
Class UserActivityClass( void );

// Create
NSUserActivity *UserActivityWithType( CFStringRef type );

// Info
CFStringRef UserActivityType( NSUserActivity *ref );
CFStringRef UserActivityTitle( NSUserActivity *ref );
void UserActivitySetTitle( NSUserActivity *ref, CFStringRef title );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFSetRef UserActivityRequiredUserInfoKeys( NSUserActivity *ref );
void UserActivitySetRequiredUserInfoKeys( NSUserActivity *ref, CFSetRef keys );
#endif

CFDictionaryRef UserActivityUserInfo( NSUserActivity *ref );
void UserActivitySetUserInfo( NSUserActivity *ref, CFDictionaryRef info );
void UserActivityAddUserInfoEntriesFromDictionary( NSUserActivity *ref, CFDictionaryRef otherDictionary );
BOOL UserActivityNeedsSave( NSUserActivity *ref );
void UserActivitySetNeedsSave( NSUserActivity *ref, BOOL flag );
//#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
//CSSearchableItemAttributeSetRef UserActivityContentAttributeSet( NSUserActivity *ref );
//void UserActivitySetContentAttributeSet( NSUserActivity *ref, CSSearchableItemAttributeSetRef attrSet );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
CFSetRef UserActivityKeywords( NSUserActivity *ref );
void UserActivitySetKeywords( NSUserActivity *ref, CFSetRef keywords );

// Eligibility
BOOL UserActivityIsEligibleForHandoff( NSUserActivity *ref );
void UserActivitySetEligibleForHandoff( NSUserActivity *ref, BOOL flag );
BOOL UserActivityIsEligibleForSearch( NSUserActivity *ref );
void UserActivitySetEligibleForSearch( NSUserActivity *ref, BOOL flag );
BOOL UserActivityIsEligibleForPublicIndexing( NSUserActivity *ref );
void UserActivitySetEligibleForPublicIndexing( NSUserActivity *ref, BOOL flag );
CFDateRef UserActivityExpirationDate( NSUserActivity *ref );
void UserActivitySetExpirationDate( NSUserActivity *ref, CFDateRef dt );
#endif// 101100

// Registering and invalidating
void UserActivityBecomeCurrent( NSUserActivity *ref );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101100
void UserActivityResignCurrent( NSUserActivity *ref );
#endif// 101100
void UserActivityInvalidate( NSUserActivity *ref );

// Delegate
void UserActivitySetDelegateCallback( NSUserActivity *ref, CUI_UserActivityDelegateCallbackType callback, void *userData );

// Web browsing
CFURLRef UserActivityWebpageURL( NSUserActivity *ref );
void UserActivitySetWebpageURL( NSUserActivity *ref, CFURLRef url );

// Streams
BOOL UserActivitySupportsContinuationStreams( NSUserActivity *ref );
void UserActivitySetSupportsContinuationStreams( NSUserActivity *ref, BOOL flag );
void UserActivityGetContinuationStreamsWithCompletionHandler( NSUserActivity *ref, CUI_UserActivityContinuationStreamsCompletionHandlerType handler, void *userData );

// Instance properties
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101304
CIBarcodeDescriptor *UserActivityDetectedBarcodeDescriptor( NSUserActivity *ref );
#endif// 101304

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101300
CFURLRef UserActivityReferrerURL( NSUserActivity *ref );
void UserActivitySetReferrerURL( NSUserActivity *ref, CFURLRef url );
#endif// 101300

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
CFStringRef UserActivityTargetContentIdentifier( NSUserActivity *ref );
void UserActivitySetTargetContentIdentifier( NSUserActivity *ref, CFStringRef identifier );
#endif// 101500

#endif// 101000
