/*
 UserDefaults.m

 Bernie Wylde
 */

#import "UserDefaults.h"

#pragma mark - private
//NSDictionary *UserDefaultsProcessDictionary( NSDictionary *inDict )
//{
//    NSMutableDictionary *dict = [inDict mutableCopy];
//    NSArray *keys = [dict allKeys];
//    for ( NSString *key in keys ) {
//        id obj = [dict objectForKey:key];
//        if ( [obj isKindOfClass:[NSColor class]] ) {
//            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
//            [dict setObject:data forKey:key];
//        } else if ( [obj isKindOfClass:[NSFont class]] ) {
//            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
//            [dict setObject:data forKey:key];
//        } else if ( [obj isKindOfClass:[NSDictionary class]] ) {
//            NSDictionary *newDict = UserDefaultsProcessDictionary( obj );
//            [dict setObject:newDict forKey:key];
//        }
//    }
//    return (NSDictionary *)dict;
//}

void CUI_UserDefaultsAddValueToDictionary( id view, NSMutableDictionary *dict )
{
    NSInteger tag = [view tag];
    
    if ( tag > 0 ) {
        // button
        if ( [(NSObject *)view class] == [NSButton class] ) {
            NSString *key = [NSString stringWithFormat:@"Button%ldState",(long)[(NSButton *)view tag]];
            [dict setObject:@([(NSButton *)view state]) forKey:key];
            return;
        }
        
        // colorwell
        if ( [(NSObject *)view class] == [NSColorWell class] ) {
            NSString *key = [NSString stringWithFormat:@"ColorWell%ldColor",(long)[(NSColorWell *)view tag]];
            [dict setObject:[NSArchiver archivedDataWithRootObject:[(NSColorWell *)view color]] forKey:key];
            return;
        }
        
        // combobox
        if ( [(NSObject *)view class] == [NSComboBox class] ) {
            NSString *key = [NSString stringWithFormat:@"ComboBox%ldStringValue",(long)[(NSComboBox *)view tag]];
            [dict setObject:[(NSComboBox *)view stringValue] forKey:key];
            key = [NSString stringWithFormat:@"ComboBox%ldObjectValues",(long)[view tag]];
            [dict setObject:[(NSComboBox *)view objectValues] forKey:key];
            return;
        }
        
        // datepicker
        if ( [(NSObject *)view class] == [NSDatePicker class] ) {
            NSString *key = [NSString stringWithFormat:@"DatePicker%ldDateValue",(long)[(NSDatePicker *)view tag]];
            [dict setObject:[(NSDatePicker *)view dateValue] forKey:key];
            return;
        }
        
        // popupbutton
        if ( [view class] == [NSPopUpButton class] ) {
            NSString *key = [NSString stringWithFormat:@"PopUpButton%ldTitleOfSelectedItem",(long)[view tag]];
            [dict setObject:[view titleOfSelectedItem] forKey:key];
            return;
        }
        
        // searchfield
        if ( [view class] == [NSSearchField class] ) {
            NSString *key = [NSString stringWithFormat:@"SearchField%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            return;
        }
        
        // segmentedcontrol
        if ( [view class] == [NSSegmentedControl class] ) {
            NSString *key = [NSString stringWithFormat:@"SegmentedControl%ldSelectedSegment",(long)[view tag]];
            [dict setObject:@([view selectedSegment]) forKey:key];
            return;
        }
        
        // slider
        if ( [view class] == [NSSlider class] ) {
            NSString *key = [NSString stringWithFormat:@"Slider%ldDoubleValue",(long)[view tag]];
            [dict setObject:@([view doubleValue]) forKey:key];
            return;
        }
        
        // stepper
        if ( [view class] == [NSStepper class]) {
            NSString *key = [NSString stringWithFormat:@"Stepper%ldDoubleValue",(long)[view tag]];
            [dict setObject:@([view doubleValue]) forKey:key];
            return;
        }
        
        // textfield
        if ( [view class] == [NSTextField class] ) {
            NSString *key = [NSString stringWithFormat:@"TextField%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            return;
        }
        
        // tokenfield
        if ( [view class] == [NSTokenField class] ) {
            NSString *key = [NSString stringWithFormat:@"TokenField%ldStringValue",(long)[view tag]];
            [dict setObject:[view stringValue] forKey:key];
            return;
        }
        
    } else {
        
        // tabview - store tabview item views
        if ( [view class] == [NSTabView class] ) {
            NSArray *items = [view tabViewItems];
            for ( NSTabViewItem *item in items ) {
                NSView *itemView = [item view];
                CUI_UserDefaultsAddSubviewValuesToDictionary( itemView, dict );
            }
            return;
        }
        
    }
}

void CUI_UserDefaultsAddSubviewValuesToDictionary( id superview, NSMutableDictionary *dict )
{
    NSArray *subviews = [superview subviews];
    for ( id obj in subviews ) {
        if ( [obj respondsToSelector:@selector(tag)] ) {
            CUI_UserDefaultsAddValueToDictionary( obj, dict );
        }
        CUI_UserDefaultsAddSubviewValuesToDictionary( obj, dict );
    }
}

void CUI_UserDefaultsStoreWindowViewValues( NSWindow *w, NSString *key )
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:0];
    CUI_UserDefaultsAddSubviewValuesToDictionary( [w contentView], dict );
    if ( !key ) key = [w title];
    [[NSUserDefaults standardUserDefaults] setObject:dict forKey:key];
}



void CUI_UserDefaultsRestoreValueFromDictionary( id view, NSDictionary *dict )
{
    NSInteger tag = [view tag];
    if ( tag > 0 ) {
        // button
        if ( [view class] == [NSButton class] ) {
            NSString *key = [NSString stringWithFormat:@"Button%ldState",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [(NSButton *)view setState:[number integerValue]];
            return;
        }
        
        // colorwell
        if ( [view class] == [NSColorWell class] ) {
            NSString *key = [NSString stringWithFormat:@"ColorWell%ldColor",(long)[view tag]];
            NSData *data = [dict objectForKey:key];
            if ( data ) [view setColor:[NSUnarchiver unarchiveObjectWithData:data]];
            return;
        }
        
        // combobox
        if ( [view class] == [NSComboBox class] ) {
            NSString *key = [NSString stringWithFormat:@"ComboBox%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            key = [NSString stringWithFormat:@"ComboBox%ldObjectValues",(long)[view tag]];
            NSArray *array = [dict objectForKey:key];
            [view removeAllItems];
            if ( key ) [view addItemsWithObjectValues:array];
            return;
        }
        
        // datepicker
        if ( [view class] == [NSDatePicker class] ) {
            NSString *key = [NSString stringWithFormat:@"DatePicker%ldDateValue",(long)[view tag]];
            NSDate *date = [dict objectForKey:key];
            if ( date ) [view setDateValue:date];
            return;
        }
        
        // popupbutton
        if ( [view class] == [NSPopUpButton class] ) {
            NSString *key = [NSString stringWithFormat:@"PopUpButton%ldTitleOfSelectedItem",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view selectItemWithTitle:string];
            return;
        }
        
        // searchfield
        if ( [view class] == [NSSearchField class] ) {
            NSString *key = [NSString stringWithFormat:@"SearchField%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            return;
        }
        
        // segmentedcontrol
        if ( [view class] == [NSSegmentedControl class] ) {
            NSString *key = [NSString stringWithFormat:@"SegmentedControl%ldSelectedSegment",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setSelectedSegment:[number integerValue]];
            return;
        }
        
        // slider
        if ( [view class] == [NSSlider class] ) {
            NSString *key = [NSString stringWithFormat:@"Slider%ldDoubleValue",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setDoubleValue:[number doubleValue]];
            return;
        }
        
        // stepper
        if ( [view class] == [NSStepper class] ) {
            NSString *key = [NSString stringWithFormat:@"Stepper%ldDoubleValue",(long)[view tag]];
            NSNumber *number = [dict objectForKey:key];
            if ( number ) [view setDoubleValue:[number doubleValue]];
            return;
        }
        
        // textfield
        if ( [view class] == [NSTextField class] ) {
            NSString *key = [NSString stringWithFormat:@"TextField%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            return;
        }
        
        // tokenfield
        if ( [view class] == [NSTokenField class] ) {
            NSString *key = [NSString stringWithFormat:@"TokenField%ldStringValue",(long)[view tag]];
            NSString *string = [dict objectForKey:key];
            if ( string ) [view setStringValue:string];
            return;
        }
        
    } else {
        
        // tabview - restore tabview item views
        if ( [view class] == [NSTabView class] ) {
            NSArray *items = [view tabViewItems];
            for ( NSTabViewItem *item in items ) {
                NSView *itemView = [item view];
                CUI_UserDefaultsRestoreSubviewValuesFromDictionary( itemView, dict );
            }
            return;
        }
        
    }
}

void CUI_UserDefaultsRestoreSubviewValuesFromDictionary( id superview, NSDictionary *dict )
{
    NSArray *subviews = [superview subviews];
    for ( id obj in subviews ) {
        if ( [obj respondsToSelector:@selector(tag)] ) {
            CUI_UserDefaultsRestoreValueFromDictionary( obj, dict );
        }
        CUI_UserDefaultsRestoreSubviewValuesFromDictionary( obj, dict );
    }
}

void CUI_UserDefaultsRestoreWindowViewValues( NSWindow *w, NSString *key )
{
    if ( !key ) key = [w title];
    NSDictionary *dict = [[NSUserDefaults standardUserDefaults] objectForKey:key];
    if ( dict ) CUI_UserDefaultsRestoreSubviewValuesFromDictionary( [w contentView], dict );
}

void CUI_UserDefaultsRemoveWindowViewValues( NSWindow *w, NSString *key )
{
    if ( !key ) key = [w title];
    [[NSUserDefaults standardUserDefaults] setObject:nil forKey:key];
}


#pragma mark - public
// Class
Class UserDefaultsClass( void )
{ return [NSUserDefaults class]; }

// Get values
CFTypeRef UserDefaultsObject( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] objectForKey:(__bridge NSString *)key]; }

BOOL UserDefaultsBool( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] boolForKey:(__bridge NSString *)key]; }

long UserDefaultsInteger( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] integerForKey:(__bridge NSString *)key]; }

double UserDefaultsDouble( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] doubleForKey:(__bridge NSString *)key]; }

float UserDefaultsFloat( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] floatForKey:(__bridge NSString *)key]; }

CFURLRef UserDefaultsURL( CFStringRef key )
{ return (__bridge CFURLRef)[[NSUserDefaults standardUserDefaults] URLForKey:(__bridge NSString *)key]; }

CFArrayRef UserDefaultsArray( CFStringRef key )
{ return (__bridge CFArrayRef)[[NSUserDefaults standardUserDefaults] arrayForKey:(__bridge NSString *)key]; }

CFDictionaryRef UserDefaultsDictionary( CFStringRef key )
{ return (__bridge CFDictionaryRef)[[NSUserDefaults standardUserDefaults] dictionaryForKey:(__bridge NSString *)key]; }

CFStringRef UserDefaultsString( CFStringRef key )
{ return (__bridge CFStringRef)[[NSUserDefaults standardUserDefaults] stringForKey:(__bridge NSString *)key]; }

CFArrayRef UserDefaultsStringArray( CFStringRef key )
{ return (__bridge CFArrayRef)[[NSUserDefaults standardUserDefaults] stringArrayForKey:(__bridge NSString *)key]; }

CFDataRef UserDefaultsData( CFStringRef key )
{ return (__bridge CFDataRef)[[NSUserDefaults standardUserDefaults] dataForKey:(__bridge NSString *)key]; }

CFDictionaryRef UserDefaultsDictionaryRepresentation( void )
{ return (__bridge CFDictionaryRef)[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]; }

// Set values
void UserDefaultsSetObject( CFStringRef key, CFTypeRef obj )
{ [[NSUserDefaults standardUserDefaults] setObject:(__bridge id)obj forKey:(__bridge NSString *)key]; }

void UserDefaultsSetBool( CFStringRef key, BOOL value )
{ [[NSUserDefaults standardUserDefaults] setBool:value forKey:(__bridge NSString *)key]; }

void UserDefaultsSetInteger( CFStringRef key, long value )
{ [[NSUserDefaults standardUserDefaults] setInteger:value forKey:(__bridge NSString *)key]; }

void UserDefaultsSetDouble( CFStringRef key, double value )
{ [[NSUserDefaults standardUserDefaults] setDouble:value forKey:(__bridge NSString *)key]; }

void UserDefaultsSetFloat( CFStringRef key, float value )
{ [[NSUserDefaults standardUserDefaults] setFloat:value forKey:(__bridge NSString *)key]; }

void UserDefaultsSetURL( CFStringRef key, CFURLRef value )
{ [[NSUserDefaults standardUserDefaults] setURL:(__bridge NSURL *)value forKey:(__bridge NSString *)key]; }

// Remove
void UserDefaultsRemoveObject( CFStringRef key )
{ [[NSUserDefaults standardUserDefaults] removeObjectForKey:(__bridge NSString *)key]; }

// Suites
void UserDefaultsAddSuiteNamed( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] addSuiteNamed:(__bridge NSString *)name]; }

void UserDefaultsRemoveSuiteNamed( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] removeSuiteNamed:(__bridge NSString *)name]; }

// Register

void UserDefaultsRegisterDefaults( CFDictionaryRef defaults )
{
    NSMutableDictionary *dict = [(__bridge NSDictionary *)defaults mutableCopy];
    NSArray *keys = [dict allKeys];
    for ( NSString *key in keys ) {
        id obj = [dict objectForKey:key];
        if ( [obj isKindOfClass:[NSColor class]] ) {
            //NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
            NSData *data;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400// 20211228
            if (@available(macOS 10.14, *)) {
                data = [NSKeyedArchiver archivedDataWithRootObject:obj requiringSecureCoding:NO error:nil];
            } else {
                data = [NSKeyedArchiver archivedDataWithRootObject:obj];
            }
#else
            data = [NSKeyedArchiver archivedDataWithRootObject:obj];
#endif// 101400
            [dict setObject:data forKey:key];
        } else if ( [obj isKindOfClass:[NSFont class]] ) {// this should be NSFontDescriptor !!!
            //NSData *data = [NSKeyedArchiver archivedDataWithRootObject:obj];
            NSData *data;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400// 20211228
            if (@available(macOS 10.14, *)) {// 20211228
                data = [NSKeyedArchiver archivedDataWithRootObject:[obj fontDescriptor] requiringSecureCoding:NO error:nil];
            } else {
                data = [NSKeyedArchiver archivedDataWithRootObject:[obj fontDescriptor]];
            }
#else
            data = [NSKeyedArchiver archivedDataWithRootObject:[obj fontDescriptor]];
#endif// 101400
            [dict setObject:data forKey:key];
        }
    }
    [[NSUserDefaults standardUserDefaults] registerDefaults:dict];
}

// Persistent domains
CFDictionaryRef UserDefaultsPersistentDomainForName( CFStringRef name )
{ return (__bridge CFDictionaryRef)[[NSUserDefaults standardUserDefaults] persistentDomainForName:(__bridge NSString *)name]; }

void UserDefaultsSetPersistentDomainForName( CFDictionaryRef domain, CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] setPersistentDomain:(__bridge NSDictionary *)domain forName:(__bridge NSString *)name]; }

void UserDefaultsRemovePersistentDomainForName( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] removePersistentDomainForName:(__bridge NSString *)name]; }

// Volatile domains
CFArrayRef UserDefaultsVolatileDomainNames( void )
{ return (__bridge CFArrayRef)[[NSUserDefaults standardUserDefaults] volatileDomainNames]; }

CFDictionaryRef UserDefaultsVolatileDomainForName( CFStringRef name )
{ return (__bridge CFDictionaryRef)[[NSUserDefaults standardUserDefaults] volatileDomainForName:(__bridge NSString *)name]; }

void UserDefaultsSetVolatileDomainForName( CFDictionaryRef domain, CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] setVolatileDomain:(__bridge NSDictionary *)domain forName:(__bridge NSString *)name]; }

void UserDefaultsRemoveVolatileDomainForName( CFStringRef name )
{ [[NSUserDefaults standardUserDefaults] removeVolatileDomainForName:(__bridge NSString *)name]; }

// Managed environment keys
BOOL UserDefaultsObjectIsForcedForKey( CFStringRef key )
{ return [[NSUserDefaults standardUserDefaults] objectIsForcedForKey:(__bridge NSString *)key]; }

BOOL UserDefaultsObjectIsForcedForKeyInDomain( CFStringRef key, CFStringRef domain )
{ return [[NSUserDefaults standardUserDefaults] objectIsForcedForKey:(__bridge NSString *)key inDomain:(__bridge NSString *)domain]; }

// Legacy
BOOL UserDefaultsSynchronize( void )
{ return [[NSUserDefaults standardUserDefaults] synchronize]; }


#pragma mark - Convenience
// Rect
CGRect UserDefaultsRect( CFStringRef key )
{ return NSRectToCGRect( NSRectFromString([[NSUserDefaults standardUserDefaults] objectForKey:(__bridge NSString *)key]) ); }

void UserDefaultsSetRect( CFStringRef key, CGRect r )
{ [[NSUserDefaults standardUserDefaults] setObject:NSStringFromRect(NSRectFromCGRect(r)) forKey:(__bridge NSString *)key]; }

// Color
NSColor *UserDefaultsColor( CFStringRef key )
{
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:(__bridge NSString *)key];
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400// 20211228
    if (@available(macOS 10.14, *)) {
        if ( data ) return [NSKeyedUnarchiver unarchivedObjectOfClass:[NSColor class] fromData:data error:nil];
    } else {
        if ( data ) return [NSKeyedUnarchiver unarchiveObjectWithData:data];
    }
#else
    if ( data ) return [NSKeyedUnarchiver unarchiveObjectWithData:data];
#endif// 101400
    return nil;
}

void UserDefaultsSetColor( CFStringRef key, NSColor *col )
{
    if ( col ) {
        NSData *data;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400// 20211228
        if (@available(macOS 10.14, *)) {
            data = [NSKeyedArchiver archivedDataWithRootObject:col requiringSecureCoding:NO error:nil];
        } else {
            data = [NSKeyedArchiver archivedDataWithRootObject:col];
        }
#else
        data = [NSKeyedArchiver archivedDataWithRootObject:col];
#endif// 101400
        if ( data ) [[NSUserDefaults standardUserDefaults] setObject:data forKey:(__bridge NSString *)key];
    }
}

// Font
CFTypeRef UserDefaultsFont( CFStringRef key )
{
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:(__bridge NSString *)key];
    if ( data ) {
        NSFontDescriptor *fd;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400// 20211228
        if (@available(macOS 10.14, *)) {
            fd = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSFontDescriptor class] fromData:data error:nil];
        } else {
            fd = [NSKeyedUnarchiver unarchiveObjectWithData:data];
        }
#else
        fd = [NSKeyedUnarchiver unarchiveObjectWithData:data];
#endif// 101400
        if ( fd ) return (__bridge CFTypeRef)[NSFont fontWithDescriptor:fd size:0];
    }
    return nil;
}

void UserDefaultsSetFont( CFStringRef key, CFTypeRef font )
{
    NSData *data;
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400// 20211228
    if (@available(macOS 10.14, *)) {// 20211228
        data = [NSKeyedArchiver archivedDataWithRootObject:[(__bridge id)font fontDescriptor] requiringSecureCoding:NO error:nil];
    } else {
        data = [NSKeyedArchiver archivedDataWithRootObject:[(__bridge id)font fontDescriptor]];
    }
#else
    data = [NSKeyedArchiver archivedDataWithRootObject:[(__bridge id)font fontDescriptor]];
#endif// 101400
    if ( data ) [[NSUserDefaults standardUserDefaults] setObject:data forKey:(__bridge NSString *)key];
}

// URL (stored as bookmark data)
CFURLRef UserDefaultsBookmarkURL( CFStringRef key )
{
    NSData *data = [[NSUserDefaults standardUserDefaults] objectForKey:(__bridge NSString *)key];
    if ( data ) return (__bridge CFURLRef)[NSURL URLByResolvingBookmarkData:data options:0 relativeToURL:nil bookmarkDataIsStale:nil error:nil];
    return nil;
}

void UserDefaultsSetBookmarkURL( CFStringRef key, CFURLRef url )
{
    NSData *data = [(__bridge NSURL *)url bookmarkDataWithOptions:NSURLBookmarkCreationMinimalBookmark includingResourceValuesForKeys:nil relativeToURL:nil error:nil];
    if ( data ) [[NSUserDefaults standardUserDefaults] setObject:data forKey:(__bridge NSString *)key];
}


// Window view values
void UserDefaultsStoreWindowViewValues( NSInteger wndTag, CFStringRef key )
{
    NSWindow *w = WindowWithTag( wndTag );
    if ( w ) CUI_UserDefaultsStoreWindowViewValues( w, (__bridge NSString *)key );
}

void UserDefaultsRestoreWindowViewValues( NSInteger wndTag, CFStringRef key )
{
    NSWindow *w = WindowWithTag( wndTag );
    if ( w ) CUI_UserDefaultsRestoreWindowViewValues( w, (__bridge NSString *)key );
}

void UserDefaultsRemoveWindowViewValues( NSInteger wndTag, CFStringRef key )
{
    NSWindow *w = WindowWithTag( wndTag );
    if ( w ) CUI_UserDefaultsRemoveWindowViewValues( w, (__bridge NSString *)key );
}


#pragma mark - Custom
// Dictation edit menu item
void UserDefaultsDictationMenuItemSetDisabled( BOOL flag )
{ [[NSUserDefaults standardUserDefaults] setBool:flag forKey:@"NSDisabledDictationMenuItem"]; }

// Character palette edit menu item
void UserDefaultsCharacterPaletteMenuItemSetDisabled( BOOL flag )
{ [[NSUserDefaults standardUserDefaults] setBool:flag forKey:@"NSDisabledCharacterPaletteMenuItem"]; }

