/*
 Workspace.h

 Bernie Wylde
 */


#import "CocoaUI.h"

typedef void (*CUI_WorkspaceNotificationCenterCallbackType)(NSNotification*);
typedef void (*CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType)(NSError*,void*);

@interface CUI_WorkspaceNotificationCenter : NSObject
@property (assign) CUI_WorkspaceNotificationCenterCallbackType callback;
@property (nonatomic,retain) NSString *name;

- (id)initWithCallback:(CUI_WorkspaceNotificationCenterCallbackType)cb name:(NSString *)nm;

@end

// -- private --
NSMutableArray *CUI_WorkspaceNotificationCenters( void );

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
typedef void (*WorkspaceRequestAuthorizationCompletionHandlerType)(NSWorkspaceAuthorization*,NSError*,void*);
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
typedef void (*CUI_WorkspaceOpenCompletionHandlerType)(NSRunningApplication*,NSError*,void*);
#endif

// Class
Class WorkspaceClass( void );

// Shared
NSWorkspace *WorkspaceShared( void );

// Notification center
NSNotificationCenter *WorkspaceNotificationCenter( void );
void WorkspaceNotificationCenterAddObserver( void *callback, CFStringRef name, CFTypeRef object );
void WorkspaceNotificationCenterRemoveObserver( void *callback, CFStringRef name );
void WorkspaceNotificationCenterPostNotification( NSNotification *note );
void WorkspaceNotificationCenterPostNotificationName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo );

// Opening files
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSRunningApplication *WorkspaceOpenURLWithOptions( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, NSError **err );
#endif
BOOL WorkspaceOpenURL( CFURLRef url );
BOOL WorkspaceOpenURLWithApplication( CFURLRef url, CFStringRef appName, BOOL deactivate );
BOOL WorkspaceOpenURLsWithApplicationAtURL( CFArrayRef urls, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration );
BOOL WorkspaceOpenURLWithApplicationAtURL( CFURLRef url, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration );
BOOL WorkspaceLaunchApplication( CFStringRef appName );

NSRunningApplication *WorkspaceLaunchApplicationAtURL( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, NSError **err );

void WorkspaceHideOtherApplications( void );
void WorkspaceDuplicateURLs( CFArrayRef urls );
void WorkspaceRecycleURL( CFURLRef url);
void WorkspaceRecycleURLs( CFArrayRef urls );

void WorkspaceActivateFileViewerSelectingURLs( CFArrayRef fileURLs );

// UTI
CFStringRef WorkspaceTypeOfFileAtURL( CFURLRef url );
CFStringRef WorkspaceLocalizedDescriptionForType( CFStringRef type );
CFStringRef WorkspacePreferredFilenameExtensionForType( CFStringRef typeName );
BOOL WorkspaceFilenameExtensionIsValidForType( CFStringRef filenameExtension, CFStringRef typeName );
BOOL WorkspaceFileTypeConformsToType( CFStringRef firstTypeName, CFStringRef secondTypeName );
BOOL WorkspaceGetInfoForFileAtURL( CFURLRef url, CFStringRef *appName, CFStringRef *type );
CFURLRef WorkspaceURLForApplicationToOpenURL( CFURLRef url );
CFURLRef WorkspaceURLForApplication( CFStringRef name );

BOOL WorkspaceGetFileSystemInfoForURL( CFURLRef url, BOOL *isRemovable, BOOL *isWritable, BOOL *isUnmountable, CFStringRef *description, CFStringRef *type );

BOOL WorkspaceIsFilePackageAtURL( CFURLRef url );
NSRunningApplication *WorkspaceFrontmostApplication( void );
CFArrayRef WorkspaceRunningApplications( void );
NSRunningApplication *WorkspaceMenuBarOwningApplication( void );
NSImage *WorkspaceIconForFileAtURL( CFURLRef url );
NSImage *WorkspaceIconForFileType( CFStringRef type );
void WorkspaceSetIconForFileAtURL( NSImage *icon, CFURLRef url, NSWorkspaceIconCreationOptions options );
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
NSImage *WorkspaceIconForContentType( UTType *contentType );
#endif// 101600

// Unmount
BOOL WorkspaceUnmountAndEjectDeviceAtURL( CFURLRef url, NSError **err );

BOOL WorkspaceShowSearchResultsForQueryString( CFStringRef string );

CFArrayRef WorkspaceFileLabelColors( void );
CFArrayRef WorkspaceFileLabels( void );

CFURLRef WorkspaceURLForApplicationWithBundleIdentifier( CFStringRef identifier );
void WorkspaceNoteFileSystemChangedAtURL( CFURLRef url );

// - desktop image -
CFURLRef WorkspaceDesktopImageURLForScreen( NSScreen *screen );
void WorkspaceSetDesktopImageURLForScreen( CFURLRef url, NSScreen *screen, CFDictionaryRef options, NSError **err );
CFDictionaryRef WorkspaceDesktopImageOptionsForScreen( NSScreen *screen );

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
void WorkspaceRequestAuthorizationOfType( NSWorkspaceAuthorizationType type, WorkspaceRequestAuthorizationCompletionHandlerType completionHandler, void *userData );
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
void WorkspaceOpenURLWithConfiguration( CFURLRef url, NSWorkspaceOpenConfiguration *configuration, CUI_WorkspaceOpenCompletionHandlerType completionHandler, void *userData );
void WorkspaceOpenURLsWithApplicationAndConfiguration( CFArrayRef urls, CFURLRef appURL, NSWorkspaceOpenConfiguration *configuration, CUI_WorkspaceOpenCompletionHandlerType completionHandler, void *userData );
void WorkspaceOpenApplicationAtURLWithConfiguration( CFURLRef url, NSWorkspaceOpenConfiguration *configuration, CUI_WorkspaceOpenCompletionHandlerType completionHandler, void *userData );
#endif// 101500

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
void WorkspaceSetDefaultApplicationAtURLToOpenContentType( CFURLRef url, UTType *contentType, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData );
void WorkspaceSetDefaultApplicationAtURLToOpenContentTypeOfFileAtURL( CFURLRef url, CFURLRef fileURL, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData );
void WorkspaceSetDefaultApplicationAtURLToOpenFileAtURL( CFURLRef url, CFURLRef fileURL, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData );
void WorkspaceSetDefaultApplicationAtURLToOpenURLsWithScheme( CFURLRef url, CFStringRef urlScheme, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData );
CFURLRef WorkspaceURLForApplicationToOpenContentType( UTType *contentType );
CFArrayRef WorkspaceURLsForApplicationsToOpenContentType( UTType *contentType );
CFArrayRef WorkspaceURLsForApplicationsToOpenURL( CFURLRef url );
CFArrayRef WorkspaceURLsForApplicationsWithBundleIdentifier( CFStringRef identifier );
#endif// 120000

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
/*
  WorkspaceOpenConfiguration
*/
// Init
NSWorkspaceOpenConfiguration *WorkspaceOpenConfigurationInit( void );

// Instance properties
BOOL WorkspaceOpenConfigurationActivates( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetActivates( NSWorkspaceOpenConfiguration *ref, BOOL flag );
BOOL WorkspaceOpenConfigurationAddsToRecentItems( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetAddsToRecentItems( NSWorkspaceOpenConfiguration *ref, BOOL flag );
BOOL WorkspaceOpenConfigurationAllowsRunningApplicationSubstition( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetAllowsRunningApplicationSubstition( NSWorkspaceOpenConfiguration *ref, BOOL flag );
NSAppleEventDescriptor *WorkspaceOpenConfigurationAppleEvent( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetAppleEvent( NSWorkspaceOpenConfiguration *ref, NSAppleEventDescriptor *ae );
SInt32 WorkspaceOpenConfigurationArchitecture( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetArchitecture( NSWorkspaceOpenConfiguration *ref, SInt32 architecture );
CFArrayRef WorkspaceOpenConfigurationArguments( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetArguments( NSWorkspaceOpenConfiguration *ref, CFArrayRef args );
BOOL WorkspaceOpenConfigurationCreatesNewApplicationInstance( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetCreatesNewApplicationInstance( NSWorkspaceOpenConfiguration *ref, BOOL flag );
CFDictionaryRef WorkspaceOpenConfigurationEnvironment( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetEnvironment( NSWorkspaceOpenConfiguration *ref, CFDictionaryRef environment );
BOOL WorkspaceOpenConfigurationHides( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetHides( NSWorkspaceOpenConfiguration *ref, BOOL flag );
BOOL WorkspaceOpenConfigurationHidesOthers( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetHidesOthers( NSWorkspaceOpenConfiguration *ref, BOOL flag );
BOOL WorkspaceOpenConfigurationIsForPrinting( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetForPrinting( NSWorkspaceOpenConfiguration *ref, BOOL flag );
BOOL WorkspaceOpenConfigurationPromptsUserIfNeeded( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetPromptsUserIfNeeded( NSWorkspaceOpenConfiguration *ref, BOOL flag );
BOOL WorkspaceOpenConfigurationRequiresUniversalLinks( NSWorkspaceOpenConfiguration *ref );
void WorkspaceOpenConfigurationSetRequiresUniversalLinks( NSWorkspaceOpenConfiguration *ref, BOOL flag );

#endif// 101500
