/*
 Workspace.m

 Bernie Wylde
 */

#import "Workspace.h"

static NSMutableArray *sCUI_WorkspaceNotificationCenters;

@implementation CUI_WorkspaceNotificationCenter : NSObject

- (id)initWithCallback:(CUI_WorkspaceNotificationCenterCallbackType)cb name:(NSString *)nm {
    if ( self = [super init] ) {
        self.callback = cb;
        self.name = nm;
    }
    return self;
}

- (void)dealloc {
    [self.name release];
    [super dealloc];
}

- (void)didReceiveNotification:(NSNotification *)notification {
    if ( _callback ) (*_callback)(notification);
}
@end

// -- private --
NSMutableArray *CUI_WorkspaceNotificationCenters( void )
{
    if ( !sCUI_WorkspaceNotificationCenters ) sCUI_WorkspaceNotificationCenters = [[NSMutableArray alloc] initWithCapacity:0];
    return sCUI_WorkspaceNotificationCenters;
}

// -- public --
// Class
Class WorkspaceClass( void )
{ return [NSWorkspace class]; }

// Shared
NSWorkspace *WorkspaceShared( void )
{ return [NSWorkspace sharedWorkspace]; }

// Notification center
NSNotificationCenter *WorkspaceNotificationCenter( void )
{ return [[NSWorkspace sharedWorkspace] notificationCenter]; }

void WorkspaceNotificationCenterAddObserver( void *callback, CFStringRef name, CFTypeRef object )
{
    NSMutableArray *centers = CUI_WorkspaceNotificationCenters();
    CUI_WorkspaceNotificationCenter *center = [[CUI_WorkspaceNotificationCenter alloc] initWithCallback:callback name:(__bridge NSString *)name];
    [centers addObject:center];
    [[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:center selector:@selector(didReceiveNotification:) name:(__bridge NSString *)name object:(__bridge id)object];
}

void WorkspaceNotificationCenterRemoveObserver( void *callback, CFStringRef name )
{
    NSMutableArray *centers = CUI_WorkspaceNotificationCenters();
    for ( CUI_WorkspaceNotificationCenter *center in centers ) {
        if ( (center.callback == callback) && ([[center name] isEqualToString:(__bridge NSString *)name]) ) {
            [[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:center name:(__bridge NSString *)name object:nil];
            [centers removeObject:center];
            break;
        }
    }
}

void WorkspaceNotificationCenterPostNotification( NSNotification *note )
{ [[[NSWorkspace sharedWorkspace] notificationCenter] postNotification:note]; }

void WorkspaceNotificationCenterPostNotificationName( CFStringRef name, CFTypeRef object, CFDictionaryRef userInfo )
{ [[[NSWorkspace sharedWorkspace] notificationCenter] postNotificationName:(__bridge NSString *)name object:object userInfo:(__bridge NSDictionary *)userInfo]; }

// Opening files
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
NSRunningApplication *WorkspaceOpenURLWithOptions( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, NSError **err )
{ return [[NSWorkspace sharedWorkspace] openURL:(__bridge NSURL *)url options:options configuration:(__bridge NSDictionary *)configuration error:err]; }
#endif

BOOL WorkspaceOpenURL( CFURLRef url )
{ return [[NSWorkspace sharedWorkspace] openURL:(__bridge NSURL *)url]; }

BOOL WorkspaceOpenURLWithApplication( CFURLRef url, CFStringRef appName, BOOL deactivate )
{ return [[NSWorkspace sharedWorkspace] openFile:[(__bridge NSURL *)url path] withApplication:(__bridge NSString *)appName andDeactivate:deactivate]; }

BOOL WorkspaceOpenURLsWithApplicationAtURL( CFArrayRef urls, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
    return ( [[NSWorkspace sharedWorkspace] openURLs:(__bridge NSArray *)urls withApplicationAtURL:(__bridge NSURL *)appURL options:options configuration:(__bridge NSDictionary *)configuration error:nil] != nil );
#endif
    return false;
}

BOOL WorkspaceOpenURLWithApplicationAtURL( CFURLRef url, CFURLRef appURL, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration )
{
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101000
    return ( [[NSWorkspace sharedWorkspace] openURLs:[NSArray arrayWithObject:(__bridge NSURL *)url] withApplicationAtURL:(__bridge NSURL *)appURL options:options configuration:(__bridge NSDictionary *)configuration error:nil] != nil );
#endif
    return false;
}

BOOL WorkspaceLaunchApplication( CFStringRef appName )
{ return [[NSWorkspace sharedWorkspace] launchApplication:(__bridge NSString *)appName]; }

NSRunningApplication *WorkspaceLaunchApplicationAtURL( CFURLRef url, NSWorkspaceLaunchOptions options, CFDictionaryRef configuration, NSError **err )
{ return [[NSWorkspace sharedWorkspace] launchApplicationAtURL:(__bridge NSURL *)url options:options configuration:(__bridge NSDictionary *)configuration error:err]; }

void WorkspaceHideOtherApplications( void )
{ [[NSWorkspace sharedWorkspace] hideOtherApplications]; }

void WorkspaceDuplicateURLs( CFArrayRef urls )
{ [[NSWorkspace sharedWorkspace] duplicateURLs:(__bridge NSArray *)urls completionHandler:nil]; }

void WorkspaceRecycleURL( CFURLRef url)
{ [[NSWorkspace sharedWorkspace] recycleURLs:[NSArray arrayWithObject:(__bridge NSURL *)url] completionHandler:nil]; }

void WorkspaceRecycleURLs( CFArrayRef urls )
{ [[NSWorkspace sharedWorkspace] recycleURLs:(__bridge NSArray *)urls completionHandler:nil]; }

void WorkspaceActivateFileViewerSelectingURLs( CFArrayRef fileURLs )
{ [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs:(__bridge NSArray *)fileURLs]; }

// UTI
CFStringRef WorkspaceTypeOfFileAtURL( CFURLRef url )
{ return (__bridge CFStringRef)[[NSWorkspace sharedWorkspace] typeOfFile:[(__bridge NSURL *)url path] error:nil]; }

CFStringRef WorkspaceLocalizedDescriptionForType( CFStringRef type )
{ return (__bridge CFStringRef)[[NSWorkspace sharedWorkspace] localizedDescriptionForType:(__bridge NSString *)type]; }

CFStringRef WorkspacePreferredFilenameExtensionForType( CFStringRef typeName )
{ return (__bridge CFStringRef)[[NSWorkspace sharedWorkspace] preferredFilenameExtensionForType:(__bridge NSString *)typeName]; }

BOOL WorkspaceFilenameExtensionIsValidForType( CFStringRef filenameExtension, CFStringRef typeName )
{ return [[NSWorkspace sharedWorkspace] filenameExtension:(__bridge NSString *)filenameExtension isValidForType:(__bridge NSString *)typeName]; }

BOOL WorkspaceFileTypeConformsToType( CFStringRef firstTypeName, CFStringRef secondTypeName )
{ return [[NSWorkspace sharedWorkspace] type:(__bridge NSString *)firstTypeName conformsToType:(__bridge NSString *)secondTypeName]; }

BOOL WorkspaceGetInfoForFileAtURL( CFURLRef url, CFStringRef *appName, CFStringRef *type )
{ return [[NSWorkspace sharedWorkspace] getInfoForFile:[(__bridge NSURL *)url path] application:(NSString **)appName type:(NSString **)type]; }

CFURLRef WorkspaceURLForApplicationToOpenURL( CFURLRef url )
{ return (__bridge CFURLRef)[[NSWorkspace sharedWorkspace] URLForApplicationToOpenURL:(__bridge NSURL *)url]; }

BOOL WorkspaceGetFileSystemInfoForURL( CFURLRef url, BOOL *isRemovable, BOOL *isWritable, BOOL *isUnmountable, CFStringRef *description, CFStringRef *type )
{ return [[NSWorkspace sharedWorkspace] getFileSystemInfoForPath:[(__bridge NSURL *)url path] isRemovable:(BOOL *)isRemovable isWritable:(BOOL *)isWritable isUnmountable:(BOOL *)isUnmountable description:(NSString **)description type:(NSString **)type]; }

BOOL WorkspaceIsFilePackageAtURL( CFURLRef url )
{ return [[NSWorkspace sharedWorkspace] isFilePackageAtPath:[(__bridge NSURL *)url path]]; }

NSRunningApplication *WorkspaceFrontmostApplication( void )
{ return [[NSWorkspace sharedWorkspace] frontmostApplication]; }

CFArrayRef WorkspaceRunningApplications( void )
{ return (__bridge CFArrayRef)[[NSWorkspace sharedWorkspace] runningApplications]; }

NSRunningApplication *WorkspaceMenuBarOwningApplication( void )
{ return [[NSWorkspace sharedWorkspace] menuBarOwningApplication]; }

NSImage *WorkspaceIconForFileAtURL( CFURLRef url )
{ return [[NSWorkspace sharedWorkspace] iconForFile:[(__bridge NSURL *)url path]]; }

NSImage *WorkspaceIconForFileType( CFStringRef type )
{ return [[NSWorkspace sharedWorkspace] iconForFileType:(__bridge NSString *)type]; }

void WorkspaceSetIconForFileAtURL( NSImage *icon, CFURLRef url, NSWorkspaceIconCreationOptions options )
{ [[NSWorkspace sharedWorkspace] setIcon:icon forFile:[(__bridge NSURL *)url path] options:options]; }

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101600
NSImage *WorkspaceIconForContentType( UTType *contentType )
{
    if (@available(macOS 11.0, *)) {// 20210101
        return [[NSWorkspace sharedWorkspace] iconForContentType:contentType];
    }
    return nil;
}
#endif// 101600

// Unmount
BOOL WorkspaceUnmountAndEjectDeviceAtURL( CFURLRef url, NSError **err )
{ return [[NSWorkspace sharedWorkspace] unmountAndEjectDeviceAtURL:(__bridge NSURL *)url error:err]; }

BOOL WorkspaceShowSearchResultsForQueryString( CFStringRef string )
{ return [[NSWorkspace sharedWorkspace] showSearchResultsForQueryString:(__bridge NSString *)string]; }

CFArrayRef WorkspaceFileLabelColors( void )
{ return (__bridge CFArrayRef)[[NSWorkspace sharedWorkspace] fileLabelColors]; }

CFArrayRef WorkspaceFileLabels( void )
{ return (__bridge CFArrayRef)[[NSWorkspace sharedWorkspace] fileLabels]; }


CFURLRef WorkspaceURLForApplicationWithBundleIdentifier( CFStringRef identifier )
{ return (__bridge CFURLRef)[[NSWorkspace sharedWorkspace] URLForApplicationWithBundleIdentifier:(__bridge NSString *)identifier]; }

CFURLRef WorkspaceURLForApplication( CFStringRef name )
{ return (__bridge CFURLRef)[NSURL fileURLWithPath:[[NSWorkspace sharedWorkspace] fullPathForApplication:(__bridge NSString *)name]]; }

void WorkspaceNoteFileSystemChangedAtURL( CFURLRef url )
{ [[NSWorkspace sharedWorkspace] noteFileSystemChanged:[(__bridge NSURL *)url path]]; }


// - desktop image -
CFURLRef WorkspaceDesktopImageURLForScreen( NSScreen *screen )
{ return (__bridge CFURLRef)[[NSWorkspace sharedWorkspace] desktopImageURLForScreen:screen]; }

void WorkspaceSetDesktopImageURLForScreen( CFURLRef url, NSScreen *screen, CFDictionaryRef options, NSError **err )
{ [[NSWorkspace sharedWorkspace] setDesktopImageURL:(__bridge NSURL *)url forScreen:screen options:(__bridge NSDictionary *)options error:err]; }

CFDictionaryRef WorkspaceDesktopImageOptionsForScreen( NSScreen *screen )
{ return (__bridge CFDictionaryRef)[[NSWorkspace sharedWorkspace] desktopImageOptionsForScreen:screen]; }

// Instance methods
#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101400
void WorkspaceRequestAuthorizationOfType( NSWorkspaceAuthorizationType type, WorkspaceRequestAuthorizationCompletionHandlerType completionHandler, void *userData )
{
    if (@available(macOS 10.14, *)) {
        [[NSWorkspace sharedWorkspace] requestAuthorizationOfType:type completionHandler:^(NSWorkspaceAuthorization *authorization, NSError *error) {
            if ( completionHandler ) (*completionHandler)(authorization,error,userData);
        }];
    }
}
#endif

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
void WorkspaceOpenURLWithConfiguration( CFURLRef url, NSWorkspaceOpenConfiguration *configuration, CUI_WorkspaceOpenCompletionHandlerType completionHandler, void *userData )
{
    if (@available(macOS 10.15, *)) {// 20210102
        if ( !configuration ) configuration = [NSWorkspaceOpenConfiguration configuration];
        [[NSWorkspace sharedWorkspace] openURL:(__bridge NSURL *)url configuration:configuration completionHandler:^(NSRunningApplication *app, NSError *err ) {
            if ( completionHandler ) (*completionHandler)(app,err,userData);
        }];
    }
}

void WorkspaceOpenURLsWithApplicationAndConfiguration( CFArrayRef urls, CFURLRef appURL, NSWorkspaceOpenConfiguration *configuration, CUI_WorkspaceOpenCompletionHandlerType completionHandler, void *userData )
{
    if (@available(macOS 10.15, *)) {// 20210102
        if ( !configuration ) configuration = [NSWorkspaceOpenConfiguration configuration];
        [[NSWorkspace sharedWorkspace] openURLs:(__bridge NSArray *)urls withApplicationAtURL:(__bridge NSURL *)appURL configuration:configuration completionHandler:^(NSRunningApplication *app, NSError *err ) {
            if ( completionHandler ) (*completionHandler)(app,err,userData);
        }];
    }
}

void WorkspaceOpenApplicationAtURLWithConfiguration( CFURLRef url, NSWorkspaceOpenConfiguration *configuration, CUI_WorkspaceOpenCompletionHandlerType completionHandler, void *userData )
{
    if (@available(macOS 10.15, *)) {// 20210102
        if ( !configuration ) configuration = [NSWorkspaceOpenConfiguration configuration];
        [[NSWorkspace sharedWorkspace] openApplicationAtURL:(__bridge NSURL *)url configuration:configuration completionHandler:^(NSRunningApplication *app, NSError *err ) {
            if ( completionHandler ) (*completionHandler)(app,err,userData);
        }];
    }
}
#endif// 101500

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
void WorkspaceSetDefaultApplicationAtURLToOpenContentType( CFURLRef url, UTType *contentType, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData )
{
    [[NSWorkspace sharedWorkspace] setDefaultApplicationAtURL:(__bridge NSURL *)url toOpenContentType:contentType completionHandler:^( NSError *error ) {
        (*completionHandler)( error, userData );
    }];
}

void WorkspaceSetDefaultApplicationAtURLToOpenContentTypeOfFileAtURL( CFURLRef url, CFURLRef fileURL, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData )
{
    [[NSWorkspace sharedWorkspace] setDefaultApplicationAtURL:(__bridge NSURL *)url toOpenContentTypeOfFileAtURL:(__bridge NSURL *)fileURL completionHandler:^( NSError *error ) {
        (*completionHandler)( error, userData );
    }];
}

void WorkspaceSetDefaultApplicationAtURLToOpenFileAtURL( CFURLRef url, CFURLRef fileURL, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData )
{
    [[NSWorkspace sharedWorkspace] setDefaultApplicationAtURL:(__bridge NSURL *)url toOpenFileAtURL:(__bridge NSURL *)fileURL completionHandler:^( NSError *error ) {
        (*completionHandler)( error, userData );
    }];
}

void WorkspaceSetDefaultApplicationAtURLToOpenURLsWithScheme( CFURLRef url, CFStringRef urlScheme, CUI_WorkspaceSetDefaultApplicationAtURLCompletionHandlerType completionHandler, void *userData )
{
    [[NSWorkspace sharedWorkspace] setDefaultApplicationAtURL:(__bridge NSURL *)url toOpenURLsWithScheme:(__bridge NSString *)urlScheme completionHandler:^( NSError *error ) {
        (*completionHandler)( error, userData );
    }];
}

CFURLRef WorkspaceURLForApplicationToOpenContentType( UTType *contentType )
{ return (__bridge CFURLRef)[[NSWorkspace sharedWorkspace] URLForApplicationToOpenContentType:contentType]; }

CFArrayRef WorkspaceURLsForApplicationsToOpenContentType( UTType *contentType )
{ return (__bridge CFArrayRef)[[NSWorkspace sharedWorkspace] URLsForApplicationsToOpenContentType:contentType]; }

CFArrayRef WorkspaceURLsForApplicationsToOpenURL( CFURLRef url )
{ return (__bridge CFArrayRef)[[NSWorkspace sharedWorkspace] URLsForApplicationsToOpenURL:(__bridge NSURL *)url]; }

CFArrayRef WorkspaceURLsForApplicationsWithBundleIdentifier( CFStringRef identifier )
{ return (__bridge CFArrayRef)[[NSWorkspace sharedWorkspace] URLsForApplicationsWithBundleIdentifier:(__bridge NSString *)identifier]; }
#endif// 120000


#pragma mark - WorkspaceOpenConfiguration

#if __MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
// Init
NSWorkspaceOpenConfiguration *WorkspaceOpenConfigurationInit( void )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [NSWorkspaceOpenConfiguration configuration];
    }
    return nil;
}

// Instance properties
BOOL WorkspaceOpenConfigurationActivates( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref activates];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetActivates( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setActivates:flag];
    }
}

BOOL WorkspaceOpenConfigurationAddsToRecentItems( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref addsToRecentItems];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetAddsToRecentItems( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setAddsToRecentItems:flag];
    }
}

BOOL WorkspaceOpenConfigurationAllowsRunningApplicationSubstition( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref allowsRunningApplicationSubstitution];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetAllowsRunningApplicationSubstition( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setAllowsRunningApplicationSubstitution:flag];
    }
}

NSAppleEventDescriptor *WorkspaceOpenConfigurationAppleEvent( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref appleEvent];
    }
    return nil;
}

void WorkspaceOpenConfigurationSetAppleEvent( NSWorkspaceOpenConfiguration *ref, NSAppleEventDescriptor *ae )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setAppleEvent:ae];
    }
}

SInt32 WorkspaceOpenConfigurationArchitecture( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref architecture];
    }
    return 0;
}

void WorkspaceOpenConfigurationSetArchitecture( NSWorkspaceOpenConfiguration *ref, SInt32 architecture )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setArchitecture:architecture];
    }
}

CFArrayRef WorkspaceOpenConfigurationArguments( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return (__bridge CFArrayRef)[ref arguments];
    }
    return nil;
}

void WorkspaceOpenConfigurationSetArguments( NSWorkspaceOpenConfiguration *ref, CFArrayRef args )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setArguments:(__bridge NSArray *)args];
    }
}

BOOL WorkspaceOpenConfigurationCreatesNewApplicationInstance( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref createsNewApplicationInstance];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetCreatesNewApplicationInstance( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setCreatesNewApplicationInstance:flag];
    }
}

CFDictionaryRef WorkspaceOpenConfigurationEnvironment( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return (__bridge CFDictionaryRef)[ref environment];
    }
    return nil;
}

void WorkspaceOpenConfigurationSetEnvironment( NSWorkspaceOpenConfiguration *ref, CFDictionaryRef environment )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setEnvironment:(__bridge NSDictionary *)environment];
    }
}

BOOL WorkspaceOpenConfigurationHides( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref hides];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetHides( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setHides:flag];
    }
}

BOOL WorkspaceOpenConfigurationHidesOthers( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref hidesOthers];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetHidesOthers( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setHidesOthers:flag];
    }
}

BOOL WorkspaceOpenConfigurationIsForPrinting( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref isForPrinting];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetForPrinting( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setForPrinting:flag];
    }
}

BOOL WorkspaceOpenConfigurationPromptsUserIfNeeded( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref promptsUserIfNeeded];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetPromptsUserIfNeeded( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setPromptsUserIfNeeded:flag];
    }
}

BOOL WorkspaceOpenConfigurationRequiresUniversalLinks( NSWorkspaceOpenConfiguration *ref )
{
    if (@available(macOS 10.15, *)) {// 20210102
        return [ref requiresUniversalLinks];
    }
    return NO;
}

void WorkspaceOpenConfigurationSetRequiresUniversalLinks( NSWorkspaceOpenConfiguration *ref, BOOL flag )
{
    if (@available(macOS 10.15, *)) {// 20210102
        [ref setRequiresUniversalLinks:flag];
    }
}

#endif// 101500
