
/*
 Time-critical BitBoard utilities
  
 KountBits()
 GetLowestBit()
 SetBit()
 
 */


#if TARGET_RT_LITTLE_ENDIAN

// Intel asm in 64- and 32-bit versions

#if __LP64__

#ifdef HAS_POPCNT

// Core i5 and i7 processors have popcnt instruction

int static inline KountBits( BitBoard b )
{
	BitBoard  tmp0, tmp1;
	
	asm("    popcnt  %1, %0     " "\n\t"
		:   "=&r"(tmp0), "=&r" (tmp1)
		:   "1"((BitBoard) (b))
		:   "cc");
	return tmp0;
}

#else

int static inline KountBits( BitBoard b )
{
	BitBoard  tmp0, tmp1, tmp2;
	
	asm("          xorq    %0, %0    " "\n\t"
		"          testq   %1, %1    " "\n\t"
		"          jz      2f        " "\n\t"
		"1:        leaq    -1(%1),%2 " "\n\t"
		"          incq    %0        " "\n\t"
		"          andq    %2, %1    " "\n\t"
		"          jnz     1b        " "\n\t"
		"2:                          " "\n\t"
		:   "=&r"(tmp0), "=&r"(tmp1), "=&r"(tmp2)
		:   "1"((BitBoard) (b))
		:   "cc");
	return tmp0;
}

#endif

int static inline GetLowestBit( BitBoard b ) // LS1B
{
	BitBoard  tmp0, tmp1;
	
	asm("    bsfq    %1, %0     " "\n\t"
		:   "=&r" (tmp0), "=&r" (tmp1)
		:   "1" ((BitBoard) (b))
		:   "cc");
	return tmp0;
}


#else

static inline int KountBits( BitBoard b )
{
	// r0 = result, %1 = tmp, %2 = b hi, %3 = b lo
	int  tmp0, tmp1, tmp2, tmp3;
	
	asm("        xorl    %0, %0      " "\n\t"
		"        testl   %2, %2      " "\n\t"
		"        jz      2f          " "\n\t"
		"1:      leal    -1(%2), %1  " "\n\t"
		"        incl    %0          " "\n\t"
		"        andl    %1, %2      " "\n\t"
		"        jnz     1b          " "\n\t"
		"2:      testl   %3, %3      " "\n\t"
		"        jz      4f          " "\n\t"
		"3:      leal    -1(%3), %1  " "\n\t"
		"        incl    %0          " "\n\t"
		"        andl    %1, %3      " "\n\t"
		"        jnz     3b          " "\n\t"
		"4:                          " "\n\t"
		:   "=&q" (tmp0), "=&q" (tmp1), "=&q" (tmp2), "=&q" (tmp3)
		:   "2" ((int) (b >> 32)), "3" ((int) b)
		:   "cc");
	return tmp0;
}


static inline int GetLowestBit( BitBoard b ) // LS1B
{
	// r0 = result, %1 = b hi, %2 = b lo
	int  tmp0, tmp1, tmp2;
	
	asm("        bsf     %2, %0      " "\n\t"
		"        jnz     2f          " "\n\t"
		"        bsf     %1, %0      " "\n\t"
		"        jnz     1f          " "\n\t"
		"        jmp     2f          " "\n\t"
		"1:      addl    $32,%0      " "\n\t"
		"2:                          " "\n\t"
		:   "=&q" (tmp0), "=&q" (tmp1), "=&q" (tmp2)
		:    "1" ((int) (b >> 32)), "2" ((int) b)
		:    "cc");
	return tmp0;
}


#endif  /* __LP64__ */


#else		/* TARGET_RT_LITTLE_ENDIAN */


// for PPC we use compiler builtins

static inline int KountBits( BitBoard b )
{
	return __builtin_popcountll( b );
} 

static inline int GetLowestBit( BitBoard b ) // LS1B
{
	return __builtin_ctzll( b );
} 



#endif		/* TARGET_RT_LITTLE_ENDIAN */



static inline void SetBit( BitBoard * b, int bitNum )
{
	*b |= 1ULL << bitNum;
} 

