//
//  CFMenu.m
//  
//  Bernie Wylde 20110802
//

#import "Cocoa/Cocoa.h"
#import "CFMenu.h"

typedef void (*CallbackType)( void );

@interface CFMenuItemController : NSObjectController {
	CallbackType	_callback;
}
- (void)menuItemAction:(id)sender;
@end

@implementation CFMenuItemController
- (void)menuItemAction:(id)sender {
	if ( _callback != nil ) (*_callback)();
}

- (NSMenuItem *)menuItemInit:(NSMenu *)menu title:(NSString *)title callback:(CallbackType)callback keyEquivalent:(NSString *)keyEquivalent {
	NSMenuItem *item = [menu addItemWithTitle:title action:@selector(menuItemAction:) keyEquivalent:keyEquivalent];
	[item setTarget:self];
	_callback = callback;
	return item;
}
@end


CFMenuRef CFMenuCreateWithTitle( CFStringRef title )
{
	return [[NSMenu alloc] initWithTitle:(NSString *)title];
}


void CFMenuSetAutoenablesItems( CFMenuRef menu, Boolean flag )
{
	[(NSMenu *)menu setAutoenablesItems:flag];
}


CFMenuItemRef CFMenuAddItemWithTitle( CFMenuRef menu, CFStringRef title, void *callback, CFStringRef keyEquivalent )
{
	NSMenuItem *item = nil;
	
	CFMenuItemController *controller = [[CFMenuItemController alloc] init];
	
	if ( title == NULL ) {		
		[(NSMenu *)menu addItem:[NSMenuItem separatorItem]];
	} else {
		if ( keyEquivalent == nil ) keyEquivalent = CFSTR( "" );
		item = [controller menuItemInit:(NSMenu *)menu title:(NSString *)title callback:callback keyEquivalent:(NSString *)keyEquivalent];
	}
	return (CFMenuItemRef)item;	
}


void CFMenuItemSetKeyEquivalentModifierMask( CFMenuItemRef item, UInt32 mask )
{
	[(NSMenuItem *)item setKeyEquivalentModifierMask:mask];
}


void CFMenuItemSetEnabled( CFMenuItemRef item, Boolean flag )
{
	[(NSMenuItem *)item setEnabled:flag];
}
