
resources "MenuTitleIcon.rsrc"

include "Tlbx IBCarbonRuntime.incl"
include "Tlbx HIView.incl"
include "Tlbx Menus.incl"
include "Tlbx CoreGraphics.incl"
include "Tlbx CFString.incl"

// images for <app>/Contents/Resources/
include resources "Alien.icns"
include resources "Leaf.png"
include resources "Leopard.jpg"
include resources "Cup.png"

include resources "main.nib" // file to be copied to <app>/Contents/Resources/en.lproj


// menus
_mEmbed = 1
_mIcon = 2
begin enum 1
_iIconRef
_iIcns
_iPng
_iJpg
_iCtx
end enum


local mode
local fn MyCreateCGBitmapContext( w as single, h as single ) as CGContextRef
'~'1
dim as CGColorSpaceRef  cs
dim as CGContextRef     ctx
dim as long             rowBytes
dim as pointer          imageBuf

ctx = 0
rowBytes = w*4
imageBuf = fn malloc( rowBytes*h )
long if ( imageBuf )
BlockZero( imageBuf, rowBytes*h )
cs = fn CGColorSpaceCreateDeviceRGB()
ctx = fn CGBitmapContextCreate( imageBuf, w, h, 8, rowBytes, cs, _kCGImageAlphaPremultipliedFirst )
fn CGColorSpaceRelease( cs )
end if
end fn = ctx


local mode
local fn MyDisposeCGBitmapContext( ctx as CGContextRef )
'~'1
dim as pointer  imageBuf

imageBuf = fn CGBitmapContextGetData( ctx )
//fn CGContextRelease( ctx )
if ( imageBuf ) then fn free( imageBuf )
end fn


local fn DrawSmileyInCGContext( ctx as CGContextRef )
'~'1
dim as CGRect  @ bounds
dim as single    degree, x, y

degree = PI / 180
bounds = fn CGRectMake( 0, 0, fn CGBitmapContextGetWidth( ctx ), fn CGBitmapContextGetHeight( ctx ) )
x = bounds.size.width / 2
y = bounds.size.height / 2

fn CGContextBeginPath( ctx )
fn CGContextSetLineWidth( ctx, 1.0 )
fn CGContextSetRGBFillColor( ctx, 1.0, 0.83, 0.0, 0.60 )
fn CGContextAddArc( ctx, x, y, 15.25, 0.0, 360 * degree, 0 )
fn CGContextDrawPath( ctx, _kCGPathFillStroke )

fn CGContextBeginPath( ctx )
fn CGContextSetRGBFillColor( ctx, 0, 0, 0, 1 )
fn CGContextAddArc( ctx, x - 5.5, y + 5.0, 3.5, 0.0, 360 * degree, 0 )
fn CGContextAddArc( ctx, x + 5.5, y + 5.0, 3.5, 0.0, 360 * degree, 0 )
fn CGContextDrawPath( ctx, _kCGPathFill )

fn CGContextBeginPath( ctx )
fn CGContextAddArc( ctx, x, y + 2.5, 11.75, 195 * degree, 345 * degree, 0 )
fn CGContextDrawPath( ctx, _kCGPathStroke )
end fn


local mode
local fn CreateCGImageFromBundleResource( fileName as CFStringRef ) as CGImageRef
'~'1
dim as CFURLRef           url
dim as CGImageRef         image
dim as CGDataProviderRef  myProvider

image = 0
url = fn CFBundleCopyResourceURL( fn CFBundleGetMainBundle(), fileName, 0, 0 )
long if ( url )
myProvider = fn CGDataProviderCreateWithURL( url )
long if ( myProvider )
// quick, and very dirty, determination of image type
select
case fn CFStringHasSuffix( fileName, fn CFSTR( ".jpg" ) )
image = fn CGImageCreateWithJPEGDataProvider( myProvider, 0, _false, _kCGRenderingIntentDefault )
case fn CFStringHasSuffix( fileName, fn CFSTR( ".png" ) )
image = fn CGImageCreateWithPNGDataProvider( myProvider, 0, _false, _kCGRenderingIntentDefault )
end select
fn CGDataProviderRelease( myProvider )
end if
CFRelease( url )
end if
end fn = image


// called once when the menu is first pulled down
local fn OwningWindowChangedHandler( nextHandler as EventHandlerCallRef, theEvent as EventRef, userData as pointer )
'~'1
dim as HIRect        bounds
dim as WindowRef     w
dim as HiViewRef     content, imageView
dim as CGImageRef    image
dim as CFStringRef   imageResName
dim as OSStatus      result

result = _eventNotHandledErr
fn GetEventParameter( theEvent, _kEventParamControlCurrentOwningWindow, _typeWindowRef, #0, sizeof( WindowRef ), #0, @w )
long if ( w )
fn HIViewFindByID( fn HIViewGetRoot( w ), kHIViewWindowContentID, @content )
fn HIViewGetFrame( content, @bounds )
imageResName = userData
image = fn CreateCGImageFromBundleResource( imageResName )
fn HIImageViewCreate( image, @imageView )
fn HIImageViewSetOpaque( imageView, _false )
fn HIImageViewSetAlpha( imageView, 0.15 )
CFRelease( image )
fn HIViewAddSubview( content, imageView )
fn HIViewSetZOrder( imageView, _kHIViewZOrderBelow, 0 )
fn HIViewSetVisible( imageView, _true )
fn HIViewSetFrame( imageView, @bounds )
result = _noErr
end if
end fn = result


local fn SetMenuEmbeddedImage( m as MenuRef, string as CFStringRef ) as OSStatus
'~'1
dim as EventTypeSpec  theEvent
dim as HIViewRef      c

// see Manipulating Menu Views in
// http://developer.apple.com/documentation/Carbon/Conceptual/HIViewDoc/HIView_tasks/chapter_3_section_1.html
fn HIMenuGetContentView( m, _kThemeMenuTypePullDown, @c )

theEvent.eventClass = _kEventClassControl
theEvent.eventKind  = _kEventControlOwningWindowChanged
end fn = fn InstallEventHandler( fn GetControlEventTarget( c ), @fn OwningWindowChangedHandler, 1, @theEvent, #(ptr)string, #0 )


local fn BuildMenus
'~'1
dim as IconSuiteRef   theIconSuite
dim as CGContextRef   ctx
dim as CGImageRef     imageRef
dim as IBNibRef       nib
dim as MenuRef        m

long if ( fn CreateNibReference( fn CFSTR( "main" ), @nib ) == _noErr )
fn SetMenuBarFromNib( nib, fn CFSTR( "MenuBar" ) )
DisposeNibReference( nib )
end if

fn SetMenuEmbeddedImage( fn GetMenuHandle(_mEmbed), fn CFSTR( "Leaf.png" ) )

m = fn GetMenuHandle( _mIcon )
fn GetIconSuite( @theIconSuite, 1234, _kSelectorAllAvailableData )
fn SetMenuTitleIcon( m, _kMenuIconSuiteType, theIconSuite )

fn SetMenuItemIconHandle( m, _iIconRef, _kMenuSystemIconSelectorType, #_kGridIcon )
//... because gcc complains unless the last argument for SetMenuItemIconHandle() is, or is cast to, a Handle
fn SetMenuItemIconHandle( m, _iIcns, _kMenuIconResourceType, #(Handle)fn CFSTR( "Alien.icns" ) )

imageRef = fn CreateCGImageFromBundleResource( fn CFSTR("Cup.png") )
fn SetMenuItemIconHandle( m, _iPng, _kMenuCGImageRefType, (Handle)imageRef )
CGImageRelease( imageRef )

imageRef = fn CreateCGImageFromBundleResource( fn CFSTR("Leopard.jpg") )
fn SetMenuItemIconHandle( m, _iJpg, _kMenuCGImageRefType, (Handle)imageRef )
CGImageRelease( imageRef )

// smiley drawn in CG
ctx = fn MyCreateCGBitmapContext( 32.0, 32.0 )
long if ( ctx )
fn DrawSmileyInCGContext( ctx )
// Convert to CGImageRef for SetMenuItemIconHandle().
imageRef = fn CGBitmapContextCreateImage( ctx ) 
fn SetMenuItemIconHandle( m, _iCtx, _kMenuCGImageRefType, #(Handle)imageRef )
CFRelease( imageRef ) // release the CGImageRef

fn MyDisposeCGBitmapContext( ctx )
fn CGContextRelease( ctx )
end if
end fn


fn BuildMenus

window 1
print "Pull down the menus"

do
HandleEvents
until gFBQuit

