//
//  CalCalendar.m
//  
//  Bernie Wylde 20110819
//

#import "CalCalendar.h"
#import <CalendarStore/CalendarStore.h>

CFArrayRef CalCopyEvents( void ) {
    NSArray *events = nil;
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    CalCalendarStore *store = [CalCalendarStore defaultCalendarStore];
    if ( store != nil ) {
        NSPredicate *allEventsPredicate = [CalCalendarStore eventPredicateWithStartDate:[NSDate date] endDate:[NSDate distantFuture] calendars:[store calendars]];
        events = [[store eventsWithPredicate:allEventsPredicate] retain];
    }
    [pool drain];
    return (CFArrayRef)events;
}

CFStringRef CalCopyEventTitle( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSString *title = [[event title] retain];
	[pool drain];
	return (CFStringRef)title;
}

CFStringRef CalCopyEventLocation( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSString *location = [[event location] retain];
	[pool drain];
	return (CFStringRef)location;
}

Boolean CalEventIsAllDay( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	BOOL flag = [event isAllDay];
	[pool drain];
	return flag;
}

CFDateRef CalCopyEventStartDate( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSDate *date = [[event startDate] retain];
	[pool drain];
	return (CFDateRef)date;
}

CFDateRef CalCopyEventEndDate( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSDate *date = [[event endDate] retain];
	[pool drain];
	return (CFDateRef)date;
}

Boolean CalEventHasAlarm( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	BOOL flag = [event hasAlarm];
	[pool drain];
	return flag;
}

CFArrayRef CalCopyEventAlarms( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSArray *alarms = [[event alarms] retain];
	[pool drain];
	return (CFArrayRef)alarms;
}

CFArrayRef CalGetEventAttendees( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSArray *attendees = [[event attendees] retain];
	[pool drain];
	return (CFArrayRef)attendees;
}

CFStringRef CalCopyEventNotes( CFArrayRef events, CFIndex index )
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	CalEvent *event = [(NSArray *)events objectAtIndex:index];
	NSString *notes = [[event notes] retain];
	[pool drain];
	return (CFStringRef)notes;
}



