
BeginCFunction
#import <Cocoa/Cocoa.h>

typedef OSStatus (*CallbackType)(int,void*);

@interface PageLayoutController : NSObject {
 CallbackType _callback;
}

- (void)setCallback:(CallbackType)callback;
@end

static PageLayoutController *sPageLayoutController;

@implementation PageLayoutController

- (void)setCallback:(CallbackType)callback {
    _callback = callback;
}

- (void)pageLayoutDidEnd:(NSPageLayout *)pageLayout returnCode:(int)returnCode  contextInfo: (id)printInfo {
	if ( returnCode == NSOKButton ) [NSPrintInfo setSharedPrintInfo:printInfo];
 if ( _callback != nil ) (*_callback)(returnCode,printInfo);
 [sPageLayoutController release];
 sPageLayoutController = nil;
}

- (void)runPageLayoutForWindow:(NSWindow *)window {
 NSPrintInfo *printInfo = [NSPrintInfo sharedPrintInfo];
 NSPageLayout *layout = [NSPageLayout pageLayout];
 [layout beginSheetWithPrintInfo:printInfo 
      modalForWindow:window 
         delegate:self 
      didEndSelector:@selector(pageLayoutDidEnd:returnCode:contextInfo:) 
      contextInfo:printInfo];
}
@end

// wrappers
void *PrintInfoGetSharedPrintInfo( void )
{
 NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 NSPrintInfo *printInfo = [NSPrintInfo sharedPrintInfo];
 [pool drain];
 return (void *)printInfo;
}

CGSize PrintInfoGetPaperSize( void *printInfo )
{
 return NSSizeToCGSize( [(NSPrintInfo *)printInfo paperSize] );
}

UInt32 PrintInfoGetOrientation( void* printInfo )
{
 return [(NSPrintInfo *)printInfo orientation];
}

void PageLayoutShowSheet( WindowRef wndRef, OSStatus (*callback)(int,void*) )
{
 NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 if ( sPageLayoutController == nil ) {
  NSWindow *window = [[NSWindow alloc] initWithWindowRef:wndRef];
  sPageLayoutController = [[PageLayoutController alloc] init];
  [sPageLayoutController setCallback:callback];
  [sPageLayoutController runPageLayoutForWindow:window];
 }
 [pool drain];
}
EndC
toolbox fn PrintInfoGetSharedPrintInfo = pointer
toolbox fn PrintInfoGetPaperSize( pointer printInfo ) = CGSize
toolbox fn PrintInfoGetOrientation( pointer printInfo ) = UInt32
toolbox PageLayoutShowSheet( WindowRef wndRef, pointer callback )
toolbox fn NSApplicationLoad = Boolean

fn NSApplicationLoad()

local fn MyPageLayoutCallback( returnCode as SInt32, printInfo as pointer )
'~'1
dim as CGSize size

print "returnCode:",returnCode
print "orientation:",fn PrintInfoGetOrientation( printInfo )
size = fn PrintInfoGetPaperSize( printInfo )
print "size:",size.width,size.height
print

end fn

local fn DoMenu
'~'1
long if ( menu( _menuID ) == 1 and menu( _itemID ) == 1 )
PageLayoutShowSheet( window( _wndRef ), @fn MyPageLayoutCallback )
end if
end fn

menu 1, 0, 1, "File"
menu 1, 1, 1, "Page Setup"

window 1

on menu fn DoMenu

RunApplicationEventLoop()


