/*
 Demonstrates Tab Control in compositing window.
 Each tab has an embedded user pane and
  controls are embedded into user panes.
 Written by S.Van Voorst  {20140803}
 Revised to include HITextView  {20150424}
 Revised to include autoresizing  {20150424}
*/

include "Tlbx HIView.incl"
Include "Tlbx MacTextEditor.incl"
// *** Important - Panes numbered starting at 1 *** //
begin enum 1
_pane1
_pane2
_pane3
end enum

begin enum 11
_btn1
_btn2
end enum

_tab = 100
_tabCount = 3

local fn ViewBindToParent( view as HIViewRef )
'~'1
dim as HILayoutInfo  layout

layout.version = _kHILayoutInfoVersionZero
fn HIViewGetLayoutInfo( view, @layout )
layout.binding.top.kind = _kHILayoutBindTop
layout.binding.left.kind = _kHILayoutBindLeft
layout.binding.bottom.kind = _kHILayoutBindBottom
layout.binding.right.kind = _kHILayoutBindRight
fn HIViewSetLayoutInfo( view, @layout )

end fn

local fn buildWnd
'~'1
dim as Rect r
dim as ControlTabInfoRec infoRec
dim as WindowAttributes attr
dim as long x
dim as str15 tabTitleStr(_tabCount)
dim as HIRect     bounds, ScrollViewRect
dim as HIViewRef   scrollView, textView
dim as TXNFrameOptions txtOptions
dim as OptionBits   scrlOptions
dim as OSStatus    err

tabTitleStr(1) = "One"
tabTitleStr(2) = "Two"
tabTitleStr(3) = "Three"

_wndW = 540
_wndH = 325

attr = _kWindowStandardFloatingAttributes
attr += _kWindowStandardHandlerAttribute
attr += _kWindowResizableAttribute
attr += _kWindowCompositingAttribute
attr += _kWindowLiveResizeAttribute
attr += _kWindowFullZoomAttribute

// **** Compositing window **** //
SetRect( @r, 0, 0, _wndW, _wndH )
appearance window -1, @"Tab demo", @r, _kDocumentWindowClass, attr

fn SetWindowBackground( _kThemeActiveDialogBackgroundBrush, _zTrue )

// **** Tab control **** //
InsetRect( r, 20, 20 )
appearance button _tab, _activeBtn, 0, 0, _tabCount,, @r, _kControlTabLargeProc

InsetRect( r, 10, 10 )
for x = 1 to _tabCount
// **** Titles for tabs **** //
infoRec.version = _kControlTabInfoVersionZero
infoRec.iconSuiteID = 0
infoRec.name = tabTitleStr(x)
fn SetButtonData( _tab, x, _kControlTabInfoTag, sizeof( infoRec ), infoRec )
// **** User Panes - only top one is initially visible **** //
long if x == 1
appearance button x,,_kControlSupportsEmbedding,,,, @r, _kControlUserPaneProc // visible
xelse
appearance button -x,,_kControlSupportsEmbedding,,,, @r, _kControlUserPaneProc // invisible
end if
fn embedbutton( x, _tab )
fn ViewBindToParent( button&(_tab))
next

// **** Controls embedded into User panes **** //
SetRect( r, 30, 30, 148, 52)
appearance button _btn1, _activeBtn, ,,, @"Pane #1", @r, _kControlPushButtonProc
fn embedbutton( _btn1, _pane1)
fn ViewBindToParent( button&(_pane1))

OffsetRect( r, 20, 20 )
appearance button _btn2, _activeBtn, ,,, @"Pane #2", @r, _kControlPushButtonProc
fn embedbutton( _btn2, _pane2)
fn ViewBindToParent( button&(_pane2))

err = fn HIViewGetBounds( button&(_pane3), @bounds)

ScrollViewRect = fn CGRectMake( bounds.origin.x, bounds.origin.y+30, bounds.size.width - 40, bounds.size.height - 80)
scrlOptions = _kHIScrollViewOptionsVertScroll
err = fn HIScrollViewCreate( scrlOptions, @scrollView )

// Turn on autohide - vertical scroll appears when full of text
err = fn HIScrollViewSetScrollBarAutoHide( scrollView, _false )
err = fn HIViewAddSubview( button&(_pane3), scrollView )
err = fn HIViewSetVisible( scrollView, _true )
err = fn HIViewSetFrame( scrollView, ScrollViewRect)
fn ViewBindToParent( button&(_pane3))

txtOptions = _kOutputTextInUnicodeEncodingMask + _kTXNAlwaysWrapAtViewEdgeMask
err = fn HITextViewCreate( ScrollViewRect, 0, txtOptions, @textView )
err = fn HIViewAddSubview( scrollView, textView )
err = fn HIViewSetVisible( textView, _true )
fn ViewBindToParent( scrollView )

// CFString
err = fn HIViewSetText( textView, @"Hello. Type a bunch of text in here.")

appearance window 1

end fn

local fn doDialog
'~'1
dim as long evnt, id, j

evnt = dialog(0)
id = dialog(evnt)

select evnt
case _btnClick
select id
case _tab
for j = 1 to _tabCount
// *** Show/hide user panes *** //
long if j == button(_tab)
button j
xelse
button -j
end if
next
case _btn1
case _btn2
end select
end select

end fn

fn buildWnd
on dialog fn doDialog

RunApplicationEventLoop()

