/*

For Bernie, Steve Van Voorst and others who may be interested, below is a
Cocoa application shell that sports a full complement of working menus:

About Menu
File Menu
Edit Menu
Text Menu
Help Menu

There were several problems I had to overcome to make this nibless. The menu
structure is completely new. But once I figured out how to call the Apple menu,
the rest fell into place. (Apple did us no favors hiding the NSApp setAppleMenu
method. But after considerable research, I figured out how to call it. BTW, Apple
still calls it load xibs.)

I first struggled with adding the other menus, but then discovered that Cocoa
has a number of automatic methods for menus such as demonstrated in the Edit.
Text and Help menus in this demo.

I'm sure this code can be optimized, and I await your comments, criticisms, additions
and suggestions.

I want to thank Steve VV for his NSTextView_Ruler demo which inspired me to take
some free time to tackle the menu issues.

I think this demo is a proof of concept that programmatically constructing a nibless
Cocoa app is well within the realm of possibility.

Let me know what you think.

Ken
*/


/*

Application Shell Demo

This demo adds a functional menus to
the Simple XML Parser first posted on the 
fbcocoa list on May 8, 2012.

(It actually is a pretty good programatically
constructed application shell.)

Ken Shmidheiser
May 9, 2012

*Special thanks to Steve Van Voorst for his 
excellent text shell and to Bernie Wylde
for his assistance.

*/

output file "Application Shell Demo"

BeginCDeclaration
@interface AppController : NSObject
{
NSTextView *txtView;
}
@property (retain) NSTextView *txtView;

- (void)aboutMenuHandler:(id)sender;
- (void)preferencesMenuHandler:(id)sender;
- (void)helpMenuOne:(id)sender;
- (void)helpMenuTwo:(id)sender;
- (void)printView:(id)sender;
- (void)saveView:(id)sender;
- (void)openFile:(id)sender;
- (void)parseXML:(id)sender;

@end //interface
EndC


BeginCDeclaration
/*

NSApplication still responds to setAppleMenu method, but for some reason 
-- probably because Xcode applications are expected to be xib-based-- the
setAppleMenu method been omitted from Apple header files and documentation.

This declaration fixes that in a way similar to adding a Toolbox function
in FB.

*/
@interface NSApplication(NiblessAdditions)
- (void)setAppleMenu:(NSMenu *)menu;
@end 
EndC


BeginCFunction
@implementation AppController
@synthesize txtView;

- (void)aboutMenuHandler:(id)sender
{
NSDictionary *options;
NSAttributedString *attribStr;
NSString *creditStr = @"Inspired by Steve Van Voorst for fbcocoa list.\nWith help from Bernie Wylde and Ken Shmidheiser.";
attribStr = [[[NSMutableAttributedString alloc] initWithString: creditStr ] autorelease];
options = [NSDictionary dictionaryWithObjectsAndKeys:
attribStr, @"Credits",
@"Application Shell Demo", @"ApplicationName",
// NULL, @"ApplicationIcon",
@"0.1 alpha for list", @"Version",
// NULL", @"ApplicationVersion",
@"Copyright 2012", @"Copyright", nil];
[[NSApplication sharedApplication] orderFrontStandardAboutPanelWithOptions:options];
}


- (void)preferencesMenuHandler:(id)sender
{
// For demo only
NSBeep();
[txtView setString:@"\nYou just clicked the Preferences menu."];
}


- (void)helpMenuOne:(id)sender
{
// For demo only
NSBeep();
[txtView setString:@"\nYou just clicked the Help Menu's first item."];
}

- (void)helpMenuTwo:(id)sender
{
// For demo only
NSBeep();
[txtView setString:@"\nYou just clicked the Help Menu's second item."];
NSBeep();
}

- (void)printView:(id)sender
{
NSPrintInfo *printInfo = [NSPrintInfo sharedPrintInfo];
[printInfo setVerticallyCentered:NO];
[printInfo setHorizontallyCentered:YES];
[printInfo setTopMargin:60];
[printInfo setBottomMargin:60];
[printInfo setLeftMargin:45];
[printInfo setRightMargin:45];
[printInfo setOrientation:NSPortraitOrientation];

NSPrintOperation *printOp;
printOp = [NSPrintOperation printOperationWithView:txtView printInfo:printInfo];

//[printOp runOperationModalForWindow:[self window]
                           //delegate:nil
                     //didRunSelector:completeSel
                        //contextInfo:NULL];


//-- Run print operation --- 
[printOp runOperation];
}

- (void)saveView:(id)sender
{

NSSavePanel *sp = [NSSavePanel savePanel];
[sp setTitle:@"Save TextView contents to file"];
[sp setAllowedFileTypes:[NSArray arrayWithObject:@"txt"]];
[sp setCanSelectHiddenExtension:YES];
[sp setAllowsOtherFileTypes:YES];
[sp setCanCreateDirectories:YES];

NSInteger panelResult = [sp runModalForDirectory:nil file:@"Untitled.txt"];
if ( panelResult == NSFileHandlingPanelOKButton )
{
NSString *viewStr = [[txtView textStorage] string];
NSURL *url = [sp URL];
[viewStr writeToURL:url atomically:YES encoding:NSUTF8StringEncoding error:NULL];
}
else
{
// User cancelled
}
}

- (void)openFile:(id)sender
{
NSOpenPanel *op = [NSOpenPanel openPanel];
[op setTitle:@"Open generic text file"];
[op setCanChooseFiles:YES];
[op setCanChooseDirectories:NO];
[op setAllowsMultipleSelection: NO];
[op setAllowedFileTypes:[NSArray arrayWithObject:@"txt"]];

if ( [op runModal] == NSOKButton )
{
NSURL *url = [op URL];
NSString *fileStr = [[NSString alloc] initWithContentsOfURL:url encoding:NSUTF8StringEncoding error:NULL];
[txtView setString:fileStr];
[fileStr release];
}
else
{
// User cancelled
}
}

- (void)parseXML:(id)sender
{
NSXMLDocument *xmlDoc; 
NSError *err = nil; 

// Get NSURL for http://www.w3schools.com/xml/plant_catalog.xml
NSString *urlString = [@"http://www.w3schools.com/xml/plant_catalog.xml" stringByExpandingTildeInPath];
[urlString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
NSURL *url = [NSURL URLWithString:urlString];

// Build NSXMLDocument from NSURL
xmlDoc = [[NSXMLDocument alloc] initWithContentsOfURL:url options:NSXMLDocumentTidyXML error:&err]; 

// Identify XML root element
NSXMLElement *rootElement = [xmlDoc rootElement];

// Determine total number of nodes-- in this case for "PLANT" nodes
NSUInteger totalElements = [[rootElement nodesForXPath:@"//CATALOG/PLANT" error:&err] count];

// Build Xpath command string adding 1 to totalElements
NSString *XpathCmdStr = [NSString stringWithFormat:@"%@%d%@", @"/CATALOG/PLANT[position()<", totalElements + 1, @"]" ]; 

/*
Build array of "PLANT" nodes using Xpath command string
Done manuually, the completed instruction would look like this:
NSArray *nodes = [xmlDoc nodesForXPath:@"/CATALOG/PLANT[position()<37]" error:&err];
*/
NSArray *nodes = [xmlDoc nodesForXPath:XpathCmdStr error:&err];

NSMutableString *xmlTreeStr = [[[NSMutableString alloc] init] autorelease];
NSXMLElement *thisUser;

int i = 1;
for ( i = 0; i < [nodes count]; i++) {

thisUser = [nodes objectAtIndex:i];

[xmlTreeStr appendString:@"\nCommon Name: "];
[xmlTreeStr appendString:[[thisUser childAtIndex:0 ] stringValue]];

[xmlTreeStr appendString:@"\nBotanical Name: "];
[xmlTreeStr appendString:[[thisUser childAtIndex:1 ] stringValue]];

[xmlTreeStr appendString:@"\nZone: "];
[xmlTreeStr appendString:[[thisUser childAtIndex:2 ] stringValue]];

[xmlTreeStr appendString:@"\nNeeded Light: "];
[xmlTreeStr appendString:[[thisUser childAtIndex:3 ] stringValue]];

[xmlTreeStr appendString:@"\nPrice: "];
[xmlTreeStr appendString:[[thisUser childAtIndex:4 ] stringValue]];

[xmlTreeStr appendString:@"\nAvailability: "];
[xmlTreeStr appendString:[[thisUser childAtIndex:5 ] stringValue]];

[xmlTreeStr appendString:@"\n\n"];

}
// Set the resulting string into the TextView
[txtView setString:xmlTreeStr];
[xmlDoc release];
}
@end
EndC

BeginCCode
NSAutoreleasePool *pool = [[NSAutoreleasePool alloc]init];
[NSApplication sharedApplication];
[NSApp setActivationPolicy:NSApplicationActivationPolicyRegular];

// Create an instance of the controller class
id myController = [[AppController alloc]init];

// ******** Main Menu ********//
NSMenu *menubar = [[NSMenu alloc] initWithTitle: @""];
if (menubar) {
[NSApp setMainMenu: menubar];
[menubar release]; 
}

// ******** About Menu (atIndex:0) ********//
NSMenuItem *aboutMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Apple Menu Demo" action:NULL keyEquivalent:@""];
NSMenu *aboutMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@"About This Demo"];
[aboutMenuItem setEnabled:YES];
[aboutMenuItem setSubmenu:aboutMenu];
[aboutMenu release];
[[NSApp mainMenu] insertItem:aboutMenuItem atIndex:0];
[aboutMenuItem release];

NSMenuItem *aboutAppMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"About Application Shell Demo" action:@selector(aboutMenuHandler:) keyEquivalent:@""];
[aboutAppMenuItem setEnabled:YES];
[aboutAppMenuItem setTarget:myController];
[aboutMenu addItem:aboutAppMenuItem];
[aboutAppMenuItem release];

// Menu separator
[aboutMenu addItem: [NSMenuItem separatorItem]];

// Preferences menu
NSMenuItem *preferencesMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Preferences (beeps for demo)..." action:@selector(preferencesMenuHandler:) keyEquivalent:@","];
[preferencesMenuItem setEnabled:YES];
[preferencesMenuItem setTarget:myController];
[aboutMenu addItem:preferencesMenuItem];
[preferencesMenuItem release];

// Menu separator
[aboutMenu addItem: [NSMenuItem separatorItem]];

// Services menu
NSMenu *menuServices = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle: @"Services"];
[NSApp setServicesMenu:menuServices]; 
NSMenuItem *menuitem = [[NSMenuItem alloc] initWithTitle: @"Services" action:NULL keyEquivalent:@""];
[menuitem setSubmenu:menuServices];
[aboutMenu addItem: menuitem];
[menuitem release];
[menuServices release];

// Menu separator
[aboutMenu addItem: [NSMenuItem separatorItem]];

// Hide menu
NSMenuItem *hideMenuitem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Hide" action:@selector(hide:) keyEquivalent:@""];
[hideMenuitem setTarget: NSApp];
[aboutMenu addItem: hideMenuitem];
[hideMenuitem release];

// Hide Others menu
NSMenuItem *hideOthersMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Hide Others" action:@selector(hideOtherApplications:) keyEquivalent:@""];
[hideOthersMenuItem setTarget: NSApp];
[aboutMenu addItem: hideOthersMenuItem];
[hideOthersMenuItem release];

// Show All menu
NSMenuItem *showAllMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Show All" action:@selector(unhideAllApplications:) keyEquivalent:@""];
[showAllMenuItem setTarget: NSApp];
[aboutMenu addItem: showAllMenuItem];
[showAllMenuItem release];

// Menu separator
[aboutMenu addItem: [NSMenuItem separatorItem]];

// Quit menu
NSMenuItem *quitMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Quit" action:@selector(terminate:) keyEquivalent:@"q"];
[quitMenuItem setTarget: NSApp];
[aboutMenu addItem: quitMenuItem];
[quitMenuItem release];


// ******** File Menu (atIndex:1) ********//
NSMenuItem *fileMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"File" action:NULL keyEquivalent:@""];
NSMenu *fileMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@"File"];
[fileMenuItem setEnabled:YES];
[fileMenuItem setSubmenu:fileMenu];
[fileMenu release];
[[NSApp mainMenu] insertItem:fileMenuItem atIndex:1];
[fileMenuItem release];

NSMenuItem *openMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Open..." action:@selector(openFile:) keyEquivalent:@"o"];
[openMenuItem setEnabled:YES];
[openMenuItem setTarget:myController];
[fileMenu addItem:openMenuItem];
[openMenuItem release];

// Menu separator
[fileMenu addItem: [NSMenuItem separatorItem]];

NSMenuItem* parseXMLMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Parse XML..." action:@selector(parseXML:) keyEquivalent:@"g"];
[parseXMLMenuItem setEnabled:YES];
[parseXMLMenuItem setTarget:myController];
[fileMenu addItem:parseXMLMenuItem];
[parseXMLMenuItem release];

// Menu separator
[fileMenu addItem: [NSMenuItem separatorItem]];

NSMenuItem* saveMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Save" action:@selector(saveView:) keyEquivalent:@"s"];
[saveMenuItem setEnabled:YES];
[saveMenuItem setTarget:myController];
[fileMenu addItem:saveMenuItem];
[saveMenuItem release];

// Menu separator
[fileMenu addItem: [NSMenuItem separatorItem]];

NSMenuItem* printMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Print" action:@selector(printView:) keyEquivalent:@"p"];
[printMenuItem setEnabled:YES];
[printMenuItem setTarget:myController];
[fileMenu addItem:printMenuItem];
[printMenuItem release];


// ******** Edit Menu (atIndex:2) ********//
NSMenuItem *editMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Edit" action:NULL keyEquivalent:@""];
NSMenu *editMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@"Edit"];
[editMenuItem setEnabled:YES];
[editMenuItem setSubmenu:editMenu];
[editMenu release];
[[NSApp mainMenu] insertItem:editMenuItem atIndex:2];
[editMenuItem release];

// Edit menu automatic system selectors:
[editMenu addItemWithTitle: @"Cut" action: @selector(cut:) keyEquivalent: @"x"];
[editMenu addItemWithTitle: @"Copy" action: @selector(copy:) keyEquivalent: @"c"];
[editMenu addItemWithTitle: @"Paste" action: @selector(paste:) keyEquivalent: @"v"];
[editMenu addItemWithTitle: @"Delete" action: @selector(delete:) keyEquivalent: @""];
// Programmer must write method for Undelete
[editMenu addItemWithTitle: @"Undelete" action: NULL keyEquivalent: @""];
[editMenu addItemWithTitle: @"Select All" action: @selector(selectAll:) keyEquivalent: @"a"];


// ******** Text Menu (atIndex:3) ********//
NSMenuItem *textMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Text" action:NULL keyEquivalent:@""];
NSMenu *textMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@"Text"];
[textMenuItem setEnabled:YES];
[textMenuItem setSubmenu:textMenu];
[textMenu release];
[[NSApp mainMenu] insertItem:textMenuItem atIndex:3];
[textMenuItem release];

// Text handler automatic system selectors:
[textMenu addItemWithTitle: @"Align Left" action: @selector(alignLeft:) keyEquivalent: @"{"];
[textMenu addItemWithTitle: @"Center" action: @selector(alignCenter:) keyEquivalent: @"-"];
[textMenu addItemWithTitle: @"Align Right" action: @selector(alignRight:) keyEquivalent: @"}"];

// Menu separator
[textMenu addItem: [NSMenuItem separatorItem]];
[textMenu addItemWithTitle: @"Show Ruler" action: @selector(toggleRuler:) keyEquivalent: @""];
[textMenu addItemWithTitle: @"Copy Ruler" action: @selector(copyRuler:) keyEquivalent: @"1"];
[textMenu addItemWithTitle: @"Paste Ruler" action: @selector(pasteRuler:) keyEquivalent: @"2"];


// ******** Help Menu (atIndex:4) ********//
NSMenuItem *helpMenuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Help" action:NULL keyEquivalent:@""];
NSMenu *helpMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:@"Help"];
[NSApp setHelpMenu:helpMenu];
[helpMenuItem setEnabled:YES];
[helpMenuItem setSubmenu:helpMenu];
[helpMenu release];
[[NSApp mainMenu] insertItem:helpMenuItem atIndex:4];
[helpMenuItem release];

NSMenuItem* helpMenuItemOne = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Help Menu Item One..." action:@selector(helpMenuOne:) keyEquivalent:@"h"];
[helpMenuItemOne setEnabled:YES];
[helpMenuItemOne setTarget:myController];
[helpMenu addItem:helpMenuItemOne];
[helpMenuItemOne release];

// Menu separator
[helpMenu addItem: [NSMenuItem separatorItem]];

NSMenuItem* helpMenuItemTwo = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@"Help Menu Item Two..." action:@selector(helpMenuTwo:) keyEquivalent:@"H"];
[helpMenuItemTwo setEnabled:YES];
[helpMenuItemTwo setTarget:myController];
[helpMenu addItem:helpMenuItemTwo];
[helpMenuItemTwo release];


// ******** Window ******** //
#define _wndW 680
#define _wndH 550
id window = [[NSWindow alloc] initWithContentRect:NSMakeRect( 0, 0, _wndW, _wndH )
styleMask:NSTitledWindowMask | NSClosableWindowMask
backing:NSBackingStoreBuffered
defer:NO];
[window center];
[window setTitle:@"NSScrollView+Ruler"];
[window makeKeyAndOrderFront:nil];

// ****** NSTextView with Scroll/Ruler ****** //
NSScrollView *scrlView = [[NSScrollView alloc] 
initWithFrame:NSMakeRect( 30, 70, _wndW - 60, _wndH - 70 )];
[scrlView setHasVerticalScroller: YES];
NSTextView *txtView = [[NSTextView alloc]
initWithFrame:NSMakeRect( 0, 120, _wndW - 60, _wndH - 200 )];
[scrlView setDocumentView: txtView];
[[window contentView] addSubview:scrlView];
[txtView insertText:@"\nWelcome to Application Shell Demo!\n\nAll menus and buttons are working. Feel free to try them all.\n\nKen Shmidheiser, May 9, 2012"];
[scrlView setHasHorizontalRuler:YES];
[scrlView setRulersVisible:YES];
[window makeFirstResponder:txtView];
[myController setTxtView:txtView];
[scrlView release];
[txtView release];

// ***** Parse XML btn ***** //
NSButton *parseXMLBtn = [[NSButton alloc]
initWithFrame:NSMakeRect( 130, 30, 120, 30 )];
[parseXMLBtn setBezelStyle:NSRoundedBezelStyle ];
[parseXMLBtn setTitle: @"Parse XML..." ];
[parseXMLBtn setTarget:myController];
[parseXMLBtn setAction:@selector(parseXML:)];
[[window contentView] addSubview: parseXMLBtn];
[parseXMLBtn release];

// ***** Print btn ***** //
NSButton *printBtn = [[NSButton alloc]
initWithFrame:NSMakeRect( 250, 30, 100, 30 )];
[printBtn setBezelStyle:NSRoundedBezelStyle ];
[printBtn setTitle: @"Print" ];
[printBtn setTarget:myController];
[printBtn setAction:@selector(printView:)];
[[window contentView] addSubview: printBtn];
[printBtn release];

// ***** Open btn ***** //
NSButton *openBtn = [[NSButton alloc]
initWithFrame:NSMakeRect( 350, 30, 100, 30 )];
[openBtn setBezelStyle:NSRoundedBezelStyle ];
[openBtn setTitle: @"Open..." ];
[openBtn setTarget:myController];
[openBtn setAction:@selector(openFile:)];
[[window contentView] addSubview: openBtn];
[openBtn release];

// ***** Save btn ***** //
NSButton *saveBtn = [[NSButton alloc]
initWithFrame:NSMakeRect( 450, 30, 100, 30 )];
[saveBtn setBezelStyle:NSRoundedBezelStyle ];
[saveBtn setTitle: @"Save" ];
[saveBtn setTarget:myController];
[saveBtn setAction:@selector(saveView:)];
[[window contentView] addSubview: saveBtn];
[saveBtn release];

// ***** Quit btn ***** //
NSButton *quitBtn = [[NSButton alloc]
initWithFrame:NSMakeRect( _wndW - 130, 30, 95, 30 )];
[quitBtn setBezelStyle:NSRoundedBezelStyle ];
[quitBtn setTitle: @"Quit" ];
[quitBtn setAction:@selector(terminate:)];
[[window contentView] addSubview: quitBtn];
[quitBtn release];

[NSApp activateIgnoringOtherApps:YES];
[NSApp run];
[myController release];
[pool drain];
EndC

