'--------------------------------------
/*
   Basic SQLite3 tutorial using C interfaces.

   Same approach as demo found at: FB_5_7_6_Examples/Miscellaneous/SQLite_demo/SQLite_proof_of_concept
   but uses CFStrings instead of pascal strings and thus avoids those limitations.


   Change the DB path @"/Users/yourUserID/Documents/MyFirstDB"  as appropriate

   N.B. minimally tested. Error handling is marginal.

   Brian S  20130407
*/

include "ConsoleWindow"
include "Tlbx SQLite3.incl"


// Function to convert CFString to C String - Ken Shmidheiser Sept. 29, 2011
local fn CopyCFStringToCString( aCFStr as CFStringRef ) as pointer
'~'1
dim as pointer  cStrBuf : cStrBuf = NULL
dim as CFIndex  length

length = fn CFStringGetMaximumSizeForEncoding( fn CFStringGetLength( aCFStr ), _kCFStringEncodingUTF8 ) + 1
cStrBuf = fn malloc( length )
if ( cStrBuf )  then fn CFStringGetCString( aCFStr, #cStrBuf, length, _kCFStringEncodingUTF8 )
end fn = cStrBuf     // User must release with free( buffer )



local fn DoSQLExec( s as CFStringRef, dbPtr as sqlite3Ptr )
'~'1
dim as long            rc
dim as pointer         cStringPtr

fn ConsolePrintCFString( s )
cStringPtr = fn CopyCFStringToCString( s )
rc = fn sqlite3_exec( dbPtr, #cStringPtr, 0, 0, 0 )
free( cStringPtr )

if ( rc )
s = fn CFStringCreateWithFormat( _kCFAllocatorDefault, #0, @"%@%ld" , @"  Error rc = ", rc )
fn ConsolePrintCFString( s )
CFRelease( s )
end if

end fn


'~'1
dim as CFStringRef      s, dbName : dbName = @"Opening MyFirstDB"
dim as sqlite3Ptr       myDbHandle
dim as long            rc
dim as pointer         cStringPtr

// announce
s = fn CFStringCreateWithFormat( _kCFAllocatorDefault, #0, @"%@ %d", @"SQLite version:" , fn sqlite3_libversion_number() )
fn ConsolePrintCFString( s )
CFRelease( s )

// open DB
fn ConsolePrintCFString( dbName )
cStringPtr = fn CopyCFStringToCString( @"/Users/brians/Documents/MyFirstDB" )
rc = fn sqlite3_open( #cStringPtr, @myDbHandle )
if ( rc )
s = fn CFStringCreateWithFormat( _kCFAllocatorDefault, #0, @"%@%ld" , @"  Error rc = ", rc )
fn ConsolePrintCFString( s )
CFRelease( s )
end if
free( cStringPtr )

// populate DB
fn DoSQLExec( @"DROP TABLE IF EXISTS memos;", myDbHandle )
fn DoSQLExec( @"CREATE TABLE memos( task TEXT, priority INTEGER );", myDbHandle )
fn DoSQLExec( @"INSERT INTO memos VALUES( 'finish this demo', 10 );", myDbHandle )
fn DoSQLExec( @"INSERT INTO memos VALUES( 'drink beer', 100 );", myDbHandle )

// query how many rows changed
s = fn CFStringCreateWithFormat( _kCFAllocatorDefault, #0, @"%@ %d", @"Rows changed:" , fn sqlite3_total_changes( myDBHandle ) )
fn ConsolePrintCFString( s )
CFRelease( s )

// close DB
fn sqlite3_close( myDbHandle )
'--------------------------------------

