'------------
/*
Print to Preview of columnar data using CoreText and Core Printing.

(1) Prints rudimentary columns with headings as a proof of concept
(2) fn PMSessionSetDestination used to send output to Preview app
(3) fn PMSessionGetCGGraphicsContext used to obtain CGContextRef.
(4) QDBeginCGContext/QDEndCGContext not required to obtain a CGContextRef due to #3
(5) Draw handler not necessary and we don't want a carbon event handler
(6) Non-deprecated APIs used such as PMSessionBeginCGDocumentNoDialog
(7) Demo will run in 64-bit with -DDECARBONATE -m64  in 'More compiler options'.

Known flaws and/or needing improvement:
(1) Pagination - page breaks are arbitrary for the demo and typically would be calculated.
(2) No error checking in the include file
(3) fn PrintLine could use more options ( color choice, italic choice etc. )

by Brian S. 14-June-2011 
*/
if ( system( _sysVers ) < 1050 ) then stop "Requires OS X 10.5 or greater"

include "Util_PrintColumnarData.incl"

_myLinesPerPage  = 40 // change to suit
_myColumnTabStop = 100// Tab stops are all 100 here but there is no need for them to be the same

local fn PrintReportHeaders( ctx as CGContextRef )
'~'1
dim as CTFontRef         font
dim as CFStringRef       string
dim as CFIndex           i
dim as CGFloat           x, y

x = 20.0 : y = 550.0// should be measured instead of hard-code
font   = fn CTFontCreateWithName( @"Lucida Grande Bold", 14.0, 0 ) 

for i = 1 to 6
string = fn CFStringCreateWithFormat( _kCFAllocatorDefault, #0, @"%@%ld", @"Head# ", i )
fn PrintLine( ctx, string, x, y, font, _true )
CFRelease( string )
x += _myColumnTabStop
next

CFRelease( font)

end fn = y

local fn DoPrinting
'~'1
dim as PMPrintSession    printSession  : printSession  = 0
dim as PMPrintSettings   printSettings : printSettings = 0
dim as CGContextRef      ctx
dim as CGFloat           x, y, ascent, descent, yOffset, xOffset
dim as PMPageFormat      pageFormat
dim as CTFontRef         font
dim as CFStringRef       string
dim as CFIndex           i,j

// fn SetUpPrinting must be called first
fn SetUpPrinting( @printSession, @printSettings, @pageFormat, @ctx, _kPMLandscape )//_kPMLandscape or _kPMPortrait

y = fn PrintReportHeaders( ctx )

font    = fn CTFontCreateWithName( @"Lucida Grande", 10.0, 0 )
ascent  = fn CTFontGetAscent( font )
descent = fn CTFontGetDescent( font )
yOffset = ascent + descent  + 1
xOffset = 20

// Outer loop is # of detail rows & inner loop # of columns in each row
for i = 1 to 85 // 85 rows
y -= yOffset :  x = xOffset

long if ( i mod _myLinesPerPage == 0 ) // page break at 40 lines per page
fn DoPageBreak( @printSession, @pageFormat, @ctx )
y = fn PrintReportHeaders( ctx )
y -= yOffset :  x = xOffset
end if

for j = 1 to 6// 6 columns of data
string = fn CFStringCreateWithFormat( _kCFAllocatorDefault, #0, @"%@ %ld %ld", @"Data# ", i,j )
fn PrintLine( ctx, string, x, y, font, _false )
CFRelease( string )
x += _myColumnTabStop
next

next

CFRelease( font )

// fn EndPrintSession must be called at the end of the print job
// It releases the 'printed' document to the final destination ( i.e. printer, Preview ap, file etc. ).
fn EndPrintSession( @printSession, @printSettings, @pageFormat )

end fn

fn DoPrinting()

end
'------------


